part of 'resource.dart';

class FhirField {
  const FhirField(this.isList, this.type);
  final bool isList;
  final String type;
}

/// Because I don't want to root around in StructureDefinitions just to see
/// what kind of an object something is or if it's a list, I'm using this
final Map<String, Map<String, FhirField>> fhirFieldMap =
    <String, Map<String, FhirField>>{
  'Element': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
  },
  'FhirExtension': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'url': const FhirField(false, 'FhirUri'),
    'urlElement': const FhirField(false, 'Element'),
    'valueBase64Binary': const FhirField(false, 'Base64Binary'),
    'valueBase64BinaryElement': const FhirField(false, 'Element'),
    'valueBoolean': const FhirField(false, 'Boolean'),
    'valueBooleanElement': const FhirField(false, 'Element'),
    'valueCanonical': const FhirField(false, 'Canonical'),
    'valueCanonicalElement': const FhirField(false, 'Element'),
    'valueCode': const FhirField(false, 'Code'),
    'valueCodeElement': const FhirField(false, 'Element'),
    'valueDate': const FhirField(false, 'Date'),
    'valueDateElement': const FhirField(false, 'Element'),
    'valueDateTime': const FhirField(false, 'FhirDateTime'),
    'valueDateTimeElement': const FhirField(false, 'Element'),
    'valueDecimal': const FhirField(false, 'Decimal'),
    'valueDecimalElement': const FhirField(false, 'Element'),
    'valueId': const FhirField(false, 'Id'),
    'valueIdElement': const FhirField(false, 'Element'),
    'valueInstant': const FhirField(false, 'Instant'),
    'valueInstantElement': const FhirField(false, 'Element'),
    'valueInteger': const FhirField(false, 'Integer'),
    'valueIntegerElement': const FhirField(false, 'Element'),
    'valueInteger64': const FhirField(false, 'Integer64'),
    'valueInteger64Element': const FhirField(false, 'Element'),
    'valueMarkdown': const FhirField(false, 'Markdown'),
    'valueMarkdownElement': const FhirField(false, 'Element'),
    'valueOid': const FhirField(false, 'Id'),
    'valueOidElement': const FhirField(false, 'Element'),
    'valuePositiveInt': const FhirField(false, 'PositiveInt'),
    'valuePositiveIntElement': const FhirField(false, 'Element'),
    'valueString': const FhirField(false, 'String'),
    'valueStringElement': const FhirField(false, 'Element'),
    'valueTime': const FhirField(false, 'Time'),
    'valueTimeElement': const FhirField(false, 'Element'),
    'valueUnsignedInt': const FhirField(false, 'UnsignedInt'),
    'valueUnsignedIntElement': const FhirField(false, 'Element'),
    'valueUri': const FhirField(false, 'FhirUri'),
    'valueUriElement': const FhirField(false, 'Element'),
    'valueUrl': const FhirField(false, 'FhirUrl'),
    'valueUrlElement': const FhirField(false, 'Element'),
    'valueUuid': const FhirField(false, 'Id'),
    'valueUuidElement': const FhirField(false, 'Element'),
    'valueAddress': const FhirField(false, 'Address'),
    'valueAge': const FhirField(false, 'Age'),
    'valueAnnotation': const FhirField(false, 'Annotation'),
    'valueAttachment': const FhirField(false, 'Attachment'),
    'valueCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'valueCodeableReference': const FhirField(false, 'CodeableReference'),
    'valueCoding': const FhirField(false, 'Coding'),
    'valueContactPoint': const FhirField(false, 'ContactPoint'),
    'valueCount': const FhirField(false, 'Count'),
    'valueDistance': const FhirField(false, 'Distance'),
    'valueDuration': const FhirField(false, 'FhirDuration'),
    'valueHumanName': const FhirField(false, 'HumanName'),
    'valueIdentifier': const FhirField(false, 'Identifier'),
    'valueMoney': const FhirField(false, 'Money'),
    'valuePeriod': const FhirField(false, 'Period'),
    'valueQuantity': const FhirField(false, 'Quantity'),
    'valueRange': const FhirField(false, 'Range'),
    'valueRatio': const FhirField(false, 'Ratio'),
    'valueRatioRange': const FhirField(false, 'RatioRange'),
    'valueReference': const FhirField(false, 'Reference'),
    'valueSampledData': const FhirField(false, 'SampledData'),
    'valueSignature': const FhirField(false, 'Signature'),
    'valueTiming': const FhirField(false, 'Timing'),
    'valueContactDetail': const FhirField(false, 'ContactDetail'),
    'valueDataRequirement': const FhirField(false, 'DataRequirement'),
    'valueExpression': const FhirField(false, 'Expression'),
    'valueParameterDefinition': const FhirField(false, 'ParameterDefinition'),
    'valueRelatedArtifact': const FhirField(false, 'RelatedArtifact'),
    'valueTriggerDefinition': const FhirField(false, 'TriggerDefinition'),
    'valueUsageContext': const FhirField(false, 'UsageContext'),
    'valueAvailability': const FhirField(false, 'Availability'),
    'valueExtendedContactDetail':
        const FhirField(false, 'ExtendedContactDetail'),
    'valueDosage': const FhirField(false, 'Dosage'),
    'valueMeta': const FhirField(false, 'Meta'),
  },
  'Narrative': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'div': const FhirField(false, 'Markdown'),
  },
  'Annotation': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'authorReference': const FhirField(false, 'Reference'),
    'authorString': const FhirField(false, 'String'),
    'authorStringElement': const FhirField(false, 'Element'),
    'time': const FhirField(false, 'FhirDateTime'),
    'timeElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Markdown'),
    'textElement': const FhirField(false, 'Element'),
  },
  'Attachment': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'contentType': const FhirField(false, 'Code'),
    'contentTypeElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'data': const FhirField(false, 'Base64Binary'),
    'dataElement': const FhirField(false, 'Element'),
    'url': const FhirField(false, 'FhirUrl'),
    'urlElement': const FhirField(false, 'Element'),
    'size': const FhirField(false, 'Integer64'),
    'sizeElement': const FhirField(false, 'Element'),
    'hash': const FhirField(false, 'Base64Binary'),
    'hashElement': const FhirField(false, 'Element'),
    'title': const FhirField(false, 'String'),
    'titleElement': const FhirField(false, 'Element'),
    'creation': const FhirField(false, 'FhirDateTime'),
    'creationElement': const FhirField(false, 'Element'),
    'height': const FhirField(false, 'PositiveInt'),
    'heightElement': const FhirField(false, 'Element'),
    'width': const FhirField(false, 'PositiveInt'),
    'widthElement': const FhirField(false, 'Element'),
    'frames': const FhirField(false, 'PositiveInt'),
    'framesElement': const FhirField(false, 'Element'),
    'duration': const FhirField(false, 'Decimal'),
    'durationElement': const FhirField(false, 'Element'),
    'pages': const FhirField(false, 'PositiveInt'),
    'pagesElement': const FhirField(false, 'Element'),
  },
  'Identifier': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'use': const FhirField(false, 'Code'),
    'useElement': const FhirField(false, 'Element'),
    'type': const FhirField(false, 'CodeableConcept'),
    'system': const FhirField(false, 'FhirUri'),
    'systemElement': const FhirField(false, 'Element'),
    'value': const FhirField(false, 'String'),
    'valueElement': const FhirField(false, 'Element'),
    'period': const FhirField(false, 'Period'),
    'assigner': const FhirField(false, 'Reference'),
  },
  'CodeableConcept': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'coding': const FhirField(true, 'Coding'),
    'text': const FhirField(false, 'String'),
    'textElement': const FhirField(false, 'Element'),
  },
  'CodeableReference': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'concept': const FhirField(false, 'CodeableConcept'),
    'reference': const FhirField(false, 'Reference'),
  },
  'Coding': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'system': const FhirField(false, 'FhirUri'),
    'systemElement': const FhirField(false, 'Element'),
    'version': const FhirField(false, 'String'),
    'versionElement': const FhirField(false, 'Element'),
    'code': const FhirField(false, 'Code'),
    'codeElement': const FhirField(false, 'Element'),
    'display': const FhirField(false, 'String'),
    'displayElement': const FhirField(false, 'Element'),
    'userSelected': const FhirField(false, 'Boolean'),
    'userSelectedElement': const FhirField(false, 'Element'),
  },
  'Quantity': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'value': const FhirField(false, 'Decimal'),
    'valueElement': const FhirField(false, 'Element'),
    'comparator': const FhirField(false, 'Code'),
    'comparatorElement': const FhirField(false, 'Element'),
    'unit': const FhirField(false, 'String'),
    'unitElement': const FhirField(false, 'Element'),
    'system': const FhirField(false, 'FhirUri'),
    'systemElement': const FhirField(false, 'Element'),
    'code': const FhirField(false, 'Code'),
    'codeElement': const FhirField(false, 'Element'),
  },
  'Distance': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'value': const FhirField(false, 'Decimal'),
    'valueElement': const FhirField(false, 'Element'),
    'comparator': const FhirField(false, 'Code'),
    'comparatorElement': const FhirField(false, 'Element'),
    'unit': const FhirField(false, 'String'),
    'unitElement': const FhirField(false, 'Element'),
    'system': const FhirField(false, 'FhirUri'),
    'systemElement': const FhirField(false, 'Element'),
    'code': const FhirField(false, 'Code'),
    'codeElement': const FhirField(false, 'Element'),
  },
  'Count': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'value': const FhirField(false, 'Decimal'),
    'valueElement': const FhirField(false, 'Element'),
    'comparator': const FhirField(false, 'Code'),
    'comparatorElement': const FhirField(false, 'Element'),
    'unit': const FhirField(false, 'String'),
    'unitElement': const FhirField(false, 'Element'),
    'system': const FhirField(false, 'FhirUri'),
    'systemElement': const FhirField(false, 'Element'),
    'code': const FhirField(false, 'Code'),
    'codeElement': const FhirField(false, 'Element'),
  },
  'FhirDuration': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'value': const FhirField(false, 'Decimal'),
    'valueElement': const FhirField(false, 'Element'),
    'comparator': const FhirField(false, 'Code'),
    'unit': const FhirField(false, 'String'),
    'unitElement': const FhirField(false, 'Element'),
    'system': const FhirField(false, 'FhirUri'),
    'systemElement': const FhirField(false, 'Element'),
    'code': const FhirField(false, 'Code'),
    'codeElement': const FhirField(false, 'Element'),
  },
  'Money': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'value': const FhirField(false, 'Decimal'),
    'valueElement': const FhirField(false, 'Element'),
    'currency': const FhirField(false, 'Code'),
    'currencyElement': const FhirField(false, 'Element'),
  },
  'Age': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'value': const FhirField(false, 'Decimal'),
    'valueElement': const FhirField(false, 'Element'),
    'comparator': const FhirField(false, 'Code'),
    'comparatorElement': const FhirField(false, 'Element'),
    'unit': const FhirField(false, 'String'),
    'unitElement': const FhirField(false, 'Element'),
    'system': const FhirField(false, 'FhirUri'),
    'systemElement': const FhirField(false, 'Element'),
    'code': const FhirField(false, 'Code'),
    'codeElement': const FhirField(false, 'Element'),
  },
  'Range': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'low': const FhirField(false, 'Quantity'),
    'high': const FhirField(false, 'Quantity'),
  },
  'Period': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'start': const FhirField(false, 'FhirDateTime'),
    'startElement': const FhirField(false, 'Element'),
    'end': const FhirField(false, 'FhirDateTime'),
    'endElement': const FhirField(false, 'Element'),
  },
  'Ratio': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'numerator': const FhirField(false, 'Quantity'),
    'denominator': const FhirField(false, 'Quantity'),
  },
  'RatioRange': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'lowNumerator': const FhirField(false, 'Quantity'),
    'highNumerator': const FhirField(false, 'Quantity'),
    'denominator': const FhirField(false, 'Quantity'),
  },
  'Reference': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'reference': const FhirField(false, 'String'),
    'referenceElement': const FhirField(false, 'Element'),
    'type': const FhirField(false, 'FhirUri'),
    'typeElement': const FhirField(false, 'Element'),
    'identifier': const FhirField(false, 'Identifier'),
    'display': const FhirField(false, 'String'),
    'displayElement': const FhirField(false, 'Element'),
  },
  'SampledData': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'origin': const FhirField(false, 'Quantity'),
    'interval': const FhirField(false, 'Decimal'),
    'intervalElement': const FhirField(false, 'Element'),
    'intervalUnit': const FhirField(false, 'Code'),
    'intervalUnitElement': const FhirField(false, 'Element'),
    'factor': const FhirField(false, 'Decimal'),
    'factorElement': const FhirField(false, 'Element'),
    'lowerLimit': const FhirField(false, 'Decimal'),
    'lowerLimitElement': const FhirField(false, 'Element'),
    'upperLimit': const FhirField(false, 'Decimal'),
    'upperLimitElement': const FhirField(false, 'Element'),
    'dimensions': const FhirField(false, 'PositiveInt'),
    'dimensionsElement': const FhirField(false, 'Element'),
    'codeMap': const FhirField(false, 'Canonical'),
    'offsets': const FhirField(false, 'String'),
    'offsetsElement': const FhirField(false, 'Element'),
    'data': const FhirField(false, 'String'),
    'dataElement': const FhirField(false, 'Element'),
  },
  'Signature': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(true, 'Coding'),
    'when': const FhirField(false, 'Instant'),
    'whenElement': const FhirField(false, 'Element'),
    'who': const FhirField(false, 'Reference'),
    'onBehalfOf': const FhirField(false, 'Reference'),
    'targetFormat': const FhirField(false, 'Code'),
    'targetFormatElement': const FhirField(false, 'Element'),
    'sigFormat': const FhirField(false, 'Code'),
    'sigFormatElement': const FhirField(false, 'Element'),
    'data': const FhirField(false, 'Base64Binary'),
    'dataElement': const FhirField(false, 'Element'),
  },
  'HumanName': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'use': const FhirField(false, 'Code'),
    'useElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'String'),
    'textElement': const FhirField(false, 'Element'),
    'family': const FhirField(false, 'String'),
    'familyElement': const FhirField(false, 'Element'),
    'given': const FhirField(true, 'String'),
    'givenElement': const FhirField(true, 'Element'),
    'prefix': const FhirField(true, 'String'),
    'prefixElement': const FhirField(true, 'Element'),
    'suffix': const FhirField(true, 'String'),
    'suffixElement': const FhirField(true, 'Element'),
    'period': const FhirField(false, 'Period'),
  },
  'Address': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'use': const FhirField(false, 'Code'),
    'useElement': const FhirField(false, 'Element'),
    'type': const FhirField(false, 'Code'),
    'typeElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'String'),
    'textElement': const FhirField(false, 'Element'),
    'line': const FhirField(true, 'String'),
    'lineElement': const FhirField(true, 'Element'),
    'city': const FhirField(false, 'String'),
    'cityElement': const FhirField(false, 'Element'),
    'district': const FhirField(false, 'String'),
    'districtElement': const FhirField(false, 'Element'),
    'state': const FhirField(false, 'String'),
    'stateElement': const FhirField(false, 'Element'),
    'postalCode': const FhirField(false, 'String'),
    'postalCodeElement': const FhirField(false, 'Element'),
    'country': const FhirField(false, 'String'),
    'countryElement': const FhirField(false, 'Element'),
    'period': const FhirField(false, 'Period'),
  },
  'ContactPoint': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'system': const FhirField(false, 'Code'),
    'systemElement': const FhirField(false, 'Element'),
    'value': const FhirField(false, 'String'),
    'valueElement': const FhirField(false, 'Element'),
    'use': const FhirField(false, 'Code'),
    'useElement': const FhirField(false, 'Element'),
    'rank': const FhirField(false, 'PositiveInt'),
    'rankElement': const FhirField(false, 'Element'),
    'period': const FhirField(false, 'Period'),
  },
  'Timing': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'event': const FhirField(true, 'FhirDateTime'),
    'eventElement': const FhirField(true, 'Element'),
    'repeat': const FhirField(false, 'TimingRepeat'),
    'code': const FhirField(false, 'CodeableConcept'),
  },
  'TimingRepeat': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'boundsDuration': const FhirField(false, 'FhirDuration'),
    'boundsRange': const FhirField(false, 'Range'),
    'boundsPeriod': const FhirField(false, 'Period'),
    'count': const FhirField(false, 'PositiveInt'),
    'countElement': const FhirField(false, 'Element'),
    'countMax': const FhirField(false, 'PositiveInt'),
    'countMaxElement': const FhirField(false, 'Element'),
    'duration': const FhirField(false, 'Decimal'),
    'durationElement': const FhirField(false, 'Element'),
    'durationMax': const FhirField(false, 'Decimal'),
    'durationMaxElement': const FhirField(false, 'Element'),
    'durationUnit': const FhirField(false, 'Code'),
    'durationUnitElement': const FhirField(false, 'Element'),
    'frequency': const FhirField(false, 'PositiveInt'),
    'frequencyElement': const FhirField(false, 'Element'),
    'frequencyMax': const FhirField(false, 'PositiveInt'),
    'frequencyMaxElement': const FhirField(false, 'Element'),
    'period': const FhirField(false, 'Decimal'),
    'periodElement': const FhirField(false, 'Element'),
    'periodMax': const FhirField(false, 'Decimal'),
    'periodMaxElement': const FhirField(false, 'Element'),
    'periodUnit': const FhirField(false, 'Code'),
    'periodUnitElement': const FhirField(false, 'Element'),
    'dayOfWeek': const FhirField(true, 'Code'),
    'dayOfWeekElement': const FhirField(true, 'Element'),
    'timeOfDay': const FhirField(true, 'Time'),
    'timeOfDayElement': const FhirField(true, 'Element'),
    'when': const FhirField(true, 'Code'),
    'whenElement': const FhirField(true, 'Element'),
    'offset': const FhirField(false, 'UnsignedInt'),
    'offsetElement': const FhirField(false, 'Element'),
  },
  'Meta': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'versionId': const FhirField(false, 'Id'),
    'versionIdElement': const FhirField(false, 'Element'),
    'lastUpdated': const FhirField(false, 'Instant'),
    'lastUpdatedElement': const FhirField(false, 'Element'),
    'source': const FhirField(false, 'FhirUri'),
    'sourceElement': const FhirField(false, 'Element'),
    'profile': const FhirField(true, 'Canonical'),
    'security': const FhirField(true, 'Coding'),
    'tag': const FhirField(true, 'Coding'),
  },
  'ContactDetail': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'name': const FhirField(false, 'String'),
    'nameElement': const FhirField(false, 'Element'),
    'telecom': const FhirField(true, 'ContactPoint'),
  },
  'ExtendedContactDetail': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'purpose': const FhirField(false, 'CodeableConcept'),
    'name': const FhirField(true, 'HumanName'),
    'telecom': const FhirField(true, 'ContactPoint'),
    'address': const FhirField(false, 'Address'),
    'organization': const FhirField(false, 'Reference'),
    'period': const FhirField(false, 'Period'),
  },
  'VirtualServiceDetail': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'channelType': const FhirField(false, 'Coding'),
    'addressUrl': const FhirField(false, 'FhirUrl'),
    'addressUrlElement': const FhirField(false, 'Element'),
    'addressString': const FhirField(false, 'String'),
    'addressStringElement': const FhirField(false, 'Element'),
    'addressContactPoint': const FhirField(false, 'ContactPoint'),
    'addressExtendedContactDetail':
        const FhirField(false, 'ExtendedContactDetail'),
    'additionalInfo': const FhirField(true, 'FhirUrl'),
    'additionalInfoElement': const FhirField(true, 'Element'),
    'maxParticipants': const FhirField(false, 'PositiveInt'),
    'maxParticipantsElement': const FhirField(false, 'Element'),
    'sessionKey': const FhirField(false, 'String'),
    'sessionKeyElement': const FhirField(false, 'Element'),
  },
  'Availability': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'availableTime': const FhirField(true, 'AvailabilityAvailableTime'),
    'notAvailableTime': const FhirField(true, 'AvailabilityNotAvailableTime'),
  },
  'AvailabilityAvailableTime': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'daysOfWeek': const FhirField(true, 'Code'),
    'daysOfWeekElement': const FhirField(true, 'Element'),
    'allDay': const FhirField(false, 'Boolean'),
    'allDayElement': const FhirField(false, 'Element'),
    'availableStartTime': const FhirField(false, 'Time'),
    'availableStartTimeElement': const FhirField(false, 'Element'),
    'availableEndTime': const FhirField(false, 'Time'),
    'availableEndTimeElement': const FhirField(false, 'Element'),
  },
  'AvailabilityNotAvailableTime': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'description': const FhirField(false, 'String'),
    'descriptionElement': const FhirField(false, 'Element'),
    'during': const FhirField(false, 'Period'),
  },
  'MonetaryComponent': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'Code'),
    'typeElement': const FhirField(false, 'Element'),
    'code': const FhirField(false, 'CodeableConcept'),
    'factor': const FhirField(false, 'Decimal'),
    'factorElement': const FhirField(false, 'Element'),
    'amount': const FhirField(false, 'Money'),
  },
  'Contributor': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'Code'),
    'typeElement': const FhirField(false, 'Element'),
    'name': const FhirField(false, 'String'),
    'nameElement': const FhirField(false, 'Element'),
    'contact': const FhirField(true, 'ContactDetail'),
  },
  'DataRequirement': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'Code'),
    'typeElement': const FhirField(false, 'Element'),
    'profile': const FhirField(true, 'Canonical'),
    'subjectCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'subjectReference': const FhirField(false, 'Reference'),
    'mustSupport': const FhirField(true, 'String'),
    'mustSupportElement': const FhirField(true, 'Element'),
    'codeFilter': const FhirField(true, 'DataRequirementCodeFilter'),
    'dateFilter': const FhirField(true, 'DataRequirementDateFilter'),
    'valueFilter': const FhirField(true, 'DataRequirementValueFilter'),
    'limit': const FhirField(false, 'PositiveInt'),
    'limitElement': const FhirField(false, 'Element'),
    'sort': const FhirField(true, 'DataRequirementSort'),
  },
  'DataRequirementCodeFilter': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'path': const FhirField(false, 'String'),
    'pathElement': const FhirField(false, 'Element'),
    'searchParam': const FhirField(false, 'String'),
    'searchParamElement': const FhirField(false, 'Element'),
    'valueSet': const FhirField(false, 'Canonical'),
    'code': const FhirField(true, 'Coding'),
  },
  'DataRequirementDateFilter': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'path': const FhirField(false, 'String'),
    'pathElement': const FhirField(false, 'Element'),
    'searchParam': const FhirField(false, 'String'),
    'searchParamElement': const FhirField(false, 'Element'),
    'valueDateTime': const FhirField(false, 'FhirDateTime'),
    'valueDateTimeElement': const FhirField(false, 'Element'),
    'valuePeriod': const FhirField(false, 'Period'),
    'valueDuration': const FhirField(false, 'FhirDuration'),
  },
  'DataRequirementValueFilter': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'path': const FhirField(false, 'String'),
    'pathElement': const FhirField(false, 'Element'),
    'searchParam': const FhirField(false, 'String'),
    'searchParamElement': const FhirField(false, 'Element'),
    'comparator': const FhirField(false, 'Code'),
    'comparatorElement': const FhirField(false, 'Element'),
    'valueDateTime': const FhirField(false, 'FhirDateTime'),
    'valueDateTimeElement': const FhirField(false, 'Element'),
    'valuePeriod': const FhirField(false, 'Period'),
    'valueDuration': const FhirField(false, 'FhirDuration'),
  },
  'DataRequirementSort': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'path': const FhirField(false, 'String'),
    'pathElement': const FhirField(false, 'Element'),
    'direction': const FhirField(false, 'Code'),
    'directionElement': const FhirField(false, 'Element'),
  },
  'ParameterDefinition': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'name': const FhirField(false, 'Code'),
    'nameElement': const FhirField(false, 'Element'),
    'use': const FhirField(false, 'Code'),
    'useElement': const FhirField(false, 'Element'),
    'min': const FhirField(false, 'Integer'),
    'minElement': const FhirField(false, 'Element'),
    'max': const FhirField(false, 'String'),
    'maxElement': const FhirField(false, 'Element'),
    'documentation': const FhirField(false, 'String'),
    'documentationElement': const FhirField(false, 'Element'),
    'type': const FhirField(false, 'Code'),
    'typeElement': const FhirField(false, 'Element'),
    'profile': const FhirField(false, 'Canonical'),
  },
  'RelatedArtifact': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'Code'),
    'typeElement': const FhirField(false, 'Element'),
    'classifier': const FhirField(true, 'CodeableConcept'),
    'label': const FhirField(false, 'String'),
    'labelElement': const FhirField(false, 'Element'),
    'display': const FhirField(false, 'String'),
    'displayElement': const FhirField(false, 'Element'),
    'citation': const FhirField(false, 'Markdown'),
    'citationElement': const FhirField(false, 'Element'),
    'document': const FhirField(false, 'Attachment'),
    'resource': const FhirField(false, 'Canonical'),
    'resourceReference': const FhirField(false, 'Reference'),
    'publicationStatus': const FhirField(false, 'Code'),
    'publicationStatusElement': const FhirField(false, 'Element'),
    'publicationDate': const FhirField(false, 'Date'),
    'publicationDateElement': const FhirField(false, 'Element'),
  },
  'TriggerDefinition': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'Code'),
    'typeElement': const FhirField(false, 'Element'),
    'name': const FhirField(false, 'String'),
    'nameElement': const FhirField(false, 'Element'),
    'code': const FhirField(false, 'CodeableConcept'),
    'subscriptionTopic': const FhirField(false, 'Canonical'),
    'timingTiming': const FhirField(false, 'Timing'),
    'timingReference': const FhirField(false, 'Reference'),
    'timingDate': const FhirField(false, 'Date'),
    'timingDateElement': const FhirField(false, 'Element'),
    'timingDateTime': const FhirField(false, 'FhirDateTime'),
    'timingDateTimeElement': const FhirField(false, 'Element'),
    'data': const FhirField(true, 'DataRequirement'),
    'condition': const FhirField(false, 'Expression'),
  },
  'UsageContext': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(false, 'Coding'),
    'valueCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'valueQuantity': const FhirField(false, 'Quantity'),
    'valueRange': const FhirField(false, 'Range'),
    'valueReference': const FhirField(false, 'Reference'),
  },
  'Dosage': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'sequence': const FhirField(false, 'Integer'),
    'sequenceElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'String'),
    'textElement': const FhirField(false, 'Element'),
    'additionalInstruction': const FhirField(true, 'CodeableConcept'),
    'patientInstruction': const FhirField(false, 'String'),
    'patientInstructionElement': const FhirField(false, 'Element'),
    'timing': const FhirField(false, 'Timing'),
    'asNeeded': const FhirField(false, 'Boolean'),
    'asNeededElement': const FhirField(false, 'Element'),
    'asNeededFor': const FhirField(true, 'CodeableConcept'),
    'site': const FhirField(false, 'CodeableConcept'),
    'route': const FhirField(false, 'CodeableConcept'),
    'method': const FhirField(false, 'CodeableConcept'),
    'doseAndRate': const FhirField(true, 'DosageDoseAndRate'),
    'maxDosePerPeriod': const FhirField(true, 'Ratio'),
    'maxDosePerAdministration': const FhirField(false, 'Quantity'),
    'maxDosePerLifetime': const FhirField(false, 'Quantity'),
  },
  'DosageDoseAndRate': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'doseRange': const FhirField(false, 'Range'),
    'doseQuantity': const FhirField(false, 'Quantity'),
    'rateRatio': const FhirField(false, 'Ratio'),
    'rateRange': const FhirField(false, 'Range'),
    'rateQuantity': const FhirField(false, 'Quantity'),
  },
  'Population': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'ageRange': const FhirField(false, 'Range'),
    'ageCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'gender': const FhirField(false, 'CodeableConcept'),
    'race': const FhirField(false, 'CodeableConcept'),
    'physiologicalCondition': const FhirField(false, 'CodeableConcept'),
  },
  'ProductShelfLife': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'periodDuration': const FhirField(false, 'FhirDuration'),
    'periodString': const FhirField(false, 'String'),
    'periodStringElement': const FhirField(false, 'Element'),
    'specialPrecautionsForStorage': const FhirField(true, 'CodeableConcept'),
  },
  'MarketingStatus': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'country': const FhirField(false, 'CodeableConcept'),
    'jurisdiction': const FhirField(false, 'CodeableConcept'),
    'status': const FhirField(false, 'CodeableConcept'),
    'dateRange': const FhirField(false, 'Period'),
    'restoreDate': const FhirField(false, 'FhirDateTime'),
    'restoreDateElement': const FhirField(false, 'Element'),
  },
  'Expression': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'description': const FhirField(false, 'String'),
    'descriptionElement': const FhirField(false, 'Element'),
    'name': const FhirField(false, 'Code'),
    'nameElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'expression': const FhirField(false, 'String'),
    'expressionElement': const FhirField(false, 'Element'),
    'reference': const FhirField(false, 'FhirUri'),
    'referenceElement': const FhirField(false, 'Element'),
  },
  'ElementDefinition': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'path': const FhirField(false, 'String'),
    'pathElement': const FhirField(false, 'Element'),
    'representation': const FhirField(true, 'Code'),
    'representationElement': const FhirField(true, 'Element'),
    'sliceName': const FhirField(false, 'String'),
    'sliceNameElement': const FhirField(false, 'Element'),
    'sliceIsConstraining': const FhirField(false, 'Boolean'),
    'sliceIsConstrainingElement': const FhirField(false, 'Element'),
    'label': const FhirField(false, 'String'),
    'labelElement': const FhirField(false, 'Element'),
    'code': const FhirField(true, 'Coding'),
    'slicing': const FhirField(false, 'ElementDefinitionSlicing'),
    'short': const FhirField(false, 'String'),
    'shortElement': const FhirField(false, 'Element'),
    'definition': const FhirField(false, 'Markdown'),
    'definitionElement': const FhirField(false, 'Element'),
    'comment': const FhirField(false, 'Markdown'),
    'commentElement': const FhirField(false, 'Element'),
    'requirements': const FhirField(false, 'Markdown'),
    'requirementsElement': const FhirField(false, 'Element'),
    'alias': const FhirField(true, 'String'),
    'aliasElement': const FhirField(true, 'Element'),
    'min': const FhirField(false, 'UnsignedInt'),
    'minElement': const FhirField(false, 'Element'),
    'max': const FhirField(false, 'String'),
    'maxElement': const FhirField(false, 'Element'),
    'base': const FhirField(false, 'ElementDefinitionBase'),
    'contentReference': const FhirField(false, 'FhirUri'),
    'contentReferenceElement': const FhirField(false, 'Element'),
    'type': const FhirField(true, 'ElementDefinitionType'),
    'defaultValueBase64Binary': const FhirField(false, 'Base64Binary'),
    'defaultValueBase64BinaryElement': const FhirField(false, 'Element'),
    'defaultValueBoolean': const FhirField(false, 'Boolean'),
    'defaultValueBooleanElement': const FhirField(false, 'Element'),
    'defaultValueCanonical': const FhirField(false, 'Canonical'),
    'defaultValueCanonicalElement': const FhirField(false, 'Element'),
    'defaultValueCode': const FhirField(false, 'Code'),
    'defaultValueCodeElement': const FhirField(false, 'Element'),
    'defaultValueDate': const FhirField(false, 'Date'),
    'defaultValueDateElement': const FhirField(false, 'Element'),
    'defaultValueDateTime': const FhirField(false, 'FhirDateTime'),
    'defaultValueDateTimeElement': const FhirField(false, 'Element'),
    'defaultValueDecimal': const FhirField(false, 'Decimal'),
    'defaultValueDecimalElement': const FhirField(false, 'Element'),
    'defaultValueId': const FhirField(false, 'Id'),
    'defaultValueIdElement': const FhirField(false, 'Element'),
    'defaultValueInstant': const FhirField(false, 'Instant'),
    'defaultValueInstantElement': const FhirField(false, 'Element'),
    'defaultValueInteger': const FhirField(false, 'Integer'),
    'defaultValueIntegerElement': const FhirField(false, 'Element'),
    'defaultValueInteger64': const FhirField(false, 'Integer64'),
    'defaultValueInteger64Element': const FhirField(false, 'Element'),
    'defaultValueMarkdown': const FhirField(false, 'Markdown'),
    'defaultValueMarkdownElement': const FhirField(false, 'Element'),
    'defaultValueOid': const FhirField(false, 'Id'),
    'defaultValueOidElement': const FhirField(false, 'Element'),
    'defaultValuePositiveInt': const FhirField(false, 'PositiveInt'),
    'defaultValuePositiveIntElement': const FhirField(false, 'Element'),
    'defaultValueString': const FhirField(false, 'String'),
    'defaultValueStringElement': const FhirField(false, 'Element'),
    'defaultValueTime': const FhirField(false, 'Time'),
    'defaultValueTimeElement': const FhirField(false, 'Element'),
    'defaultValueUnsignedInt': const FhirField(false, 'UnsignedInt'),
    'defaultValueUnsignedIntElement': const FhirField(false, 'Element'),
    'defaultValueUri': const FhirField(false, 'FhirUri'),
    'defaultValueUriElement': const FhirField(false, 'Element'),
    'defaultValueUrl': const FhirField(false, 'FhirUrl'),
    'defaultValueUrlElement': const FhirField(false, 'Element'),
    'defaultValueUuid': const FhirField(false, 'Id'),
    'defaultValueUuidElement': const FhirField(false, 'Element'),
    'defaultValueAddress': const FhirField(false, 'Address'),
    'defaultValueAge': const FhirField(false, 'Age'),
    'defaultValueAnnotation': const FhirField(false, 'Annotation'),
    'defaultValueAttachment': const FhirField(false, 'Attachment'),
    'defaultValueCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'defaultValueCodeableReference':
        const FhirField(false, 'CodeableReference'),
    'defaultValueCoding': const FhirField(false, 'Coding'),
    'defaultValueContactPoint': const FhirField(false, 'ContactPoint'),
    'defaultValueCount': const FhirField(false, 'Count'),
    'defaultValueDistance': const FhirField(false, 'Distance'),
    'defaultValueDuration': const FhirField(false, 'FhirDuration'),
    'defaultValueHumanName': const FhirField(false, 'HumanName'),
    'defaultValueIdentifier': const FhirField(false, 'Identifier'),
    'defaultValueMoney': const FhirField(false, 'Money'),
    'defaultValuePeriod': const FhirField(false, 'Period'),
    'defaultValueQuantity': const FhirField(false, 'Quantity'),
    'defaultValueRange': const FhirField(false, 'Range'),
    'defaultValueRatio': const FhirField(false, 'Ratio'),
    'defaultValueRatioRange': const FhirField(false, 'RatioRange'),
    'defaultValueReference': const FhirField(false, 'Reference'),
    'defaultValueSampledData': const FhirField(false, 'SampledData'),
    'defaultValueSignature': const FhirField(false, 'Signature'),
    'defaultValueTiming': const FhirField(false, 'Timing'),
    'defaultValueContactDetail': const FhirField(false, 'ContactDetail'),
    'defaultValueDataRequirement': const FhirField(false, 'DataRequirement'),
    'defaultValueExpression': const FhirField(false, 'Expression'),
    'defaultValueParameterDefinition':
        const FhirField(false, 'ParameterDefinition'),
    'defaultValueRelatedArtifact': const FhirField(false, 'RelatedArtifact'),
    'defaultValueTriggerDefinition':
        const FhirField(false, 'TriggerDefinition'),
    'defaultValueUsageContext': const FhirField(false, 'UsageContext'),
    'defaultValueAvailability': const FhirField(false, 'Availability'),
    'defaultValueExtendedContactDetail':
        const FhirField(false, 'ExtendedContactDetail'),
    'defaultValueDosage': const FhirField(false, 'Dosage'),
    'defaultValueMeta': const FhirField(false, 'Meta'),
    'meaningWhenMissing': const FhirField(false, 'Markdown'),
    'meaningWhenMissingElement': const FhirField(false, 'Element'),
    'orderMeaning': const FhirField(false, 'String'),
    'orderMeaningElement': const FhirField(false, 'Element'),
    'fixedBase64Binary': const FhirField(false, 'Base64Binary'),
    'fixedBase64BinaryElement': const FhirField(false, 'Element'),
    'fixedBoolean': const FhirField(false, 'Boolean'),
    'fixedBooleanElement': const FhirField(false, 'Element'),
    'fixedCanonical': const FhirField(false, 'Canonical'),
    'fixedCanonicalElement': const FhirField(false, 'Element'),
    'fixedCode': const FhirField(false, 'Code'),
    'fixedCodeElement': const FhirField(false, 'Element'),
    'fixedDate': const FhirField(false, 'Date'),
    'fixedDateElement': const FhirField(false, 'Element'),
    'fixedDateTime': const FhirField(false, 'FhirDateTime'),
    'fixedDateTimeElement': const FhirField(false, 'Element'),
    'fixedDecimal': const FhirField(false, 'Decimal'),
    'fixedDecimalElement': const FhirField(false, 'Element'),
    'fixedId': const FhirField(false, 'Id'),
    'fixedIdElement': const FhirField(false, 'Element'),
    'fixedInstant': const FhirField(false, 'Instant'),
    'fixedInstantElement': const FhirField(false, 'Element'),
    'fixedInteger': const FhirField(false, 'Integer'),
    'fixedIntegerElement': const FhirField(false, 'Element'),
    'fixedInteger64': const FhirField(false, 'Integer64'),
    'fixedInteger64Element': const FhirField(false, 'Element'),
    'fixedMarkdown': const FhirField(false, 'Markdown'),
    'fixedMarkdownElement': const FhirField(false, 'Element'),
    'fixedOid': const FhirField(false, 'Id'),
    'fixedOidElement': const FhirField(false, 'Element'),
    'fixedPositiveInt': const FhirField(false, 'PositiveInt'),
    'fixedPositiveIntElement': const FhirField(false, 'Element'),
    'fixedString': const FhirField(false, 'String'),
    'fixedStringElement': const FhirField(false, 'Element'),
    'fixedTime': const FhirField(false, 'Time'),
    'fixedTimeElement': const FhirField(false, 'Element'),
    'fixedUnsignedInt': const FhirField(false, 'UnsignedInt'),
    'fixedUnsignedIntElement': const FhirField(false, 'Element'),
    'fixedUri': const FhirField(false, 'FhirUri'),
    'fixedUriElement': const FhirField(false, 'Element'),
    'fixedUrl': const FhirField(false, 'FhirUrl'),
    'fixedUrlElement': const FhirField(false, 'Element'),
    'fixedUuid': const FhirField(false, 'Id'),
    'fixedUuidElement': const FhirField(false, 'Element'),
    'fixedAddress': const FhirField(false, 'Address'),
    'fixedAge': const FhirField(false, 'Age'),
    'fixedAnnotation': const FhirField(false, 'Annotation'),
    'fixedAttachment': const FhirField(false, 'Attachment'),
    'fixedCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'fixedCodeableReference': const FhirField(false, 'CodeableReference'),
    'fixedCoding': const FhirField(false, 'Coding'),
    'fixedContactPoint': const FhirField(false, 'ContactPoint'),
    'fixedCount': const FhirField(false, 'Count'),
    'fixedDistance': const FhirField(false, 'Distance'),
    'fixedDuration': const FhirField(false, 'FhirDuration'),
    'fixedHumanName': const FhirField(false, 'HumanName'),
    'fixedIdentifier': const FhirField(false, 'Identifier'),
    'fixedMoney': const FhirField(false, 'Money'),
    'fixedPeriod': const FhirField(false, 'Period'),
    'fixedQuantity': const FhirField(false, 'Quantity'),
    'fixedRange': const FhirField(false, 'Range'),
    'fixedRatio': const FhirField(false, 'Ratio'),
    'fixedRatioRange': const FhirField(false, 'RatioRange'),
    'fixedReference': const FhirField(false, 'Reference'),
    'fixedSampledData': const FhirField(false, 'SampledData'),
    'fixedSignature': const FhirField(false, 'Signature'),
    'fixedTiming': const FhirField(false, 'Timing'),
    'fixedContactDetail': const FhirField(false, 'ContactDetail'),
    'fixedDataRequirement': const FhirField(false, 'DataRequirement'),
    'fixedExpression': const FhirField(false, 'Expression'),
    'fixedParameterDefinition': const FhirField(false, 'ParameterDefinition'),
    'fixedRelatedArtifact': const FhirField(false, 'RelatedArtifact'),
    'fixedTriggerDefinition': const FhirField(false, 'TriggerDefinition'),
    'fixedUsageContext': const FhirField(false, 'UsageContext'),
    'fixedAvailability': const FhirField(false, 'Availability'),
    'fixedExtendedContactDetail':
        const FhirField(false, 'ExtendedContactDetail'),
    'fixedDosage': const FhirField(false, 'Dosage'),
    'fixedMeta': const FhirField(false, 'Meta'),
    'patternBase64Binary': const FhirField(false, 'Base64Binary'),
    'patternBase64BinaryElement': const FhirField(false, 'Element'),
    'patternBoolean': const FhirField(false, 'Boolean'),
    'patternBooleanElement': const FhirField(false, 'Element'),
    'patternCanonical': const FhirField(false, 'Canonical'),
    'patternCanonicalElement': const FhirField(false, 'Element'),
    'patternCode': const FhirField(false, 'Code'),
    'patternCodeElement': const FhirField(false, 'Element'),
    'patternDate': const FhirField(false, 'Date'),
    'patternDateElement': const FhirField(false, 'Element'),
    'patternDateTime': const FhirField(false, 'FhirDateTime'),
    'patternDateTimeElement': const FhirField(false, 'Element'),
    'patternDecimal': const FhirField(false, 'Decimal'),
    'patternDecimalElement': const FhirField(false, 'Element'),
    'patternId': const FhirField(false, 'Id'),
    'patternIdElement': const FhirField(false, 'Element'),
    'patternInstant': const FhirField(false, 'Instant'),
    'patternInstantElement': const FhirField(false, 'Element'),
    'patternInteger': const FhirField(false, 'Integer'),
    'patternIntegerElement': const FhirField(false, 'Element'),
    'patternInteger64': const FhirField(false, 'Integer64'),
    'patternInteger64Element': const FhirField(false, 'Element'),
    'patternMarkdown': const FhirField(false, 'Markdown'),
    'patternMarkdownElement': const FhirField(false, 'Element'),
    'patternOid': const FhirField(false, 'Id'),
    'patternOidElement': const FhirField(false, 'Element'),
    'patternPositiveInt': const FhirField(false, 'PositiveInt'),
    'patternPositiveIntElement': const FhirField(false, 'Element'),
    'patternString': const FhirField(false, 'String'),
    'patternStringElement': const FhirField(false, 'Element'),
    'patternTime': const FhirField(false, 'Time'),
    'patternTimeElement': const FhirField(false, 'Element'),
    'patternUnsignedInt': const FhirField(false, 'UnsignedInt'),
    'patternUnsignedIntElement': const FhirField(false, 'Element'),
    'patternUri': const FhirField(false, 'FhirUri'),
    'patternUriElement': const FhirField(false, 'Element'),
    'patternUrl': const FhirField(false, 'FhirUrl'),
    'patternUrlElement': const FhirField(false, 'Element'),
    'patternUuid': const FhirField(false, 'Id'),
    'patternUuidElement': const FhirField(false, 'Element'),
    'patternAddress': const FhirField(false, 'Address'),
    'patternAge': const FhirField(false, 'Age'),
    'patternAnnotation': const FhirField(false, 'Annotation'),
    'patternAttachment': const FhirField(false, 'Attachment'),
    'patternCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'patternCodeableReference': const FhirField(false, 'CodeableReference'),
    'patternCoding': const FhirField(false, 'Coding'),
    'patternContactPoint': const FhirField(false, 'ContactPoint'),
    'patternCount': const FhirField(false, 'Count'),
    'patternDistance': const FhirField(false, 'Distance'),
    'patternDuration': const FhirField(false, 'FhirDuration'),
    'patternHumanName': const FhirField(false, 'HumanName'),
    'patternIdentifier': const FhirField(false, 'Identifier'),
    'patternMoney': const FhirField(false, 'Money'),
    'patternPeriod': const FhirField(false, 'Period'),
    'patternQuantity': const FhirField(false, 'Quantity'),
    'patternRange': const FhirField(false, 'Range'),
    'patternRatio': const FhirField(false, 'Ratio'),
    'patternRatioRange': const FhirField(false, 'RatioRange'),
    'patternReference': const FhirField(false, 'Reference'),
    'patternSampledData': const FhirField(false, 'SampledData'),
    'patternSignature': const FhirField(false, 'Signature'),
    'patternTiming': const FhirField(false, 'Timing'),
    'patternContactDetail': const FhirField(false, 'ContactDetail'),
    'patternDataRequirement': const FhirField(false, 'DataRequirement'),
    'patternExpression': const FhirField(false, 'Expression'),
    'patternParameterDefinition': const FhirField(false, 'ParameterDefinition'),
    'patternRelatedArtifact': const FhirField(false, 'RelatedArtifact'),
    'patternTriggerDefinition': const FhirField(false, 'TriggerDefinition'),
    'patternUsageContext': const FhirField(false, 'UsageContext'),
    'patternAvailability': const FhirField(false, 'Availability'),
    'patternExtendedContactDetail':
        const FhirField(false, 'ExtendedContactDetail'),
    'patternDosage': const FhirField(false, 'Dosage'),
    'patternMeta': const FhirField(false, 'Meta'),
    'example': const FhirField(true, 'ElementDefinitionExample'),
    'minValueDate': const FhirField(false, 'Date'),
    'minValueDateElement': const FhirField(false, 'Element'),
    'minValueDateTime': const FhirField(false, 'FhirDateTime'),
    'minValueDateTimeElement': const FhirField(false, 'Element'),
    'minValueInstant': const FhirField(false, 'Instant'),
    'minValueInstantElement': const FhirField(false, 'Element'),
    'minValueTime': const FhirField(false, 'Time'),
    'minValueTimeElement': const FhirField(false, 'Element'),
    'minValueDecimal': const FhirField(false, 'Decimal'),
    'minValueDecimalElement': const FhirField(false, 'Element'),
    'minValueInteger': const FhirField(false, 'Integer'),
    'minValueIntegerElement': const FhirField(false, 'Element'),
    'minValueInteger64': const FhirField(false, 'Integer64'),
    'minValueInteger64Element': const FhirField(false, 'Element'),
    'minValuePositiveInt': const FhirField(false, 'PositiveInt'),
    'minValuePositiveIntElement': const FhirField(false, 'Element'),
    'minValueUnsignedInt': const FhirField(false, 'UnsignedInt'),
    'minValueUnsignedIntElement': const FhirField(false, 'Element'),
    'minValueQuantity': const FhirField(false, 'Quantity'),
    'maxValueDate': const FhirField(false, 'Date'),
    'maxValueDateElement': const FhirField(false, 'Element'),
    'maxValueDateTime': const FhirField(false, 'FhirDateTime'),
    'maxValueDateTimeElement': const FhirField(false, 'Element'),
    'maxValueInstant': const FhirField(false, 'Instant'),
    'maxValueInstantElement': const FhirField(false, 'Element'),
    'maxValueTime': const FhirField(false, 'Time'),
    'maxValueTimeElement': const FhirField(false, 'Element'),
    'maxValueDecimal': const FhirField(false, 'Decimal'),
    'maxValueDecimalElement': const FhirField(false, 'Element'),
    'maxValueInteger': const FhirField(false, 'Integer'),
    'maxValueIntegerElement': const FhirField(false, 'Element'),
    'maxValueInteger64': const FhirField(false, 'Integer64'),
    'maxValueInteger64Element': const FhirField(false, 'Element'),
    'maxValuePositiveInt': const FhirField(false, 'PositiveInt'),
    'maxValuePositiveIntElement': const FhirField(false, 'Element'),
    'maxValueUnsignedInt': const FhirField(false, 'UnsignedInt'),
    'maxValueUnsignedIntElement': const FhirField(false, 'Element'),
    'maxValueQuantity': const FhirField(false, 'Quantity'),
    'maxLength': const FhirField(false, 'Integer'),
    'maxLengthElement': const FhirField(false, 'Element'),
    'condition': const FhirField(true, 'Id'),
    'conditionElement': const FhirField(true, 'Element'),
    'constraint': const FhirField(true, 'ElementDefinitionConstraint'),
    'mustHaveValue': const FhirField(false, 'Boolean'),
    'mustHaveValueElement': const FhirField(false, 'Element'),
    'valueAlternatives': const FhirField(true, 'Canonical'),
    'mustSupport': const FhirField(false, 'Boolean'),
    'mustSupportElement': const FhirField(false, 'Element'),
    'isModifier': const FhirField(false, 'Boolean'),
    'isModifierElement': const FhirField(false, 'Element'),
    'isModifierReason': const FhirField(false, 'String'),
    'isModifierReasonElement': const FhirField(false, 'Element'),
    'isSummary': const FhirField(false, 'Boolean'),
    'isSummaryElement': const FhirField(false, 'Element'),
    'binding': const FhirField(false, 'ElementDefinitionBinding'),
    'mapping': const FhirField(true, 'ElementDefinitionMapping'),
  },
  'ElementDefinitionSlicing': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'discriminator': const FhirField(true, 'ElementDefinitionDiscriminator'),
    'description': const FhirField(false, 'String'),
    'descriptionElement': const FhirField(false, 'Element'),
    'ordered': const FhirField(false, 'Boolean'),
    'orderedElement': const FhirField(false, 'Element'),
    'rules': const FhirField(false, 'Code'),
    'rulesElement': const FhirField(false, 'Element'),
  },
  'ElementDefinitionDiscriminator': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'Code'),
    'typeElement': const FhirField(false, 'Element'),
    'path': const FhirField(false, 'String'),
    'pathElement': const FhirField(false, 'Element'),
  },
  'ElementDefinitionBase': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'path': const FhirField(false, 'String'),
    'pathElement': const FhirField(false, 'Element'),
    'min': const FhirField(false, 'UnsignedInt'),
    'minElement': const FhirField(false, 'Element'),
    'max': const FhirField(false, 'String'),
    'maxElement': const FhirField(false, 'Element'),
  },
  'ElementDefinitionType': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(false, 'FhirUri'),
    'codeElement': const FhirField(false, 'Element'),
    'profile': const FhirField(true, 'Canonical'),
    'targetProfile': const FhirField(true, 'Canonical'),
    'aggregation': const FhirField(true, 'ElementDefinitionTypeAggregation'),
    'aggregationElement': const FhirField(true, 'Element'),
    'versioning': const FhirField(false, 'Code'),
    'versioningElement': const FhirField(false, 'Element'),
  },
  'ElementDefinitionExample': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'label': const FhirField(false, 'String'),
    'labelElement': const FhirField(false, 'Element'),
    'valueBase64Binary': const FhirField(false, 'Base64Binary'),
    'valueBase64BinaryElement': const FhirField(false, 'Element'),
    'valueBoolean': const FhirField(false, 'Boolean'),
    'valueBooleanElement': const FhirField(false, 'Element'),
    'valueCanonical': const FhirField(false, 'Canonical'),
    'valueCanonicalElement': const FhirField(false, 'Element'),
    'valueCode': const FhirField(false, 'Code'),
    'valueCodeElement': const FhirField(false, 'Element'),
    'valueDate': const FhirField(false, 'Date'),
    'valueDateElement': const FhirField(false, 'Element'),
    'valueDateTime': const FhirField(false, 'FhirDateTime'),
    'valueDateTimeElement': const FhirField(false, 'Element'),
    'valueDecimal': const FhirField(false, 'Decimal'),
    'valueDecimalElement': const FhirField(false, 'Element'),
    'valueId': const FhirField(false, 'Id'),
    'valueIdElement': const FhirField(false, 'Element'),
    'valueInstant': const FhirField(false, 'Instant'),
    'valueInstantElement': const FhirField(false, 'Element'),
    'valueInteger': const FhirField(false, 'Integer'),
    'valueIntegerElement': const FhirField(false, 'Element'),
    'valueInteger64': const FhirField(false, 'Integer64'),
    'valueInteger64Element': const FhirField(false, 'Element'),
    'valueMarkdown': const FhirField(false, 'Markdown'),
    'valueMarkdownElement': const FhirField(false, 'Element'),
    'valueOid': const FhirField(false, 'Id'),
    'valueOidElement': const FhirField(false, 'Element'),
    'valuePositiveInt': const FhirField(false, 'PositiveInt'),
    'valuePositiveIntElement': const FhirField(false, 'Element'),
    'valueString': const FhirField(false, 'String'),
    'valueStringElement': const FhirField(false, 'Element'),
    'valueTime': const FhirField(false, 'Time'),
    'valueTimeElement': const FhirField(false, 'Element'),
    'valueUnsignedInt': const FhirField(false, 'UnsignedInt'),
    'valueUnsignedIntElement': const FhirField(false, 'Element'),
    'valueUri': const FhirField(false, 'FhirUri'),
    'valueUriElement': const FhirField(false, 'Element'),
    'valueUrl': const FhirField(false, 'FhirUrl'),
    'valueUrlElement': const FhirField(false, 'Element'),
    'valueUuid': const FhirField(false, 'Id'),
    'valueUuidElement': const FhirField(false, 'Element'),
    'valueAddress': const FhirField(false, 'Address'),
    'valueAge': const FhirField(false, 'Age'),
    'valueAnnotation': const FhirField(false, 'Annotation'),
    'valueAttachment': const FhirField(false, 'Attachment'),
    'valueCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'valueCodeableReference': const FhirField(false, 'CodeableReference'),
    'valueCoding': const FhirField(false, 'Coding'),
    'valueContactPoint': const FhirField(false, 'ContactPoint'),
    'valueCount': const FhirField(false, 'Count'),
    'valueDistance': const FhirField(false, 'Distance'),
    'valueDuration': const FhirField(false, 'FhirDuration'),
    'valueHumanName': const FhirField(false, 'HumanName'),
    'valueIdentifier': const FhirField(false, 'Identifier'),
    'valueMoney': const FhirField(false, 'Money'),
    'valuePeriod': const FhirField(false, 'Period'),
    'valueQuantity': const FhirField(false, 'Quantity'),
    'valueRange': const FhirField(false, 'Range'),
    'valueRatio': const FhirField(false, 'Ratio'),
    'valueRatioRange': const FhirField(false, 'RatioRange'),
    'valueReference': const FhirField(false, 'Reference'),
    'valueSampledData': const FhirField(false, 'SampledData'),
    'valueSignature': const FhirField(false, 'Signature'),
    'valueTiming': const FhirField(false, 'Timing'),
    'valueContactDetail': const FhirField(false, 'ContactDetail'),
    'valueDataRequirement': const FhirField(false, 'DataRequirement'),
    'valueExpression': const FhirField(false, 'Expression'),
    'valueParameterDefinition': const FhirField(false, 'ParameterDefinition'),
    'valueRelatedArtifact': const FhirField(false, 'RelatedArtifact'),
    'valueTriggerDefinition': const FhirField(false, 'TriggerDefinition'),
    'valueUsageContext': const FhirField(false, 'UsageContext'),
    'valueAvailability': const FhirField(false, 'Availability'),
    'valueExtendedContactDetail':
        const FhirField(false, 'ExtendedContactDetail'),
    'valueDosage': const FhirField(false, 'Dosage'),
    'valueMeta': const FhirField(false, 'Meta'),
  },
  'ElementDefinitionConstraint': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'key': const FhirField(false, 'Id'),
    'keyElement': const FhirField(false, 'Element'),
    'requirements': const FhirField(false, 'Markdown'),
    'requirementsElement': const FhirField(false, 'Element'),
    'severity': const FhirField(false, 'Code'),
    'severityElement': const FhirField(false, 'Element'),
    'suppress': const FhirField(false, 'Boolean'),
    'suppressElement': const FhirField(false, 'Element'),
    'human': const FhirField(false, 'String'),
    'humanElement': const FhirField(false, 'Element'),
    'expression': const FhirField(false, 'String'),
    'expressionElement': const FhirField(false, 'Element'),
    'source': const FhirField(false, 'Canonical'),
  },
  'ElementDefinitionObligation': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(false, 'Coding'),
    'actor': const FhirField(true, 'Canonical'),
    'documentation': const FhirField(false, 'Markdown'),
    'documentationElement': const FhirField(false, 'Element'),
    'usage': const FhirField(true, 'UsageContext'),
    'filter': const FhirField(false, 'String'),
    'filterElement': const FhirField(false, 'Element'),
    'filterDocumentation': const FhirField(false, 'String'),
    'filterDocumentationElement': const FhirField(false, 'Element'),
    'process': const FhirField(true, 'FhirUri'),
    'processElement': const FhirField(true, 'Element'),
  },
  'ElementDefinitionBinding': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'strength': const FhirField(false, 'Code'),
    'strengthElement': const FhirField(false, 'Element'),
    'description': const FhirField(false, 'Markdown'),
    'descriptionElement': const FhirField(false, 'Element'),
    'valueSet': const FhirField(false, 'Canonical'),
    'additional': const FhirField(true, 'ElementDefinitionAdditional'),
  },
  'ElementDefinitionAdditional': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'purpose': const FhirField(false, 'Code'),
    'purposeElement': const FhirField(false, 'Element'),
    'valueSet': const FhirField(false, 'Canonical'),
    'documentation': const FhirField(false, 'Markdown'),
    'documentationElement': const FhirField(false, 'Element'),
    'shortDoco': const FhirField(false, 'String'),
    'shortDocoElement': const FhirField(false, 'Element'),
    'usage': const FhirField(true, 'UsageContext'),
    'any': const FhirField(false, 'Boolean'),
    'anyElement': const FhirField(false, 'Element'),
  },
  'ElementDefinitionMapping': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identity': const FhirField(false, 'Id'),
    'identityElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'map': const FhirField(false, 'String'),
    'mapElement': const FhirField(false, 'Element'),
    'comment': const FhirField(false, 'Markdown'),
    'commentElement': const FhirField(false, 'Element'),
  },
  'Account': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'billingStatus': const FhirField(false, 'CodeableConcept'),
    'type': const FhirField(false, 'CodeableConcept'),
    'name': const FhirField(false, 'String'),
    'nameElement': const FhirField(false, 'Element'),
    'subject': const FhirField(true, 'Reference'),
    'servicePeriod': const FhirField(false, 'Period'),
    'coverage': const FhirField(true, 'AccountCoverage'),
    'owner': const FhirField(false, 'Reference'),
    'description': const FhirField(false, 'Markdown'),
    'descriptionElement': const FhirField(false, 'Element'),
    'guarantor': const FhirField(true, 'AccountGuarantor'),
    'diagnosis': const FhirField(true, 'AccountDiagnosis'),
    'procedure': const FhirField(true, 'AccountProcedure'),
    'relatedAccount': const FhirField(true, 'AccountRelatedAccount'),
    'currency': const FhirField(false, 'CodeableConcept'),
    'balance': const FhirField(true, 'AccountBalance'),
    'calculatedAt': const FhirField(false, 'Instant'),
    'calculatedAtElement': const FhirField(false, 'Element'),
  },
  'AccountCoverage': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'coverage': const FhirField(false, 'Reference'),
    'priority': const FhirField(false, 'PositiveInt'),
    'priorityElement': const FhirField(false, 'Element'),
  },
  'AccountGuarantor': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'party': const FhirField(false, 'Reference'),
    'onHold': const FhirField(false, 'Boolean'),
    'onHoldElement': const FhirField(false, 'Element'),
    'period': const FhirField(false, 'Period'),
  },
  'AccountDiagnosis': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'sequence': const FhirField(false, 'PositiveInt'),
    'sequenceElement': const FhirField(false, 'Element'),
    'condition': const FhirField(false, 'CodeableReference'),
    'dateOfDiagnosis': const FhirField(false, 'FhirDateTime'),
    'dateOfDiagnosisElement': const FhirField(false, 'Element'),
    'type': const FhirField(true, 'CodeableConcept'),
    'onAdmission': const FhirField(false, 'Boolean'),
    'onAdmissionElement': const FhirField(false, 'Element'),
    'packageCode': const FhirField(true, 'CodeableConcept'),
  },
  'AccountProcedure': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'sequence': const FhirField(false, 'PositiveInt'),
    'sequenceElement': const FhirField(false, 'Element'),
    'code': const FhirField(false, 'CodeableReference'),
    'dateOfService': const FhirField(false, 'FhirDateTime'),
    'dateOfServiceElement': const FhirField(false, 'Element'),
    'type': const FhirField(true, 'CodeableConcept'),
    'packageCode': const FhirField(true, 'CodeableConcept'),
    'device': const FhirField(true, 'Reference'),
  },
  'AccountRelatedAccount': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'relationship': const FhirField(false, 'CodeableConcept'),
    'account': const FhirField(false, 'Reference'),
  },
  'AccountBalance': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'aggregate': const FhirField(false, 'CodeableConcept'),
    'term': const FhirField(false, 'CodeableConcept'),
    'estimate': const FhirField(false, 'Boolean'),
    'estimateElement': const FhirField(false, 'Element'),
    'amount': const FhirField(false, 'Money'),
  },
  'ActivityDefinition': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'url': const FhirField(false, 'FhirUri'),
    'urlElement': const FhirField(false, 'Element'),
    'identifier': const FhirField(true, 'Identifier'),
    'version': const FhirField(false, 'String'),
    'versionElement': const FhirField(false, 'Element'),
    'versionAlgorithmString': const FhirField(false, 'String'),
    'versionAlgorithmStringElement': const FhirField(false, 'Element'),
    'versionAlgorithmCoding': const FhirField(false, 'Coding'),
    'name': const FhirField(false, 'String'),
    'nameElement': const FhirField(false, 'Element'),
    'title': const FhirField(false, 'String'),
    'titleElement': const FhirField(false, 'Element'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'experimental': const FhirField(false, 'Boolean'),
    'experimentalElement': const FhirField(false, 'Element'),
    'date': const FhirField(false, 'FhirDateTime'),
    'dateElement': const FhirField(false, 'Element'),
    'publisher': const FhirField(false, 'String'),
    'publisherElement': const FhirField(false, 'Element'),
    'contact': const FhirField(true, 'ContactDetail'),
    'description': const FhirField(false, 'Markdown'),
    'descriptionElement': const FhirField(false, 'Element'),
    'useContext': const FhirField(true, 'UsageContext'),
    'jurisdiction': const FhirField(true, 'CodeableConcept'),
    'purpose': const FhirField(false, 'Markdown'),
    'purposeElement': const FhirField(false, 'Element'),
    'copyright': const FhirField(false, 'Markdown'),
    'copyrightElement': const FhirField(false, 'Element'),
    'copyrightLabel': const FhirField(false, 'String'),
    'copyrightLabelElement': const FhirField(false, 'Element'),
    'approvalDate': const FhirField(false, 'Date'),
    'approvalDateElement': const FhirField(false, 'Element'),
    'lastReviewDate': const FhirField(false, 'Date'),
    'lastReviewDateElement': const FhirField(false, 'Element'),
    'effectivePeriod': const FhirField(false, 'Period'),
    'topic': const FhirField(true, 'CodeableConcept'),
    'author': const FhirField(true, 'ContactDetail'),
    'editor': const FhirField(true, 'ContactDetail'),
    'reviewer': const FhirField(true, 'ContactDetail'),
    'endorser': const FhirField(true, 'ContactDetail'),
    'relatedArtifact': const FhirField(true, 'RelatedArtifact'),
    'subtitle': const FhirField(false, 'String'),
    'subtitleElement': const FhirField(false, 'Element'),
    'subjectCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'subjectReference': const FhirField(false, 'Reference'),
    'subjectCanonical': const FhirField(false, 'Canonical'),
    'subjectCanonicalElement': const FhirField(false, 'Element'),
    'usage': const FhirField(false, 'Markdown'),
    'usageElement': const FhirField(false, 'Element'),
    'library': const FhirField(true, 'Canonical'),
    'kind': const FhirField(false, 'Code'),
    'kindElement': const FhirField(false, 'Element'),
    'profile': const FhirField(false, 'Canonical'),
    'code': const FhirField(false, 'CodeableConcept'),
    'intent': const FhirField(false, 'Code'),
    'intentElement': const FhirField(false, 'Element'),
    'priority': const FhirField(false, 'Code'),
    'priorityElement': const FhirField(false, 'Element'),
    'doNotPerform': const FhirField(false, 'Boolean'),
    'doNotPerformElement': const FhirField(false, 'Element'),
    'timingTiming': const FhirField(false, 'Timing'),
    'timingAge': const FhirField(false, 'Age'),
    'timingRange': const FhirField(false, 'Range'),
    'timingDuration': const FhirField(false, 'FhirDuration'),
    'asNeededBoolean': const FhirField(false, 'Boolean'),
    'asNeededBooleanElement': const FhirField(false, 'Element'),
    'asNeededCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'location': const FhirField(false, 'CodeableReference'),
    'participant': const FhirField(true, 'ActivityDefinitionParticipant'),
    'productReference': const FhirField(false, 'Reference'),
    'productCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'quantity': const FhirField(false, 'Quantity'),
    'dosage': const FhirField(true, 'Dosage'),
    'bodySite': const FhirField(true, 'CodeableConcept'),
    'specimenRequirement': const FhirField(true, 'Canonical'),
    'observationRequirement': const FhirField(true, 'Canonical'),
    'observationResultRequirement': const FhirField(true, 'Canonical'),
    'transform': const FhirField(false, 'Canonical'),
    'dynamicValue': const FhirField(true, 'ActivityDefinitionDynamicValue'),
  },
  'ActivityDefinitionParticipant': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'Code'),
    'typeElement': const FhirField(false, 'Element'),
    'typeCanonical': const FhirField(false, 'Canonical'),
    'typeReference': const FhirField(false, 'Reference'),
    'role': const FhirField(false, 'CodeableConcept'),
    'function': const FhirField(false, 'CodeableConcept'),
  },
  'ActivityDefinitionDynamicValue': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'path': const FhirField(false, 'String'),
    'pathElement': const FhirField(false, 'Element'),
    'expression': const FhirField(false, 'Expression'),
  },
  'ActorDefinition': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'url': const FhirField(false, 'FhirUri'),
    'urlElement': const FhirField(false, 'Element'),
    'identifier': const FhirField(true, 'Identifier'),
    'version': const FhirField(false, 'String'),
    'versionElement': const FhirField(false, 'Element'),
    'versionAlgorithmString': const FhirField(false, 'String'),
    'versionAlgorithmStringElement': const FhirField(false, 'Element'),
    'versionAlgorithmCoding': const FhirField(false, 'Coding'),
    'name': const FhirField(false, 'String'),
    'nameElement': const FhirField(false, 'Element'),
    'title': const FhirField(false, 'String'),
    'titleElement': const FhirField(false, 'Element'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'experimental': const FhirField(false, 'Boolean'),
    'experimentalElement': const FhirField(false, 'Element'),
    'date': const FhirField(false, 'FhirDateTime'),
    'dateElement': const FhirField(false, 'Element'),
    'publisher': const FhirField(false, 'String'),
    'publisherElement': const FhirField(false, 'Element'),
    'contact': const FhirField(true, 'ContactDetail'),
    'description': const FhirField(false, 'Markdown'),
    'descriptionElement': const FhirField(false, 'Element'),
    'useContext': const FhirField(true, 'UsageContext'),
    'jurisdiction': const FhirField(true, 'CodeableConcept'),
    'purpose': const FhirField(false, 'Markdown'),
    'purposeElement': const FhirField(false, 'Element'),
    'copyright': const FhirField(false, 'Markdown'),
    'copyrightElement': const FhirField(false, 'Element'),
    'copyrightLabel': const FhirField(false, 'String'),
    'copyrightLabelElement': const FhirField(false, 'Element'),
    'type': const FhirField(false, 'Code'),
    'typeElement': const FhirField(false, 'Element'),
    'documentation': const FhirField(false, 'Markdown'),
    'documentationElement': const FhirField(false, 'Element'),
    'reference': const FhirField(true, 'FhirUrl'),
    'referenceElement': const FhirField(true, 'Element'),
    'capabilities': const FhirField(false, 'Canonical'),
    'derivedFrom': const FhirField(true, 'Canonical'),
  },
  'AdministrableProductDefinition': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'formOf': const FhirField(true, 'Reference'),
    'administrableDoseForm': const FhirField(false, 'CodeableConcept'),
    'unitOfPresentation': const FhirField(false, 'CodeableConcept'),
    'producedFrom': const FhirField(true, 'Reference'),
    'ingredient': const FhirField(true, 'CodeableConcept'),
    'device': const FhirField(false, 'Reference'),
    'description': const FhirField(false, 'Markdown'),
    'descriptionElement': const FhirField(false, 'Element'),
    'property': const FhirField(true, 'AdministrableProductDefinitionProperty'),
    'routeOfAdministration': const FhirField(
        true, 'AdministrableProductDefinitionRouteOfAdministration'),
  },
  'AdministrableProductDefinitionProperty': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'valueCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'valueQuantity': const FhirField(false, 'Quantity'),
    'valueDate': const FhirField(false, 'Date'),
    'valueDateElement': const FhirField(false, 'Element'),
    'valueBoolean': const FhirField(false, 'Boolean'),
    'valueBooleanElement': const FhirField(false, 'Element'),
    'valueMarkdown': const FhirField(false, 'Markdown'),
    'valueMarkdownElement': const FhirField(false, 'Element'),
    'valueAttachment': const FhirField(false, 'Attachment'),
    'valueReference': const FhirField(false, 'Reference'),
    'status': const FhirField(false, 'CodeableConcept'),
  },
  'AdministrableProductDefinitionRouteOfAdministration': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(false, 'CodeableConcept'),
    'firstDose': const FhirField(false, 'Quantity'),
    'maxSingleDose': const FhirField(false, 'Quantity'),
    'maxDosePerDay': const FhirField(false, 'Quantity'),
    'maxDosePerTreatmentPeriod': const FhirField(false, 'Ratio'),
    'maxTreatmentPeriod': const FhirField(false, 'FhirDuration'),
    'targetSpecies':
        const FhirField(true, 'AdministrableProductDefinitionTargetSpecies'),
  },
  'AdministrableProductDefinitionTargetSpecies': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(false, 'CodeableConcept'),
    'withdrawalPeriod':
        const FhirField(true, 'AdministrableProductDefinitionWithdrawalPeriod'),
  },
  'AdministrableProductDefinitionWithdrawalPeriod': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'tissue': const FhirField(false, 'CodeableConcept'),
    'value': const FhirField(false, 'Quantity'),
    'supportingInformation': const FhirField(false, 'String'),
    'supportingInformationElement': const FhirField(false, 'Element'),
  },
  'AdverseEvent': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'actuality': const FhirField(false, 'Code'),
    'actualityElement': const FhirField(false, 'Element'),
    'category': const FhirField(true, 'CodeableConcept'),
    'code': const FhirField(false, 'CodeableConcept'),
    'subject': const FhirField(false, 'Reference'),
    'encounter': const FhirField(false, 'Reference'),
    'occurrenceDateTime': const FhirField(false, 'FhirDateTime'),
    'occurrenceDateTimeElement': const FhirField(false, 'Element'),
    'occurrencePeriod': const FhirField(false, 'Period'),
    'occurrenceTiming': const FhirField(false, 'Timing'),
    'detected': const FhirField(false, 'FhirDateTime'),
    'detectedElement': const FhirField(false, 'Element'),
    'recordedDate': const FhirField(false, 'FhirDateTime'),
    'recordedDateElement': const FhirField(false, 'Element'),
    'resultingEffect': const FhirField(true, 'Reference'),
    'location': const FhirField(false, 'Reference'),
    'seriousness': const FhirField(false, 'CodeableConcept'),
    'outcome': const FhirField(true, 'CodeableConcept'),
    'recorder': const FhirField(false, 'Reference'),
    'participant': const FhirField(true, 'AdverseEventParticipant'),
    'study': const FhirField(true, 'Reference'),
    'expectedInResearchStudy': const FhirField(false, 'Boolean'),
    'expectedInResearchStudyElement': const FhirField(false, 'Element'),
    'suspectEntity': const FhirField(true, 'AdverseEventSuspectEntity'),
    'contributingFactor':
        const FhirField(true, 'AdverseEventContributingFactor'),
    'preventiveAction': const FhirField(true, 'AdverseEventPreventiveAction'),
    'mitigatingAction': const FhirField(true, 'AdverseEventMitigatingAction'),
    'supportingInfo': const FhirField(true, 'AdverseEventSupportingInfo'),
    'note': const FhirField(true, 'Annotation'),
  },
  'AdverseEventParticipant': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'function': const FhirField(false, 'CodeableConcept'),
    'actor': const FhirField(false, 'Reference'),
  },
  'AdverseEventSuspectEntity': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'instanceCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'instanceReference': const FhirField(false, 'Reference'),
    'causality': const FhirField(false, 'AdverseEventCausality'),
  },
  'AdverseEventCausality': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'assessmentMethod': const FhirField(false, 'CodeableConcept'),
    'entityRelatedness': const FhirField(false, 'CodeableConcept'),
    'author': const FhirField(false, 'Reference'),
  },
  'AdverseEventContributingFactor': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'itemReference': const FhirField(false, 'Reference'),
    'itemCodeableConcept': const FhirField(false, 'CodeableConcept'),
  },
  'AdverseEventPreventiveAction': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'itemReference': const FhirField(false, 'Reference'),
    'itemCodeableConcept': const FhirField(false, 'CodeableConcept'),
  },
  'AdverseEventMitigatingAction': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'itemReference': const FhirField(false, 'Reference'),
    'itemCodeableConcept': const FhirField(false, 'CodeableConcept'),
  },
  'AdverseEventSupportingInfo': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'itemReference': const FhirField(false, 'Reference'),
    'itemCodeableConcept': const FhirField(false, 'CodeableConcept'),
  },
  'AllergyIntolerance': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'clinicalStatus': const FhirField(false, 'CodeableConcept'),
    'verificationStatus': const FhirField(false, 'CodeableConcept'),
    'type': const FhirField(false, 'CodeableConcept'),
    'category': const FhirField(true, 'Code'),
    'categoryElement': const FhirField(true, 'Element'),
    'criticality': const FhirField(false, 'Code'),
    'criticalityElement': const FhirField(false, 'Element'),
    'code': const FhirField(false, 'CodeableConcept'),
    'patient': const FhirField(false, 'Reference'),
    'encounter': const FhirField(false, 'Reference'),
    'onsetDateTime': const FhirField(false, 'FhirDateTime'),
    'onsetDateTimeElement': const FhirField(false, 'Element'),
    'onsetAge': const FhirField(false, 'Age'),
    'onsetPeriod': const FhirField(false, 'Period'),
    'onsetRange': const FhirField(false, 'Range'),
    'onsetString': const FhirField(false, 'String'),
    'onsetStringElement': const FhirField(false, 'Element'),
    'recordedDate': const FhirField(false, 'FhirDateTime'),
    'recordedDateElement': const FhirField(false, 'Element'),
    'participant': const FhirField(true, 'AllergyIntoleranceParticipant'),
    'lastOccurrence': const FhirField(false, 'FhirDateTime'),
    'lastOccurrenceElement': const FhirField(false, 'Element'),
    'note': const FhirField(true, 'Annotation'),
    'reaction': const FhirField(true, 'AllergyIntoleranceReaction'),
  },
  'AllergyIntoleranceParticipant': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'function': const FhirField(false, 'CodeableConcept'),
    'actor': const FhirField(false, 'Reference'),
  },
  'AllergyIntoleranceReaction': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'substance': const FhirField(false, 'CodeableConcept'),
    'manifestation': const FhirField(true, 'CodeableReference'),
    'description': const FhirField(false, 'String'),
    'descriptionElement': const FhirField(false, 'Element'),
    'onset': const FhirField(false, 'FhirDateTime'),
    'onsetElement': const FhirField(false, 'Element'),
    'severity': const FhirField(false, 'Code'),
    'severityElement': const FhirField(false, 'Element'),
    'exposureRoute': const FhirField(false, 'CodeableConcept'),
    'note': const FhirField(true, 'Annotation'),
  },
  'Appointment': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'cancellationReason': const FhirField(false, 'CodeableConcept'),
    'class': const FhirField(true, 'CodeableConcept'),
    'serviceCategory': const FhirField(true, 'CodeableConcept'),
    'serviceType': const FhirField(true, 'CodeableReference'),
    'specialty': const FhirField(true, 'CodeableConcept'),
    'appointmentType': const FhirField(false, 'CodeableConcept'),
    'reason': const FhirField(true, 'CodeableReference'),
    'priority': const FhirField(false, 'CodeableConcept'),
    'description': const FhirField(false, 'String'),
    'descriptionElement': const FhirField(false, 'Element'),
    'replaces': const FhirField(true, 'Reference'),
    'virtualService': const FhirField(true, 'VirtualServiceDetail'),
    'supportingInformation': const FhirField(true, 'Reference'),
    'previousAppointment': const FhirField(false, 'Reference'),
    'originatingAppointment': const FhirField(false, 'Reference'),
    'start': const FhirField(false, 'Instant'),
    'startElement': const FhirField(false, 'Element'),
    'end': const FhirField(false, 'Instant'),
    'endElement': const FhirField(false, 'Element'),
    'minutesDuration': const FhirField(false, 'PositiveInt'),
    'minutesDurationElement': const FhirField(false, 'Element'),
    'requestedPeriod': const FhirField(true, 'Period'),
    'slot': const FhirField(true, 'Reference'),
    'account': const FhirField(true, 'Reference'),
    'created': const FhirField(false, 'FhirDateTime'),
    'createdElement': const FhirField(false, 'Element'),
    'cancellationDate': const FhirField(false, 'FhirDateTime'),
    'cancellationDateElement': const FhirField(false, 'Element'),
    'note': const FhirField(true, 'Annotation'),
    'patientInstruction': const FhirField(true, 'CodeableReference'),
    'basedOn': const FhirField(true, 'Reference'),
    'subject': const FhirField(false, 'Reference'),
    'participant': const FhirField(true, 'AppointmentParticipant'),
    'recurrenceId': const FhirField(false, 'PositiveInt'),
    'recurrenceIdElement': const FhirField(false, 'Element'),
    'occurrenceChanged': const FhirField(false, 'Boolean'),
    'occurrenceChangedElement': const FhirField(false, 'Element'),
    'recurrenceTemplate':
        const FhirField(true, 'AppointmentRecurrenceTemplate'),
  },
  'AppointmentParticipant': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(true, 'CodeableConcept'),
    'period': const FhirField(false, 'Period'),
    'actor': const FhirField(false, 'Reference'),
    'required': const FhirField(false, 'Boolean'),
    'requiredElement': const FhirField(false, 'Element'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
  },
  'AppointmentRecurrenceTemplate': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'timezone': const FhirField(false, 'CodeableConcept'),
    'recurrenceType': const FhirField(false, 'CodeableConcept'),
    'lastOccurrenceDate': const FhirField(false, 'Date'),
    'lastOccurrenceDateElement': const FhirField(false, 'Element'),
    'occurrenceCount': const FhirField(false, 'PositiveInt'),
    'occurrenceCountElement': const FhirField(false, 'Element'),
    'occurrenceDate': const FhirField(true, 'Date'),
    'occurrenceDateElement': const FhirField(true, 'Element'),
    'weeklyTemplate': const FhirField(false, 'AppointmentWeeklyTemplate'),
    'monthlyTemplate': const FhirField(false, 'AppointmentMonthlyTemplate'),
    'yearlyTemplate': const FhirField(false, 'AppointmentYearlyTemplate'),
    'excludingDate': const FhirField(true, 'Date'),
    'excludingDateElement': const FhirField(true, 'Element'),
    'excludingRecurrenceId': const FhirField(true, 'PositiveInt'),
    'excludingRecurrenceIdElement': const FhirField(true, 'Element'),
  },
  'AppointmentWeeklyTemplate': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'monday': const FhirField(false, 'Boolean'),
    'mondayElement': const FhirField(false, 'Element'),
    'tuesday': const FhirField(false, 'Boolean'),
    'tuesdayElement': const FhirField(false, 'Element'),
    'wednesday': const FhirField(false, 'Boolean'),
    'wednesdayElement': const FhirField(false, 'Element'),
    'thursday': const FhirField(false, 'Boolean'),
    'thursdayElement': const FhirField(false, 'Element'),
    'friday': const FhirField(false, 'Boolean'),
    'fridayElement': const FhirField(false, 'Element'),
    'saturday': const FhirField(false, 'Boolean'),
    'saturdayElement': const FhirField(false, 'Element'),
    'sunday': const FhirField(false, 'Boolean'),
    'sundayElement': const FhirField(false, 'Element'),
    'weekInterval': const FhirField(false, 'PositiveInt'),
    'weekIntervalElement': const FhirField(false, 'Element'),
  },
  'AppointmentMonthlyTemplate': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'dayOfMonth': const FhirField(false, 'PositiveInt'),
    'dayOfMonthElement': const FhirField(false, 'Element'),
    'nthWeekOfMonth': const FhirField(false, 'Coding'),
    'dayOfWeek': const FhirField(false, 'Coding'),
    'monthInterval': const FhirField(false, 'PositiveInt'),
    'monthIntervalElement': const FhirField(false, 'Element'),
  },
  'AppointmentYearlyTemplate': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'yearInterval': const FhirField(false, 'PositiveInt'),
    'yearIntervalElement': const FhirField(false, 'Element'),
  },
  'AppointmentResponse': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'appointment': const FhirField(false, 'Reference'),
    'proposedNewTime': const FhirField(false, 'Boolean'),
    'proposedNewTimeElement': const FhirField(false, 'Element'),
    'start': const FhirField(false, 'Instant'),
    'startElement': const FhirField(false, 'Element'),
    'end': const FhirField(false, 'Instant'),
    'endElement': const FhirField(false, 'Element'),
    'participantType': const FhirField(true, 'CodeableConcept'),
    'actor': const FhirField(false, 'Reference'),
    'participantStatus': const FhirField(false, 'Code'),
    'participantStatusElement': const FhirField(false, 'Element'),
    'comment': const FhirField(false, 'Markdown'),
    'commentElement': const FhirField(false, 'Element'),
    'recurring': const FhirField(false, 'Boolean'),
    'recurringElement': const FhirField(false, 'Element'),
    'occurrenceDate': const FhirField(false, 'Date'),
    'occurrenceDateElement': const FhirField(false, 'Element'),
    'recurrenceId': const FhirField(false, 'PositiveInt'),
    'recurrenceIdElement': const FhirField(false, 'Element'),
  },
  'ArtifactAssessment': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'title': const FhirField(false, 'String'),
    'titleElement': const FhirField(false, 'Element'),
    'citeAsReference': const FhirField(false, 'Reference'),
    'citeAsMarkdown': const FhirField(false, 'Markdown'),
    'citeAsMarkdownElement': const FhirField(false, 'Element'),
    'date': const FhirField(false, 'FhirDateTime'),
    'dateElement': const FhirField(false, 'Element'),
    'copyright': const FhirField(false, 'Markdown'),
    'copyrightElement': const FhirField(false, 'Element'),
    'approvalDate': const FhirField(false, 'Date'),
    'approvalDateElement': const FhirField(false, 'Element'),
    'lastReviewDate': const FhirField(false, 'Date'),
    'lastReviewDateElement': const FhirField(false, 'Element'),
    'artifactReference': const FhirField(false, 'Reference'),
    'artifactCanonical': const FhirField(false, 'Canonical'),
    'artifactCanonicalElement': const FhirField(false, 'Element'),
    'artifactUri': const FhirField(false, 'FhirUri'),
    'artifactUriElement': const FhirField(false, 'Element'),
    'content': const FhirField(true, 'ArtifactAssessmentContent'),
    'workflowStatus': const FhirField(false, 'Code'),
    'workflowStatusElement': const FhirField(false, 'Element'),
    'disposition': const FhirField(false, 'Code'),
    'dispositionElement': const FhirField(false, 'Element'),
  },
  'ArtifactAssessmentContent': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'informationType': const FhirField(false, 'Code'),
    'informationTypeElement': const FhirField(false, 'Element'),
    'summary': const FhirField(false, 'Markdown'),
    'summaryElement': const FhirField(false, 'Element'),
    'type': const FhirField(false, 'CodeableConcept'),
    'classifier': const FhirField(true, 'CodeableConcept'),
    'quantity': const FhirField(false, 'Quantity'),
    'author': const FhirField(false, 'Reference'),
    'path': const FhirField(true, 'FhirUri'),
    'pathElement': const FhirField(true, 'Element'),
    'relatedArtifact': const FhirField(true, 'RelatedArtifact'),
    'freeToShare': const FhirField(false, 'Boolean'),
    'freeToShareElement': const FhirField(false, 'Element'),
    'component': const FhirField(true, 'ArtifactAssessmentContent'),
  },
  'AuditEvent': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'category': const FhirField(true, 'CodeableConcept'),
    'code': const FhirField(false, 'CodeableConcept'),
    'action': const FhirField(false, 'Code'),
    'actionElement': const FhirField(false, 'Element'),
    'severity': const FhirField(false, 'Code'),
    'severityElement': const FhirField(false, 'Element'),
    'occurredPeriod': const FhirField(false, 'Period'),
    'occurredDateTime': const FhirField(false, 'FhirDateTime'),
    'occurredDateTimeElement': const FhirField(false, 'Element'),
    'recorded': const FhirField(false, 'Instant'),
    'recordedElement': const FhirField(false, 'Element'),
    'outcome': const FhirField(false, 'AuditEventOutcome'),
    'authorization': const FhirField(true, 'CodeableConcept'),
    'basedOn': const FhirField(true, 'Reference'),
    'patient': const FhirField(false, 'Reference'),
    'encounter': const FhirField(false, 'Reference'),
    'agent': const FhirField(true, 'AuditEventAgent'),
    'source': const FhirField(false, 'AuditEventSource'),
    'entity': const FhirField(true, 'AuditEventEntity'),
  },
  'AuditEventOutcome': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(false, 'Coding'),
    'detail': const FhirField(true, 'CodeableConcept'),
  },
  'AuditEventAgent': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'role': const FhirField(true, 'CodeableConcept'),
    'who': const FhirField(false, 'Reference'),
    'requestor': const FhirField(false, 'Boolean'),
    'requestorElement': const FhirField(false, 'Element'),
    'location': const FhirField(false, 'Reference'),
    'policy': const FhirField(true, 'FhirUri'),
    'policyElement': const FhirField(true, 'Element'),
    'networkReference': const FhirField(false, 'Reference'),
    'networkUri': const FhirField(false, 'FhirUri'),
    'networkUriElement': const FhirField(false, 'Element'),
    'networkString': const FhirField(false, 'String'),
    'networkStringElement': const FhirField(false, 'Element'),
    'authorization': const FhirField(true, 'CodeableConcept'),
  },
  'AuditEventSource': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'site': const FhirField(false, 'Reference'),
    'observer': const FhirField(false, 'Reference'),
    'type': const FhirField(true, 'CodeableConcept'),
  },
  'AuditEventEntity': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'what': const FhirField(false, 'Reference'),
    'role': const FhirField(false, 'CodeableConcept'),
    'securityLabel': const FhirField(true, 'CodeableConcept'),
    'query': const FhirField(false, 'Base64Binary'),
    'queryElement': const FhirField(false, 'Element'),
    'detail': const FhirField(true, 'AuditEventDetail'),
    'agent': const FhirField(true, 'AuditEventAgent'),
  },
  'AuditEventDetail': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'valueQuantity': const FhirField(false, 'Quantity'),
    'valueCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'valueString': const FhirField(false, 'String'),
    'valueStringElement': const FhirField(false, 'Element'),
    'valueBoolean': const FhirField(false, 'Boolean'),
    'valueBooleanElement': const FhirField(false, 'Element'),
    'valueInteger': const FhirField(false, 'Integer'),
    'valueIntegerElement': const FhirField(false, 'Element'),
    'valueRange': const FhirField(false, 'Range'),
    'valueRatio': const FhirField(false, 'Ratio'),
    'valueTime': const FhirField(false, 'Time'),
    'valueTimeElement': const FhirField(false, 'Element'),
    'valueDateTime': const FhirField(false, 'FhirDateTime'),
    'valueDateTimeElement': const FhirField(false, 'Element'),
    'valuePeriod': const FhirField(false, 'Period'),
    'valueBase64Binary': const FhirField(false, 'Base64Binary'),
    'valueBase64BinaryElement': const FhirField(false, 'Element'),
  },
  'Basic': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'code': const FhirField(false, 'CodeableConcept'),
    'subject': const FhirField(false, 'Reference'),
    'created': const FhirField(false, 'FhirDateTime'),
    'createdElement': const FhirField(false, 'Element'),
    'author': const FhirField(false, 'Reference'),
  },
  'Binary': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'contentType': const FhirField(false, 'Code'),
    'contentTypeElement': const FhirField(false, 'Element'),
    'securityContext': const FhirField(false, 'Reference'),
    'data': const FhirField(false, 'Base64Binary'),
    'dataElement': const FhirField(false, 'Element'),
  },
  'BiologicallyDerivedProduct': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'productCategory': const FhirField(false, 'Coding'),
    'productCode': const FhirField(false, 'CodeableConcept'),
    'parent': const FhirField(true, 'Reference'),
    'request': const FhirField(true, 'Reference'),
    'identifier': const FhirField(true, 'Identifier'),
    'biologicalSourceEvent': const FhirField(false, 'Identifier'),
    'processingFacility': const FhirField(true, 'Reference'),
    'division': const FhirField(false, 'String'),
    'divisionElement': const FhirField(false, 'Element'),
    'productStatus': const FhirField(false, 'Coding'),
    'expirationDate': const FhirField(false, 'FhirDateTime'),
    'expirationDateElement': const FhirField(false, 'Element'),
    'collection':
        const FhirField(false, 'BiologicallyDerivedProductCollection'),
    'storageTempRequirements': const FhirField(false, 'Range'),
    'property': const FhirField(true, 'BiologicallyDerivedProductProperty'),
  },
  'BiologicallyDerivedProductCollection': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'collector': const FhirField(false, 'Reference'),
    'source': const FhirField(false, 'Reference'),
    'collectedDateTime': const FhirField(false, 'FhirDateTime'),
    'collectedDateTimeElement': const FhirField(false, 'Element'),
    'collectedPeriod': const FhirField(false, 'Period'),
  },
  'BiologicallyDerivedProductProperty': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'valueBoolean': const FhirField(false, 'Boolean'),
    'valueBooleanElement': const FhirField(false, 'Element'),
    'valueInteger': const FhirField(false, 'Integer'),
    'valueIntegerElement': const FhirField(false, 'Element'),
    'valueCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'valuePeriod': const FhirField(false, 'Period'),
    'valueQuantity': const FhirField(false, 'Quantity'),
    'valueRange': const FhirField(false, 'Range'),
    'valueRatio': const FhirField(false, 'Ratio'),
    'valueString': const FhirField(false, 'String'),
    'valueStringElement': const FhirField(false, 'Element'),
    'valueAttachment': const FhirField(false, 'Attachment'),
  },
  'BiologicallyDerivedProductDispense': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'basedOn': const FhirField(true, 'Reference'),
    'partOf': const FhirField(true, 'Reference'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'originRelationshipType': const FhirField(false, 'CodeableConcept'),
    'product': const FhirField(false, 'Reference'),
    'patient': const FhirField(false, 'Reference'),
    'matchStatus': const FhirField(false, 'CodeableConcept'),
    'performer':
        const FhirField(true, 'BiologicallyDerivedProductDispensePerformer'),
    'location': const FhirField(false, 'Reference'),
    'quantity': const FhirField(false, 'Quantity'),
    'preparedDate': const FhirField(false, 'FhirDateTime'),
    'preparedDateElement': const FhirField(false, 'Element'),
    'whenHandedOver': const FhirField(false, 'FhirDateTime'),
    'whenHandedOverElement': const FhirField(false, 'Element'),
    'destination': const FhirField(false, 'Reference'),
    'note': const FhirField(true, 'Annotation'),
    'usageInstruction': const FhirField(false, 'String'),
    'usageInstructionElement': const FhirField(false, 'Element'),
  },
  'BiologicallyDerivedProductDispensePerformer': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'function': const FhirField(false, 'CodeableConcept'),
    'actor': const FhirField(false, 'Reference'),
  },
  'BodyStructure': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'active': const FhirField(false, 'Boolean'),
    'activeElement': const FhirField(false, 'Element'),
    'morphology': const FhirField(false, 'CodeableConcept'),
    'includedStructure':
        const FhirField(true, 'BodyStructureIncludedStructure'),
    'excludedStructure':
        const FhirField(true, 'BodyStructureIncludedStructure'),
    'description': const FhirField(false, 'Markdown'),
    'descriptionElement': const FhirField(false, 'Element'),
    'image': const FhirField(true, 'Attachment'),
    'patient': const FhirField(false, 'Reference'),
  },
  'BodyStructureIncludedStructure': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'structure': const FhirField(false, 'CodeableConcept'),
    'laterality': const FhirField(false, 'CodeableConcept'),
    'bodyLandmarkOrientation':
        const FhirField(true, 'BodyStructureBodyLandmarkOrientation'),
    'spatialReference': const FhirField(true, 'Reference'),
    'qualifier': const FhirField(true, 'CodeableConcept'),
  },
  'BodyStructureBodyLandmarkOrientation': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'landmarkDescription': const FhirField(true, 'CodeableConcept'),
    'clockFacePosition': const FhirField(true, 'CodeableConcept'),
    'distanceFromLandmark':
        const FhirField(true, 'BodyStructureDistanceFromLandmark'),
    'surfaceOrientation': const FhirField(true, 'CodeableConcept'),
  },
  'BodyStructureDistanceFromLandmark': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'device': const FhirField(true, 'CodeableReference'),
    'value': const FhirField(true, 'Quantity'),
  },
  'Bundle': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'identifier': const FhirField(false, 'Identifier'),
    'type': const FhirField(false, 'Code'),
    'typeElement': const FhirField(false, 'Element'),
    'timestamp': const FhirField(false, 'Instant'),
    'timestampElement': const FhirField(false, 'Element'),
    'total': const FhirField(false, 'UnsignedInt'),
    'totalElement': const FhirField(false, 'Element'),
    'link': const FhirField(true, 'BundleLink'),
    'entry': const FhirField(true, 'BundleEntry'),
    'signature': const FhirField(false, 'Signature'),
    'issues': const FhirField(false, 'ResourceList'),
  },
  'BundleLink': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'relation': const FhirField(false, 'Code'),
    'relationElement': const FhirField(false, 'Element'),
    'url': const FhirField(false, 'FhirUri'),
    'urlElement': const FhirField(false, 'Element'),
  },
  'BundleEntry': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'link': const FhirField(true, 'BundleLink'),
    'fullUrl': const FhirField(false, 'FhirUri'),
    'fullUrlElement': const FhirField(false, 'Element'),
    'resource': const FhirField(false, 'ResourceList'),
    'search': const FhirField(false, 'BundleSearch'),
    'request': const FhirField(false, 'BundleRequest'),
    'response': const FhirField(false, 'BundleResponse'),
  },
  'BundleSearch': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'mode': const FhirField(false, 'Code'),
    'modeElement': const FhirField(false, 'Element'),
    'score': const FhirField(false, 'Decimal'),
    'scoreElement': const FhirField(false, 'Element'),
  },
  'BundleRequest': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'method': const FhirField(false, 'Code'),
    'methodElement': const FhirField(false, 'Element'),
    'url': const FhirField(false, 'FhirUri'),
    'urlElement': const FhirField(false, 'Element'),
    'ifNoneMatch': const FhirField(false, 'String'),
    'ifNoneMatchElement': const FhirField(false, 'Element'),
    'ifModifiedSince': const FhirField(false, 'Instant'),
    'ifModifiedSinceElement': const FhirField(false, 'Element'),
    'ifMatch': const FhirField(false, 'String'),
    'ifMatchElement': const FhirField(false, 'Element'),
    'ifNoneExist': const FhirField(false, 'String'),
    'ifNoneExistElement': const FhirField(false, 'Element'),
  },
  'BundleResponse': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'status': const FhirField(false, 'String'),
    'statusElement': const FhirField(false, 'Element'),
    'location': const FhirField(false, 'FhirUri'),
    'locationElement': const FhirField(false, 'Element'),
    'etag': const FhirField(false, 'String'),
    'etagElement': const FhirField(false, 'Element'),
    'lastModified': const FhirField(false, 'Instant'),
    'lastModifiedElement': const FhirField(false, 'Element'),
    'outcome': const FhirField(false, 'ResourceList'),
  },
  'CapabilityStatement': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'url': const FhirField(false, 'FhirUri'),
    'urlElement': const FhirField(false, 'Element'),
    'identifier': const FhirField(true, 'Identifier'),
    'version': const FhirField(false, 'String'),
    'versionElement': const FhirField(false, 'Element'),
    'versionAlgorithmString': const FhirField(false, 'String'),
    'versionAlgorithmStringElement': const FhirField(false, 'Element'),
    'versionAlgorithmCoding': const FhirField(false, 'Coding'),
    'name': const FhirField(false, 'String'),
    'nameElement': const FhirField(false, 'Element'),
    'title': const FhirField(false, 'String'),
    'titleElement': const FhirField(false, 'Element'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'experimental': const FhirField(false, 'Boolean'),
    'experimentalElement': const FhirField(false, 'Element'),
    'date': const FhirField(false, 'FhirDateTime'),
    'dateElement': const FhirField(false, 'Element'),
    'publisher': const FhirField(false, 'String'),
    'publisherElement': const FhirField(false, 'Element'),
    'contact': const FhirField(true, 'ContactDetail'),
    'description': const FhirField(false, 'Markdown'),
    'descriptionElement': const FhirField(false, 'Element'),
    'useContext': const FhirField(true, 'UsageContext'),
    'jurisdiction': const FhirField(true, 'CodeableConcept'),
    'purpose': const FhirField(false, 'Markdown'),
    'purposeElement': const FhirField(false, 'Element'),
    'copyright': const FhirField(false, 'Markdown'),
    'copyrightElement': const FhirField(false, 'Element'),
    'copyrightLabel': const FhirField(false, 'String'),
    'copyrightLabelElement': const FhirField(false, 'Element'),
    'kind': const FhirField(false, 'Code'),
    'kindElement': const FhirField(false, 'Element'),
    'instantiates': const FhirField(true, 'Canonical'),
    'imports': const FhirField(true, 'Canonical'),
    'software': const FhirField(false, 'CapabilityStatementSoftware'),
    'implementation':
        const FhirField(false, 'CapabilityStatementImplementation'),
    'fhirVersion': const FhirField(false, 'Code'),
    'fhirVersionElement': const FhirField(false, 'Element'),
    'format': const FhirField(true, 'Code'),
    'formatElement': const FhirField(true, 'Element'),
    'patchFormat': const FhirField(true, 'Code'),
    'patchFormatElement': const FhirField(true, 'Element'),
    'acceptLanguage': const FhirField(true, 'Code'),
    'acceptLanguageElement': const FhirField(true, 'Element'),
    'implementationGuide': const FhirField(true, 'Canonical'),
    'rest': const FhirField(true, 'CapabilityStatementRest'),
    'messaging': const FhirField(true, 'CapabilityStatementMessaging'),
    'document': const FhirField(true, 'CapabilityStatementDocument'),
  },
  'CapabilityStatementSoftware': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'name': const FhirField(false, 'String'),
    'nameElement': const FhirField(false, 'Element'),
    'version': const FhirField(false, 'String'),
    'versionElement': const FhirField(false, 'Element'),
    'releaseDate': const FhirField(false, 'FhirDateTime'),
    'releaseDateElement': const FhirField(false, 'Element'),
  },
  'CapabilityStatementImplementation': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'description': const FhirField(false, 'Markdown'),
    'descriptionElement': const FhirField(false, 'Element'),
    'url': const FhirField(false, 'FhirUrl'),
    'urlElement': const FhirField(false, 'Element'),
    'custodian': const FhirField(false, 'Reference'),
  },
  'CapabilityStatementRest': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'mode': const FhirField(false, 'Code'),
    'modeElement': const FhirField(false, 'Element'),
    'documentation': const FhirField(false, 'Markdown'),
    'documentationElement': const FhirField(false, 'Element'),
    'security': const FhirField(false, 'CapabilityStatementSecurity'),
    'resource': const FhirField(true, 'CapabilityStatementResource'),
    'interaction': const FhirField(true, 'CapabilityStatementInteraction1'),
    'searchParam': const FhirField(true, 'CapabilityStatementSearchParam'),
    'operation': const FhirField(true, 'CapabilityStatementOperation'),
    'compartment': const FhirField(true, 'Canonical'),
  },
  'CapabilityStatementSecurity': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'cors': const FhirField(false, 'Boolean'),
    'corsElement': const FhirField(false, 'Element'),
    'service': const FhirField(true, 'CodeableConcept'),
    'description': const FhirField(false, 'Markdown'),
    'descriptionElement': const FhirField(false, 'Element'),
  },
  'CapabilityStatementResource': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'Code'),
    'typeElement': const FhirField(false, 'Element'),
    'profile': const FhirField(false, 'Canonical'),
    'supportedProfile': const FhirField(true, 'Canonical'),
    'documentation': const FhirField(false, 'Markdown'),
    'documentationElement': const FhirField(false, 'Element'),
    'interaction': const FhirField(true, 'CapabilityStatementInteraction'),
    'versioning': const FhirField(false, 'Code'),
    'versioningElement': const FhirField(false, 'Element'),
    'readHistory': const FhirField(false, 'Boolean'),
    'readHistoryElement': const FhirField(false, 'Element'),
    'updateCreate': const FhirField(false, 'Boolean'),
    'updateCreateElement': const FhirField(false, 'Element'),
    'conditionalCreate': const FhirField(false, 'Boolean'),
    'conditionalCreateElement': const FhirField(false, 'Element'),
    'conditionalRead': const FhirField(false, 'Code'),
    'conditionalReadElement': const FhirField(false, 'Element'),
    'conditionalUpdate': const FhirField(false, 'Boolean'),
    'conditionalUpdateElement': const FhirField(false, 'Element'),
    'conditionalPatch': const FhirField(false, 'Boolean'),
    'conditionalPatchElement': const FhirField(false, 'Element'),
    'conditionalDelete': const FhirField(false, 'Code'),
    'conditionalDeleteElement': const FhirField(false, 'Element'),
    'referencePolicy': const FhirField(true, 'Code'),
    'referencePolicyElement': const FhirField(true, 'Element'),
    'searchInclude': const FhirField(true, 'String'),
    'searchIncludeElement': const FhirField(true, 'Element'),
    'searchRevInclude': const FhirField(true, 'String'),
    'searchRevIncludeElement': const FhirField(true, 'Element'),
    'searchParam': const FhirField(true, 'CapabilityStatementSearchParam'),
    'operation': const FhirField(true, 'CapabilityStatementOperation'),
  },
  'CapabilityStatementInteraction': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(false, 'Code'),
    'codeElement': const FhirField(false, 'Element'),
    'documentation': const FhirField(false, 'Markdown'),
    'documentationElement': const FhirField(false, 'Element'),
  },
  'CapabilityStatementSearchParam': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'name': const FhirField(false, 'String'),
    'nameElement': const FhirField(false, 'Element'),
    'definition': const FhirField(false, 'Canonical'),
    'type': const FhirField(false, 'Code'),
    'typeElement': const FhirField(false, 'Element'),
    'documentation': const FhirField(false, 'Markdown'),
    'documentationElement': const FhirField(false, 'Element'),
  },
  'CapabilityStatementOperation': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'name': const FhirField(false, 'String'),
    'nameElement': const FhirField(false, 'Element'),
    'definition': const FhirField(false, 'Canonical'),
    'documentation': const FhirField(false, 'Markdown'),
    'documentationElement': const FhirField(false, 'Element'),
  },
  'CapabilityStatementInteraction1': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(false, 'Code'),
    'codeElement': const FhirField(false, 'Element'),
    'documentation': const FhirField(false, 'Markdown'),
    'documentationElement': const FhirField(false, 'Element'),
  },
  'CapabilityStatementMessaging': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'endpoint': const FhirField(true, 'CapabilityStatementEndpoint'),
    'reliableCache': const FhirField(false, 'UnsignedInt'),
    'reliableCacheElement': const FhirField(false, 'Element'),
    'documentation': const FhirField(false, 'Markdown'),
    'documentationElement': const FhirField(false, 'Element'),
    'supportedMessage':
        const FhirField(true, 'CapabilityStatementSupportedMessage'),
  },
  'CapabilityStatementEndpoint': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'protocol': const FhirField(false, 'Coding'),
    'address': const FhirField(false, 'FhirUrl'),
    'addressElement': const FhirField(false, 'Element'),
  },
  'CapabilityStatementSupportedMessage': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'mode': const FhirField(false, 'Code'),
    'modeElement': const FhirField(false, 'Element'),
    'definition': const FhirField(false, 'Canonical'),
  },
  'CapabilityStatementDocument': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'mode': const FhirField(false, 'Code'),
    'modeElement': const FhirField(false, 'Element'),
    'documentation': const FhirField(false, 'Markdown'),
    'documentationElement': const FhirField(false, 'Element'),
    'profile': const FhirField(false, 'Canonical'),
  },
  'CarePlan': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'instantiatesCanonical': const FhirField(true, 'Canonical'),
    'instantiatesUri': const FhirField(true, 'FhirUri'),
    'instantiatesUriElement': const FhirField(true, 'Element'),
    'basedOn': const FhirField(true, 'Reference'),
    'replaces': const FhirField(true, 'Reference'),
    'partOf': const FhirField(true, 'Reference'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'intent': const FhirField(false, 'Code'),
    'intentElement': const FhirField(false, 'Element'),
    'category': const FhirField(true, 'CodeableConcept'),
    'title': const FhirField(false, 'String'),
    'titleElement': const FhirField(false, 'Element'),
    'description': const FhirField(false, 'String'),
    'descriptionElement': const FhirField(false, 'Element'),
    'subject': const FhirField(false, 'Reference'),
    'encounter': const FhirField(false, 'Reference'),
    'period': const FhirField(false, 'Period'),
    'created': const FhirField(false, 'FhirDateTime'),
    'createdElement': const FhirField(false, 'Element'),
    'custodian': const FhirField(false, 'Reference'),
    'contributor': const FhirField(true, 'Reference'),
    'careTeam': const FhirField(true, 'Reference'),
    'addresses': const FhirField(true, 'CodeableReference'),
    'supportingInfo': const FhirField(true, 'Reference'),
    'goal': const FhirField(true, 'Reference'),
    'activity': const FhirField(true, 'CarePlanActivity'),
    'note': const FhirField(true, 'Annotation'),
  },
  'CarePlanActivity': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'performedActivity': const FhirField(true, 'CodeableReference'),
    'progress': const FhirField(true, 'Annotation'),
    'plannedActivityReference': const FhirField(false, 'Reference'),
  },
  'CarePlanPlannedActivityDetail': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'kind': const FhirField(false, 'Code'),
    'kindElement': const FhirField(false, 'Element'),
    'instantiatesCanonical': const FhirField(true, 'Canonical'),
    'instantiatesUri': const FhirField(true, 'FhirUri'),
    'instantiatesUriElement': const FhirField(true, 'Element'),
    'code': const FhirField(false, 'CodeableConcept'),
    'reason': const FhirField(true, 'CodeableReference'),
    'goal': const FhirField(true, 'Reference'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'statusReason': const FhirField(false, 'CodeableConcept'),
    'doNotPerform': const FhirField(false, 'Boolean'),
    'doNotPerformElement': const FhirField(false, 'Element'),
    'scheduledTiming': const FhirField(false, 'Timing'),
    'scheduledPeriod': const FhirField(false, 'Period'),
    'scheduledString': const FhirField(false, 'String'),
    'scheduledStringElement': const FhirField(false, 'Element'),
    'location': const FhirField(false, 'CodeableReference'),
    'reportedBoolean': const FhirField(false, 'Boolean'),
    'reportedBooleanElement': const FhirField(false, 'Element'),
    'reportedReference': const FhirField(false, 'Reference'),
    'performer': const FhirField(true, 'Reference'),
    'productCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'productReference': const FhirField(false, 'Reference'),
    'dailyAmount': const FhirField(false, 'Quantity'),
    'quantity': const FhirField(false, 'Quantity'),
    'description': const FhirField(false, 'String'),
    'descriptionElement': const FhirField(false, 'Element'),
  },
  'CareTeam': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'category': const FhirField(true, 'CodeableConcept'),
    'name': const FhirField(false, 'String'),
    'nameElement': const FhirField(false, 'Element'),
    'subject': const FhirField(false, 'Reference'),
    'period': const FhirField(false, 'Period'),
    'participant': const FhirField(true, 'CareTeamParticipant'),
    'reason': const FhirField(true, 'CodeableReference'),
    'managingOrganization': const FhirField(true, 'Reference'),
    'telecom': const FhirField(true, 'ContactPoint'),
    'note': const FhirField(true, 'Annotation'),
  },
  'CareTeamParticipant': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'role': const FhirField(false, 'CodeableConcept'),
    'member': const FhirField(false, 'Reference'),
    'onBehalfOf': const FhirField(false, 'Reference'),
    'coveragePeriod': const FhirField(false, 'Period'),
    'coverageTiming': const FhirField(false, 'Timing'),
  },
  'ChargeItem': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'definitionUri': const FhirField(true, 'FhirUri'),
    'definitionUriElement': const FhirField(true, 'Element'),
    'definitionCanonical': const FhirField(true, 'Canonical'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'partOf': const FhirField(true, 'Reference'),
    'code': const FhirField(false, 'CodeableConcept'),
    'subject': const FhirField(false, 'Reference'),
    'encounter': const FhirField(false, 'Reference'),
    'occurrenceDateTime': const FhirField(false, 'FhirDateTime'),
    'occurrenceDateTimeElement': const FhirField(false, 'Element'),
    'occurrencePeriod': const FhirField(false, 'Period'),
    'occurrenceTiming': const FhirField(false, 'Timing'),
    'performer': const FhirField(true, 'ChargeItemPerformer'),
    'performingOrganization': const FhirField(false, 'Reference'),
    'requestingOrganization': const FhirField(false, 'Reference'),
    'costCenter': const FhirField(false, 'Reference'),
    'quantity': const FhirField(false, 'Quantity'),
    'bodysite': const FhirField(true, 'CodeableConcept'),
    'unitPriceComponent': const FhirField(false, 'MonetaryComponent'),
    'totalPriceComponent': const FhirField(false, 'MonetaryComponent'),
    'overrideReason': const FhirField(false, 'CodeableConcept'),
    'enterer': const FhirField(false, 'Reference'),
    'enteredDate': const FhirField(false, 'FhirDateTime'),
    'enteredDateElement': const FhirField(false, 'Element'),
    'reason': const FhirField(true, 'CodeableConcept'),
    'service': const FhirField(true, 'CodeableReference'),
    'product': const FhirField(true, 'CodeableReference'),
    'account': const FhirField(true, 'Reference'),
    'note': const FhirField(true, 'Annotation'),
    'supportingInformation': const FhirField(true, 'Reference'),
  },
  'ChargeItemPerformer': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'function': const FhirField(false, 'CodeableConcept'),
    'actor': const FhirField(false, 'Reference'),
  },
  'ChargeItemDefinition': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'url': const FhirField(false, 'FhirUri'),
    'urlElement': const FhirField(false, 'Element'),
    'identifier': const FhirField(true, 'Identifier'),
    'version': const FhirField(false, 'String'),
    'versionElement': const FhirField(false, 'Element'),
    'versionAlgorithmString': const FhirField(false, 'String'),
    'versionAlgorithmStringElement': const FhirField(false, 'Element'),
    'versionAlgorithmCoding': const FhirField(false, 'Coding'),
    'name': const FhirField(false, 'String'),
    'nameElement': const FhirField(false, 'Element'),
    'title': const FhirField(false, 'String'),
    'titleElement': const FhirField(false, 'Element'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'experimental': const FhirField(false, 'Boolean'),
    'experimentalElement': const FhirField(false, 'Element'),
    'date': const FhirField(false, 'FhirDateTime'),
    'dateElement': const FhirField(false, 'Element'),
    'publisher': const FhirField(false, 'String'),
    'publisherElement': const FhirField(false, 'Element'),
    'contact': const FhirField(true, 'ContactDetail'),
    'description': const FhirField(false, 'Markdown'),
    'descriptionElement': const FhirField(false, 'Element'),
    'useContext': const FhirField(true, 'UsageContext'),
    'jurisdiction': const FhirField(true, 'CodeableConcept'),
    'purpose': const FhirField(false, 'Markdown'),
    'purposeElement': const FhirField(false, 'Element'),
    'copyright': const FhirField(false, 'Markdown'),
    'copyrightElement': const FhirField(false, 'Element'),
    'copyrightLabel': const FhirField(false, 'String'),
    'copyrightLabelElement': const FhirField(false, 'Element'),
    'approvalDate': const FhirField(false, 'Date'),
    'approvalDateElement': const FhirField(false, 'Element'),
    'lastReviewDate': const FhirField(false, 'Date'),
    'lastReviewDateElement': const FhirField(false, 'Element'),
    'effectivePeriod': const FhirField(false, 'Period'),
    'topic': const FhirField(true, 'CodeableConcept'),
    'author': const FhirField(true, 'ContactDetail'),
    'editor': const FhirField(true, 'ContactDetail'),
    'reviewer': const FhirField(true, 'ContactDetail'),
    'endorser': const FhirField(true, 'ContactDetail'),
    'relatedArtifact': const FhirField(true, 'RelatedArtifact'),
    'derivedFromUri': const FhirField(true, 'FhirUri'),
    'derivedFromUriElement': const FhirField(true, 'Element'),
    'partOf': const FhirField(true, 'Canonical'),
    'replaces': const FhirField(true, 'Canonical'),
    'code': const FhirField(false, 'CodeableConcept'),
    'instance': const FhirField(true, 'Reference'),
    'applicability': const FhirField(true, 'ChargeItemDefinitionApplicability'),
    'propertyGroup': const FhirField(true, 'ChargeItemDefinitionPropertyGroup'),
  },
  'ChargeItemDefinitionApplicability': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'condition': const FhirField(false, 'Expression'),
    'effectivePeriod': const FhirField(false, 'Period'),
    'relatedArtifact': const FhirField(false, 'RelatedArtifact'),
  },
  'ChargeItemDefinitionPropertyGroup': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'applicability': const FhirField(true, 'ChargeItemDefinitionApplicability'),
    'priceComponent': const FhirField(true, 'MonetaryComponent'),
  },
  'Citation': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'url': const FhirField(false, 'FhirUri'),
    'urlElement': const FhirField(false, 'Element'),
    'identifier': const FhirField(true, 'Identifier'),
    'version': const FhirField(false, 'String'),
    'versionElement': const FhirField(false, 'Element'),
    'versionAlgorithmString': const FhirField(false, 'String'),
    'versionAlgorithmStringElement': const FhirField(false, 'Element'),
    'versionAlgorithmCoding': const FhirField(false, 'Coding'),
    'name': const FhirField(false, 'String'),
    'nameElement': const FhirField(false, 'Element'),
    'title': const FhirField(false, 'String'),
    'titleElement': const FhirField(false, 'Element'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'experimental': const FhirField(false, 'Boolean'),
    'experimentalElement': const FhirField(false, 'Element'),
    'date': const FhirField(false, 'FhirDateTime'),
    'dateElement': const FhirField(false, 'Element'),
    'publisher': const FhirField(false, 'String'),
    'publisherElement': const FhirField(false, 'Element'),
    'contact': const FhirField(true, 'ContactDetail'),
    'description': const FhirField(false, 'Markdown'),
    'descriptionElement': const FhirField(false, 'Element'),
    'useContext': const FhirField(true, 'UsageContext'),
    'jurisdiction': const FhirField(true, 'CodeableConcept'),
    'purpose': const FhirField(false, 'Markdown'),
    'purposeElement': const FhirField(false, 'Element'),
    'copyright': const FhirField(false, 'Markdown'),
    'copyrightElement': const FhirField(false, 'Element'),
    'copyrightLabel': const FhirField(false, 'String'),
    'copyrightLabelElement': const FhirField(false, 'Element'),
    'approvalDate': const FhirField(false, 'Date'),
    'approvalDateElement': const FhirField(false, 'Element'),
    'lastReviewDate': const FhirField(false, 'Date'),
    'lastReviewDateElement': const FhirField(false, 'Element'),
    'effectivePeriod': const FhirField(false, 'Period'),
    'topic': const FhirField(true, 'CodeableConcept'),
    'author': const FhirField(true, 'ContactDetail'),
    'editor': const FhirField(true, 'ContactDetail'),
    'reviewer': const FhirField(true, 'ContactDetail'),
    'endorser': const FhirField(true, 'ContactDetail'),
    'relatedArtifact': const FhirField(true, 'RelatedArtifact'),
    'summary': const FhirField(true, 'CitationSummary'),
    'classification': const FhirField(true, 'CitationClassification'),
    'note': const FhirField(true, 'Annotation'),
    'currentState': const FhirField(true, 'CodeableConcept'),
    'statusDate': const FhirField(true, 'Date'),
    'citedArtifact': const FhirField(false, 'CitationCitedArtifact'),
  },
  'CitationSummary': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'style': const FhirField(false, 'CodeableConcept'),
    'text': const FhirField(false, 'Markdown'),
    'textElement': const FhirField(false, 'Element'),
  },
  'CitationClassification': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'classifier': const FhirField(true, 'CodeableConcept'),
  },
  'CitationStatusDate': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'activity': const FhirField(false, 'CodeableConcept'),
    'actual': const FhirField(false, 'Boolean'),
    'actualElement': const FhirField(false, 'Element'),
    'period': const FhirField(false, 'Period'),
  },
  'CitationCitedArtifact': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'relatedIdentifier': const FhirField(true, 'Identifier'),
    'dateAccessed': const FhirField(false, 'FhirDateTime'),
    'dateAccessedElement': const FhirField(false, 'Element'),
    'version': const FhirField(false, 'CitationVersion'),
    'currentState': const FhirField(true, 'CodeableConcept'),
    'statusDate': const FhirField(true, 'CitationStatusDate1'),
    'title': const FhirField(true, 'CitationTitle'),
    'abstract': const FhirField(true, 'CitationAbstract'),
    'part': const FhirField(false, 'CitationPart'),
    'relatesTo': const FhirField(true, 'CitationRelatesTo'),
    'publicationForm': const FhirField(true, 'CitationPublicationForm'),
    'webLocation': const FhirField(true, 'CitationWebLocation'),
    'classification': const FhirField(true, 'CitationClassification1'),
    'contributorship': const FhirField(false, 'CitationContributorship'),
    'note': const FhirField(true, 'Annotation'),
  },
  'CitationVersion': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'value': const FhirField(false, 'String'),
    'valueElement': const FhirField(false, 'Element'),
    'baseCitation': const FhirField(false, 'Reference'),
  },
  'CitationStatusDate1': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'activity': const FhirField(false, 'CodeableConcept'),
    'actual': const FhirField(false, 'Boolean'),
    'actualElement': const FhirField(false, 'Element'),
    'period': const FhirField(false, 'Period'),
  },
  'CitationTitle': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(true, 'CodeableConcept'),
    'language': const FhirField(false, 'CodeableConcept'),
    'text': const FhirField(false, 'Markdown'),
    'textElement': const FhirField(false, 'Element'),
  },
  'CitationAbstract': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'language': const FhirField(false, 'CodeableConcept'),
    'text': const FhirField(false, 'Markdown'),
    'textElement': const FhirField(false, 'Element'),
    'copyright': const FhirField(false, 'Markdown'),
    'copyrightElement': const FhirField(false, 'Element'),
  },
  'CitationPart': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'value': const FhirField(false, 'String'),
    'valueElement': const FhirField(false, 'Element'),
    'baseCitation': const FhirField(false, 'Reference'),
  },
  'CitationRelatesTo': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'Code'),
    'typeElement': const FhirField(false, 'Element'),
    'classifier': const FhirField(true, 'CodeableConcept'),
    'label': const FhirField(false, 'String'),
    'labelElement': const FhirField(false, 'Element'),
    'display': const FhirField(false, 'String'),
    'displayElement': const FhirField(false, 'Element'),
    'citation': const FhirField(false, 'Markdown'),
    'citationElement': const FhirField(false, 'Element'),
    'document': const FhirField(false, 'Attachment'),
    'resource': const FhirField(false, 'Canonical'),
    'resourceElement': const FhirField(false, 'Element'),
    'resourceReference': const FhirField(false, 'Reference'),
  },
  'CitationPublicationForm': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'publishedIn': const FhirField(false, 'CitationPublishedIn'),
    'citedMedium': const FhirField(false, 'CodeableConcept'),
    'volume': const FhirField(false, 'String'),
    'volumeElement': const FhirField(false, 'Element'),
    'issue': const FhirField(false, 'String'),
    'issueElement': const FhirField(false, 'Element'),
    'articleDate': const FhirField(false, 'FhirDateTime'),
    'articleDateElement': const FhirField(false, 'Element'),
    'publicationDateText': const FhirField(false, 'String'),
    'publicationDateTextElement': const FhirField(false, 'Element'),
    'publicationDateSeason': const FhirField(false, 'String'),
    'publicationDateSeasonElement': const FhirField(false, 'Element'),
    'lastRevisionDate': const FhirField(false, 'FhirDateTime'),
    'lastRevisionDateElement': const FhirField(false, 'Element'),
    'language': const FhirField(true, 'CodeableConcept'),
    'accessionNumber': const FhirField(false, 'String'),
    'accessionNumberElement': const FhirField(false, 'Element'),
    'pageString': const FhirField(false, 'String'),
    'pageStringElement': const FhirField(false, 'Element'),
    'firstPage': const FhirField(false, 'String'),
    'firstPageElement': const FhirField(false, 'Element'),
    'lastPage': const FhirField(false, 'String'),
    'lastPageElement': const FhirField(false, 'Element'),
    'pageCount': const FhirField(false, 'String'),
    'pageCountElement': const FhirField(false, 'Element'),
    'copyright': const FhirField(false, 'Markdown'),
    'copyrightElement': const FhirField(false, 'Element'),
  },
  'CitationPublishedIn': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'identifier': const FhirField(true, 'Identifier'),
    'title': const FhirField(false, 'String'),
    'titleElement': const FhirField(false, 'Element'),
    'publisher': const FhirField(false, 'Reference'),
    'publisherLocation': const FhirField(false, 'String'),
    'publisherLocationElement': const FhirField(false, 'Element'),
  },
  'CitationWebLocation': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'classifier': const FhirField(true, 'CodeableConcept'),
    'url': const FhirField(false, 'FhirUri'),
    'urlElement': const FhirField(false, 'Element'),
  },
  'CitationClassification1': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'classifier': const FhirField(true, 'CodeableConcept'),
    'artifactAssessment': const FhirField(true, 'Reference'),
  },
  'CitationContributorship': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'complete': const FhirField(false, 'Boolean'),
    'completeElement': const FhirField(false, 'Element'),
    'entry': const FhirField(true, 'CitationEntry'),
    'summary': const FhirField(true, 'CitationSummary1'),
  },
  'CitationEntry': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'contributor': const FhirField(false, 'Reference'),
    'forenameInitials': const FhirField(false, 'String'),
    'forenameInitialsElement': const FhirField(false, 'Element'),
    'affiliation': const FhirField(true, 'Reference'),
    'contributionType': const FhirField(true, 'CodeableConcept'),
    'role': const FhirField(false, 'CodeableConcept'),
    'contributionInstance':
        const FhirField(true, 'CitationContributionInstance'),
    'correspondingContact': const FhirField(false, 'Boolean'),
    'correspondingContactElement': const FhirField(false, 'Element'),
    'rankingOrder': const FhirField(false, 'PositiveInt'),
    'rankingOrderElement': const FhirField(false, 'Element'),
  },
  'CitationContributionInstance': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'time': const FhirField(false, 'FhirDateTime'),
    'timeElement': const FhirField(false, 'Element'),
  },
  'CitationSummary1': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'style': const FhirField(false, 'CodeableConcept'),
    'source': const FhirField(false, 'CodeableConcept'),
    'value': const FhirField(false, 'Markdown'),
    'valueElement': const FhirField(false, 'Element'),
  },
  'Claim': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'traceNumber': const FhirField(true, 'Identifier'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'type': const FhirField(false, 'CodeableConcept'),
    'subType': const FhirField(false, 'CodeableConcept'),
    'use': const FhirField(false, 'Code'),
    'useElement': const FhirField(false, 'Element'),
    'patient': const FhirField(false, 'Reference'),
    'billablePeriod': const FhirField(false, 'Period'),
    'created': const FhirField(false, 'FhirDateTime'),
    'createdElement': const FhirField(false, 'Element'),
    'enterer': const FhirField(false, 'Reference'),
    'insurer': const FhirField(false, 'Reference'),
    'provider': const FhirField(false, 'Reference'),
    'priority': const FhirField(false, 'CodeableConcept'),
    'fundsReserve': const FhirField(false, 'CodeableConcept'),
    'related': const FhirField(true, 'ClaimRelated'),
    'prescription': const FhirField(false, 'Reference'),
    'originalPrescription': const FhirField(false, 'Reference'),
    'payee': const FhirField(false, 'ClaimPayee'),
    'referral': const FhirField(false, 'Reference'),
    'encounter': const FhirField(true, 'Reference'),
    'facility': const FhirField(false, 'Reference'),
    'diagnosisRelatedGroup': const FhirField(false, 'CodeableConcept'),
    'event': const FhirField(true, 'ClaimEvent'),
    'careTeam': const FhirField(true, 'ClaimCareTeam'),
    'supportingInfo': const FhirField(true, 'ClaimSupportingInfo'),
    'diagnosis': const FhirField(true, 'ClaimDiagnosis'),
    'procedure': const FhirField(true, 'ClaimProcedure'),
    'insurance': const FhirField(true, 'ClaimInsurance'),
    'accident': const FhirField(false, 'ClaimAccident'),
    'patientPaid': const FhirField(false, 'Money'),
    'item': const FhirField(true, 'ClaimItem'),
    'total': const FhirField(false, 'Money'),
  },
  'ClaimRelated': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'claim': const FhirField(false, 'Reference'),
    'relationship': const FhirField(false, 'CodeableConcept'),
    'reference': const FhirField(false, 'Identifier'),
  },
  'ClaimPayee': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'party': const FhirField(false, 'Reference'),
  },
  'ClaimEvent': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'whenDateTime': const FhirField(false, 'FhirDateTime'),
    'whenDateTimeElement': const FhirField(false, 'Element'),
    'whenPeriod': const FhirField(false, 'Period'),
  },
  'ClaimCareTeam': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'sequence': const FhirField(false, 'PositiveInt'),
    'sequenceElement': const FhirField(false, 'Element'),
    'provider': const FhirField(false, 'Reference'),
    'responsible': const FhirField(false, 'Boolean'),
    'responsibleElement': const FhirField(false, 'Element'),
    'role': const FhirField(false, 'CodeableConcept'),
    'specialty': const FhirField(false, 'CodeableConcept'),
  },
  'ClaimSupportingInfo': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'sequence': const FhirField(false, 'PositiveInt'),
    'sequenceElement': const FhirField(false, 'Element'),
    'category': const FhirField(false, 'CodeableConcept'),
    'code': const FhirField(false, 'CodeableConcept'),
    'timingDate': const FhirField(false, 'Date'),
    'timingDateElement': const FhirField(false, 'Element'),
    'timingPeriod': const FhirField(false, 'Period'),
    'valueBoolean': const FhirField(false, 'Boolean'),
    'valueBooleanElement': const FhirField(false, 'Element'),
    'valueString': const FhirField(false, 'String'),
    'valueStringElement': const FhirField(false, 'Element'),
    'valueQuantity': const FhirField(false, 'Quantity'),
    'valueAttachment': const FhirField(false, 'Attachment'),
    'valueReference': const FhirField(false, 'Reference'),
    'valueIdentifier': const FhirField(false, 'Identifier'),
    'reason': const FhirField(false, 'CodeableConcept'),
  },
  'ClaimDiagnosis': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'sequence': const FhirField(false, 'PositiveInt'),
    'sequenceElement': const FhirField(false, 'Element'),
    'diagnosisCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'diagnosisReference': const FhirField(false, 'Reference'),
    'type': const FhirField(true, 'CodeableConcept'),
    'onAdmission': const FhirField(false, 'CodeableConcept'),
  },
  'ClaimProcedure': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'sequence': const FhirField(false, 'PositiveInt'),
    'sequenceElement': const FhirField(false, 'Element'),
    'type': const FhirField(true, 'CodeableConcept'),
    'date': const FhirField(false, 'FhirDateTime'),
    'dateElement': const FhirField(false, 'Element'),
    'procedureCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'procedureReference': const FhirField(false, 'Reference'),
    'udi': const FhirField(true, 'Reference'),
  },
  'ClaimInsurance': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'sequence': const FhirField(false, 'PositiveInt'),
    'sequenceElement': const FhirField(false, 'Element'),
    'focal': const FhirField(false, 'Boolean'),
    'focalElement': const FhirField(false, 'Element'),
    'identifier': const FhirField(false, 'Identifier'),
    'coverage': const FhirField(false, 'Reference'),
    'businessArrangement': const FhirField(false, 'String'),
    'businessArrangementElement': const FhirField(false, 'Element'),
    'preAuthRef': const FhirField(true, 'String'),
    'preAuthRefElement': const FhirField(true, 'Element'),
    'claimResponse': const FhirField(false, 'Reference'),
  },
  'ClaimAccident': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'date': const FhirField(false, 'Date'),
    'dateElement': const FhirField(false, 'Element'),
    'type': const FhirField(false, 'CodeableConcept'),
    'locationAddress': const FhirField(false, 'Address'),
    'locationReference': const FhirField(false, 'Reference'),
  },
  'ClaimItem': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'sequence': const FhirField(false, 'PositiveInt'),
    'sequenceElement': const FhirField(false, 'Element'),
    'traceNumber': const FhirField(true, 'Identifier'),
    'careTeamSequence': const FhirField(true, 'PositiveInt'),
    'careTeamSequenceElement': const FhirField(true, 'Element'),
    'diagnosisSequence': const FhirField(true, 'PositiveInt'),
    'diagnosisSequenceElement': const FhirField(true, 'Element'),
    'procedureSequence': const FhirField(true, 'PositiveInt'),
    'procedureSequenceElement': const FhirField(true, 'Element'),
    'informationSequence': const FhirField(true, 'PositiveInt'),
    'informationSequenceElement': const FhirField(true, 'Element'),
    'revenue': const FhirField(false, 'CodeableConcept'),
    'category': const FhirField(false, 'CodeableConcept'),
    'productOrService': const FhirField(false, 'CodeableConcept'),
    'productOrServiceEnd': const FhirField(false, 'CodeableConcept'),
    'request': const FhirField(true, 'Reference'),
    'modifier': const FhirField(true, 'CodeableConcept'),
    'programCode': const FhirField(true, 'CodeableConcept'),
    'servicedDate': const FhirField(false, 'Date'),
    'servicedDateElement': const FhirField(false, 'Element'),
    'servicedPeriod': const FhirField(false, 'Period'),
    'locationCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'locationAddress': const FhirField(false, 'Address'),
    'locationReference': const FhirField(false, 'Reference'),
    'patientPaid': const FhirField(false, 'Money'),
    'quantity': const FhirField(false, 'Quantity'),
    'unitPrice': const FhirField(false, 'Money'),
    'factor': const FhirField(false, 'Decimal'),
    'factorElement': const FhirField(false, 'Element'),
    'tax': const FhirField(false, 'Money'),
    'net': const FhirField(false, 'Money'),
    'udi': const FhirField(true, 'Reference'),
    'bodySite': const FhirField(true, 'ClaimBodySite'),
    'encounter': const FhirField(true, 'Reference'),
    'detail': const FhirField(true, 'ClaimDetail'),
  },
  'ClaimBodySite': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'site': const FhirField(true, 'CodeableReference'),
    'subSite': const FhirField(true, 'CodeableConcept'),
  },
  'ClaimDetail': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'sequence': const FhirField(false, 'PositiveInt'),
    'sequenceElement': const FhirField(false, 'Element'),
    'traceNumber': const FhirField(true, 'Identifier'),
    'revenue': const FhirField(false, 'CodeableConcept'),
    'category': const FhirField(false, 'CodeableConcept'),
    'productOrService': const FhirField(false, 'CodeableConcept'),
    'productOrServiceEnd': const FhirField(false, 'CodeableConcept'),
    'modifier': const FhirField(true, 'CodeableConcept'),
    'programCode': const FhirField(true, 'CodeableConcept'),
    'patientPaid': const FhirField(false, 'Money'),
    'quantity': const FhirField(false, 'Quantity'),
    'unitPrice': const FhirField(false, 'Money'),
    'factor': const FhirField(false, 'Decimal'),
    'factorElement': const FhirField(false, 'Element'),
    'tax': const FhirField(false, 'Money'),
    'net': const FhirField(false, 'Money'),
    'udi': const FhirField(true, 'Reference'),
    'subDetail': const FhirField(true, 'ClaimSubDetail'),
  },
  'ClaimSubDetail': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'sequence': const FhirField(false, 'PositiveInt'),
    'sequenceElement': const FhirField(false, 'Element'),
    'traceNumber': const FhirField(true, 'Identifier'),
    'revenue': const FhirField(false, 'CodeableConcept'),
    'category': const FhirField(false, 'CodeableConcept'),
    'productOrService': const FhirField(false, 'CodeableConcept'),
    'productOrServiceEnd': const FhirField(false, 'CodeableConcept'),
    'modifier': const FhirField(true, 'CodeableConcept'),
    'programCode': const FhirField(true, 'CodeableConcept'),
    'patientPaid': const FhirField(false, 'Money'),
    'quantity': const FhirField(false, 'Quantity'),
    'unitPrice': const FhirField(false, 'Money'),
    'factor': const FhirField(false, 'Decimal'),
    'factorElement': const FhirField(false, 'Element'),
    'tax': const FhirField(false, 'Money'),
    'net': const FhirField(false, 'Money'),
    'udi': const FhirField(true, 'Reference'),
  },
  'ClaimResponse': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'traceNumber': const FhirField(true, 'Identifier'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'type': const FhirField(false, 'CodeableConcept'),
    'subType': const FhirField(false, 'CodeableConcept'),
    'use': const FhirField(false, 'Code'),
    'useElement': const FhirField(false, 'Element'),
    'patient': const FhirField(false, 'Reference'),
    'created': const FhirField(false, 'FhirDateTime'),
    'createdElement': const FhirField(false, 'Element'),
    'insurer': const FhirField(false, 'Reference'),
    'requestor': const FhirField(false, 'Reference'),
    'request': const FhirField(false, 'Reference'),
    'outcome': const FhirField(false, 'Code'),
    'outcomeElement': const FhirField(false, 'Element'),
    'decision': const FhirField(false, 'CodeableConcept'),
    'disposition': const FhirField(false, 'String'),
    'dispositionElement': const FhirField(false, 'Element'),
    'preAuthRef': const FhirField(false, 'String'),
    'preAuthRefElement': const FhirField(false, 'Element'),
    'preAuthPeriod': const FhirField(false, 'Period'),
    'event': const FhirField(true, 'ClaimResponseEvent'),
    'payeeType': const FhirField(false, 'CodeableConcept'),
    'encounter': const FhirField(true, 'Reference'),
    'diagnosisRelatedGroup': const FhirField(false, 'CodeableConcept'),
    'item': const FhirField(true, 'ClaimResponseItem'),
    'addItem': const FhirField(true, 'ClaimResponseAddItem'),
    'adjudication': const FhirField(true, 'ClaimResponseAdjudication'),
    'total': const FhirField(true, 'ClaimResponseTotal'),
    'payment': const FhirField(false, 'ClaimResponsePayment'),
    'fundsReserve': const FhirField(false, 'CodeableConcept'),
    'formCode': const FhirField(false, 'CodeableConcept'),
    'form': const FhirField(false, 'Attachment'),
    'processNote': const FhirField(true, 'ClaimResponseProcessNote'),
    'communicationRequest': const FhirField(true, 'Reference'),
    'insurance': const FhirField(true, 'ClaimResponseInsurance'),
    'error': const FhirField(true, 'ClaimResponseError'),
  },
  'ClaimResponseEvent': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'whenDateTime': const FhirField(false, 'FhirDateTime'),
    'whenDateTimeElement': const FhirField(false, 'Element'),
    'whenPeriod': const FhirField(false, 'Period'),
  },
  'ClaimResponseItem': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'itemSequence': const FhirField(false, 'PositiveInt'),
    'itemSequenceElement': const FhirField(false, 'Element'),
    'traceNumber': const FhirField(true, 'Identifier'),
    'noteNumber': const FhirField(true, 'PositiveInt'),
    'noteNumberElement': const FhirField(true, 'Element'),
    'reviewOutcome': const FhirField(false, 'ClaimResponseReviewOutcome'),
    'adjudication': const FhirField(true, 'ClaimResponseAdjudication'),
    'detail': const FhirField(true, 'ClaimResponseDetail'),
  },
  'ClaimResponseReviewOutcome': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'decision': const FhirField(false, 'CodeableConcept'),
    'reason': const FhirField(true, 'CodeableConcept'),
    'preAuthRef': const FhirField(false, 'String'),
    'preAuthRefElement': const FhirField(false, 'Element'),
    'preAuthPeriod': const FhirField(false, 'Period'),
  },
  'ClaimResponseAdjudication': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'category': const FhirField(false, 'CodeableConcept'),
    'reason': const FhirField(false, 'CodeableConcept'),
    'amount': const FhirField(false, 'Money'),
    'quantity': const FhirField(false, 'Quantity'),
  },
  'ClaimResponseDetail': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'detailSequence': const FhirField(false, 'PositiveInt'),
    'detailSequenceElement': const FhirField(false, 'Element'),
    'traceNumber': const FhirField(true, 'Identifier'),
    'noteNumber': const FhirField(true, 'PositiveInt'),
    'noteNumberElement': const FhirField(true, 'Element'),
    'reviewOutcome': const FhirField(false, 'ClaimResponseReviewOutcome'),
    'adjudication': const FhirField(true, 'ClaimResponseAdjudication'),
    'subDetail': const FhirField(true, 'ClaimResponseSubDetail'),
  },
  'ClaimResponseSubDetail': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'subDetailSequence': const FhirField(false, 'PositiveInt'),
    'subDetailSequenceElement': const FhirField(false, 'Element'),
    'traceNumber': const FhirField(true, 'Identifier'),
    'noteNumber': const FhirField(true, 'PositiveInt'),
    'noteNumberElement': const FhirField(true, 'Element'),
    'reviewOutcome': const FhirField(false, 'ClaimResponseReviewOutcome'),
    'adjudication': const FhirField(true, 'ClaimResponseAdjudication'),
  },
  'ClaimResponseAddItem': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'itemSequence': const FhirField(true, 'PositiveInt'),
    'itemSequenceElement': const FhirField(true, 'Element'),
    'detailSequence': const FhirField(true, 'PositiveInt'),
    'detailSequenceElement': const FhirField(true, 'Element'),
    'subdetailSequence': const FhirField(true, 'PositiveInt'),
    'subdetailSequenceElement': const FhirField(true, 'Element'),
    'traceNumber': const FhirField(true, 'Identifier'),
    'provider': const FhirField(true, 'Reference'),
    'revenue': const FhirField(false, 'CodeableConcept'),
    'productOrService': const FhirField(false, 'CodeableConcept'),
    'productOrServiceEnd': const FhirField(false, 'CodeableConcept'),
    'request': const FhirField(true, 'Reference'),
    'modifier': const FhirField(true, 'CodeableConcept'),
    'programCode': const FhirField(true, 'CodeableConcept'),
    'servicedDate': const FhirField(false, 'Date'),
    'servicedDateElement': const FhirField(false, 'Element'),
    'servicedPeriod': const FhirField(false, 'Period'),
    'locationCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'locationAddress': const FhirField(false, 'Address'),
    'locationReference': const FhirField(false, 'Reference'),
    'quantity': const FhirField(false, 'Quantity'),
    'unitPrice': const FhirField(false, 'Money'),
    'factor': const FhirField(false, 'Decimal'),
    'factorElement': const FhirField(false, 'Element'),
    'tax': const FhirField(false, 'Money'),
    'net': const FhirField(false, 'Money'),
    'bodySite': const FhirField(true, 'ClaimResponseBodySite'),
    'noteNumber': const FhirField(true, 'PositiveInt'),
    'noteNumberElement': const FhirField(true, 'Element'),
    'reviewOutcome': const FhirField(false, 'ClaimResponseReviewOutcome'),
    'adjudication': const FhirField(true, 'ClaimResponseAdjudication'),
    'detail': const FhirField(true, 'ClaimResponseDetail1'),
  },
  'ClaimResponseBodySite': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'site': const FhirField(true, 'CodeableReference'),
    'subSite': const FhirField(true, 'CodeableConcept'),
  },
  'ClaimResponseDetail1': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'traceNumber': const FhirField(true, 'Identifier'),
    'revenue': const FhirField(false, 'CodeableConcept'),
    'productOrService': const FhirField(false, 'CodeableConcept'),
    'productOrServiceEnd': const FhirField(false, 'CodeableConcept'),
    'modifier': const FhirField(true, 'CodeableConcept'),
    'quantity': const FhirField(false, 'Quantity'),
    'unitPrice': const FhirField(false, 'Money'),
    'factor': const FhirField(false, 'Decimal'),
    'factorElement': const FhirField(false, 'Element'),
    'tax': const FhirField(false, 'Money'),
    'net': const FhirField(false, 'Money'),
    'noteNumber': const FhirField(true, 'PositiveInt'),
    'noteNumberElement': const FhirField(true, 'Element'),
    'reviewOutcome': const FhirField(false, 'ClaimResponseReviewOutcome'),
    'adjudication': const FhirField(true, 'ClaimResponseAdjudication'),
    'subDetail': const FhirField(true, 'ClaimResponseSubDetail1'),
  },
  'ClaimResponseSubDetail1': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'traceNumber': const FhirField(true, 'Identifier'),
    'revenue': const FhirField(false, 'CodeableConcept'),
    'productOrService': const FhirField(false, 'CodeableConcept'),
    'productOrServiceEnd': const FhirField(false, 'CodeableConcept'),
    'modifier': const FhirField(true, 'CodeableConcept'),
    'quantity': const FhirField(false, 'Quantity'),
    'unitPrice': const FhirField(false, 'Money'),
    'factor': const FhirField(false, 'Decimal'),
    'factorElement': const FhirField(false, 'Element'),
    'tax': const FhirField(false, 'Money'),
    'net': const FhirField(false, 'Money'),
    'noteNumber': const FhirField(true, 'PositiveInt'),
    'noteNumberElement': const FhirField(true, 'Element'),
    'reviewOutcome': const FhirField(false, 'ClaimResponseReviewOutcome'),
    'adjudication': const FhirField(true, 'ClaimResponseAdjudication'),
  },
  'ClaimResponseTotal': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'category': const FhirField(false, 'CodeableConcept'),
    'amount': const FhirField(false, 'Money'),
  },
  'ClaimResponsePayment': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'adjustment': const FhirField(false, 'Money'),
    'adjustmentReason': const FhirField(false, 'CodeableConcept'),
    'date': const FhirField(false, 'Date'),
    'dateElement': const FhirField(false, 'Element'),
    'amount': const FhirField(false, 'Money'),
    'identifier': const FhirField(false, 'Identifier'),
  },
  'ClaimResponseProcessNote': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'number': const FhirField(false, 'PositiveInt'),
    'numberElement': const FhirField(false, 'Element'),
    'type': const FhirField(false, 'CodeableConcept'),
    'text': const FhirField(false, 'String'),
    'textElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'CodeableConcept'),
  },
  'ClaimResponseInsurance': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'sequence': const FhirField(false, 'PositiveInt'),
    'sequenceElement': const FhirField(false, 'Element'),
    'focal': const FhirField(false, 'Boolean'),
    'focalElement': const FhirField(false, 'Element'),
    'coverage': const FhirField(false, 'Reference'),
    'businessArrangement': const FhirField(false, 'String'),
    'businessArrangementElement': const FhirField(false, 'Element'),
    'claimResponse': const FhirField(false, 'Reference'),
  },
  'ClaimResponseError': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'itemSequence': const FhirField(false, 'PositiveInt'),
    'itemSequenceElement': const FhirField(false, 'Element'),
    'detailSequence': const FhirField(false, 'PositiveInt'),
    'detailSequenceElement': const FhirField(false, 'Element'),
    'subDetailSequence': const FhirField(false, 'PositiveInt'),
    'subDetailSequenceElement': const FhirField(false, 'Element'),
    'code': const FhirField(false, 'CodeableConcept'),
    'expression': const FhirField(true, 'String'),
    'expressionElement': const FhirField(true, 'Element'),
  },
  'ClinicalImpression': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'statusReason': const FhirField(false, 'CodeableConcept'),
    'description': const FhirField(false, 'String'),
    'descriptionElement': const FhirField(false, 'Element'),
    'subject': const FhirField(false, 'Reference'),
    'encounter': const FhirField(false, 'Reference'),
    'effectiveDateTime': const FhirField(false, 'FhirDateTime'),
    'effectiveDateTimeElement': const FhirField(false, 'Element'),
    'effectivePeriod': const FhirField(false, 'Period'),
    'date': const FhirField(false, 'FhirDateTime'),
    'dateElement': const FhirField(false, 'Element'),
    'performer': const FhirField(false, 'Reference'),
    'previous': const FhirField(false, 'Reference'),
    'problem': const FhirField(true, 'Reference'),
    'changePattern': const FhirField(false, 'CodeableConcept'),
    'protocol': const FhirField(true, 'FhirUri'),
    'protocolElement': const FhirField(true, 'Element'),
    'summary': const FhirField(false, 'String'),
    'summaryElement': const FhirField(false, 'Element'),
    'finding': const FhirField(true, 'ClinicalImpressionFinding'),
    'prognosisCodeableConcept': const FhirField(true, 'CodeableConcept'),
    'prognosisReference': const FhirField(true, 'Reference'),
    'supportingInfo': const FhirField(true, 'Reference'),
    'note': const FhirField(true, 'Annotation'),
  },
  'ClinicalImpressionFinding': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'item': const FhirField(false, 'CodeableReference'),
    'basis': const FhirField(false, 'String'),
    'basisElement': const FhirField(false, 'Element'),
  },
  'ClinicalUseDefinition': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'type': const FhirField(false, 'Code'),
    'typeElement': const FhirField(false, 'Element'),
    'category': const FhirField(true, 'CodeableConcept'),
    'subject': const FhirField(true, 'Reference'),
    'status': const FhirField(false, 'CodeableConcept'),
    'contraindication':
        const FhirField(false, 'ClinicalUseDefinitionContraindication'),
    'indication': const FhirField(false, 'ClinicalUseDefinitionIndication'),
    'interaction': const FhirField(false, 'ClinicalUseDefinitionInteraction'),
    'population': const FhirField(true, 'Reference'),
    'library': const FhirField(true, 'Canonical'),
    'undesirableEffect':
        const FhirField(false, 'ClinicalUseDefinitionUndesirableEffect'),
    'warning': const FhirField(false, 'ClinicalUseDefinitionWarning'),
  },
  'ClinicalUseDefinitionContraindication': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'diseaseSymptomProcedure': const FhirField(false, 'CodeableReference'),
    'diseaseStatus': const FhirField(false, 'CodeableReference'),
    'comorbidity': const FhirField(true, 'CodeableReference'),
    'indication': const FhirField(true, 'Reference'),
    'applicability': const FhirField(false, 'Expression'),
    'otherTherapy': const FhirField(true, 'ClinicalUseDefinitionOtherTherapy'),
  },
  'ClinicalUseDefinitionOtherTherapy': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'relationshipType': const FhirField(false, 'CodeableConcept'),
    'treatment': const FhirField(false, 'CodeableReference'),
  },
  'ClinicalUseDefinitionIndication': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'diseaseSymptomProcedure': const FhirField(false, 'CodeableReference'),
    'diseaseStatus': const FhirField(false, 'CodeableReference'),
    'comorbidity': const FhirField(true, 'CodeableReference'),
    'intendedEffect': const FhirField(false, 'CodeableReference'),
    'durationRange': const FhirField(false, 'Range'),
    'durationString': const FhirField(false, 'String'),
    'durationStringElement': const FhirField(false, 'Element'),
    'undesirableEffect': const FhirField(true, 'Reference'),
    'applicability': const FhirField(false, 'Expression'),
    'otherTherapy': const FhirField(true, 'ClinicalUseDefinitionOtherTherapy'),
  },
  'ClinicalUseDefinitionInteraction': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'interactant': const FhirField(true, 'ClinicalUseDefinitionInteractant'),
    'type': const FhirField(false, 'CodeableConcept'),
    'effect': const FhirField(false, 'CodeableReference'),
    'incidence': const FhirField(false, 'CodeableConcept'),
    'management': const FhirField(true, 'CodeableConcept'),
  },
  'ClinicalUseDefinitionInteractant': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'itemReference': const FhirField(false, 'Reference'),
    'itemCodeableConcept': const FhirField(false, 'CodeableConcept'),
  },
  'ClinicalUseDefinitionUndesirableEffect': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'symptomConditionEffect': const FhirField(false, 'CodeableReference'),
    'classification': const FhirField(false, 'CodeableConcept'),
    'frequencyOfOccurrence': const FhirField(false, 'CodeableConcept'),
  },
  'ClinicalUseDefinitionWarning': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'description': const FhirField(false, 'Markdown'),
    'descriptionElement': const FhirField(false, 'Element'),
    'code': const FhirField(false, 'CodeableConcept'),
  },
  'CodeSystem': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'url': const FhirField(false, 'FhirUri'),
    'urlElement': const FhirField(false, 'Element'),
    'identifier': const FhirField(true, 'Identifier'),
    'version': const FhirField(false, 'String'),
    'versionElement': const FhirField(false, 'Element'),
    'versionAlgorithmString': const FhirField(false, 'String'),
    'versionAlgorithmStringElement': const FhirField(false, 'Element'),
    'versionAlgorithmCoding': const FhirField(false, 'Coding'),
    'name': const FhirField(false, 'String'),
    'nameElement': const FhirField(false, 'Element'),
    'title': const FhirField(false, 'String'),
    'titleElement': const FhirField(false, 'Element'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'experimental': const FhirField(false, 'Boolean'),
    'experimentalElement': const FhirField(false, 'Element'),
    'date': const FhirField(false, 'FhirDateTime'),
    'dateElement': const FhirField(false, 'Element'),
    'publisher': const FhirField(false, 'String'),
    'publisherElement': const FhirField(false, 'Element'),
    'contact': const FhirField(true, 'ContactDetail'),
    'description': const FhirField(false, 'Markdown'),
    'descriptionElement': const FhirField(false, 'Element'),
    'useContext': const FhirField(true, 'UsageContext'),
    'jurisdiction': const FhirField(true, 'CodeableConcept'),
    'purpose': const FhirField(false, 'Markdown'),
    'purposeElement': const FhirField(false, 'Element'),
    'copyright': const FhirField(false, 'Markdown'),
    'copyrightElement': const FhirField(false, 'Element'),
    'copyrightLabel': const FhirField(false, 'String'),
    'copyrightLabelElement': const FhirField(false, 'Element'),
    'approvalDate': const FhirField(false, 'Date'),
    'approvalDateElement': const FhirField(false, 'Element'),
    'lastReviewDate': const FhirField(false, 'Date'),
    'lastReviewDateElement': const FhirField(false, 'Element'),
    'effectivePeriod': const FhirField(false, 'Period'),
    'topic': const FhirField(true, 'CodeableConcept'),
    'author': const FhirField(true, 'ContactDetail'),
    'editor': const FhirField(true, 'ContactDetail'),
    'reviewer': const FhirField(true, 'ContactDetail'),
    'endorser': const FhirField(true, 'ContactDetail'),
    'relatedArtifact': const FhirField(true, 'RelatedArtifact'),
    'caseSensitive': const FhirField(false, 'Boolean'),
    'caseSensitiveElement': const FhirField(false, 'Element'),
    'valueSet': const FhirField(false, 'Canonical'),
    'hierarchyMeaning': const FhirField(false, 'Code'),
    'hierarchyMeaningElement': const FhirField(false, 'Element'),
    'compositional': const FhirField(false, 'Boolean'),
    'compositionalElement': const FhirField(false, 'Element'),
    'versionNeeded': const FhirField(false, 'Boolean'),
    'versionNeededElement': const FhirField(false, 'Element'),
    'content': const FhirField(false, 'Code'),
    'contentElement': const FhirField(false, 'Element'),
    'supplements': const FhirField(false, 'Canonical'),
    'count': const FhirField(false, 'UnsignedInt'),
    'countElement': const FhirField(false, 'Element'),
    'filter': const FhirField(true, 'CodeSystemFilter'),
    'property': const FhirField(true, 'CodeSystemProperty'),
    'concept': const FhirField(true, 'CodeSystemConcept'),
  },
  'CodeSystemFilter': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(false, 'Code'),
    'codeElement': const FhirField(false, 'Element'),
    'description': const FhirField(false, 'String'),
    'descriptionElement': const FhirField(false, 'Element'),
    'operator': const FhirField(true, 'Code'),
    'operatorElement': const FhirField(true, 'Element'),
    'value': const FhirField(false, 'String'),
    'valueElement': const FhirField(false, 'Element'),
  },
  'CodeSystemProperty': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(false, 'Code'),
    'codeElement': const FhirField(false, 'Element'),
    'uri': const FhirField(false, 'FhirUri'),
    'uriElement': const FhirField(false, 'Element'),
    'description': const FhirField(false, 'String'),
    'descriptionElement': const FhirField(false, 'Element'),
    'type': const FhirField(false, 'Code'),
    'typeElement': const FhirField(false, 'Element'),
  },
  'CodeSystemConcept': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(false, 'Code'),
    'codeElement': const FhirField(false, 'Element'),
    'display': const FhirField(false, 'String'),
    'displayElement': const FhirField(false, 'Element'),
    'definition': const FhirField(false, 'String'),
    'definitionElement': const FhirField(false, 'Element'),
    'designation': const FhirField(true, 'CodeSystemDesignation'),
    'property': const FhirField(true, 'CodeSystemProperty1'),
    'concept': const FhirField(true, 'CodeSystemConcept'),
  },
  'CodeSystemDesignation': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'use': const FhirField(false, 'Coding'),
    'additionalUse': const FhirField(true, 'Coding'),
    'value': const FhirField(false, 'String'),
    'valueElement': const FhirField(false, 'Element'),
  },
  'CodeSystemProperty1': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(false, 'Code'),
    'codeElement': const FhirField(false, 'Element'),
    'valueCode': const FhirField(false, 'Code'),
    'valueCodeElement': const FhirField(false, 'Element'),
    'valueCoding': const FhirField(false, 'Coding'),
    'valueString': const FhirField(false, 'String'),
    'valueStringElement': const FhirField(false, 'Element'),
    'valueInteger': const FhirField(false, 'Integer'),
    'valueIntegerElement': const FhirField(false, 'Element'),
    'valueBoolean': const FhirField(false, 'Boolean'),
    'valueBooleanElement': const FhirField(false, 'Element'),
    'valueDateTime': const FhirField(false, 'FhirDateTime'),
    'valueDateTimeElement': const FhirField(false, 'Element'),
    'valueDecimal': const FhirField(false, 'Decimal'),
    'valueDecimalElement': const FhirField(false, 'Element'),
  },
  'Communication': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'instantiatesCanonical': const FhirField(true, 'Canonical'),
    'instantiatesUri': const FhirField(true, 'FhirUri'),
    'instantiatesUriElement': const FhirField(true, 'Element'),
    'basedOn': const FhirField(true, 'Reference'),
    'partOf': const FhirField(true, 'Reference'),
    'inResponseTo': const FhirField(true, 'Reference'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'statusReason': const FhirField(false, 'CodeableConcept'),
    'category': const FhirField(true, 'CodeableConcept'),
    'priority': const FhirField(false, 'Code'),
    'priorityElement': const FhirField(false, 'Element'),
    'medium': const FhirField(true, 'CodeableConcept'),
    'subject': const FhirField(false, 'Reference'),
    'topic': const FhirField(false, 'CodeableConcept'),
    'about': const FhirField(true, 'Reference'),
    'encounter': const FhirField(false, 'Reference'),
    'sent': const FhirField(false, 'FhirDateTime'),
    'sentElement': const FhirField(false, 'Element'),
    'received': const FhirField(false, 'FhirDateTime'),
    'receivedElement': const FhirField(false, 'Element'),
    'recipient': const FhirField(true, 'Reference'),
    'sender': const FhirField(false, 'Reference'),
    'reason': const FhirField(true, 'CodeableReference'),
    'payload': const FhirField(true, 'CommunicationPayload'),
    'note': const FhirField(true, 'Annotation'),
  },
  'CommunicationPayload': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'contentAttachment': const FhirField(false, 'Attachment'),
    'contentReference': const FhirField(false, 'Reference'),
    'contentCodeableConcept': const FhirField(false, 'CodeableConcept'),
  },
  'CommunicationRequest': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'basedOn': const FhirField(true, 'Reference'),
    'replaces': const FhirField(true, 'Reference'),
    'groupIdentifier': const FhirField(false, 'Identifier'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'statusReason': const FhirField(false, 'CodeableConcept'),
    'intent': const FhirField(false, 'Code'),
    'intentElement': const FhirField(false, 'Element'),
    'category': const FhirField(true, 'CodeableConcept'),
    'priority': const FhirField(false, 'Code'),
    'priorityElement': const FhirField(false, 'Element'),
    'doNotPerform': const FhirField(false, 'Boolean'),
    'doNotPerformElement': const FhirField(false, 'Element'),
    'medium': const FhirField(true, 'CodeableConcept'),
    'subject': const FhirField(false, 'Reference'),
    'about': const FhirField(true, 'Reference'),
    'encounter': const FhirField(false, 'Reference'),
    'payload': const FhirField(true, 'CommunicationRequestPayload'),
    'occurrenceDateTime': const FhirField(false, 'FhirDateTime'),
    'occurrenceDateTimeElement': const FhirField(false, 'Element'),
    'occurrencePeriod': const FhirField(false, 'Period'),
    'authoredOn': const FhirField(false, 'FhirDateTime'),
    'authoredOnElement': const FhirField(false, 'Element'),
    'requester': const FhirField(false, 'Reference'),
    'recipient': const FhirField(true, 'Reference'),
    'informationProvider': const FhirField(true, 'Reference'),
    'reason': const FhirField(true, 'CodeableReference'),
    'note': const FhirField(true, 'Annotation'),
  },
  'CommunicationRequestPayload': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'contentAttachment': const FhirField(false, 'Attachment'),
    'contentReference': const FhirField(false, 'Reference'),
    'contentCodeableConcept': const FhirField(false, 'CodeableConcept'),
  },
  'CompartmentDefinition': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'url': const FhirField(false, 'FhirUri'),
    'urlElement': const FhirField(false, 'Element'),
    'identifier': const FhirField(true, 'Identifier'),
    'version': const FhirField(false, 'String'),
    'versionElement': const FhirField(false, 'Element'),
    'versionAlgorithmString': const FhirField(false, 'String'),
    'versionAlgorithmStringElement': const FhirField(false, 'Element'),
    'versionAlgorithmCoding': const FhirField(false, 'Coding'),
    'name': const FhirField(false, 'String'),
    'nameElement': const FhirField(false, 'Element'),
    'title': const FhirField(false, 'String'),
    'titleElement': const FhirField(false, 'Element'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'experimental': const FhirField(false, 'Boolean'),
    'experimentalElement': const FhirField(false, 'Element'),
    'date': const FhirField(false, 'FhirDateTime'),
    'dateElement': const FhirField(false, 'Element'),
    'publisher': const FhirField(false, 'String'),
    'publisherElement': const FhirField(false, 'Element'),
    'contact': const FhirField(true, 'ContactDetail'),
    'description': const FhirField(false, 'Markdown'),
    'descriptionElement': const FhirField(false, 'Element'),
    'useContext': const FhirField(true, 'UsageContext'),
    'jurisdiction': const FhirField(true, 'CodeableConcept'),
    'purpose': const FhirField(false, 'Markdown'),
    'purposeElement': const FhirField(false, 'Element'),
    'copyright': const FhirField(false, 'Markdown'),
    'copyrightElement': const FhirField(false, 'Element'),
    'copyrightLabel': const FhirField(false, 'String'),
    'copyrightLabelElement': const FhirField(false, 'Element'),
    'code': const FhirField(false, 'Code'),
    'codeElement': const FhirField(false, 'Element'),
    'search': const FhirField(false, 'Boolean'),
    'searchElement': const FhirField(false, 'Element'),
    'resource': const FhirField(true, 'CompartmentDefinitionResource'),
  },
  'CompartmentDefinitionResource': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(false, 'Code'),
    'codeElement': const FhirField(false, 'Element'),
    'param': const FhirField(true, 'String'),
    'paramElement': const FhirField(true, 'Element'),
    'documentation': const FhirField(false, 'String'),
    'documentationElement': const FhirField(false, 'Element'),
    'startParam': const FhirField(false, 'FhirUri'),
    'startParamElement': const FhirField(false, 'Element'),
    'endParam': const FhirField(false, 'FhirUri'),
    'endParamElement': const FhirField(false, 'Element'),
  },
  'Composition': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'url': const FhirField(false, 'FhirUri'),
    'urlElement': const FhirField(false, 'Element'),
    'identifier': const FhirField(true, 'Identifier'),
    'version': const FhirField(false, 'String'),
    'versionElement': const FhirField(false, 'Element'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'type': const FhirField(false, 'CodeableConcept'),
    'category': const FhirField(true, 'CodeableConcept'),
    'subject': const FhirField(true, 'Reference'),
    'encounter': const FhirField(false, 'Reference'),
    'date': const FhirField(false, 'FhirDateTime'),
    'dateElement': const FhirField(false, 'Element'),
    'useContext': const FhirField(true, 'UsageContext'),
    'author': const FhirField(true, 'Reference'),
    'name': const FhirField(false, 'String'),
    'nameElement': const FhirField(false, 'Element'),
    'title': const FhirField(false, 'String'),
    'titleElement': const FhirField(false, 'Element'),
    'note': const FhirField(true, 'Annotation'),
    'attester': const FhirField(true, 'CompositionAttester'),
    'custodian': const FhirField(false, 'Reference'),
    'relatesTo': const FhirField(true, 'RelatedArtifact'),
    'event': const FhirField(true, 'CompositionEvent'),
    'section': const FhirField(true, 'CompositionSection'),
  },
  'CompositionAttester': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'mode': const FhirField(false, 'CodeableConcept'),
    'time': const FhirField(false, 'FhirDateTime'),
    'timeElement': const FhirField(false, 'Element'),
    'party': const FhirField(false, 'Reference'),
  },
  'CompositionEvent': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'period': const FhirField(false, 'Period'),
    'detail': const FhirField(true, 'CodeableReference'),
  },
  'CompositionSection': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'title': const FhirField(false, 'String'),
    'titleElement': const FhirField(false, 'Element'),
    'code': const FhirField(false, 'CodeableConcept'),
    'author': const FhirField(true, 'Reference'),
    'focus': const FhirField(false, 'Reference'),
    'text': const FhirField(false, 'Narrative'),
    'orderedBy': const FhirField(false, 'CodeableConcept'),
    'entry': const FhirField(true, 'Reference'),
    'emptyReason': const FhirField(false, 'CodeableConcept'),
    'section': const FhirField(true, 'CompositionSection'),
  },
  'ConceptMap': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'url': const FhirField(false, 'FhirUri'),
    'urlElement': const FhirField(false, 'Element'),
    'identifier': const FhirField(true, 'Identifier'),
    'version': const FhirField(false, 'String'),
    'versionElement': const FhirField(false, 'Element'),
    'versionAlgorithmString': const FhirField(false, 'String'),
    'versionAlgorithmStringElement': const FhirField(false, 'Element'),
    'versionAlgorithmCoding': const FhirField(false, 'Coding'),
    'name': const FhirField(false, 'String'),
    'nameElement': const FhirField(false, 'Element'),
    'title': const FhirField(false, 'String'),
    'titleElement': const FhirField(false, 'Element'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'experimental': const FhirField(false, 'Boolean'),
    'experimentalElement': const FhirField(false, 'Element'),
    'date': const FhirField(false, 'FhirDateTime'),
    'dateElement': const FhirField(false, 'Element'),
    'publisher': const FhirField(false, 'String'),
    'publisherElement': const FhirField(false, 'Element'),
    'contact': const FhirField(true, 'ContactDetail'),
    'description': const FhirField(false, 'Markdown'),
    'descriptionElement': const FhirField(false, 'Element'),
    'useContext': const FhirField(true, 'UsageContext'),
    'jurisdiction': const FhirField(true, 'CodeableConcept'),
    'purpose': const FhirField(false, 'Markdown'),
    'purposeElement': const FhirField(false, 'Element'),
    'copyright': const FhirField(false, 'Markdown'),
    'copyrightElement': const FhirField(false, 'Element'),
    'copyrightLabel': const FhirField(false, 'String'),
    'copyrightLabelElement': const FhirField(false, 'Element'),
    'approvalDate': const FhirField(false, 'Date'),
    'approvalDateElement': const FhirField(false, 'Element'),
    'lastReviewDate': const FhirField(false, 'Date'),
    'lastReviewDateElement': const FhirField(false, 'Element'),
    'effectivePeriod': const FhirField(false, 'Period'),
    'topic': const FhirField(true, 'CodeableConcept'),
    'author': const FhirField(true, 'ContactDetail'),
    'editor': const FhirField(true, 'ContactDetail'),
    'reviewer': const FhirField(true, 'ContactDetail'),
    'endorser': const FhirField(true, 'ContactDetail'),
    'relatedArtifact': const FhirField(true, 'RelatedArtifact'),
    'property': const FhirField(true, 'ConceptMapProperty'),
    'additionalAttribute':
        const FhirField(true, 'ConceptMapAdditionalAttribute'),
    'sourceScopeUri': const FhirField(false, 'FhirUri'),
    'sourceScopeUriElement': const FhirField(false, 'Element'),
    'sourceScopeCanonical': const FhirField(false, 'Canonical'),
    'sourceScopeCanonicalElement': const FhirField(false, 'Element'),
    'targetScopeUri': const FhirField(false, 'FhirUri'),
    'targetScopeUriElement': const FhirField(false, 'Element'),
    'targetScopeCanonical': const FhirField(false, 'Canonical'),
    'targetScopeCanonicalElement': const FhirField(false, 'Element'),
    'group': const FhirField(true, 'ConceptMapGroup'),
  },
  'ConceptMapProperty': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(false, 'Code'),
    'codeElement': const FhirField(false, 'Element'),
    'uri': const FhirField(false, 'FhirUri'),
    'uriElement': const FhirField(false, 'Element'),
    'description': const FhirField(false, 'String'),
    'descriptionElement': const FhirField(false, 'Element'),
    'type': const FhirField(false, 'Code'),
    'typeElement': const FhirField(false, 'Element'),
    'system': const FhirField(false, 'Canonical'),
  },
  'ConceptMapAdditionalAttribute': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(false, 'Code'),
    'codeElement': const FhirField(false, 'Element'),
    'uri': const FhirField(false, 'FhirUri'),
    'uriElement': const FhirField(false, 'Element'),
    'description': const FhirField(false, 'String'),
    'descriptionElement': const FhirField(false, 'Element'),
    'type': const FhirField(false, 'Code'),
    'typeElement': const FhirField(false, 'Element'),
  },
  'ConceptMapGroup': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'source': const FhirField(false, 'Canonical'),
    'target': const FhirField(false, 'Canonical'),
    'element': const FhirField(true, 'ConceptMapElement'),
    'unmapped': const FhirField(false, 'ConceptMapUnmapped'),
  },
  'ConceptMapElement': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(false, 'Code'),
    'codeElement': const FhirField(false, 'Element'),
    'display': const FhirField(false, 'String'),
    'displayElement': const FhirField(false, 'Element'),
    'valueSet': const FhirField(false, 'Canonical'),
    'noMap': const FhirField(false, 'Boolean'),
    'noMapElement': const FhirField(false, 'Element'),
    'target': const FhirField(true, 'ConceptMapTarget'),
  },
  'ConceptMapTarget': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(false, 'Code'),
    'codeElement': const FhirField(false, 'Element'),
    'display': const FhirField(false, 'String'),
    'displayElement': const FhirField(false, 'Element'),
    'valueSet': const FhirField(false, 'Canonical'),
    'relationship': const FhirField(false, 'Code'),
    'relationshipElement': const FhirField(false, 'Element'),
    'comment': const FhirField(false, 'String'),
    'commentElement': const FhirField(false, 'Element'),
    'property': const FhirField(true, 'ConceptMapProperty1'),
    'dependsOn': const FhirField(true, 'ConceptMapDependsOn'),
    'product': const FhirField(true, 'ConceptMapDependsOn'),
  },
  'ConceptMapProperty1': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(false, 'Code'),
    'codeElement': const FhirField(false, 'Element'),
    'valueCoding': const FhirField(false, 'Coding'),
    'valueString': const FhirField(false, 'String'),
    'valueStringElement': const FhirField(false, 'Element'),
    'valueInteger': const FhirField(false, 'Integer'),
    'valueIntegerElement': const FhirField(false, 'Element'),
    'valueBoolean': const FhirField(false, 'Boolean'),
    'valueBooleanElement': const FhirField(false, 'Element'),
    'valueDateTime': const FhirField(false, 'FhirDateTime'),
    'valueDateTimeElement': const FhirField(false, 'Element'),
    'valueDecimal': const FhirField(false, 'Decimal'),
    'valueDecimalElement': const FhirField(false, 'Element'),
    'valueCode': const FhirField(false, 'Code'),
    'valueCodeElement': const FhirField(false, 'Element'),
  },
  'ConceptMapDependsOn': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'attribute': const FhirField(false, 'Code'),
    'attributeElement': const FhirField(false, 'Element'),
    'valueCode': const FhirField(false, 'Code'),
    'valueCodeElement': const FhirField(false, 'Element'),
    'valueCoding': const FhirField(false, 'Coding'),
    'valueString': const FhirField(false, 'String'),
    'valueStringElement': const FhirField(false, 'Element'),
    'valueBoolean': const FhirField(false, 'Boolean'),
    'valueBooleanElement': const FhirField(false, 'Element'),
    'valueQuantity': const FhirField(false, 'Quantity'),
    'valueSet': const FhirField(false, 'Canonical'),
  },
  'ConceptMapUnmapped': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'mode': const FhirField(false, 'Code'),
    'modeElement': const FhirField(false, 'Element'),
    'code': const FhirField(false, 'Code'),
    'codeElement': const FhirField(false, 'Element'),
    'display': const FhirField(false, 'String'),
    'displayElement': const FhirField(false, 'Element'),
    'valueSet': const FhirField(false, 'Canonical'),
    'relationship': const FhirField(false, 'Code'),
    'relationshipElement': const FhirField(false, 'Element'),
    'otherMap': const FhirField(false, 'Canonical'),
  },
  'Condition': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'clinicalStatus': const FhirField(false, 'CodeableConcept'),
    'verificationStatus': const FhirField(false, 'CodeableConcept'),
    'category': const FhirField(true, 'CodeableConcept'),
    'severity': const FhirField(false, 'CodeableConcept'),
    'code': const FhirField(false, 'CodeableConcept'),
    'bodySite': const FhirField(true, 'CodeableConcept'),
    'subject': const FhirField(false, 'Reference'),
    'encounter': const FhirField(false, 'Reference'),
    'onsetDateTime': const FhirField(false, 'FhirDateTime'),
    'onsetDateTimeElement': const FhirField(false, 'Element'),
    'onsetAge': const FhirField(false, 'Age'),
    'onsetPeriod': const FhirField(false, 'Period'),
    'onsetRange': const FhirField(false, 'Range'),
    'onsetString': const FhirField(false, 'String'),
    'onsetStringElement': const FhirField(false, 'Element'),
    'abatementDateTime': const FhirField(false, 'FhirDateTime'),
    'abatementDateTimeElement': const FhirField(false, 'Element'),
    'abatementAge': const FhirField(false, 'Age'),
    'abatementPeriod': const FhirField(false, 'Period'),
    'abatementRange': const FhirField(false, 'Range'),
    'abatementString': const FhirField(false, 'String'),
    'abatementStringElement': const FhirField(false, 'Element'),
    'recordedDate': const FhirField(false, 'FhirDateTime'),
    'recordedDateElement': const FhirField(false, 'Element'),
    'participant': const FhirField(true, 'ConditionParticipant'),
    'stage': const FhirField(true, 'ConditionStage'),
    'evidence': const FhirField(true, 'CodeableReference'),
    'note': const FhirField(true, 'Annotation'),
  },
  'ConditionParticipant': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'function': const FhirField(false, 'CodeableConcept'),
    'actor': const FhirField(false, 'Reference'),
  },
  'ConditionStage': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'summary': const FhirField(false, 'CodeableConcept'),
    'assessment': const FhirField(true, 'Reference'),
    'type': const FhirField(false, 'CodeableConcept'),
  },
  'ConditionDefinition': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'url': const FhirField(false, 'FhirUri'),
    'urlElement': const FhirField(false, 'Element'),
    'identifier': const FhirField(true, 'Identifier'),
    'version': const FhirField(false, 'String'),
    'versionElement': const FhirField(false, 'Element'),
    'versionAlgorithmString': const FhirField(false, 'String'),
    'versionAlgorithmStringElement': const FhirField(false, 'Element'),
    'versionAlgorithmCoding': const FhirField(false, 'Coding'),
    'name': const FhirField(false, 'String'),
    'nameElement': const FhirField(false, 'Element'),
    'title': const FhirField(false, 'String'),
    'titleElement': const FhirField(false, 'Element'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'experimental': const FhirField(false, 'Boolean'),
    'experimentalElement': const FhirField(false, 'Element'),
    'date': const FhirField(false, 'FhirDateTime'),
    'dateElement': const FhirField(false, 'Element'),
    'publisher': const FhirField(false, 'String'),
    'publisherElement': const FhirField(false, 'Element'),
    'contact': const FhirField(true, 'ContactDetail'),
    'description': const FhirField(false, 'Markdown'),
    'descriptionElement': const FhirField(false, 'Element'),
    'useContext': const FhirField(true, 'UsageContext'),
    'jurisdiction': const FhirField(true, 'CodeableConcept'),
    'purpose': const FhirField(false, 'Markdown'),
    'purposeElement': const FhirField(false, 'Element'),
    'copyright': const FhirField(false, 'Markdown'),
    'copyrightElement': const FhirField(false, 'Element'),
    'copyrightLabel': const FhirField(false, 'String'),
    'copyrightLabelElement': const FhirField(false, 'Element'),
    'approvalDate': const FhirField(false, 'Date'),
    'approvalDateElement': const FhirField(false, 'Element'),
    'lastReviewDate': const FhirField(false, 'Date'),
    'lastReviewDateElement': const FhirField(false, 'Element'),
    'effectivePeriod': const FhirField(false, 'Period'),
    'topic': const FhirField(true, 'CodeableConcept'),
    'author': const FhirField(true, 'ContactDetail'),
    'editor': const FhirField(true, 'ContactDetail'),
    'reviewer': const FhirField(true, 'ContactDetail'),
    'endorser': const FhirField(true, 'ContactDetail'),
    'relatedArtifact': const FhirField(true, 'RelatedArtifact'),
    'subtitle': const FhirField(false, 'String'),
    'subtitleElement': const FhirField(false, 'Element'),
    'code': const FhirField(false, 'CodeableConcept'),
    'severity': const FhirField(false, 'CodeableConcept'),
    'bodySite': const FhirField(false, 'CodeableConcept'),
    'stage': const FhirField(false, 'CodeableConcept'),
    'hasSeverity': const FhirField(false, 'Boolean'),
    'hasSeverityElement': const FhirField(false, 'Element'),
    'hasBodySite': const FhirField(false, 'Boolean'),
    'hasBodySiteElement': const FhirField(false, 'Element'),
    'hasStage': const FhirField(false, 'Boolean'),
    'hasStageElement': const FhirField(false, 'Element'),
    'definition': const FhirField(true, 'FhirUri'),
    'definitionElement': const FhirField(true, 'Element'),
    'observation': const FhirField(true, 'ConditionDefinitionObservation'),
    'medication': const FhirField(true, 'ConditionDefinitionMedication'),
    'precondition': const FhirField(true, 'ConditionDefinitionPrecondition'),
    'team': const FhirField(true, 'Reference'),
    'questionnaire': const FhirField(true, 'ConditionDefinitionQuestionnaire'),
    'plan': const FhirField(true, 'ConditionDefinitionPlan'),
  },
  'ConditionDefinitionObservation': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'category': const FhirField(false, 'CodeableConcept'),
    'code': const FhirField(false, 'CodeableConcept'),
  },
  'ConditionDefinitionMedication': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'category': const FhirField(false, 'CodeableConcept'),
    'code': const FhirField(false, 'CodeableConcept'),
  },
  'ConditionDefinitionPrecondition': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'Code'),
    'typeElement': const FhirField(false, 'Element'),
    'code': const FhirField(false, 'CodeableConcept'),
    'valueCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'valueQuantity': const FhirField(false, 'Quantity'),
  },
  'ConditionDefinitionQuestionnaire': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'purpose': const FhirField(false, 'Code'),
    'purposeElement': const FhirField(false, 'Element'),
    'reference': const FhirField(false, 'Reference'),
  },
  'ConditionDefinitionPlan': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'role': const FhirField(false, 'CodeableConcept'),
    'reference': const FhirField(false, 'Reference'),
  },
  'Consent': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'category': const FhirField(true, 'CodeableConcept'),
    'subject': const FhirField(false, 'Reference'),
    'date': const FhirField(false, 'Date'),
    'dateElement': const FhirField(false, 'Element'),
    'period': const FhirField(false, 'Period'),
    'grantor': const FhirField(true, 'Reference'),
    'grantee': const FhirField(true, 'Reference'),
    'manager': const FhirField(true, 'Reference'),
    'controller': const FhirField(true, 'Reference'),
    'sourceAttachment': const FhirField(true, 'Attachment'),
    'sourceReference': const FhirField(true, 'Reference'),
    'regulatoryBasis': const FhirField(true, 'CodeableConcept'),
    'policyBasis': const FhirField(false, 'ConsentPolicyBasis'),
    'policyText': const FhirField(true, 'Reference'),
    'verification': const FhirField(true, 'ConsentVerification'),
    'decision': const FhirField(false, 'Code'),
    'decisionElement': const FhirField(false, 'Element'),
    'provision': const FhirField(true, 'ConsentProvision'),
  },
  'ConsentPolicyBasis': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'reference': const FhirField(false, 'Reference'),
    'url': const FhirField(false, 'FhirUrl'),
    'urlElement': const FhirField(false, 'Element'),
  },
  'ConsentVerification': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'verified': const FhirField(false, 'Boolean'),
    'verifiedElement': const FhirField(false, 'Element'),
    'verificationType': const FhirField(false, 'CodeableConcept'),
    'verifiedBy': const FhirField(false, 'Reference'),
    'verifiedWith': const FhirField(false, 'Reference'),
    'verificationDate': const FhirField(true, 'FhirDateTime'),
    'verificationDateElement': const FhirField(true, 'Element'),
  },
  'ConsentProvision': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'period': const FhirField(false, 'Period'),
    'actor': const FhirField(true, 'ConsentActor'),
    'action': const FhirField(true, 'CodeableConcept'),
    'securityLabel': const FhirField(true, 'Coding'),
    'purpose': const FhirField(true, 'Coding'),
    'documentType': const FhirField(true, 'Coding'),
    'resourceType': const FhirField(true, 'Coding'),
    'code': const FhirField(true, 'CodeableConcept'),
    'dataPeriod': const FhirField(false, 'Period'),
    'data': const FhirField(true, 'ConsentData'),
    'expression': const FhirField(false, 'Expression'),
    'provision': const FhirField(true, 'ConsentProvision'),
  },
  'ConsentActor': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'role': const FhirField(false, 'CodeableConcept'),
    'reference': const FhirField(false, 'Reference'),
  },
  'ConsentData': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'meaning': const FhirField(false, 'Code'),
    'meaningElement': const FhirField(false, 'Element'),
    'reference': const FhirField(false, 'Reference'),
  },
  'Contract': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'url': const FhirField(false, 'FhirUri'),
    'urlElement': const FhirField(false, 'Element'),
    'version': const FhirField(false, 'String'),
    'versionElement': const FhirField(false, 'Element'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'legalState': const FhirField(false, 'CodeableConcept'),
    'instantiatesCanonical': const FhirField(false, 'Reference'),
    'instantiatesUri': const FhirField(false, 'FhirUri'),
    'instantiatesUriElement': const FhirField(false, 'Element'),
    'contentDerivative': const FhirField(false, 'CodeableConcept'),
    'issued': const FhirField(false, 'FhirDateTime'),
    'issuedElement': const FhirField(false, 'Element'),
    'applies': const FhirField(false, 'Period'),
    'expirationType': const FhirField(false, 'CodeableConcept'),
    'subject': const FhirField(true, 'Reference'),
    'authority': const FhirField(true, 'Reference'),
    'domain': const FhirField(true, 'Reference'),
    'site': const FhirField(true, 'Reference'),
    'name': const FhirField(false, 'String'),
    'nameElement': const FhirField(false, 'Element'),
    'title': const FhirField(false, 'String'),
    'titleElement': const FhirField(false, 'Element'),
    'subtitle': const FhirField(false, 'String'),
    'subtitleElement': const FhirField(false, 'Element'),
    'alias': const FhirField(true, 'String'),
    'aliasElement': const FhirField(true, 'Element'),
    'author': const FhirField(false, 'Reference'),
    'scope': const FhirField(false, 'CodeableConcept'),
    'topicCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'topicReference': const FhirField(false, 'Reference'),
    'type': const FhirField(false, 'CodeableConcept'),
    'subType': const FhirField(true, 'CodeableConcept'),
    'contentDefinition': const FhirField(false, 'ContractContentDefinition'),
    'term': const FhirField(true, 'ContractTerm'),
    'supportingInfo': const FhirField(true, 'Reference'),
    'relevantHistory': const FhirField(true, 'Reference'),
    'signer': const FhirField(true, 'ContractSigner'),
    'friendly': const FhirField(true, 'ContractFriendly'),
    'legal': const FhirField(true, 'ContractLegal'),
    'rule': const FhirField(true, 'ContractRule'),
    'legallyBindingAttachment': const FhirField(false, 'Attachment'),
    'legallyBindingReference': const FhirField(false, 'Reference'),
  },
  'ContractContentDefinition': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'subType': const FhirField(false, 'CodeableConcept'),
    'publisher': const FhirField(false, 'Reference'),
    'publicationDate': const FhirField(false, 'FhirDateTime'),
    'publicationDateElement': const FhirField(false, 'Element'),
    'publicationStatus': const FhirField(false, 'Code'),
    'publicationStatusElement': const FhirField(false, 'Element'),
    'copyright': const FhirField(false, 'Markdown'),
    'copyrightElement': const FhirField(false, 'Element'),
  },
  'ContractTerm': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(false, 'Identifier'),
    'issued': const FhirField(false, 'FhirDateTime'),
    'issuedElement': const FhirField(false, 'Element'),
    'applies': const FhirField(false, 'Period'),
    'topicCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'topicReference': const FhirField(false, 'Reference'),
    'type': const FhirField(false, 'CodeableConcept'),
    'subType': const FhirField(false, 'CodeableConcept'),
    'text': const FhirField(false, 'String'),
    'textElement': const FhirField(false, 'Element'),
    'securityLabel': const FhirField(true, 'ContractSecurityLabel'),
    'offer': const FhirField(false, 'ContractOffer'),
    'asset': const FhirField(true, 'ContractAsset'),
    'action': const FhirField(true, 'ContractAction'),
    'group': const FhirField(true, 'ContractTerm'),
  },
  'ContractSecurityLabel': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'number': const FhirField(true, 'UnsignedInt'),
    'numberElement': const FhirField(true, 'Element'),
    'classification': const FhirField(false, 'Coding'),
    'category': const FhirField(true, 'Coding'),
    'control': const FhirField(true, 'Coding'),
  },
  'ContractOffer': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'party': const FhirField(true, 'ContractParty'),
    'topic': const FhirField(false, 'Reference'),
    'type': const FhirField(false, 'CodeableConcept'),
    'decision': const FhirField(false, 'CodeableConcept'),
    'decisionMode': const FhirField(true, 'CodeableConcept'),
    'answer': const FhirField(true, 'ContractAnswer'),
    'text': const FhirField(false, 'String'),
    'textElement': const FhirField(false, 'Element'),
    'linkId': const FhirField(true, 'String'),
    'linkIdElement': const FhirField(true, 'Element'),
    'securityLabelNumber': const FhirField(true, 'UnsignedInt'),
    'securityLabelNumberElement': const FhirField(true, 'Element'),
  },
  'ContractParty': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'reference': const FhirField(true, 'Reference'),
    'role': const FhirField(false, 'CodeableConcept'),
  },
  'ContractAnswer': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'valueBoolean': const FhirField(false, 'Boolean'),
    'valueBooleanElement': const FhirField(false, 'Element'),
    'valueDecimal': const FhirField(false, 'Decimal'),
    'valueDecimalElement': const FhirField(false, 'Element'),
    'valueInteger': const FhirField(false, 'Integer'),
    'valueIntegerElement': const FhirField(false, 'Element'),
    'valueDate': const FhirField(false, 'Date'),
    'valueDateElement': const FhirField(false, 'Element'),
    'valueDateTime': const FhirField(false, 'FhirDateTime'),
    'valueDateTimeElement': const FhirField(false, 'Element'),
    'valueTime': const FhirField(false, 'Time'),
    'valueTimeElement': const FhirField(false, 'Element'),
    'valueString': const FhirField(false, 'String'),
    'valueStringElement': const FhirField(false, 'Element'),
    'valueUri': const FhirField(false, 'FhirUri'),
    'valueUriElement': const FhirField(false, 'Element'),
    'valueAttachment': const FhirField(false, 'Attachment'),
    'valueCoding': const FhirField(false, 'Coding'),
    'valueQuantity': const FhirField(false, 'Quantity'),
    'valueReference': const FhirField(false, 'Reference'),
  },
  'ContractAsset': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'scope': const FhirField(false, 'CodeableConcept'),
    'type': const FhirField(true, 'CodeableConcept'),
    'typeReference': const FhirField(true, 'Reference'),
    'subtype': const FhirField(true, 'CodeableConcept'),
    'relationship': const FhirField(false, 'Coding'),
    'context': const FhirField(true, 'ContractContext'),
    'condition': const FhirField(false, 'String'),
    'conditionElement': const FhirField(false, 'Element'),
    'periodType': const FhirField(true, 'CodeableConcept'),
    'period': const FhirField(true, 'Period'),
    'usePeriod': const FhirField(true, 'Period'),
    'text': const FhirField(false, 'String'),
    'textElement': const FhirField(false, 'Element'),
    'linkId': const FhirField(true, 'String'),
    'linkIdElement': const FhirField(true, 'Element'),
    'answer': const FhirField(true, 'ContractAnswer'),
    'securityLabelNumber': const FhirField(true, 'UnsignedInt'),
    'securityLabelNumberElement': const FhirField(true, 'Element'),
    'valuedItem': const FhirField(true, 'ContractValuedItem'),
  },
  'ContractContext': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'reference': const FhirField(false, 'Reference'),
    'code': const FhirField(true, 'CodeableConcept'),
    'text': const FhirField(false, 'String'),
    'textElement': const FhirField(false, 'Element'),
  },
  'ContractValuedItem': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'entityCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'entityReference': const FhirField(false, 'Reference'),
    'identifier': const FhirField(false, 'Identifier'),
    'effectiveTime': const FhirField(false, 'FhirDateTime'),
    'effectiveTimeElement': const FhirField(false, 'Element'),
    'quantity': const FhirField(false, 'Quantity'),
    'unitPrice': const FhirField(false, 'Money'),
    'factor': const FhirField(false, 'Decimal'),
    'factorElement': const FhirField(false, 'Element'),
    'points': const FhirField(false, 'Decimal'),
    'pointsElement': const FhirField(false, 'Element'),
    'net': const FhirField(false, 'Money'),
    'payment': const FhirField(false, 'String'),
    'paymentElement': const FhirField(false, 'Element'),
    'paymentDate': const FhirField(false, 'FhirDateTime'),
    'paymentDateElement': const FhirField(false, 'Element'),
    'responsible': const FhirField(false, 'Reference'),
    'recipient': const FhirField(false, 'Reference'),
    'linkId': const FhirField(true, 'String'),
    'linkIdElement': const FhirField(true, 'Element'),
    'securityLabelNumber': const FhirField(true, 'UnsignedInt'),
    'securityLabelNumberElement': const FhirField(true, 'Element'),
  },
  'ContractAction': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'doNotPerform': const FhirField(false, 'Boolean'),
    'doNotPerformElement': const FhirField(false, 'Element'),
    'type': const FhirField(false, 'CodeableConcept'),
    'subject': const FhirField(true, 'ContractSubject'),
    'intent': const FhirField(false, 'CodeableConcept'),
    'linkId': const FhirField(true, 'String'),
    'linkIdElement': const FhirField(true, 'Element'),
    'status': const FhirField(false, 'CodeableConcept'),
    'context': const FhirField(false, 'Reference'),
    'contextLinkId': const FhirField(true, 'String'),
    'contextLinkIdElement': const FhirField(true, 'Element'),
    'occurrenceDateTime': const FhirField(false, 'FhirDateTime'),
    'occurrenceDateTimeElement': const FhirField(false, 'Element'),
    'occurrencePeriod': const FhirField(false, 'Period'),
    'occurrenceTiming': const FhirField(false, 'Timing'),
    'requester': const FhirField(true, 'Reference'),
    'requesterLinkId': const FhirField(true, 'String'),
    'requesterLinkIdElement': const FhirField(true, 'Element'),
    'performerType': const FhirField(true, 'CodeableConcept'),
    'performerRole': const FhirField(false, 'CodeableConcept'),
    'performer': const FhirField(false, 'Reference'),
    'performerLinkId': const FhirField(true, 'String'),
    'performerLinkIdElement': const FhirField(true, 'Element'),
    'reason': const FhirField(true, 'CodeableReference'),
    'reasonLinkId': const FhirField(true, 'String'),
    'reasonLinkIdElement': const FhirField(true, 'Element'),
    'note': const FhirField(true, 'Annotation'),
    'securityLabelNumber': const FhirField(true, 'UnsignedInt'),
    'securityLabelNumberElement': const FhirField(true, 'Element'),
  },
  'ContractSubject': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'reference': const FhirField(true, 'Reference'),
    'role': const FhirField(false, 'CodeableConcept'),
  },
  'ContractSigner': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'Coding'),
    'party': const FhirField(false, 'Reference'),
    'signature': const FhirField(true, 'Signature'),
  },
  'ContractFriendly': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'contentAttachment': const FhirField(false, 'Attachment'),
    'contentReference': const FhirField(false, 'Reference'),
  },
  'ContractLegal': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'contentAttachment': const FhirField(false, 'Attachment'),
    'contentReference': const FhirField(false, 'Reference'),
  },
  'ContractRule': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'contentAttachment': const FhirField(false, 'Attachment'),
    'contentReference': const FhirField(false, 'Reference'),
  },
  'Coverage': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'kind': const FhirField(false, 'Code'),
    'kindElement': const FhirField(false, 'Element'),
    'paymentBy': const FhirField(true, 'CoveragePaymentBy'),
    'type': const FhirField(false, 'CodeableConcept'),
    'policyHolder': const FhirField(false, 'Reference'),
    'subscriber': const FhirField(false, 'Reference'),
    'subscriberId': const FhirField(true, 'Identifier'),
    'beneficiary': const FhirField(false, 'Reference'),
    'dependent': const FhirField(false, 'String'),
    'dependentElement': const FhirField(false, 'Element'),
    'relationship': const FhirField(false, 'CodeableConcept'),
    'period': const FhirField(false, 'Period'),
    'insurer': const FhirField(false, 'Reference'),
    'class': const FhirField(true, 'CoverageClass'),
    'order': const FhirField(false, 'PositiveInt'),
    'orderElement': const FhirField(false, 'Element'),
    'network': const FhirField(false, 'String'),
    'networkElement': const FhirField(false, 'Element'),
    'costToBeneficiary': const FhirField(true, 'CoverageCostToBeneficiary'),
    'subrogation': const FhirField(false, 'Boolean'),
    'subrogationElement': const FhirField(false, 'Element'),
    'contract': const FhirField(true, 'Reference'),
    'insurancePlan': const FhirField(false, 'Reference'),
  },
  'CoveragePaymentBy': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'party': const FhirField(false, 'Reference'),
    'responsibility': const FhirField(false, 'String'),
    'responsibilityElement': const FhirField(false, 'Element'),
  },
  'CoverageClass': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'value': const FhirField(false, 'Identifier'),
    'name': const FhirField(false, 'String'),
    'nameElement': const FhirField(false, 'Element'),
  },
  'CoverageCostToBeneficiary': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'category': const FhirField(false, 'CodeableConcept'),
    'network': const FhirField(false, 'CodeableConcept'),
    'unit': const FhirField(false, 'CodeableConcept'),
    'term': const FhirField(false, 'CodeableConcept'),
    'valueQuantity': const FhirField(false, 'Quantity'),
    'valueMoney': const FhirField(false, 'Money'),
    'exception': const FhirField(true, 'CoverageException'),
  },
  'CoverageException': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'period': const FhirField(false, 'Period'),
  },
  'CoverageEligibilityRequest': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'priority': const FhirField(false, 'CodeableConcept'),
    'purpose': const FhirField(true, 'Code'),
    'purposeElement': const FhirField(true, 'Element'),
    'patient': const FhirField(false, 'Reference'),
    'event': const FhirField(true, 'CoverageEligibilityRequestEvent'),
    'servicedDate': const FhirField(false, 'Date'),
    'servicedDateElement': const FhirField(false, 'Element'),
    'servicedPeriod': const FhirField(false, 'Period'),
    'created': const FhirField(false, 'FhirDateTime'),
    'createdElement': const FhirField(false, 'Element'),
    'enterer': const FhirField(false, 'Reference'),
    'provider': const FhirField(false, 'Reference'),
    'insurer': const FhirField(false, 'Reference'),
    'facility': const FhirField(false, 'Reference'),
    'supportingInfo':
        const FhirField(true, 'CoverageEligibilityRequestSupportingInfo'),
    'insurance': const FhirField(true, 'CoverageEligibilityRequestInsurance'),
    'item': const FhirField(true, 'CoverageEligibilityRequestItem'),
  },
  'CoverageEligibilityRequestEvent': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'whenDateTime': const FhirField(false, 'FhirDateTime'),
    'whenDateTimeElement': const FhirField(false, 'Element'),
    'whenPeriod': const FhirField(false, 'Period'),
  },
  'CoverageEligibilityRequestSupportingInfo': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'sequence': const FhirField(false, 'PositiveInt'),
    'sequenceElement': const FhirField(false, 'Element'),
    'information': const FhirField(false, 'Reference'),
    'appliesToAll': const FhirField(false, 'Boolean'),
    'appliesToAllElement': const FhirField(false, 'Element'),
  },
  'CoverageEligibilityRequestInsurance': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'focal': const FhirField(false, 'Boolean'),
    'focalElement': const FhirField(false, 'Element'),
    'coverage': const FhirField(false, 'Reference'),
    'businessArrangement': const FhirField(false, 'String'),
    'businessArrangementElement': const FhirField(false, 'Element'),
  },
  'CoverageEligibilityRequestItem': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'supportingInfoSequence': const FhirField(true, 'PositiveInt'),
    'supportingInfoSequenceElement': const FhirField(true, 'Element'),
    'category': const FhirField(false, 'CodeableConcept'),
    'productOrService': const FhirField(false, 'CodeableConcept'),
    'modifier': const FhirField(true, 'CodeableConcept'),
    'provider': const FhirField(false, 'Reference'),
    'quantity': const FhirField(false, 'Quantity'),
    'unitPrice': const FhirField(false, 'Money'),
    'facility': const FhirField(false, 'Reference'),
    'diagnosis': const FhirField(true, 'CoverageEligibilityRequestDiagnosis'),
    'detail': const FhirField(true, 'Reference'),
  },
  'CoverageEligibilityRequestDiagnosis': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'diagnosisCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'diagnosisReference': const FhirField(false, 'Reference'),
  },
  'CoverageEligibilityResponse': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'purpose': const FhirField(true, 'Code'),
    'purposeElement': const FhirField(true, 'Element'),
    'patient': const FhirField(false, 'Reference'),
    'event': const FhirField(true, 'CoverageEligibilityResponseEvent'),
    'servicedDate': const FhirField(false, 'Date'),
    'servicedDateElement': const FhirField(false, 'Element'),
    'servicedPeriod': const FhirField(false, 'Period'),
    'created': const FhirField(false, 'FhirDateTime'),
    'createdElement': const FhirField(false, 'Element'),
    'requestor': const FhirField(false, 'Reference'),
    'request': const FhirField(false, 'Reference'),
    'outcome': const FhirField(false, 'Code'),
    'outcomeElement': const FhirField(false, 'Element'),
    'disposition': const FhirField(false, 'String'),
    'dispositionElement': const FhirField(false, 'Element'),
    'insurer': const FhirField(false, 'Reference'),
    'insurance': const FhirField(true, 'CoverageEligibilityResponseInsurance'),
    'preAuthRef': const FhirField(false, 'String'),
    'preAuthRefElement': const FhirField(false, 'Element'),
    'form': const FhirField(false, 'CodeableConcept'),
    'error': const FhirField(true, 'CoverageEligibilityResponseError'),
  },
  'CoverageEligibilityResponseEvent': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'whenDateTime': const FhirField(false, 'FhirDateTime'),
    'whenDateTimeElement': const FhirField(false, 'Element'),
    'whenPeriod': const FhirField(false, 'Period'),
  },
  'CoverageEligibilityResponseInsurance': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'coverage': const FhirField(false, 'Reference'),
    'inforce': const FhirField(false, 'Boolean'),
    'inforceElement': const FhirField(false, 'Element'),
    'benefitPeriod': const FhirField(false, 'Period'),
    'item': const FhirField(true, 'CoverageEligibilityResponseItem'),
  },
  'CoverageEligibilityResponseItem': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'category': const FhirField(false, 'CodeableConcept'),
    'productOrService': const FhirField(false, 'CodeableConcept'),
    'modifier': const FhirField(true, 'CodeableConcept'),
    'provider': const FhirField(false, 'Reference'),
    'excluded': const FhirField(false, 'Boolean'),
    'excludedElement': const FhirField(false, 'Element'),
    'name': const FhirField(false, 'String'),
    'nameElement': const FhirField(false, 'Element'),
    'description': const FhirField(false, 'String'),
    'descriptionElement': const FhirField(false, 'Element'),
    'network': const FhirField(false, 'CodeableConcept'),
    'unit': const FhirField(false, 'CodeableConcept'),
    'term': const FhirField(false, 'CodeableConcept'),
    'benefit': const FhirField(true, 'CoverageEligibilityResponseBenefit'),
    'authorizationRequired': const FhirField(false, 'Boolean'),
    'authorizationRequiredElement': const FhirField(false, 'Element'),
    'authorizationSupporting': const FhirField(true, 'CodeableConcept'),
    'authorizationUrl': const FhirField(false, 'FhirUri'),
    'authorizationUrlElement': const FhirField(false, 'Element'),
  },
  'CoverageEligibilityResponseBenefit': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'allowedUnsignedInt': const FhirField(false, 'UnsignedInt'),
    'allowedUnsignedIntElement': const FhirField(false, 'Element'),
    'allowedString': const FhirField(false, 'String'),
    'allowedStringElement': const FhirField(false, 'Element'),
    'allowedMoney': const FhirField(false, 'Money'),
    'usedUnsignedInt': const FhirField(false, 'UnsignedInt'),
    'usedUnsignedIntElement': const FhirField(false, 'Element'),
    'usedString': const FhirField(false, 'String'),
    'usedStringElement': const FhirField(false, 'Element'),
    'usedMoney': const FhirField(false, 'Money'),
  },
  'CoverageEligibilityResponseError': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(false, 'CodeableConcept'),
    'expression': const FhirField(true, 'String'),
    'expressionElement': const FhirField(true, 'Element'),
  },
  'DetectedIssue': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'category': const FhirField(true, 'CodeableConcept'),
    'code': const FhirField(false, 'CodeableConcept'),
    'severity': const FhirField(false, 'Code'),
    'severityElement': const FhirField(false, 'Element'),
    'subject': const FhirField(false, 'Reference'),
    'encounter': const FhirField(false, 'Reference'),
    'identifiedDateTime': const FhirField(false, 'FhirDateTime'),
    'identifiedDateTimeElement': const FhirField(false, 'Element'),
    'identifiedPeriod': const FhirField(false, 'Period'),
    'author': const FhirField(false, 'Reference'),
    'implicated': const FhirField(true, 'Reference'),
    'evidence': const FhirField(true, 'DetectedIssueEvidence'),
    'detail': const FhirField(false, 'Markdown'),
    'detailElement': const FhirField(false, 'Element'),
    'reference': const FhirField(false, 'FhirUri'),
    'referenceElement': const FhirField(false, 'Element'),
    'mitigation': const FhirField(true, 'DetectedIssueMitigation'),
  },
  'DetectedIssueEvidence': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(true, 'CodeableConcept'),
    'detail': const FhirField(true, 'Reference'),
  },
  'DetectedIssueMitigation': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'action': const FhirField(false, 'CodeableConcept'),
    'date': const FhirField(false, 'FhirDateTime'),
    'dateElement': const FhirField(false, 'Element'),
    'author': const FhirField(false, 'Reference'),
    'note': const FhirField(true, 'Annotation'),
  },
  'Device': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'displayName': const FhirField(false, 'String'),
    'displayNameElement': const FhirField(false, 'Element'),
    'definition': const FhirField(false, 'CodeableReference'),
    'udiCarrier': const FhirField(true, 'DeviceUdiCarrier'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'availabilityStatus': const FhirField(false, 'CodeableConcept'),
    'biologicalSourceEvent': const FhirField(false, 'Identifier'),
    'manufacturer': const FhirField(false, 'String'),
    'manufacturerElement': const FhirField(false, 'Element'),
    'manufactureDate': const FhirField(false, 'FhirDateTime'),
    'manufactureDateElement': const FhirField(false, 'Element'),
    'expirationDate': const FhirField(false, 'FhirDateTime'),
    'expirationDateElement': const FhirField(false, 'Element'),
    'lotNumber': const FhirField(false, 'String'),
    'lotNumberElement': const FhirField(false, 'Element'),
    'serialNumber': const FhirField(false, 'String'),
    'serialNumberElement': const FhirField(false, 'Element'),
    'name': const FhirField(true, 'DeviceName'),
    'modelNumber': const FhirField(false, 'String'),
    'modelNumberElement': const FhirField(false, 'Element'),
    'partNumber': const FhirField(false, 'String'),
    'partNumberElement': const FhirField(false, 'Element'),
    'category': const FhirField(true, 'CodeableConcept'),
    'type': const FhirField(true, 'CodeableConcept'),
    'version': const FhirField(true, 'DeviceVersion'),
    'conformsTo': const FhirField(true, 'DeviceConformsTo'),
    'property': const FhirField(true, 'DeviceProperty'),
    'mode': const FhirField(false, 'CodeableConcept'),
    'cycle': const FhirField(false, 'Count'),
    'duration': const FhirField(false, 'FhirDuration'),
    'owner': const FhirField(false, 'Reference'),
    'contact': const FhirField(true, 'ContactPoint'),
    'location': const FhirField(false, 'Reference'),
    'url': const FhirField(false, 'FhirUri'),
    'urlElement': const FhirField(false, 'Element'),
    'endpoint': const FhirField(true, 'Reference'),
    'gateway': const FhirField(true, 'CodeableReference'),
    'note': const FhirField(true, 'Annotation'),
    'safety': const FhirField(true, 'CodeableConcept'),
    'parent': const FhirField(false, 'Reference'),
  },
  'DeviceUdiCarrier': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'deviceIdentifier': const FhirField(false, 'String'),
    'deviceIdentifierElement': const FhirField(false, 'Element'),
    'issuer': const FhirField(false, 'FhirUri'),
    'issuerElement': const FhirField(false, 'Element'),
    'jurisdiction': const FhirField(false, 'FhirUri'),
    'jurisdictionElement': const FhirField(false, 'Element'),
    'carrierAIDC': const FhirField(false, 'Base64Binary'),
    'carrierAIDCElement': const FhirField(false, 'Element'),
    'carrierHRF': const FhirField(false, 'String'),
    'carrierHRFElement': const FhirField(false, 'Element'),
    'entryType': const FhirField(false, 'Code'),
    'entryTypeElement': const FhirField(false, 'Element'),
  },
  'DeviceName': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'value': const FhirField(false, 'String'),
    'valueElement': const FhirField(false, 'Element'),
    'type': const FhirField(false, 'Code'),
    'typeElement': const FhirField(false, 'Element'),
    'display': const FhirField(false, 'Boolean'),
    'displayElement': const FhirField(false, 'Element'),
  },
  'DeviceVersion': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'component': const FhirField(false, 'Identifier'),
    'installDate': const FhirField(false, 'FhirDateTime'),
    'installDateElement': const FhirField(false, 'Element'),
    'value': const FhirField(false, 'String'),
    'valueElement': const FhirField(false, 'Element'),
  },
  'DeviceConformsTo': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'category': const FhirField(false, 'CodeableConcept'),
    'specification': const FhirField(false, 'CodeableConcept'),
    'version': const FhirField(false, 'String'),
    'versionElement': const FhirField(false, 'Element'),
  },
  'DeviceProperty': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'valueQuantity': const FhirField(false, 'Quantity'),
    'valueCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'valueString': const FhirField(false, 'String'),
    'valueStringElement': const FhirField(false, 'Element'),
    'valueBoolean': const FhirField(false, 'Boolean'),
    'valueBooleanElement': const FhirField(false, 'Element'),
    'valueInteger': const FhirField(false, 'Integer'),
    'valueIntegerElement': const FhirField(false, 'Element'),
    'valueRange': const FhirField(false, 'Range'),
    'valueAttachment': const FhirField(false, 'Attachment'),
  },
  'DeviceAssociation': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'device': const FhirField(false, 'Reference'),
    'category': const FhirField(true, 'CodeableConcept'),
    'status': const FhirField(false, 'CodeableConcept'),
    'statusReason': const FhirField(true, 'CodeableConcept'),
    'subject': const FhirField(false, 'Reference'),
    'bodyStructure': const FhirField(false, 'Reference'),
    'period': const FhirField(false, 'Period'),
    'operation': const FhirField(true, 'DeviceAssociationOperation'),
  },
  'DeviceAssociationOperation': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'status': const FhirField(false, 'CodeableConcept'),
    'operator': const FhirField(true, 'Reference'),
    'period': const FhirField(false, 'Period'),
  },
  'DeviceDefinition': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'description': const FhirField(false, 'Markdown'),
    'descriptionElement': const FhirField(false, 'Element'),
    'identifier': const FhirField(true, 'Identifier'),
    'udiDeviceIdentifier':
        const FhirField(true, 'DeviceDefinitionUdiDeviceIdentifier'),
    'regulatoryIdentifier':
        const FhirField(true, 'DeviceDefinitionRegulatoryIdentifier'),
    'partNumber': const FhirField(false, 'String'),
    'partNumberElement': const FhirField(false, 'Element'),
    'manufacturer': const FhirField(false, 'Reference'),
    'deviceName': const FhirField(true, 'DeviceDefinitionDeviceName'),
    'modelNumber': const FhirField(false, 'String'),
    'modelNumberElement': const FhirField(false, 'Element'),
    'classification': const FhirField(true, 'DeviceDefinitionClassification'),
    'conformsTo': const FhirField(true, 'DeviceDefinitionConformsTo'),
    'hasPart': const FhirField(true, 'DeviceDefinitionHasPart'),
    'packaging': const FhirField(true, 'DeviceDefinitionPackaging'),
    'version': const FhirField(true, 'DeviceDefinitionVersion'),
    'safety': const FhirField(true, 'CodeableConcept'),
    'shelfLifeStorage': const FhirField(true, 'ProductShelfLife'),
    'languageCode': const FhirField(true, 'CodeableConcept'),
    'property': const FhirField(true, 'DeviceDefinitionProperty'),
    'owner': const FhirField(false, 'Reference'),
    'contact': const FhirField(true, 'ContactPoint'),
    'link': const FhirField(true, 'DeviceDefinitionLink'),
    'note': const FhirField(true, 'Annotation'),
    'material': const FhirField(true, 'DeviceDefinitionMaterial'),
    'productionIdentifierInUDI': const FhirField(true, 'Code'),
    'productionIdentifierInUDIElement': const FhirField(true, 'Element'),
    'guideline': const FhirField(false, 'DeviceDefinitionGuideline'),
    'correctiveAction':
        const FhirField(false, 'DeviceDefinitionCorrectiveAction'),
    'chargeItem': const FhirField(true, 'DeviceDefinitionChargeItem'),
  },
  'DeviceDefinitionUdiDeviceIdentifier': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'deviceIdentifier': const FhirField(false, 'String'),
    'deviceIdentifierElement': const FhirField(false, 'Element'),
    'issuer': const FhirField(false, 'FhirUri'),
    'issuerElement': const FhirField(false, 'Element'),
    'jurisdiction': const FhirField(false, 'FhirUri'),
    'jurisdictionElement': const FhirField(false, 'Element'),
    'marketDistribution':
        const FhirField(true, 'DeviceDefinitionMarketDistribution'),
  },
  'DeviceDefinitionMarketDistribution': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'marketPeriod': const FhirField(false, 'Period'),
    'subJurisdiction': const FhirField(false, 'FhirUri'),
    'subJurisdictionElement': const FhirField(false, 'Element'),
  },
  'DeviceDefinitionRegulatoryIdentifier': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'Code'),
    'typeElement': const FhirField(false, 'Element'),
    'deviceIdentifier': const FhirField(false, 'String'),
    'deviceIdentifierElement': const FhirField(false, 'Element'),
    'issuer': const FhirField(false, 'FhirUri'),
    'issuerElement': const FhirField(false, 'Element'),
    'jurisdiction': const FhirField(false, 'FhirUri'),
    'jurisdictionElement': const FhirField(false, 'Element'),
  },
  'DeviceDefinitionDeviceName': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'name': const FhirField(false, 'String'),
    'nameElement': const FhirField(false, 'Element'),
    'type': const FhirField(false, 'Code'),
    'typeElement': const FhirField(false, 'Element'),
  },
  'DeviceDefinitionClassification': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'justification': const FhirField(true, 'RelatedArtifact'),
  },
  'DeviceDefinitionConformsTo': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'category': const FhirField(false, 'CodeableConcept'),
    'specification': const FhirField(false, 'CodeableConcept'),
    'version': const FhirField(true, 'String'),
    'versionElement': const FhirField(true, 'Element'),
    'source': const FhirField(true, 'RelatedArtifact'),
  },
  'DeviceDefinitionHasPart': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'reference': const FhirField(false, 'Reference'),
    'count': const FhirField(false, 'Integer'),
    'countElement': const FhirField(false, 'Element'),
  },
  'DeviceDefinitionPackaging': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(false, 'Identifier'),
    'type': const FhirField(false, 'CodeableConcept'),
    'count': const FhirField(false, 'Integer'),
    'countElement': const FhirField(false, 'Element'),
    'distributor': const FhirField(true, 'DeviceDefinitionDistributor'),
    'udiDeviceIdentifier':
        const FhirField(true, 'DeviceDefinitionUdiDeviceIdentifier'),
    'packaging': const FhirField(true, 'DeviceDefinitionPackaging'),
  },
  'DeviceDefinitionDistributor': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'name': const FhirField(false, 'String'),
    'nameElement': const FhirField(false, 'Element'),
    'organizationReference': const FhirField(true, 'Reference'),
  },
  'DeviceDefinitionVersion': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'component': const FhirField(false, 'Identifier'),
    'value': const FhirField(false, 'String'),
    'valueElement': const FhirField(false, 'Element'),
  },
  'DeviceDefinitionProperty': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'valueQuantity': const FhirField(false, 'Quantity'),
    'valueCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'valueString': const FhirField(false, 'String'),
    'valueStringElement': const FhirField(false, 'Element'),
    'valueBoolean': const FhirField(false, 'Boolean'),
    'valueBooleanElement': const FhirField(false, 'Element'),
    'valueInteger': const FhirField(false, 'Integer'),
    'valueIntegerElement': const FhirField(false, 'Element'),
    'valueRange': const FhirField(false, 'Range'),
    'valueAttachment': const FhirField(false, 'Attachment'),
  },
  'DeviceDefinitionLink': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'relation': const FhirField(false, 'Coding'),
    'relatedDevice': const FhirField(false, 'CodeableReference'),
  },
  'DeviceDefinitionMaterial': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'substance': const FhirField(false, 'CodeableConcept'),
    'alternate': const FhirField(false, 'Boolean'),
    'alternateElement': const FhirField(false, 'Element'),
    'allergenicIndicator': const FhirField(false, 'Boolean'),
    'allergenicIndicatorElement': const FhirField(false, 'Element'),
  },
  'DeviceDefinitionGuideline': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'useContext': const FhirField(true, 'UsageContext'),
    'usageInstruction': const FhirField(false, 'Markdown'),
    'usageInstructionElement': const FhirField(false, 'Element'),
    'relatedArtifact': const FhirField(true, 'RelatedArtifact'),
    'indication': const FhirField(true, 'CodeableConcept'),
    'contraindication': const FhirField(true, 'CodeableConcept'),
    'warning': const FhirField(true, 'CodeableConcept'),
    'intendedUse': const FhirField(false, 'String'),
    'intendedUseElement': const FhirField(false, 'Element'),
  },
  'DeviceDefinitionCorrectiveAction': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'recall': const FhirField(false, 'Boolean'),
    'recallElement': const FhirField(false, 'Element'),
    'scope': const FhirField(false, 'Code'),
    'scopeElement': const FhirField(false, 'Element'),
    'period': const FhirField(false, 'Period'),
  },
  'DeviceDefinitionChargeItem': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'chargeItemCode': const FhirField(false, 'CodeableReference'),
    'count': const FhirField(false, 'Quantity'),
    'effectivePeriod': const FhirField(false, 'Period'),
    'useContext': const FhirField(true, 'UsageContext'),
  },
  'DeviceDispense': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'basedOn': const FhirField(true, 'Reference'),
    'partOf': const FhirField(true, 'Reference'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'statusReason': const FhirField(false, 'CodeableReference'),
    'category': const FhirField(true, 'CodeableConcept'),
    'device': const FhirField(false, 'CodeableReference'),
    'subject': const FhirField(false, 'Reference'),
    'receiver': const FhirField(false, 'Reference'),
    'encounter': const FhirField(false, 'Reference'),
    'supportingInformation': const FhirField(true, 'Reference'),
    'performer': const FhirField(true, 'DeviceDispensePerformer'),
    'location': const FhirField(false, 'Reference'),
    'type': const FhirField(false, 'CodeableConcept'),
    'quantity': const FhirField(false, 'Quantity'),
    'preparedDate': const FhirField(false, 'FhirDateTime'),
    'preparedDateElement': const FhirField(false, 'Element'),
    'whenHandedOver': const FhirField(false, 'FhirDateTime'),
    'whenHandedOverElement': const FhirField(false, 'Element'),
    'destination': const FhirField(false, 'Reference'),
    'note': const FhirField(true, 'Annotation'),
    'usageInstruction': const FhirField(false, 'Markdown'),
    'usageInstructionElement': const FhirField(false, 'Element'),
    'eventHistory': const FhirField(true, 'Reference'),
  },
  'DeviceDispensePerformer': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'function': const FhirField(false, 'CodeableConcept'),
    'actor': const FhirField(false, 'Reference'),
  },
  'DeviceMetric': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'type': const FhirField(false, 'CodeableConcept'),
    'unit': const FhirField(false, 'CodeableConcept'),
    'device': const FhirField(false, 'Reference'),
    'operationalStatus': const FhirField(false, 'Code'),
    'operationalStatusElement': const FhirField(false, 'Element'),
    'color': const FhirField(false, 'Code'),
    'colorElement': const FhirField(false, 'Element'),
    'category': const FhirField(false, 'Code'),
    'categoryElement': const FhirField(false, 'Element'),
    'measurementFrequency': const FhirField(false, 'Quantity'),
    'calibration': const FhirField(true, 'DeviceMetricCalibration'),
  },
  'DeviceMetricCalibration': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'Code'),
    'typeElement': const FhirField(false, 'Element'),
    'state': const FhirField(false, 'Code'),
    'stateElement': const FhirField(false, 'Element'),
    'time': const FhirField(false, 'Instant'),
    'timeElement': const FhirField(false, 'Element'),
  },
  'DeviceRequest': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'instantiatesCanonical': const FhirField(true, 'Canonical'),
    'instantiatesUri': const FhirField(true, 'FhirUri'),
    'instantiatesUriElement': const FhirField(true, 'Element'),
    'basedOn': const FhirField(true, 'Reference'),
    'replaces': const FhirField(true, 'Reference'),
    'groupIdentifier': const FhirField(false, 'Identifier'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'intent': const FhirField(false, 'Code'),
    'intentElement': const FhirField(false, 'Element'),
    'priority': const FhirField(false, 'Code'),
    'priorityElement': const FhirField(false, 'Element'),
    'doNotPerform': const FhirField(false, 'Boolean'),
    'doNotPerformElement': const FhirField(false, 'Element'),
    'code': const FhirField(false, 'CodeableReference'),
    'quantity': const FhirField(false, 'Integer'),
    'quantityElement': const FhirField(false, 'Element'),
    'parameter': const FhirField(true, 'DeviceRequestParameter'),
    'subject': const FhirField(false, 'Reference'),
    'encounter': const FhirField(false, 'Reference'),
    'occurrenceDateTime': const FhirField(false, 'FhirDateTime'),
    'occurrenceDateTimeElement': const FhirField(false, 'Element'),
    'occurrencePeriod': const FhirField(false, 'Period'),
    'occurrenceTiming': const FhirField(false, 'Timing'),
    'authoredOn': const FhirField(false, 'FhirDateTime'),
    'authoredOnElement': const FhirField(false, 'Element'),
    'requester': const FhirField(false, 'Reference'),
    'performer': const FhirField(false, 'CodeableReference'),
    'reason': const FhirField(true, 'CodeableReference'),
    'asNeeded': const FhirField(false, 'Boolean'),
    'asNeededElement': const FhirField(false, 'Element'),
    'asNeededFor': const FhirField(false, 'CodeableConcept'),
    'insurance': const FhirField(true, 'Reference'),
    'supportingInfo': const FhirField(true, 'Reference'),
    'note': const FhirField(true, 'Annotation'),
    'relevantHistory': const FhirField(true, 'Reference'),
  },
  'DeviceRequestParameter': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(false, 'CodeableConcept'),
    'valueCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'valueQuantity': const FhirField(false, 'Quantity'),
    'valueRange': const FhirField(false, 'Range'),
    'valueBoolean': const FhirField(false, 'Boolean'),
    'valueBooleanElement': const FhirField(false, 'Element'),
  },
  'DeviceUsage': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'basedOn': const FhirField(true, 'Reference'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'category': const FhirField(true, 'CodeableConcept'),
    'patient': const FhirField(false, 'Reference'),
    'derivedFrom': const FhirField(true, 'Reference'),
    'context': const FhirField(false, 'Reference'),
    'timingTiming': const FhirField(false, 'Timing'),
    'timingPeriod': const FhirField(false, 'Period'),
    'timingDateTime': const FhirField(false, 'FhirDateTime'),
    'timingDateTimeElement': const FhirField(false, 'Element'),
    'dateAsserted': const FhirField(false, 'FhirDateTime'),
    'dateAssertedElement': const FhirField(false, 'Element'),
    'usageStatus': const FhirField(false, 'CodeableConcept'),
    'usageReason': const FhirField(true, 'CodeableConcept'),
    'adherence': const FhirField(false, 'DeviceUsageAdherence'),
    'informationSource': const FhirField(false, 'Reference'),
    'device': const FhirField(false, 'CodeableReference'),
    'reason': const FhirField(true, 'CodeableReference'),
    'bodySite': const FhirField(false, 'CodeableReference'),
    'note': const FhirField(true, 'Annotation'),
  },
  'DeviceUsageAdherence': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(false, 'CodeableConcept'),
    'reason': const FhirField(true, 'CodeableConcept'),
  },
  'DiagnosticReport': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'basedOn': const FhirField(true, 'Reference'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'category': const FhirField(true, 'CodeableConcept'),
    'code': const FhirField(false, 'CodeableConcept'),
    'subject': const FhirField(false, 'Reference'),
    'encounter': const FhirField(false, 'Reference'),
    'effectiveDateTime': const FhirField(false, 'FhirDateTime'),
    'effectiveDateTimeElement': const FhirField(false, 'Element'),
    'effectivePeriod': const FhirField(false, 'Period'),
    'issued': const FhirField(false, 'Instant'),
    'issuedElement': const FhirField(false, 'Element'),
    'performer': const FhirField(true, 'Reference'),
    'resultsInterpreter': const FhirField(true, 'Reference'),
    'specimen': const FhirField(true, 'Reference'),
    'result': const FhirField(true, 'Reference'),
    'note': const FhirField(true, 'Annotation'),
    'study': const FhirField(true, 'Reference'),
    'supportingInfo': const FhirField(true, 'DiagnosticReportSupportingInfo'),
    'media': const FhirField(true, 'DiagnosticReportMedia'),
    'composition': const FhirField(false, 'Reference'),
    'conclusion': const FhirField(false, 'Markdown'),
    'conclusionElement': const FhirField(false, 'Element'),
    'conclusionCode': const FhirField(true, 'CodeableConcept'),
    'presentedForm': const FhirField(true, 'Attachment'),
  },
  'DiagnosticReportSupportingInfo': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'reference': const FhirField(false, 'Reference'),
  },
  'DiagnosticReportMedia': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'comment': const FhirField(false, 'String'),
    'commentElement': const FhirField(false, 'Element'),
    'link': const FhirField(false, 'Reference'),
  },
  'DocumentManifest': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'masterIdentifier': const FhirField(false, 'Identifier'),
    'identifier': const FhirField(true, 'Identifier'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'type': const FhirField(false, 'CodeableConcept'),
    'subject': const FhirField(false, 'Reference'),
    'created': const FhirField(false, 'FhirDateTime'),
    'createdElement': const FhirField(false, 'Element'),
    'author': const FhirField(true, 'Reference'),
    'recipient': const FhirField(true, 'Reference'),
    'source': const FhirField(false, 'FhirUri'),
    'sourceElement': const FhirField(false, 'Element'),
    'description': const FhirField(false, 'String'),
    'descriptionElement': const FhirField(false, 'Element'),
    'content': const FhirField(true, 'Reference'),
    'related': const FhirField(true, 'DocumentManifestRelated'),
  },
  'DocumentManifestRelated': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(false, 'Identifier'),
    'ref': const FhirField(false, 'Reference'),
  },
  'DocumentReference': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'version': const FhirField(false, 'String'),
    'versionElement': const FhirField(false, 'Element'),
    'basedOn': const FhirField(true, 'Reference'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'docStatus': const FhirField(false, 'Code'),
    'docStatusElement': const FhirField(false, 'Element'),
    'modality': const FhirField(true, 'CodeableConcept'),
    'type': const FhirField(false, 'CodeableConcept'),
    'category': const FhirField(true, 'CodeableConcept'),
    'subject': const FhirField(false, 'Reference'),
    'context': const FhirField(true, 'Reference'),
    'event': const FhirField(true, 'CodeableReference'),
    'bodySite': const FhirField(true, 'CodeableReference'),
    'facilityType': const FhirField(false, 'CodeableConcept'),
    'practiceSetting': const FhirField(false, 'CodeableConcept'),
    'period': const FhirField(false, 'Period'),
    'date': const FhirField(false, 'Instant'),
    'dateElement': const FhirField(false, 'Element'),
    'author': const FhirField(true, 'Reference'),
    'attester': const FhirField(true, 'DocumentReferenceAttester'),
    'custodian': const FhirField(false, 'Reference'),
    'relatesTo': const FhirField(true, 'DocumentReferenceRelatesTo'),
    'description': const FhirField(false, 'Markdown'),
    'descriptionElement': const FhirField(false, 'Element'),
    'securityLabel': const FhirField(true, 'CodeableConcept'),
    'content': const FhirField(true, 'DocumentReferenceContent'),
  },
  'DocumentReferenceAttester': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'mode': const FhirField(false, 'CodeableConcept'),
    'time': const FhirField(false, 'FhirDateTime'),
    'timeElement': const FhirField(false, 'Element'),
    'party': const FhirField(false, 'Reference'),
  },
  'DocumentReferenceRelatesTo': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(false, 'CodeableConcept'),
    'target': const FhirField(false, 'Reference'),
  },
  'DocumentReferenceContent': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'attachment': const FhirField(false, 'Attachment'),
    'profile': const FhirField(true, 'DocumentReferenceProfile'),
  },
  'DocumentReferenceProfile': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'valueCoding': const FhirField(false, 'Coding'),
    'valueUri': const FhirField(false, 'FhirUri'),
    'valueUriElement': const FhirField(false, 'Element'),
    'valueCanonical': const FhirField(false, 'Canonical'),
    'valueCanonicalElement': const FhirField(false, 'Element'),
  },
  'Encounter': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'class': const FhirField(true, 'CodeableConcept'),
    'priority': const FhirField(false, 'CodeableConcept'),
    'type': const FhirField(true, 'CodeableConcept'),
    'serviceType': const FhirField(true, 'CodeableReference'),
    'subject': const FhirField(false, 'Reference'),
    'subjectStatus': const FhirField(false, 'CodeableConcept'),
    'episodeOfCare': const FhirField(true, 'Reference'),
    'basedOn': const FhirField(true, 'Reference'),
    'careTeam': const FhirField(true, 'Reference'),
    'partOf': const FhirField(false, 'Reference'),
    'serviceProvider': const FhirField(false, 'Reference'),
    'participant': const FhirField(true, 'EncounterParticipant'),
    'appointment': const FhirField(true, 'Reference'),
    'virtualService': const FhirField(true, 'VirtualServiceDetail'),
    'actualPeriod': const FhirField(false, 'Period'),
    'plannedStartDate': const FhirField(false, 'FhirDateTime'),
    'plannedStartDateElement': const FhirField(false, 'Element'),
    'plannedEndDate': const FhirField(false, 'FhirDateTime'),
    'plannedEndDateElement': const FhirField(false, 'Element'),
    'length': const FhirField(false, 'FhirDuration'),
    'reason': const FhirField(true, 'EncounterReason'),
    'diagnosis': const FhirField(true, 'EncounterDiagnosis'),
    'account': const FhirField(true, 'Reference'),
    'dietPreference': const FhirField(true, 'CodeableConcept'),
    'specialArrangement': const FhirField(true, 'CodeableConcept'),
    'specialCourtesy': const FhirField(true, 'CodeableConcept'),
    'admission': const FhirField(false, 'EncounterAdmission'),
    'location': const FhirField(true, 'EncounterLocation'),
  },
  'EncounterParticipant': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(true, 'CodeableConcept'),
    'period': const FhirField(false, 'Period'),
    'actor': const FhirField(false, 'Reference'),
  },
  'EncounterReason': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'use': const FhirField(true, 'CodeableConcept'),
    'value': const FhirField(true, 'CodeableReference'),
  },
  'EncounterDiagnosis': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'condition': const FhirField(true, 'CodeableReference'),
    'use': const FhirField(true, 'CodeableConcept'),
  },
  'EncounterAdmission': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'preAdmissionIdentifier': const FhirField(false, 'Identifier'),
    'origin': const FhirField(false, 'Reference'),
    'admitSource': const FhirField(false, 'CodeableConcept'),
    'reAdmission': const FhirField(false, 'CodeableConcept'),
    'destination': const FhirField(false, 'Reference'),
    'dischargeDisposition': const FhirField(false, 'CodeableConcept'),
  },
  'EncounterLocation': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'location': const FhirField(false, 'Reference'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'form': const FhirField(false, 'CodeableConcept'),
    'period': const FhirField(false, 'Period'),
  },
  'EncounterHistory': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'encounter': const FhirField(false, 'Reference'),
    'identifier': const FhirField(true, 'Identifier'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'class': const FhirField(false, 'CodeableConcept'),
    'type': const FhirField(true, 'CodeableConcept'),
    'serviceType': const FhirField(true, 'CodeableReference'),
    'subject': const FhirField(false, 'Reference'),
    'subjectStatus': const FhirField(false, 'CodeableConcept'),
    'actualPeriod': const FhirField(false, 'Period'),
    'plannedStartDate': const FhirField(false, 'FhirDateTime'),
    'plannedStartDateElement': const FhirField(false, 'Element'),
    'plannedEndDate': const FhirField(false, 'FhirDateTime'),
    'plannedEndDateElement': const FhirField(false, 'Element'),
    'length': const FhirField(false, 'FhirDuration'),
    'location': const FhirField(true, 'EncounterHistoryLocation'),
  },
  'EncounterHistoryLocation': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'location': const FhirField(false, 'Reference'),
    'form': const FhirField(false, 'CodeableConcept'),
  },
  'Endpoint': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'connectionType': const FhirField(true, 'CodeableConcept'),
    'name': const FhirField(false, 'String'),
    'nameElement': const FhirField(false, 'Element'),
    'description': const FhirField(false, 'String'),
    'descriptionElement': const FhirField(false, 'Element'),
    'environmentType': const FhirField(true, 'CodeableConcept'),
    'managingOrganization': const FhirField(false, 'Reference'),
    'contact': const FhirField(true, 'ContactPoint'),
    'period': const FhirField(false, 'Period'),
    'payload': const FhirField(true, 'EndpointPayload'),
    'address': const FhirField(false, 'FhirUrl'),
    'addressElement': const FhirField(false, 'Element'),
    'header': const FhirField(true, 'String'),
    'headerElement': const FhirField(true, 'Element'),
  },
  'EndpointPayload': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(true, 'CodeableConcept'),
    'mimeType': const FhirField(true, 'Code'),
    'mimeTypeElement': const FhirField(true, 'Element'),
  },
  'EnrollmentRequest': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'created': const FhirField(false, 'FhirDateTime'),
    'createdElement': const FhirField(false, 'Element'),
    'insurer': const FhirField(false, 'Reference'),
    'provider': const FhirField(false, 'Reference'),
    'candidate': const FhirField(false, 'Reference'),
    'coverage': const FhirField(false, 'Reference'),
  },
  'EnrollmentResponse': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'request': const FhirField(false, 'Reference'),
    'outcome': const FhirField(false, 'Code'),
    'outcomeElement': const FhirField(false, 'Element'),
    'disposition': const FhirField(false, 'String'),
    'dispositionElement': const FhirField(false, 'Element'),
    'created': const FhirField(false, 'FhirDateTime'),
    'createdElement': const FhirField(false, 'Element'),
    'organization': const FhirField(false, 'Reference'),
    'requestProvider': const FhirField(false, 'Reference'),
  },
  'EpisodeOfCare': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'statusHistory': const FhirField(true, 'EpisodeOfCareStatusHistory'),
    'type': const FhirField(true, 'CodeableConcept'),
    'reason': const FhirField(true, 'EpisodeOfCareReason'),
    'diagnosis': const FhirField(true, 'EpisodeOfCareDiagnosis'),
    'patient': const FhirField(false, 'Reference'),
    'managingOrganization': const FhirField(false, 'Reference'),
    'period': const FhirField(false, 'Period'),
    'referralRequest': const FhirField(true, 'Reference'),
    'careManager': const FhirField(false, 'Reference'),
    'careTeam': const FhirField(true, 'Reference'),
    'account': const FhirField(true, 'Reference'),
  },
  'EpisodeOfCareStatusHistory': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'period': const FhirField(false, 'Period'),
  },
  'EpisodeOfCareReason': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'use': const FhirField(false, 'CodeableConcept'),
    'value': const FhirField(true, 'CodeableReference'),
  },
  'EpisodeOfCareDiagnosis': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'condition': const FhirField(true, 'CodeableReference'),
    'use': const FhirField(false, 'CodeableConcept'),
  },
  'EventDefinition': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'url': const FhirField(false, 'FhirUri'),
    'urlElement': const FhirField(false, 'Element'),
    'identifier': const FhirField(true, 'Identifier'),
    'version': const FhirField(false, 'String'),
    'versionElement': const FhirField(false, 'Element'),
    'versionAlgorithmString': const FhirField(false, 'String'),
    'versionAlgorithmStringElement': const FhirField(false, 'Element'),
    'versionAlgorithmCoding': const FhirField(false, 'Coding'),
    'name': const FhirField(false, 'String'),
    'nameElement': const FhirField(false, 'Element'),
    'title': const FhirField(false, 'String'),
    'titleElement': const FhirField(false, 'Element'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'experimental': const FhirField(false, 'Boolean'),
    'experimentalElement': const FhirField(false, 'Element'),
    'date': const FhirField(false, 'FhirDateTime'),
    'dateElement': const FhirField(false, 'Element'),
    'publisher': const FhirField(false, 'String'),
    'publisherElement': const FhirField(false, 'Element'),
    'contact': const FhirField(true, 'ContactDetail'),
    'description': const FhirField(false, 'Markdown'),
    'descriptionElement': const FhirField(false, 'Element'),
    'useContext': const FhirField(true, 'UsageContext'),
    'jurisdiction': const FhirField(true, 'CodeableConcept'),
    'purpose': const FhirField(false, 'Markdown'),
    'purposeElement': const FhirField(false, 'Element'),
    'copyright': const FhirField(false, 'Markdown'),
    'copyrightElement': const FhirField(false, 'Element'),
    'copyrightLabel': const FhirField(false, 'String'),
    'copyrightLabelElement': const FhirField(false, 'Element'),
    'approvalDate': const FhirField(false, 'Date'),
    'approvalDateElement': const FhirField(false, 'Element'),
    'lastReviewDate': const FhirField(false, 'Date'),
    'lastReviewDateElement': const FhirField(false, 'Element'),
    'effectivePeriod': const FhirField(false, 'Period'),
    'topic': const FhirField(true, 'CodeableConcept'),
    'author': const FhirField(true, 'ContactDetail'),
    'editor': const FhirField(true, 'ContactDetail'),
    'reviewer': const FhirField(true, 'ContactDetail'),
    'endorser': const FhirField(true, 'ContactDetail'),
    'relatedArtifact': const FhirField(true, 'RelatedArtifact'),
    'subtitle': const FhirField(false, 'String'),
    'subtitleElement': const FhirField(false, 'Element'),
    'subjectCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'subjectReference': const FhirField(false, 'Reference'),
    'usage': const FhirField(false, 'Markdown'),
    'usageElement': const FhirField(false, 'Element'),
    'trigger': const FhirField(true, 'TriggerDefinition'),
  },
  'Evidence': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'url': const FhirField(false, 'FhirUri'),
    'urlElement': const FhirField(false, 'Element'),
    'identifier': const FhirField(true, 'Identifier'),
    'version': const FhirField(false, 'String'),
    'versionElement': const FhirField(false, 'Element'),
    'versionAlgorithmString': const FhirField(false, 'String'),
    'versionAlgorithmStringElement': const FhirField(false, 'Element'),
    'versionAlgorithmCoding': const FhirField(false, 'Coding'),
    'name': const FhirField(false, 'String'),
    'nameElement': const FhirField(false, 'Element'),
    'title': const FhirField(false, 'String'),
    'titleElement': const FhirField(false, 'Element'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'experimental': const FhirField(false, 'Boolean'),
    'experimentalElement': const FhirField(false, 'Element'),
    'date': const FhirField(false, 'FhirDateTime'),
    'dateElement': const FhirField(false, 'Element'),
    'publisher': const FhirField(false, 'String'),
    'publisherElement': const FhirField(false, 'Element'),
    'contact': const FhirField(true, 'ContactDetail'),
    'description': const FhirField(false, 'Markdown'),
    'descriptionElement': const FhirField(false, 'Element'),
    'useContext': const FhirField(true, 'UsageContext'),
    'jurisdiction': const FhirField(true, 'CodeableConcept'),
    'purpose': const FhirField(false, 'Markdown'),
    'purposeElement': const FhirField(false, 'Element'),
    'copyright': const FhirField(false, 'Markdown'),
    'copyrightElement': const FhirField(false, 'Element'),
    'copyrightLabel': const FhirField(false, 'String'),
    'copyrightLabelElement': const FhirField(false, 'Element'),
    'approvalDate': const FhirField(false, 'Date'),
    'approvalDateElement': const FhirField(false, 'Element'),
    'lastReviewDate': const FhirField(false, 'Date'),
    'lastReviewDateElement': const FhirField(false, 'Element'),
    'effectivePeriod': const FhirField(false, 'Period'),
    'topic': const FhirField(true, 'CodeableConcept'),
    'author': const FhirField(true, 'ContactDetail'),
    'editor': const FhirField(true, 'ContactDetail'),
    'reviewer': const FhirField(true, 'ContactDetail'),
    'endorser': const FhirField(true, 'ContactDetail'),
    'relatedArtifact': const FhirField(true, 'RelatedArtifact'),
    'citeAsReference': const FhirField(false, 'Reference'),
    'citeAsMarkdown': const FhirField(false, 'Markdown'),
    'citeAsMarkdownElement': const FhirField(false, 'Element'),
    'assertion': const FhirField(false, 'Markdown'),
    'assertionElement': const FhirField(false, 'Element'),
    'note': const FhirField(true, 'Annotation'),
    'variableDefinition': const FhirField(true, 'EvidenceVariableDefinition'),
    'synthesisType': const FhirField(false, 'CodeableConcept'),
    'studyDesign': const FhirField(true, 'CodeableConcept'),
    'statistic': const FhirField(true, 'EvidenceStatistic'),
    'certainty': const FhirField(true, 'EvidenceCertainty'),
  },
  'EvidenceVariableDefinition': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'description': const FhirField(false, 'Markdown'),
    'descriptionElement': const FhirField(false, 'Element'),
    'note': const FhirField(true, 'Annotation'),
    'variableRole': const FhirField(false, 'CodeableConcept'),
    'observed': const FhirField(false, 'Reference'),
    'intended': const FhirField(false, 'Reference'),
    'directnessMatch': const FhirField(false, 'CodeableConcept'),
  },
  'EvidenceStatistic': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'description': const FhirField(false, 'Markdown'),
    'descriptionElement': const FhirField(false, 'Element'),
    'note': const FhirField(true, 'Annotation'),
    'statisticType': const FhirField(false, 'CodeableConcept'),
    'category': const FhirField(false, 'CodeableConcept'),
    'quantity': const FhirField(false, 'Quantity'),
    'numberOfEvents': const FhirField(false, 'UnsignedInt'),
    'numberOfEventsElement': const FhirField(false, 'Element'),
    'numberAffected': const FhirField(false, 'UnsignedInt'),
    'numberAffectedElement': const FhirField(false, 'Element'),
    'sampleSize': const FhirField(false, 'EvidenceSampleSize'),
    'attributeEstimate': const FhirField(true, 'EvidenceAttributeEstimate'),
    'modelCharacteristic': const FhirField(true, 'EvidenceModelCharacteristic'),
  },
  'EvidenceSampleSize': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'description': const FhirField(false, 'Markdown'),
    'descriptionElement': const FhirField(false, 'Element'),
    'note': const FhirField(true, 'Annotation'),
    'numberOfStudies': const FhirField(false, 'UnsignedInt'),
    'numberOfStudiesElement': const FhirField(false, 'Element'),
    'numberOfParticipants': const FhirField(false, 'UnsignedInt'),
    'numberOfParticipantsElement': const FhirField(false, 'Element'),
    'knownDataCount': const FhirField(false, 'UnsignedInt'),
    'knownDataCountElement': const FhirField(false, 'Element'),
  },
  'EvidenceAttributeEstimate': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'description': const FhirField(false, 'Markdown'),
    'descriptionElement': const FhirField(false, 'Element'),
    'note': const FhirField(true, 'Annotation'),
    'type': const FhirField(false, 'CodeableConcept'),
    'quantity': const FhirField(false, 'Quantity'),
    'level': const FhirField(false, 'Decimal'),
    'levelElement': const FhirField(false, 'Element'),
    'range': const FhirField(false, 'Range'),
    'attributeEstimate': const FhirField(true, 'EvidenceAttributeEstimate'),
  },
  'EvidenceModelCharacteristic': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(false, 'CodeableConcept'),
    'value': const FhirField(false, 'Quantity'),
    'variable': const FhirField(true, 'EvidenceVariable'),
    'attributeEstimate': const FhirField(true, 'EvidenceAttributeEstimate'),
  },
  'EvidenceCertainty': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'description': const FhirField(false, 'Markdown'),
    'descriptionElement': const FhirField(false, 'Element'),
    'note': const FhirField(true, 'Annotation'),
    'type': const FhirField(false, 'CodeableConcept'),
    'rating': const FhirField(false, 'CodeableConcept'),
    'rater': const FhirField(false, 'String'),
    'raterElement': const FhirField(false, 'Element'),
    'subcomponent': const FhirField(true, 'EvidenceCertainty'),
  },
  'EvidenceReport': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'url': const FhirField(false, 'FhirUri'),
    'urlElement': const FhirField(false, 'Element'),
    'identifier': const FhirField(true, 'Identifier'),
    'version': const FhirField(false, 'String'),
    'versionElement': const FhirField(false, 'Element'),
    'versionAlgorithmString': const FhirField(false, 'String'),
    'versionAlgorithmStringElement': const FhirField(false, 'Element'),
    'versionAlgorithmCoding': const FhirField(false, 'Coding'),
    'name': const FhirField(false, 'String'),
    'nameElement': const FhirField(false, 'Element'),
    'title': const FhirField(false, 'String'),
    'titleElement': const FhirField(false, 'Element'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'experimental': const FhirField(false, 'Boolean'),
    'experimentalElement': const FhirField(false, 'Element'),
    'date': const FhirField(false, 'FhirDateTime'),
    'dateElement': const FhirField(false, 'Element'),
    'publisher': const FhirField(false, 'String'),
    'publisherElement': const FhirField(false, 'Element'),
    'contact': const FhirField(true, 'ContactDetail'),
    'description': const FhirField(false, 'Markdown'),
    'descriptionElement': const FhirField(false, 'Element'),
    'useContext': const FhirField(true, 'UsageContext'),
    'jurisdiction': const FhirField(true, 'CodeableConcept'),
    'purpose': const FhirField(false, 'Markdown'),
    'purposeElement': const FhirField(false, 'Element'),
    'copyright': const FhirField(false, 'Markdown'),
    'copyrightElement': const FhirField(false, 'Element'),
    'copyrightLabel': const FhirField(false, 'String'),
    'copyrightLabelElement': const FhirField(false, 'Element'),
    'approvalDate': const FhirField(false, 'Date'),
    'approvalDateElement': const FhirField(false, 'Element'),
    'lastReviewDate': const FhirField(false, 'Date'),
    'lastReviewDateElement': const FhirField(false, 'Element'),
    'effectivePeriod': const FhirField(false, 'Period'),
    'topic': const FhirField(true, 'CodeableConcept'),
    'author': const FhirField(true, 'ContactDetail'),
    'editor': const FhirField(true, 'ContactDetail'),
    'reviewer': const FhirField(true, 'ContactDetail'),
    'endorser': const FhirField(true, 'ContactDetail'),
    'relatedArtifact': const FhirField(true, 'RelatedArtifact'),
    'relatedIdentifier': const FhirField(true, 'Identifier'),
    'citeAsReference': const FhirField(false, 'Reference'),
    'citeAsMarkdown': const FhirField(false, 'Markdown'),
    'citeAsMarkdownElement': const FhirField(false, 'Element'),
    'type': const FhirField(false, 'CodeableConcept'),
    'note': const FhirField(true, 'Annotation'),
    'subject': const FhirField(false, 'EvidenceReportSubject'),
    'relatesTo': const FhirField(true, 'EvidenceReportRelatesTo'),
    'section': const FhirField(true, 'EvidenceReportSection'),
  },
  'EvidenceReportSubject': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'characteristic': const FhirField(true, 'EvidenceReportCharacteristic'),
    'note': const FhirField(true, 'Annotation'),
  },
  'EvidenceReportCharacteristic': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(false, 'CodeableConcept'),
    'valueReference': const FhirField(false, 'Reference'),
    'valueCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'valueBoolean': const FhirField(false, 'Boolean'),
    'valueBooleanElement': const FhirField(false, 'Element'),
    'valueQuantity': const FhirField(false, 'Quantity'),
    'valueRange': const FhirField(false, 'Range'),
    'exclude': const FhirField(false, 'Boolean'),
    'excludeElement': const FhirField(false, 'Element'),
    'period': const FhirField(false, 'Period'),
  },
  'EvidenceReportRelatesTo': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(false, 'Code'),
    'codeElement': const FhirField(false, 'Element'),
    'target': const FhirField(false, 'EvidenceReportTarget'),
  },
  'EvidenceReportTarget': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'url': const FhirField(false, 'FhirUri'),
    'urlElement': const FhirField(false, 'Element'),
    'identifier': const FhirField(false, 'Identifier'),
    'display': const FhirField(false, 'Markdown'),
    'displayElement': const FhirField(false, 'Element'),
    'resource': const FhirField(false, 'Reference'),
  },
  'EvidenceReportSection': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'title': const FhirField(false, 'String'),
    'titleElement': const FhirField(false, 'Element'),
    'focus': const FhirField(false, 'CodeableConcept'),
    'focusReference': const FhirField(false, 'Reference'),
    'author': const FhirField(true, 'Reference'),
    'text': const FhirField(false, 'Narrative'),
    'mode': const FhirField(false, 'Code'),
    'modeElement': const FhirField(false, 'Element'),
    'orderedBy': const FhirField(false, 'CodeableConcept'),
    'entryClassifier': const FhirField(true, 'CodeableConcept'),
    'entryReference': const FhirField(true, 'Reference'),
    'entryQuantity': const FhirField(true, 'Quantity'),
    'emptyReason': const FhirField(false, 'CodeableConcept'),
    'section': const FhirField(true, 'EvidenceReportSection'),
  },
  'EvidenceVariable': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'url': const FhirField(false, 'FhirUri'),
    'urlElement': const FhirField(false, 'Element'),
    'identifier': const FhirField(true, 'Identifier'),
    'version': const FhirField(false, 'String'),
    'versionElement': const FhirField(false, 'Element'),
    'versionAlgorithmString': const FhirField(false, 'String'),
    'versionAlgorithmStringElement': const FhirField(false, 'Element'),
    'versionAlgorithmCoding': const FhirField(false, 'Coding'),
    'name': const FhirField(false, 'String'),
    'nameElement': const FhirField(false, 'Element'),
    'title': const FhirField(false, 'String'),
    'titleElement': const FhirField(false, 'Element'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'experimental': const FhirField(false, 'Boolean'),
    'experimentalElement': const FhirField(false, 'Element'),
    'date': const FhirField(false, 'FhirDateTime'),
    'dateElement': const FhirField(false, 'Element'),
    'publisher': const FhirField(false, 'String'),
    'publisherElement': const FhirField(false, 'Element'),
    'contact': const FhirField(true, 'ContactDetail'),
    'description': const FhirField(false, 'Markdown'),
    'descriptionElement': const FhirField(false, 'Element'),
    'useContext': const FhirField(true, 'UsageContext'),
    'jurisdiction': const FhirField(true, 'CodeableConcept'),
    'purpose': const FhirField(false, 'Markdown'),
    'purposeElement': const FhirField(false, 'Element'),
    'copyright': const FhirField(false, 'Markdown'),
    'copyrightElement': const FhirField(false, 'Element'),
    'copyrightLabel': const FhirField(false, 'String'),
    'copyrightLabelElement': const FhirField(false, 'Element'),
    'approvalDate': const FhirField(false, 'Date'),
    'approvalDateElement': const FhirField(false, 'Element'),
    'lastReviewDate': const FhirField(false, 'Date'),
    'lastReviewDateElement': const FhirField(false, 'Element'),
    'effectivePeriod': const FhirField(false, 'Period'),
    'topic': const FhirField(true, 'CodeableConcept'),
    'author': const FhirField(true, 'ContactDetail'),
    'editor': const FhirField(true, 'ContactDetail'),
    'reviewer': const FhirField(true, 'ContactDetail'),
    'endorser': const FhirField(true, 'ContactDetail'),
    'relatedArtifact': const FhirField(true, 'RelatedArtifact'),
    'shortTitle': const FhirField(false, 'String'),
    'shortTitleElement': const FhirField(false, 'Element'),
    'note': const FhirField(true, 'Annotation'),
    'actual': const FhirField(false, 'Boolean'),
    'actualElement': const FhirField(false, 'Element'),
    'characteristic': const FhirField(true, 'EvidenceVariableCharacteristic'),
    'handling': const FhirField(false, 'Code'),
    'handlingElement': const FhirField(false, 'Element'),
    'category': const FhirField(true, 'EvidenceVariableCategory'),
  },
  'EvidenceVariableCharacteristic': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'linkId': const FhirField(false, 'Id'),
    'linkIdElement': const FhirField(false, 'Element'),
    'description': const FhirField(false, 'Markdown'),
    'descriptionElement': const FhirField(false, 'Element'),
    'note': const FhirField(true, 'Annotation'),
    'exclude': const FhirField(false, 'Boolean'),
    'excludeElement': const FhirField(false, 'Element'),
    'definitionReference': const FhirField(false, 'Reference'),
    'definitionCanonical': const FhirField(false, 'Canonical'),
    'definitionCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'definitionExpression': const FhirField(false, 'Expression'),
    'definitionId': const FhirField(false, 'Id'),
    'definitionIdElement': const FhirField(false, 'Element'),
    'definitionByTypeAndValue':
        const FhirField(false, 'EvidenceVariableDefinitionByTypeAndValue'),
    'definitionByCombination':
        const FhirField(false, 'EvidenceVariableDefinitionByCombination'),
    'instancesQuantity': const FhirField(false, 'Quantity'),
    'instancesRange': const FhirField(false, 'Range'),
    'durationQuantity': const FhirField(false, 'Quantity'),
    'durationRange': const FhirField(false, 'Range'),
    'timeFromEvent': const FhirField(true, 'EvidenceVariableTimeFromEvent'),
  },
  'EvidenceVariableDefinitionByTypeAndValue': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'method': const FhirField(true, 'CodeableConcept'),
    'device': const FhirField(false, 'Reference'),
    'valueCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'valueBoolean': const FhirField(false, 'Boolean'),
    'valueBooleanElement': const FhirField(false, 'Element'),
    'valueQuantity': const FhirField(false, 'Quantity'),
    'valueRange': const FhirField(false, 'Range'),
    'valueReference': const FhirField(false, 'Reference'),
    'valueId': const FhirField(false, 'Id'),
    'valueIdElement': const FhirField(false, 'Element'),
    'offset': const FhirField(false, 'CodeableConcept'),
  },
  'EvidenceVariableDefinitionByCombination': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(false, 'Code'),
    'codeElement': const FhirField(false, 'Element'),
    'threshold': const FhirField(false, 'PositiveInt'),
    'thresholdElement': const FhirField(false, 'Element'),
    'characteristic': const FhirField(true, 'EvidenceVariableCharacteristic'),
  },
  'EvidenceVariableTimeFromEvent': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'description': const FhirField(false, 'Markdown'),
    'descriptionElement': const FhirField(false, 'Element'),
    'note': const FhirField(true, 'Annotation'),
    'eventCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'eventReference': const FhirField(false, 'Reference'),
    'eventDateTime': const FhirField(false, 'FhirDateTime'),
    'eventDateTimeElement': const FhirField(false, 'Element'),
    'eventId': const FhirField(false, 'Id'),
    'eventIdElement': const FhirField(false, 'Element'),
    'quantity': const FhirField(false, 'Quantity'),
    'range': const FhirField(false, 'Range'),
  },
  'EvidenceVariableCategory': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'name': const FhirField(false, 'String'),
    'nameElement': const FhirField(false, 'Element'),
    'valueCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'valueQuantity': const FhirField(false, 'Quantity'),
    'valueRange': const FhirField(false, 'Range'),
  },
  'ExampleScenario': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'url': const FhirField(false, 'FhirUri'),
    'urlElement': const FhirField(false, 'Element'),
    'identifier': const FhirField(true, 'Identifier'),
    'version': const FhirField(false, 'String'),
    'versionElement': const FhirField(false, 'Element'),
    'versionAlgorithmString': const FhirField(false, 'String'),
    'versionAlgorithmStringElement': const FhirField(false, 'Element'),
    'versionAlgorithmCoding': const FhirField(false, 'Coding'),
    'name': const FhirField(false, 'String'),
    'nameElement': const FhirField(false, 'Element'),
    'title': const FhirField(false, 'String'),
    'titleElement': const FhirField(false, 'Element'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'experimental': const FhirField(false, 'Boolean'),
    'experimentalElement': const FhirField(false, 'Element'),
    'date': const FhirField(false, 'FhirDateTime'),
    'dateElement': const FhirField(false, 'Element'),
    'publisher': const FhirField(false, 'String'),
    'publisherElement': const FhirField(false, 'Element'),
    'contact': const FhirField(true, 'ContactDetail'),
    'description': const FhirField(false, 'Markdown'),
    'descriptionElement': const FhirField(false, 'Element'),
    'useContext': const FhirField(true, 'UsageContext'),
    'jurisdiction': const FhirField(true, 'CodeableConcept'),
    'purpose': const FhirField(false, 'Markdown'),
    'purposeElement': const FhirField(false, 'Element'),
    'copyright': const FhirField(false, 'Markdown'),
    'copyrightElement': const FhirField(false, 'Element'),
    'copyrightLabel': const FhirField(false, 'String'),
    'copyrightLabelElement': const FhirField(false, 'Element'),
    'actor': const FhirField(true, 'ExampleScenarioActor'),
    'instance': const FhirField(true, 'ExampleScenarioInstance'),
    'process': const FhirField(true, 'ExampleScenarioProcess'),
  },
  'ExampleScenarioActor': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'key': const FhirField(false, 'String'),
    'keyElement': const FhirField(false, 'Element'),
    'type': const FhirField(false, 'Code'),
    'typeElement': const FhirField(false, 'Element'),
    'title': const FhirField(false, 'String'),
    'titleElement': const FhirField(false, 'Element'),
    'description': const FhirField(false, 'Markdown'),
    'descriptionElement': const FhirField(false, 'Element'),
  },
  'ExampleScenarioInstance': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'key': const FhirField(false, 'String'),
    'keyElement': const FhirField(false, 'Element'),
    'structureType': const FhirField(false, 'Coding'),
    'structureVersion': const FhirField(false, 'String'),
    'structureVersionElement': const FhirField(false, 'Element'),
    'structureProfileCanonical': const FhirField(false, 'Canonical'),
    'structureProfileCanonicalElement': const FhirField(false, 'Element'),
    'structureProfileUri': const FhirField(false, 'FhirUri'),
    'structureProfileUriElement': const FhirField(false, 'Element'),
    'title': const FhirField(false, 'String'),
    'titleElement': const FhirField(false, 'Element'),
    'description': const FhirField(false, 'Markdown'),
    'descriptionElement': const FhirField(false, 'Element'),
    'content': const FhirField(false, 'Reference'),
    'version': const FhirField(true, 'ExampleScenarioVersion'),
    'containedInstance':
        const FhirField(true, 'ExampleScenarioContainedInstance'),
  },
  'ExampleScenarioVersion': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'key': const FhirField(false, 'String'),
    'keyElement': const FhirField(false, 'Element'),
    'title': const FhirField(false, 'String'),
    'titleElement': const FhirField(false, 'Element'),
    'description': const FhirField(false, 'Markdown'),
    'descriptionElement': const FhirField(false, 'Element'),
    'content': const FhirField(false, 'Reference'),
  },
  'ExampleScenarioContainedInstance': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'instanceReference': const FhirField(false, 'String'),
    'instanceReferenceElement': const FhirField(false, 'Element'),
    'versionReference': const FhirField(false, 'String'),
    'versionReferenceElement': const FhirField(false, 'Element'),
  },
  'ExampleScenarioProcess': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'title': const FhirField(false, 'String'),
    'titleElement': const FhirField(false, 'Element'),
    'description': const FhirField(false, 'Markdown'),
    'descriptionElement': const FhirField(false, 'Element'),
    'preConditions': const FhirField(false, 'Markdown'),
    'preConditionsElement': const FhirField(false, 'Element'),
    'postConditions': const FhirField(false, 'Markdown'),
    'postConditionsElement': const FhirField(false, 'Element'),
    'step': const FhirField(true, 'ExampleScenarioStep'),
  },
  'ExampleScenarioStep': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'number': const FhirField(false, 'String'),
    'numberElement': const FhirField(false, 'Element'),
    'process': const FhirField(false, 'ExampleScenarioProcess'),
    'workflow': const FhirField(false, 'Canonical'),
    'operation': const FhirField(false, 'ExampleScenarioOperation'),
    'alternative': const FhirField(true, 'ExampleScenarioAlternative'),
    'pause': const FhirField(false, 'Boolean'),
    'pauseElement': const FhirField(false, 'Element'),
  },
  'ExampleScenarioOperation': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'Coding'),
    'title': const FhirField(false, 'String'),
    'titleElement': const FhirField(false, 'Element'),
    'initiator': const FhirField(false, 'String'),
    'initiatorElement': const FhirField(false, 'Element'),
    'receiver': const FhirField(false, 'String'),
    'receiverElement': const FhirField(false, 'Element'),
    'description': const FhirField(false, 'Markdown'),
    'descriptionElement': const FhirField(false, 'Element'),
    'initiatorActive': const FhirField(false, 'Boolean'),
    'initiatorActiveElement': const FhirField(false, 'Element'),
    'receiverActive': const FhirField(false, 'Boolean'),
    'receiverActiveElement': const FhirField(false, 'Element'),
    'request': const FhirField(false, 'ExampleScenarioContainedInstance'),
    'response': const FhirField(false, 'ExampleScenarioContainedInstance'),
  },
  'ExampleScenarioAlternative': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'title': const FhirField(false, 'String'),
    'titleElement': const FhirField(false, 'Element'),
    'description': const FhirField(false, 'Markdown'),
    'descriptionElement': const FhirField(false, 'Element'),
    'step': const FhirField(true, 'ExampleScenarioStep'),
  },
  'ExplanationOfBenefit': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'traceNumber': const FhirField(true, 'Identifier'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'type': const FhirField(false, 'CodeableConcept'),
    'subType': const FhirField(false, 'CodeableConcept'),
    'use': const FhirField(false, 'Code'),
    'useElement': const FhirField(false, 'Element'),
    'patient': const FhirField(false, 'Reference'),
    'billablePeriod': const FhirField(false, 'Period'),
    'created': const FhirField(false, 'FhirDateTime'),
    'createdElement': const FhirField(false, 'Element'),
    'enterer': const FhirField(false, 'Reference'),
    'insurer': const FhirField(false, 'Reference'),
    'provider': const FhirField(false, 'Reference'),
    'priority': const FhirField(false, 'CodeableConcept'),
    'fundsReserveRequested': const FhirField(false, 'CodeableConcept'),
    'fundsReserve': const FhirField(false, 'CodeableConcept'),
    'related': const FhirField(true, 'ExplanationOfBenefitRelated'),
    'prescription': const FhirField(false, 'Reference'),
    'originalPrescription': const FhirField(false, 'Reference'),
    'event': const FhirField(true, 'ExplanationOfBenefitEvent'),
    'payee': const FhirField(false, 'ExplanationOfBenefitPayee'),
    'referral': const FhirField(false, 'Reference'),
    'encounter': const FhirField(true, 'Reference'),
    'facility': const FhirField(false, 'Reference'),
    'claim': const FhirField(false, 'Reference'),
    'claimResponse': const FhirField(false, 'Reference'),
    'outcome': const FhirField(false, 'Code'),
    'outcomeElement': const FhirField(false, 'Element'),
    'decision': const FhirField(false, 'CodeableConcept'),
    'disposition': const FhirField(false, 'String'),
    'dispositionElement': const FhirField(false, 'Element'),
    'preAuthRef': const FhirField(true, 'String'),
    'preAuthRefElement': const FhirField(true, 'Element'),
    'preAuthRefPeriod': const FhirField(true, 'Period'),
    'diagnosisRelatedGroup': const FhirField(false, 'CodeableConcept'),
    'careTeam': const FhirField(true, 'ExplanationOfBenefitCareTeam'),
    'supportingInfo':
        const FhirField(true, 'ExplanationOfBenefitSupportingInfo'),
    'diagnosis': const FhirField(true, 'ExplanationOfBenefitDiagnosis'),
    'procedure': const FhirField(true, 'ExplanationOfBenefitProcedure'),
    'precedence': const FhirField(false, 'PositiveInt'),
    'precedenceElement': const FhirField(false, 'Element'),
    'insurance': const FhirField(true, 'ExplanationOfBenefitInsurance'),
    'accident': const FhirField(false, 'ExplanationOfBenefitAccident'),
    'patientPaid': const FhirField(false, 'Money'),
    'item': const FhirField(true, 'ExplanationOfBenefitItem'),
    'addItem': const FhirField(true, 'ExplanationOfBenefitAddItem'),
    'adjudication': const FhirField(true, 'ExplanationOfBenefitAdjudication'),
    'total': const FhirField(true, 'ExplanationOfBenefitTotal'),
    'payment': const FhirField(false, 'ExplanationOfBenefitPayment'),
    'formCode': const FhirField(false, 'CodeableConcept'),
    'form': const FhirField(false, 'Attachment'),
    'processNote': const FhirField(true, 'ExplanationOfBenefitProcessNote'),
    'benefitPeriod': const FhirField(false, 'Period'),
    'benefitBalance':
        const FhirField(true, 'ExplanationOfBenefitBenefitBalance'),
  },
  'ExplanationOfBenefitRelated': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'claim': const FhirField(false, 'Reference'),
    'relationship': const FhirField(false, 'CodeableConcept'),
    'reference': const FhirField(false, 'Identifier'),
  },
  'ExplanationOfBenefitEvent': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'whenDateTime': const FhirField(false, 'FhirDateTime'),
    'whenDateTimeElement': const FhirField(false, 'Element'),
    'whenPeriod': const FhirField(false, 'Period'),
  },
  'ExplanationOfBenefitPayee': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'party': const FhirField(false, 'Reference'),
  },
  'ExplanationOfBenefitCareTeam': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'sequence': const FhirField(false, 'PositiveInt'),
    'sequenceElement': const FhirField(false, 'Element'),
    'provider': const FhirField(false, 'Reference'),
    'responsible': const FhirField(false, 'Boolean'),
    'responsibleElement': const FhirField(false, 'Element'),
    'role': const FhirField(false, 'CodeableConcept'),
    'specialty': const FhirField(false, 'CodeableConcept'),
  },
  'ExplanationOfBenefitSupportingInfo': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'sequence': const FhirField(false, 'PositiveInt'),
    'sequenceElement': const FhirField(false, 'Element'),
    'category': const FhirField(false, 'CodeableConcept'),
    'code': const FhirField(false, 'CodeableConcept'),
    'timingDate': const FhirField(false, 'Date'),
    'timingDateElement': const FhirField(false, 'Element'),
    'timingPeriod': const FhirField(false, 'Period'),
    'valueBoolean': const FhirField(false, 'Boolean'),
    'valueBooleanElement': const FhirField(false, 'Element'),
    'valueString': const FhirField(false, 'String'),
    'valueStringElement': const FhirField(false, 'Element'),
    'valueQuantity': const FhirField(false, 'Quantity'),
    'valueAttachment': const FhirField(false, 'Attachment'),
    'valueReference': const FhirField(false, 'Reference'),
    'valueIdentifier': const FhirField(false, 'Identifier'),
    'reason': const FhirField(false, 'Coding'),
  },
  'ExplanationOfBenefitDiagnosis': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'sequence': const FhirField(false, 'PositiveInt'),
    'sequenceElement': const FhirField(false, 'Element'),
    'diagnosisCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'diagnosisReference': const FhirField(false, 'Reference'),
    'type': const FhirField(true, 'CodeableConcept'),
    'onAdmission': const FhirField(false, 'CodeableConcept'),
  },
  'ExplanationOfBenefitProcedure': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'sequence': const FhirField(false, 'PositiveInt'),
    'sequenceElement': const FhirField(false, 'Element'),
    'type': const FhirField(true, 'CodeableConcept'),
    'date': const FhirField(false, 'FhirDateTime'),
    'dateElement': const FhirField(false, 'Element'),
    'procedureCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'procedureReference': const FhirField(false, 'Reference'),
    'udi': const FhirField(true, 'Reference'),
  },
  'ExplanationOfBenefitInsurance': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'focal': const FhirField(false, 'Boolean'),
    'focalElement': const FhirField(false, 'Element'),
    'coverage': const FhirField(false, 'Reference'),
    'preAuthRef': const FhirField(true, 'String'),
    'preAuthRefElement': const FhirField(true, 'Element'),
  },
  'ExplanationOfBenefitAccident': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'date': const FhirField(false, 'Date'),
    'dateElement': const FhirField(false, 'Element'),
    'type': const FhirField(false, 'CodeableConcept'),
    'locationAddress': const FhirField(false, 'Address'),
    'locationReference': const FhirField(false, 'Reference'),
  },
  'ExplanationOfBenefitItem': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'sequence': const FhirField(false, 'PositiveInt'),
    'sequenceElement': const FhirField(false, 'Element'),
    'careTeamSequence': const FhirField(true, 'PositiveInt'),
    'careTeamSequenceElement': const FhirField(true, 'Element'),
    'diagnosisSequence': const FhirField(true, 'PositiveInt'),
    'diagnosisSequenceElement': const FhirField(true, 'Element'),
    'procedureSequence': const FhirField(true, 'PositiveInt'),
    'procedureSequenceElement': const FhirField(true, 'Element'),
    'informationSequence': const FhirField(true, 'PositiveInt'),
    'informationSequenceElement': const FhirField(true, 'Element'),
    'traceNumber': const FhirField(true, 'Identifier'),
    'revenue': const FhirField(false, 'CodeableConcept'),
    'category': const FhirField(false, 'CodeableConcept'),
    'productOrService': const FhirField(false, 'CodeableConcept'),
    'productOrServiceEnd': const FhirField(false, 'CodeableConcept'),
    'request': const FhirField(true, 'Reference'),
    'modifier': const FhirField(true, 'CodeableConcept'),
    'programCode': const FhirField(true, 'CodeableConcept'),
    'servicedDate': const FhirField(false, 'Date'),
    'servicedDateElement': const FhirField(false, 'Element'),
    'servicedPeriod': const FhirField(false, 'Period'),
    'locationCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'locationAddress': const FhirField(false, 'Address'),
    'locationReference': const FhirField(false, 'Reference'),
    'patientPaid': const FhirField(false, 'Money'),
    'quantity': const FhirField(false, 'Quantity'),
    'unitPrice': const FhirField(false, 'Money'),
    'factor': const FhirField(false, 'Decimal'),
    'factorElement': const FhirField(false, 'Element'),
    'tax': const FhirField(false, 'Money'),
    'net': const FhirField(false, 'Money'),
    'udi': const FhirField(true, 'Reference'),
    'bodySite': const FhirField(true, 'ExplanationOfBenefitBodySite'),
    'encounter': const FhirField(true, 'Reference'),
    'noteNumber': const FhirField(true, 'PositiveInt'),
    'noteNumberElement': const FhirField(true, 'Element'),
    'reviewOutcome':
        const FhirField(false, 'ExplanationOfBenefitReviewOutcome'),
    'adjudication': const FhirField(true, 'ExplanationOfBenefitAdjudication'),
    'detail': const FhirField(true, 'ExplanationOfBenefitDetail'),
  },
  'ExplanationOfBenefitBodySite': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'site': const FhirField(true, 'CodeableReference'),
    'subSite': const FhirField(true, 'CodeableConcept'),
  },
  'ExplanationOfBenefitReviewOutcome': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'decision': const FhirField(false, 'CodeableConcept'),
    'reason': const FhirField(true, 'CodeableConcept'),
    'preAuthRef': const FhirField(false, 'String'),
    'preAuthRefElement': const FhirField(false, 'Element'),
    'preAuthPeriod': const FhirField(false, 'Period'),
  },
  'ExplanationOfBenefitAdjudication': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'category': const FhirField(false, 'CodeableConcept'),
    'reason': const FhirField(false, 'CodeableConcept'),
    'amount': const FhirField(false, 'Money'),
    'quantity': const FhirField(false, 'Quantity'),
  },
  'ExplanationOfBenefitDetail': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'sequence': const FhirField(false, 'PositiveInt'),
    'sequenceElement': const FhirField(false, 'Element'),
    'traceNumber': const FhirField(true, 'Identifier'),
    'revenue': const FhirField(false, 'CodeableConcept'),
    'category': const FhirField(false, 'CodeableConcept'),
    'productOrService': const FhirField(false, 'CodeableConcept'),
    'productOrServiceEnd': const FhirField(false, 'CodeableConcept'),
    'modifier': const FhirField(true, 'CodeableConcept'),
    'programCode': const FhirField(true, 'CodeableConcept'),
    'patientPaid': const FhirField(false, 'Money'),
    'quantity': const FhirField(false, 'Quantity'),
    'unitPrice': const FhirField(false, 'Money'),
    'factor': const FhirField(false, 'Decimal'),
    'factorElement': const FhirField(false, 'Element'),
    'tax': const FhirField(false, 'Money'),
    'net': const FhirField(false, 'Money'),
    'udi': const FhirField(true, 'Reference'),
    'noteNumber': const FhirField(true, 'PositiveInt'),
    'noteNumberElement': const FhirField(true, 'Element'),
    'reviewOutcome':
        const FhirField(false, 'ExplanationOfBenefitReviewOutcome'),
    'adjudication': const FhirField(true, 'ExplanationOfBenefitAdjudication'),
    'subDetail': const FhirField(true, 'ExplanationOfBenefitSubDetail'),
  },
  'ExplanationOfBenefitSubDetail': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'sequence': const FhirField(false, 'PositiveInt'),
    'sequenceElement': const FhirField(false, 'Element'),
    'traceNumber': const FhirField(true, 'Identifier'),
    'revenue': const FhirField(false, 'CodeableConcept'),
    'category': const FhirField(false, 'CodeableConcept'),
    'productOrService': const FhirField(false, 'CodeableConcept'),
    'productOrServiceEnd': const FhirField(false, 'CodeableConcept'),
    'modifier': const FhirField(true, 'CodeableConcept'),
    'programCode': const FhirField(true, 'CodeableConcept'),
    'patientPaid': const FhirField(false, 'Money'),
    'quantity': const FhirField(false, 'Quantity'),
    'unitPrice': const FhirField(false, 'Money'),
    'factor': const FhirField(false, 'Decimal'),
    'factorElement': const FhirField(false, 'Element'),
    'tax': const FhirField(false, 'Money'),
    'net': const FhirField(false, 'Money'),
    'udi': const FhirField(true, 'Reference'),
    'noteNumber': const FhirField(true, 'PositiveInt'),
    'noteNumberElement': const FhirField(true, 'Element'),
    'reviewOutcome':
        const FhirField(false, 'ExplanationOfBenefitReviewOutcome'),
    'adjudication': const FhirField(true, 'ExplanationOfBenefitAdjudication'),
  },
  'ExplanationOfBenefitAddItem': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'itemSequence': const FhirField(true, 'PositiveInt'),
    'itemSequenceElement': const FhirField(true, 'Element'),
    'detailSequence': const FhirField(true, 'PositiveInt'),
    'detailSequenceElement': const FhirField(true, 'Element'),
    'subDetailSequence': const FhirField(true, 'PositiveInt'),
    'subDetailSequenceElement': const FhirField(true, 'Element'),
    'traceNumber': const FhirField(true, 'Identifier'),
    'provider': const FhirField(true, 'Reference'),
    'revenue': const FhirField(false, 'CodeableConcept'),
    'productOrService': const FhirField(false, 'CodeableConcept'),
    'productOrServiceEnd': const FhirField(false, 'CodeableConcept'),
    'request': const FhirField(true, 'Reference'),
    'modifier': const FhirField(true, 'CodeableConcept'),
    'programCode': const FhirField(true, 'CodeableConcept'),
    'servicedDate': const FhirField(false, 'Date'),
    'servicedDateElement': const FhirField(false, 'Element'),
    'servicedPeriod': const FhirField(false, 'Period'),
    'locationCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'locationAddress': const FhirField(false, 'Address'),
    'locationReference': const FhirField(false, 'Reference'),
    'patientPaid': const FhirField(false, 'Money'),
    'quantity': const FhirField(false, 'Quantity'),
    'unitPrice': const FhirField(false, 'Money'),
    'factor': const FhirField(false, 'Decimal'),
    'factorElement': const FhirField(false, 'Element'),
    'tax': const FhirField(false, 'Money'),
    'net': const FhirField(false, 'Money'),
    'bodySite': const FhirField(true, 'ExplanationOfBenefitBodySite1'),
    'noteNumber': const FhirField(true, 'PositiveInt'),
    'noteNumberElement': const FhirField(true, 'Element'),
    'reviewOutcome':
        const FhirField(false, 'ExplanationOfBenefitReviewOutcome'),
    'adjudication': const FhirField(true, 'ExplanationOfBenefitAdjudication'),
    'detail': const FhirField(true, 'ExplanationOfBenefitDetail1'),
  },
  'ExplanationOfBenefitBodySite1': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'site': const FhirField(true, 'CodeableReference'),
    'subSite': const FhirField(true, 'CodeableConcept'),
  },
  'ExplanationOfBenefitDetail1': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'traceNumber': const FhirField(true, 'Identifier'),
    'revenue': const FhirField(false, 'CodeableConcept'),
    'productOrService': const FhirField(false, 'CodeableConcept'),
    'productOrServiceEnd': const FhirField(false, 'CodeableConcept'),
    'modifier': const FhirField(true, 'CodeableConcept'),
    'patientPaid': const FhirField(false, 'Money'),
    'quantity': const FhirField(false, 'Quantity'),
    'unitPrice': const FhirField(false, 'Money'),
    'factor': const FhirField(false, 'Decimal'),
    'factorElement': const FhirField(false, 'Element'),
    'tax': const FhirField(false, 'Money'),
    'net': const FhirField(false, 'Money'),
    'noteNumber': const FhirField(true, 'PositiveInt'),
    'noteNumberElement': const FhirField(true, 'Element'),
    'reviewOutcome':
        const FhirField(false, 'ExplanationOfBenefitReviewOutcome'),
    'adjudication': const FhirField(true, 'ExplanationOfBenefitAdjudication'),
    'subDetail': const FhirField(true, 'ExplanationOfBenefitSubDetail1'),
  },
  'ExplanationOfBenefitSubDetail1': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'traceNumber': const FhirField(true, 'Identifier'),
    'revenue': const FhirField(false, 'CodeableConcept'),
    'productOrService': const FhirField(false, 'CodeableConcept'),
    'productOrServiceEnd': const FhirField(false, 'CodeableConcept'),
    'modifier': const FhirField(true, 'CodeableConcept'),
    'patientPaid': const FhirField(false, 'Money'),
    'quantity': const FhirField(false, 'Quantity'),
    'unitPrice': const FhirField(false, 'Money'),
    'factor': const FhirField(false, 'Decimal'),
    'factorElement': const FhirField(false, 'Element'),
    'tax': const FhirField(false, 'Money'),
    'net': const FhirField(false, 'Money'),
    'noteNumber': const FhirField(true, 'PositiveInt'),
    'noteNumberElement': const FhirField(true, 'Element'),
    'reviewOutcome':
        const FhirField(false, 'ExplanationOfBenefitReviewOutcome'),
    'adjudication': const FhirField(true, 'ExplanationOfBenefitAdjudication'),
  },
  'ExplanationOfBenefitTotal': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'category': const FhirField(false, 'CodeableConcept'),
    'amount': const FhirField(false, 'Money'),
  },
  'ExplanationOfBenefitPayment': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'adjustment': const FhirField(false, 'Money'),
    'adjustmentReason': const FhirField(false, 'CodeableConcept'),
    'date': const FhirField(false, 'Date'),
    'dateElement': const FhirField(false, 'Element'),
    'amount': const FhirField(false, 'Money'),
    'identifier': const FhirField(false, 'Identifier'),
  },
  'ExplanationOfBenefitProcessNote': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'number': const FhirField(false, 'PositiveInt'),
    'numberElement': const FhirField(false, 'Element'),
    'type': const FhirField(false, 'CodeableConcept'),
    'text': const FhirField(false, 'String'),
    'textElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'CodeableConcept'),
  },
  'ExplanationOfBenefitBenefitBalance': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'category': const FhirField(false, 'CodeableConcept'),
    'excluded': const FhirField(false, 'Boolean'),
    'excludedElement': const FhirField(false, 'Element'),
    'name': const FhirField(false, 'String'),
    'nameElement': const FhirField(false, 'Element'),
    'description': const FhirField(false, 'String'),
    'descriptionElement': const FhirField(false, 'Element'),
    'network': const FhirField(false, 'CodeableConcept'),
    'unit': const FhirField(false, 'CodeableConcept'),
    'term': const FhirField(false, 'CodeableConcept'),
    'financial': const FhirField(true, 'ExplanationOfBenefitFinancial'),
  },
  'ExplanationOfBenefitFinancial': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'allowedUnsignedInt': const FhirField(false, 'UnsignedInt'),
    'allowedUnsignedIntElement': const FhirField(false, 'Element'),
    'allowedString': const FhirField(false, 'String'),
    'allowedStringElement': const FhirField(false, 'Element'),
    'allowedMoney': const FhirField(false, 'Money'),
    'usedUnsignedInt': const FhirField(false, 'UnsignedInt'),
    'usedUnsignedIntElement': const FhirField(false, 'Element'),
    'usedMoney': const FhirField(false, 'Money'),
  },
  'FamilyMemberHistory': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'instantiatesCanonical': const FhirField(true, 'Canonical'),
    'instantiatesUri': const FhirField(true, 'FhirUri'),
    'instantiatesUriElement': const FhirField(true, 'Element'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'dataAbsentReason': const FhirField(false, 'CodeableConcept'),
    'patient': const FhirField(false, 'Reference'),
    'date': const FhirField(false, 'FhirDateTime'),
    'dateElement': const FhirField(false, 'Element'),
    'participant': const FhirField(true, 'FamilyMemberHistoryParticipant'),
    'name': const FhirField(false, 'String'),
    'nameElement': const FhirField(false, 'Element'),
    'relationship': const FhirField(false, 'CodeableConcept'),
    'sex': const FhirField(false, 'CodeableConcept'),
    'bornPeriod': const FhirField(false, 'Period'),
    'bornDate': const FhirField(false, 'Date'),
    'bornDateElement': const FhirField(false, 'Element'),
    'bornString': const FhirField(false, 'String'),
    'bornStringElement': const FhirField(false, 'Element'),
    'ageAge': const FhirField(false, 'Age'),
    'ageRange': const FhirField(false, 'Range'),
    'ageString': const FhirField(false, 'String'),
    'ageStringElement': const FhirField(false, 'Element'),
    'estimatedAge': const FhirField(false, 'Boolean'),
    'estimatedAgeElement': const FhirField(false, 'Element'),
    'deceasedBoolean': const FhirField(false, 'Boolean'),
    'deceasedBooleanElement': const FhirField(false, 'Element'),
    'deceasedAge': const FhirField(false, 'Age'),
    'deceasedRange': const FhirField(false, 'Range'),
    'deceasedDate': const FhirField(false, 'Date'),
    'deceasedDateElement': const FhirField(false, 'Element'),
    'deceasedString': const FhirField(false, 'String'),
    'deceasedStringElement': const FhirField(false, 'Element'),
    'reason': const FhirField(true, 'CodeableReference'),
    'note': const FhirField(true, 'Annotation'),
    'condition': const FhirField(true, 'FamilyMemberHistoryCondition'),
    'procedure': const FhirField(true, 'FamilyMemberHistoryProcedure'),
  },
  'FamilyMemberHistoryParticipant': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'function': const FhirField(false, 'CodeableConcept'),
    'actor': const FhirField(false, 'Reference'),
  },
  'FamilyMemberHistoryCondition': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(false, 'CodeableConcept'),
    'outcome': const FhirField(false, 'CodeableConcept'),
    'contributedToDeath': const FhirField(false, 'Boolean'),
    'contributedToDeathElement': const FhirField(false, 'Element'),
    'onsetAge': const FhirField(false, 'Age'),
    'onsetRange': const FhirField(false, 'Range'),
    'onsetPeriod': const FhirField(false, 'Period'),
    'onsetString': const FhirField(false, 'String'),
    'onsetStringElement': const FhirField(false, 'Element'),
    'note': const FhirField(true, 'Annotation'),
  },
  'FamilyMemberHistoryProcedure': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(false, 'CodeableConcept'),
    'outcome': const FhirField(false, 'CodeableConcept'),
    'contributedToDeath': const FhirField(false, 'Boolean'),
    'contributedToDeathElement': const FhirField(false, 'Element'),
    'performedAge': const FhirField(false, 'Age'),
    'performedRange': const FhirField(false, 'Range'),
    'performedPeriod': const FhirField(false, 'Period'),
    'performedString': const FhirField(false, 'String'),
    'performedStringElement': const FhirField(false, 'Element'),
    'performedDateTime': const FhirField(false, 'FhirDateTime'),
    'performedDateTimeElement': const FhirField(false, 'Element'),
    'note': const FhirField(true, 'Annotation'),
  },
  'Flag': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'category': const FhirField(true, 'CodeableConcept'),
    'code': const FhirField(false, 'CodeableConcept'),
    'subject': const FhirField(false, 'Reference'),
    'period': const FhirField(false, 'Period'),
    'encounter': const FhirField(false, 'Reference'),
    'author': const FhirField(false, 'Reference'),
  },
  'FormularyItem': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'code': const FhirField(false, 'CodeableConcept'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
  },
  'GenomicStudy': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'type': const FhirField(true, 'CodeableConcept'),
    'subject': const FhirField(false, 'Reference'),
    'encounter': const FhirField(false, 'Reference'),
    'startDate': const FhirField(false, 'FhirDateTime'),
    'startDateElement': const FhirField(false, 'Element'),
    'basedOn': const FhirField(true, 'Reference'),
    'referrer': const FhirField(false, 'Reference'),
    'interpreter': const FhirField(true, 'Reference'),
    'reason': const FhirField(true, 'CodeableReference'),
    'instantiatesCanonical': const FhirField(false, 'Canonical'),
    'instantiatesUri': const FhirField(false, 'FhirUri'),
    'instantiatesUriElement': const FhirField(false, 'Element'),
    'note': const FhirField(true, 'Annotation'),
    'description': const FhirField(false, 'Markdown'),
    'descriptionElement': const FhirField(false, 'Element'),
    'analysis': const FhirField(true, 'GenomicStudyAnalysis'),
  },
  'GenomicStudyAnalysis': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'methodType': const FhirField(true, 'CodeableConcept'),
    'changeType': const FhirField(true, 'CodeableConcept'),
    'genomeBuild': const FhirField(false, 'CodeableConcept'),
    'instantiatesCanonical': const FhirField(false, 'Canonical'),
    'instantiatesUri': const FhirField(false, 'FhirUri'),
    'instantiatesUriElement': const FhirField(false, 'Element'),
    'title': const FhirField(false, 'String'),
    'titleElement': const FhirField(false, 'Element'),
    'focus': const FhirField(true, 'Reference'),
    'specimen': const FhirField(true, 'Reference'),
    'date': const FhirField(false, 'FhirDateTime'),
    'dateElement': const FhirField(false, 'Element'),
    'note': const FhirField(true, 'Annotation'),
    'protocolPerformed': const FhirField(false, 'Reference'),
    'regionsStudied': const FhirField(true, 'Reference'),
    'regionsCalled': const FhirField(true, 'Reference'),
    'input': const FhirField(true, 'GenomicStudyInput'),
    'output': const FhirField(true, 'GenomicStudyOutput'),
    'performer': const FhirField(true, 'GenomicStudyPerformer'),
    'device': const FhirField(true, 'GenomicStudyDevice'),
  },
  'GenomicStudyInput': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'file': const FhirField(false, 'Reference'),
    'type': const FhirField(false, 'CodeableConcept'),
    'generatedByIdentifier': const FhirField(false, 'Identifier'),
    'generatedByReference': const FhirField(false, 'Reference'),
  },
  'GenomicStudyOutput': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'file': const FhirField(false, 'Reference'),
    'type': const FhirField(false, 'CodeableConcept'),
  },
  'GenomicStudyPerformer': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'actor': const FhirField(false, 'Reference'),
    'role': const FhirField(false, 'CodeableConcept'),
  },
  'GenomicStudyDevice': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'device': const FhirField(false, 'Reference'),
    'function': const FhirField(false, 'CodeableConcept'),
  },
  'Goal': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'lifecycleStatus': const FhirField(false, 'Code'),
    'lifecycleStatusElement': const FhirField(false, 'Element'),
    'achievementStatus': const FhirField(false, 'CodeableConcept'),
    'category': const FhirField(true, 'CodeableConcept'),
    'continuous': const FhirField(false, 'Boolean'),
    'continuousElement': const FhirField(false, 'Element'),
    'priority': const FhirField(false, 'CodeableConcept'),
    'description': const FhirField(false, 'CodeableConcept'),
    'subject': const FhirField(false, 'Reference'),
    'startDate': const FhirField(false, 'Date'),
    'startDateElement': const FhirField(false, 'Element'),
    'startCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'target': const FhirField(true, 'GoalTarget'),
    'statusDate': const FhirField(false, 'Date'),
    'statusDateElement': const FhirField(false, 'Element'),
    'statusReason': const FhirField(false, 'String'),
    'statusReasonElement': const FhirField(false, 'Element'),
    'source': const FhirField(false, 'Reference'),
    'addresses': const FhirField(true, 'Reference'),
    'note': const FhirField(true, 'Annotation'),
    'outcome': const FhirField(true, 'CodeableReference'),
  },
  'GoalTarget': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'measure': const FhirField(false, 'CodeableConcept'),
    'detailQuantity': const FhirField(false, 'Quantity'),
    'detailRange': const FhirField(false, 'Range'),
    'detailCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'detailString': const FhirField(false, 'String'),
    'detailStringElement': const FhirField(false, 'Element'),
    'detailBoolean': const FhirField(false, 'Boolean'),
    'detailBooleanElement': const FhirField(false, 'Element'),
    'detailInteger': const FhirField(false, 'Integer'),
    'detailIntegerElement': const FhirField(false, 'Element'),
    'detailRatio': const FhirField(false, 'Ratio'),
    'dueDate': const FhirField(false, 'Date'),
    'dueDateElement': const FhirField(false, 'Element'),
    'dueDuration': const FhirField(false, 'FhirDuration'),
  },
  'GraphDefinition': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'url': const FhirField(false, 'FhirUri'),
    'urlElement': const FhirField(false, 'Element'),
    'identifier': const FhirField(true, 'Identifier'),
    'version': const FhirField(false, 'String'),
    'versionElement': const FhirField(false, 'Element'),
    'versionAlgorithmString': const FhirField(false, 'String'),
    'versionAlgorithmStringElement': const FhirField(false, 'Element'),
    'versionAlgorithmCoding': const FhirField(false, 'Coding'),
    'name': const FhirField(false, 'String'),
    'nameElement': const FhirField(false, 'Element'),
    'title': const FhirField(false, 'String'),
    'titleElement': const FhirField(false, 'Element'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'experimental': const FhirField(false, 'Boolean'),
    'experimentalElement': const FhirField(false, 'Element'),
    'date': const FhirField(false, 'FhirDateTime'),
    'dateElement': const FhirField(false, 'Element'),
    'publisher': const FhirField(false, 'String'),
    'publisherElement': const FhirField(false, 'Element'),
    'contact': const FhirField(true, 'ContactDetail'),
    'description': const FhirField(false, 'Markdown'),
    'descriptionElement': const FhirField(false, 'Element'),
    'useContext': const FhirField(true, 'UsageContext'),
    'jurisdiction': const FhirField(true, 'CodeableConcept'),
    'purpose': const FhirField(false, 'Markdown'),
    'purposeElement': const FhirField(false, 'Element'),
    'copyright': const FhirField(false, 'Markdown'),
    'copyrightElement': const FhirField(false, 'Element'),
    'copyrightLabel': const FhirField(false, 'String'),
    'copyrightLabelElement': const FhirField(false, 'Element'),
    'start': const FhirField(false, 'Id'),
    'startElement': const FhirField(false, 'Element'),
    'node': const FhirField(true, 'GraphDefinitionNode'),
    'link': const FhirField(true, 'GraphDefinitionLink'),
  },
  'GraphDefinitionNode': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'nodeId': const FhirField(false, 'Id'),
    'nodeIdElement': const FhirField(false, 'Element'),
    'description': const FhirField(false, 'String'),
    'descriptionElement': const FhirField(false, 'Element'),
    'type': const FhirField(false, 'Code'),
    'typeElement': const FhirField(false, 'Element'),
    'profile': const FhirField(false, 'Canonical'),
  },
  'GraphDefinitionLink': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'description': const FhirField(false, 'String'),
    'descriptionElement': const FhirField(false, 'Element'),
    'min': const FhirField(false, 'Integer'),
    'minElement': const FhirField(false, 'Element'),
    'max': const FhirField(false, 'String'),
    'maxElement': const FhirField(false, 'Element'),
    'sourceId': const FhirField(false, 'Id'),
    'sourceIdElement': const FhirField(false, 'Element'),
    'path': const FhirField(false, 'String'),
    'pathElement': const FhirField(false, 'Element'),
    'sliceName': const FhirField(false, 'String'),
    'sliceNameElement': const FhirField(false, 'Element'),
    'targetId': const FhirField(false, 'Id'),
    'targetIdElement': const FhirField(false, 'Element'),
    'params': const FhirField(false, 'String'),
    'paramsElement': const FhirField(false, 'Element'),
    'compartment': const FhirField(true, 'GraphDefinitionCompartment'),
  },
  'GraphDefinitionCompartment': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'use': const FhirField(false, 'Code'),
    'useElement': const FhirField(false, 'Element'),
    'rule': const FhirField(false, 'Code'),
    'ruleElement': const FhirField(false, 'Element'),
    'code': const FhirField(false, 'Code'),
    'codeElement': const FhirField(false, 'Element'),
    'expression': const FhirField(false, 'String'),
    'expressionElement': const FhirField(false, 'Element'),
    'description': const FhirField(false, 'String'),
    'descriptionElement': const FhirField(false, 'Element'),
  },
  'Group': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'active': const FhirField(false, 'Boolean'),
    'activeElement': const FhirField(false, 'Element'),
    'type': const FhirField(false, 'Code'),
    'typeElement': const FhirField(false, 'Element'),
    'membership': const FhirField(false, 'Code'),
    'membershipElement': const FhirField(false, 'Element'),
    'code': const FhirField(false, 'CodeableConcept'),
    'name': const FhirField(false, 'String'),
    'nameElement': const FhirField(false, 'Element'),
    'description': const FhirField(false, 'Markdown'),
    'descriptionElement': const FhirField(false, 'Element'),
    'quantity': const FhirField(false, 'UnsignedInt'),
    'quantityElement': const FhirField(false, 'Element'),
    'managingEntity': const FhirField(false, 'Reference'),
    'characteristic': const FhirField(true, 'GroupCharacteristic'),
    'member': const FhirField(true, 'GroupMember'),
  },
  'GroupCharacteristic': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(false, 'CodeableConcept'),
    'valueCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'valueBoolean': const FhirField(false, 'Boolean'),
    'valueBooleanElement': const FhirField(false, 'Element'),
    'valueQuantity': const FhirField(false, 'Quantity'),
    'valueRange': const FhirField(false, 'Range'),
    'valueReference': const FhirField(false, 'Reference'),
    'exclude': const FhirField(false, 'Boolean'),
    'excludeElement': const FhirField(false, 'Element'),
    'period': const FhirField(false, 'Period'),
  },
  'GroupMember': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'entity': const FhirField(false, 'Reference'),
    'period': const FhirField(false, 'Period'),
    'inactive': const FhirField(false, 'Boolean'),
    'inactiveElement': const FhirField(false, 'Element'),
  },
  'GuidanceResponse': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'requestIdentifier': const FhirField(false, 'Identifier'),
    'identifier': const FhirField(true, 'Identifier'),
    'moduleUri': const FhirField(false, 'FhirUri'),
    'moduleUriElement': const FhirField(false, 'Element'),
    'moduleCanonical': const FhirField(false, 'Canonical'),
    'moduleCanonicalElement': const FhirField(false, 'Element'),
    'moduleCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'subject': const FhirField(false, 'Reference'),
    'encounter': const FhirField(false, 'Reference'),
    'occurrenceDateTime': const FhirField(false, 'FhirDateTime'),
    'occurrenceDateTimeElement': const FhirField(false, 'Element'),
    'performer': const FhirField(false, 'Reference'),
    'reason': const FhirField(true, 'CodeableReference'),
    'note': const FhirField(true, 'Annotation'),
    'evaluationMessage': const FhirField(false, 'Reference'),
    'outputParameters': const FhirField(false, 'Reference'),
    'result': const FhirField(true, 'Reference'),
    'dataRequirement': const FhirField(true, 'DataRequirement'),
  },
  'HealthcareService': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'active': const FhirField(false, 'Boolean'),
    'activeElement': const FhirField(false, 'Element'),
    'providedBy': const FhirField(false, 'Reference'),
    'offeredIn': const FhirField(true, 'Reference'),
    'category': const FhirField(true, 'CodeableConcept'),
    'type': const FhirField(true, 'CodeableConcept'),
    'specialty': const FhirField(true, 'CodeableConcept'),
    'location': const FhirField(true, 'Reference'),
    'name': const FhirField(false, 'String'),
    'nameElement': const FhirField(false, 'Element'),
    'comment': const FhirField(false, 'Markdown'),
    'commentElement': const FhirField(false, 'Element'),
    'extraDetails': const FhirField(false, 'Markdown'),
    'extraDetailsElement': const FhirField(false, 'Element'),
    'photo': const FhirField(false, 'Attachment'),
    'contact': const FhirField(true, 'ExtendedContactDetail'),
    'coverageArea': const FhirField(true, 'Reference'),
    'serviceProvisionCode': const FhirField(true, 'CodeableConcept'),
    'eligibility': const FhirField(true, 'HealthcareServiceEligibility'),
    'program': const FhirField(true, 'CodeableConcept'),
    'characteristic': const FhirField(true, 'CodeableConcept'),
    'communication': const FhirField(true, 'CodeableConcept'),
    'referralMethod': const FhirField(true, 'CodeableConcept'),
    'appointmentRequired': const FhirField(false, 'Boolean'),
    'appointmentRequiredElement': const FhirField(false, 'Element'),
    'availability': const FhirField(true, 'Availability'),
    'endpoint': const FhirField(true, 'Reference'),
  },
  'HealthcareServiceEligibility': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(false, 'CodeableConcept'),
    'comment': const FhirField(false, 'Markdown'),
    'commentElement': const FhirField(false, 'Element'),
  },
  'ImagingSelection': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'subject': const FhirField(false, 'Reference'),
    'issued': const FhirField(false, 'Instant'),
    'issuedElement': const FhirField(false, 'Element'),
    'performer': const FhirField(true, 'ImagingSelectionPerformer'),
    'basedOn': const FhirField(true, 'Reference'),
    'category': const FhirField(true, 'CodeableConcept'),
    'code': const FhirField(false, 'CodeableConcept'),
    'studyUid': const FhirField(false, 'Id'),
    'studyUidElement': const FhirField(false, 'Element'),
    'derivedFrom': const FhirField(true, 'Reference'),
    'endpoint': const FhirField(true, 'Reference'),
    'seriesUid': const FhirField(false, 'Id'),
    'seriesUidElement': const FhirField(false, 'Element'),
    'seriesNumber': const FhirField(false, 'UnsignedInt'),
    'seriesNumberElement': const FhirField(false, 'Element'),
    'frameOfReferenceUid': const FhirField(false, 'Id'),
    'frameOfReferenceUidElement': const FhirField(false, 'Element'),
    'bodySite': const FhirField(false, 'CodeableReference'),
    'focus': const FhirField(true, 'Reference'),
    'instance': const FhirField(true, 'ImagingSelectionInstance'),
  },
  'ImagingSelectionPerformer': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'function': const FhirField(false, 'CodeableConcept'),
    'actor': const FhirField(false, 'Reference'),
  },
  'ImagingSelectionInstance': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'uid': const FhirField(false, 'Id'),
    'uidElement': const FhirField(false, 'Element'),
    'number': const FhirField(false, 'UnsignedInt'),
    'numberElement': const FhirField(false, 'Element'),
    'sopClass': const FhirField(false, 'Coding'),
    'subset': const FhirField(true, 'String'),
    'subsetElement': const FhirField(true, 'Element'),
    'imageRegion2D': const FhirField(true, 'ImagingSelectionImageRegion2D'),
    'imageRegion3D': const FhirField(true, 'ImagingSelectionImageRegion3D'),
  },
  'ImagingSelectionImageRegion2D': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'regionType': const FhirField(false, 'Code'),
    'regionTypeElement': const FhirField(false, 'Element'),
    'coordinate': const FhirField(true, 'Decimal'),
    'coordinateElement': const FhirField(true, 'Element'),
  },
  'ImagingSelectionImageRegion3D': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'regionType': const FhirField(false, 'Code'),
    'regionTypeElement': const FhirField(false, 'Element'),
    'coordinate': const FhirField(true, 'Decimal'),
    'coordinateElement': const FhirField(true, 'Element'),
  },
  'ImagingStudy': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'modality': const FhirField(true, 'CodeableConcept'),
    'subject': const FhirField(false, 'Reference'),
    'encounter': const FhirField(false, 'Reference'),
    'started': const FhirField(false, 'FhirDateTime'),
    'startedElement': const FhirField(false, 'Element'),
    'basedOn': const FhirField(true, 'Reference'),
    'partOf': const FhirField(true, 'Reference'),
    'referrer': const FhirField(false, 'Reference'),
    'endpoint': const FhirField(true, 'Reference'),
    'numberOfSeries': const FhirField(false, 'UnsignedInt'),
    'numberOfSeriesElement': const FhirField(false, 'Element'),
    'numberOfInstances': const FhirField(false, 'UnsignedInt'),
    'numberOfInstancesElement': const FhirField(false, 'Element'),
    'procedure': const FhirField(true, 'CodeableReference'),
    'location': const FhirField(false, 'Reference'),
    'reason': const FhirField(true, 'CodeableReference'),
    'note': const FhirField(true, 'Annotation'),
    'description': const FhirField(false, 'String'),
    'descriptionElement': const FhirField(false, 'Element'),
    'series': const FhirField(true, 'ImagingStudySeries'),
  },
  'ImagingStudySeries': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'uid': const FhirField(false, 'Id'),
    'uidElement': const FhirField(false, 'Element'),
    'number': const FhirField(false, 'UnsignedInt'),
    'numberElement': const FhirField(false, 'Element'),
    'modality': const FhirField(false, 'CodeableConcept'),
    'description': const FhirField(false, 'String'),
    'descriptionElement': const FhirField(false, 'Element'),
    'numberOfInstances': const FhirField(false, 'UnsignedInt'),
    'numberOfInstancesElement': const FhirField(false, 'Element'),
    'endpoint': const FhirField(true, 'Reference'),
    'bodySite': const FhirField(false, 'CodeableReference'),
    'laterality': const FhirField(false, 'CodeableConcept'),
    'specimen': const FhirField(true, 'Reference'),
    'started': const FhirField(false, 'FhirDateTime'),
    'startedElement': const FhirField(false, 'Element'),
    'performer': const FhirField(true, 'ImagingStudyPerformer'),
    'instance': const FhirField(true, 'ImagingStudyInstance'),
  },
  'ImagingStudyPerformer': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'function': const FhirField(false, 'CodeableConcept'),
    'actor': const FhirField(false, 'Reference'),
  },
  'ImagingStudyInstance': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'uid': const FhirField(false, 'Id'),
    'uidElement': const FhirField(false, 'Element'),
    'sopClass': const FhirField(false, 'Coding'),
    'number': const FhirField(false, 'UnsignedInt'),
    'numberElement': const FhirField(false, 'Element'),
    'title': const FhirField(false, 'String'),
    'titleElement': const FhirField(false, 'Element'),
  },
  'Immunization': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'basedOn': const FhirField(true, 'Reference'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'statusReason': const FhirField(false, 'CodeableConcept'),
    'vaccineCode': const FhirField(false, 'CodeableConcept'),
    'administeredProduct': const FhirField(false, 'CodeableReference'),
    'manufacturer': const FhirField(false, 'CodeableReference'),
    'lotNumber': const FhirField(false, 'String'),
    'lotNumberElement': const FhirField(false, 'Element'),
    'expirationDate': const FhirField(false, 'Date'),
    'expirationDateElement': const FhirField(false, 'Element'),
    'patient': const FhirField(false, 'Reference'),
    'encounter': const FhirField(false, 'Reference'),
    'supportingInformation': const FhirField(true, 'Reference'),
    'occurrenceDateTime': const FhirField(false, 'FhirDateTime'),
    'occurrenceDateTimeElement': const FhirField(false, 'Element'),
    'occurrenceString': const FhirField(false, 'String'),
    'occurrenceStringElement': const FhirField(false, 'Element'),
    'primarySource': const FhirField(false, 'Boolean'),
    'primarySourceElement': const FhirField(false, 'Element'),
    'informationSource': const FhirField(false, 'CodeableReference'),
    'location': const FhirField(false, 'Reference'),
    'site': const FhirField(false, 'CodeableConcept'),
    'route': const FhirField(false, 'CodeableConcept'),
    'doseQuantity': const FhirField(false, 'Quantity'),
    'performer': const FhirField(true, 'ImmunizationPerformer'),
    'note': const FhirField(true, 'Annotation'),
    'reason': const FhirField(true, 'CodeableReference'),
    'isSubpotent': const FhirField(false, 'Boolean'),
    'isSubpotentElement': const FhirField(false, 'Element'),
    'subpotentReason': const FhirField(true, 'CodeableConcept'),
    'programEligibility':
        const FhirField(true, 'ImmunizationProgramEligibility'),
    'fundingSource': const FhirField(false, 'CodeableConcept'),
    'reaction': const FhirField(true, 'ImmunizationReaction'),
    'protocolApplied': const FhirField(true, 'ImmunizationProtocolApplied'),
  },
  'ImmunizationPerformer': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'function': const FhirField(false, 'CodeableConcept'),
    'actor': const FhirField(false, 'Reference'),
  },
  'ImmunizationProgramEligibility': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'program': const FhirField(false, 'CodeableConcept'),
    'programStatus': const FhirField(false, 'CodeableConcept'),
  },
  'ImmunizationReaction': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'date': const FhirField(false, 'FhirDateTime'),
    'dateElement': const FhirField(false, 'Element'),
    'manifestation': const FhirField(false, 'CodeableReference'),
    'reported': const FhirField(false, 'Boolean'),
    'reportedElement': const FhirField(false, 'Element'),
  },
  'ImmunizationProtocolApplied': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'series': const FhirField(false, 'String'),
    'seriesElement': const FhirField(false, 'Element'),
    'authority': const FhirField(false, 'Reference'),
    'targetDisease': const FhirField(true, 'CodeableConcept'),
    'doseNumber': const FhirField(false, 'String'),
    'doseNumberElement': const FhirField(false, 'Element'),
    'seriesDoses': const FhirField(false, 'String'),
    'seriesDosesElement': const FhirField(false, 'Element'),
  },
  'ImmunizationEvaluation': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'patient': const FhirField(false, 'Reference'),
    'date': const FhirField(false, 'FhirDateTime'),
    'dateElement': const FhirField(false, 'Element'),
    'authority': const FhirField(false, 'Reference'),
    'targetDisease': const FhirField(false, 'CodeableConcept'),
    'immunizationEvent': const FhirField(false, 'Reference'),
    'doseStatus': const FhirField(false, 'CodeableConcept'),
    'doseStatusReason': const FhirField(true, 'CodeableConcept'),
    'description': const FhirField(false, 'Markdown'),
    'descriptionElement': const FhirField(false, 'Element'),
    'series': const FhirField(false, 'String'),
    'seriesElement': const FhirField(false, 'Element'),
    'doseNumber': const FhirField(false, 'String'),
    'doseNumberElement': const FhirField(false, 'Element'),
    'seriesDoses': const FhirField(false, 'String'),
    'seriesDosesElement': const FhirField(false, 'Element'),
  },
  'ImmunizationRecommendation': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'patient': const FhirField(false, 'Reference'),
    'date': const FhirField(false, 'FhirDateTime'),
    'dateElement': const FhirField(false, 'Element'),
    'authority': const FhirField(false, 'Reference'),
    'recommendation':
        const FhirField(true, 'ImmunizationRecommendationRecommendation'),
  },
  'ImmunizationRecommendationRecommendation': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'vaccineCode': const FhirField(true, 'CodeableConcept'),
    'targetDisease': const FhirField(true, 'CodeableConcept'),
    'contraindicatedVaccineCode': const FhirField(true, 'CodeableConcept'),
    'forecastStatus': const FhirField(false, 'CodeableConcept'),
    'forecastReason': const FhirField(true, 'CodeableConcept'),
    'dateCriterion':
        const FhirField(true, 'ImmunizationRecommendationDateCriterion'),
    'description': const FhirField(false, 'Markdown'),
    'descriptionElement': const FhirField(false, 'Element'),
    'series': const FhirField(false, 'String'),
    'seriesElement': const FhirField(false, 'Element'),
    'doseNumber': const FhirField(false, 'String'),
    'doseNumberElement': const FhirField(false, 'Element'),
    'seriesDoses': const FhirField(false, 'String'),
    'seriesDosesElement': const FhirField(false, 'Element'),
    'supportingImmunization': const FhirField(true, 'Reference'),
    'supportingPatientInformation': const FhirField(true, 'Reference'),
  },
  'ImmunizationRecommendationDateCriterion': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(false, 'CodeableConcept'),
    'value': const FhirField(false, 'FhirDateTime'),
    'valueElement': const FhirField(false, 'Element'),
  },
  'ImplementationGuide': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'url': const FhirField(false, 'FhirUri'),
    'urlElement': const FhirField(false, 'Element'),
    'identifier': const FhirField(true, 'Identifier'),
    'version': const FhirField(false, 'String'),
    'versionElement': const FhirField(false, 'Element'),
    'versionAlgorithmString': const FhirField(false, 'String'),
    'versionAlgorithmStringElement': const FhirField(false, 'Element'),
    'versionAlgorithmCoding': const FhirField(false, 'Coding'),
    'name': const FhirField(false, 'String'),
    'nameElement': const FhirField(false, 'Element'),
    'title': const FhirField(false, 'String'),
    'titleElement': const FhirField(false, 'Element'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'experimental': const FhirField(false, 'Boolean'),
    'experimentalElement': const FhirField(false, 'Element'),
    'date': const FhirField(false, 'FhirDateTime'),
    'dateElement': const FhirField(false, 'Element'),
    'publisher': const FhirField(false, 'String'),
    'publisherElement': const FhirField(false, 'Element'),
    'contact': const FhirField(true, 'ContactDetail'),
    'description': const FhirField(false, 'Markdown'),
    'descriptionElement': const FhirField(false, 'Element'),
    'useContext': const FhirField(true, 'UsageContext'),
    'jurisdiction': const FhirField(true, 'CodeableConcept'),
    'purpose': const FhirField(false, 'Markdown'),
    'purposeElement': const FhirField(false, 'Element'),
    'copyright': const FhirField(false, 'Markdown'),
    'copyrightElement': const FhirField(false, 'Element'),
    'copyrightLabel': const FhirField(false, 'String'),
    'copyrightLabelElement': const FhirField(false, 'Element'),
    'packageId': const FhirField(false, 'Id'),
    'packageIdElement': const FhirField(false, 'Element'),
    'license': const FhirField(false, 'Code'),
    'licenseElement': const FhirField(false, 'Element'),
    'fhirVersion': const FhirField(true, 'Code'),
    'fhirVersionElement': const FhirField(true, 'Element'),
    'dependsOn': const FhirField(true, 'ImplementationGuideDependsOn'),
    'global': const FhirField(true, 'ImplementationGuideGlobal'),
    'definition': const FhirField(false, 'ImplementationGuideDefinition'),
    'manifest': const FhirField(false, 'ImplementationGuideManifest'),
  },
  'ImplementationGuideDependsOn': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'uri': const FhirField(false, 'Canonical'),
    'packageId': const FhirField(false, 'Id'),
    'packageIdElement': const FhirField(false, 'Element'),
    'version': const FhirField(false, 'String'),
    'versionElement': const FhirField(false, 'Element'),
    'reason': const FhirField(false, 'Markdown'),
    'reasonElement': const FhirField(false, 'Element'),
  },
  'ImplementationGuideGlobal': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'Code'),
    'typeElement': const FhirField(false, 'Element'),
    'profile': const FhirField(false, 'Canonical'),
  },
  'ImplementationGuideDefinition': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'grouping': const FhirField(true, 'ImplementationGuideGrouping'),
    'resource': const FhirField(true, 'ImplementationGuideResource'),
    'page': const FhirField(false, 'ImplementationGuidePage'),
    'parameter': const FhirField(true, 'ImplementationGuideParameter'),
    'template': const FhirField(true, 'ImplementationGuideTemplate'),
  },
  'ImplementationGuideGrouping': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'name': const FhirField(false, 'String'),
    'nameElement': const FhirField(false, 'Element'),
    'description': const FhirField(false, 'Markdown'),
    'descriptionElement': const FhirField(false, 'Element'),
  },
  'ImplementationGuideResource': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'reference': const FhirField(false, 'Reference'),
    'fhirVersion': const FhirField(true, 'Code'),
    'fhirVersionElement': const FhirField(true, 'Element'),
    'name': const FhirField(false, 'String'),
    'nameElement': const FhirField(false, 'Element'),
    'description': const FhirField(false, 'Markdown'),
    'descriptionElement': const FhirField(false, 'Element'),
    'isExample': const FhirField(false, 'Boolean'),
    'isExampleElement': const FhirField(false, 'Element'),
    'profile': const FhirField(true, 'Canonical'),
    'groupingId': const FhirField(false, 'Id'),
    'groupingIdElement': const FhirField(false, 'Element'),
  },
  'ImplementationGuidePage': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'sourceUrl': const FhirField(false, 'FhirUrl'),
    'sourceUrlElement': const FhirField(false, 'Element'),
    'sourceString': const FhirField(false, 'String'),
    'sourceStringElement': const FhirField(false, 'Element'),
    'sourceMarkdown': const FhirField(false, 'Markdown'),
    'sourceMarkdownElement': const FhirField(false, 'Element'),
    'name': const FhirField(false, 'FhirUrl'),
    'nameElement': const FhirField(false, 'Element'),
    'title': const FhirField(false, 'String'),
    'titleElement': const FhirField(false, 'Element'),
    'generation': const FhirField(false, 'Code'),
    'generationElement': const FhirField(false, 'Element'),
    'page': const FhirField(true, 'ImplementationGuidePage'),
  },
  'ImplementationGuideParameter': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(false, 'Coding'),
    'value': const FhirField(false, 'String'),
    'valueElement': const FhirField(false, 'Element'),
  },
  'ImplementationGuideTemplate': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(false, 'Code'),
    'codeElement': const FhirField(false, 'Element'),
    'source': const FhirField(false, 'String'),
    'sourceElement': const FhirField(false, 'Element'),
    'scope': const FhirField(false, 'String'),
    'scopeElement': const FhirField(false, 'Element'),
  },
  'ImplementationGuideManifest': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'rendering': const FhirField(false, 'FhirUrl'),
    'renderingElement': const FhirField(false, 'Element'),
    'resource': const FhirField(true, 'ImplementationGuideResource1'),
    'page': const FhirField(true, 'ImplementationGuidePage1'),
    'image': const FhirField(true, 'String'),
    'imageElement': const FhirField(true, 'Element'),
    'other': const FhirField(true, 'String'),
    'otherElement': const FhirField(true, 'Element'),
  },
  'ImplementationGuideResource1': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'reference': const FhirField(false, 'Reference'),
    'isExample': const FhirField(false, 'Boolean'),
    'isExampleElement': const FhirField(false, 'Element'),
    'profile': const FhirField(true, 'Canonical'),
    'relativePath': const FhirField(false, 'FhirUrl'),
    'relativePathElement': const FhirField(false, 'Element'),
  },
  'ImplementationGuidePage1': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'name': const FhirField(false, 'String'),
    'nameElement': const FhirField(false, 'Element'),
    'title': const FhirField(false, 'String'),
    'titleElement': const FhirField(false, 'Element'),
    'anchor': const FhirField(true, 'String'),
    'anchorElement': const FhirField(true, 'Element'),
  },
  'Ingredient': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(false, 'Identifier'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'for': const FhirField(true, 'Reference'),
    'role': const FhirField(false, 'CodeableConcept'),
    'function': const FhirField(true, 'CodeableConcept'),
    'group': const FhirField(false, 'CodeableConcept'),
    'allergenicIndicator': const FhirField(false, 'Boolean'),
    'allergenicIndicatorElement': const FhirField(false, 'Element'),
    'comment': const FhirField(false, 'Markdown'),
    'commentElement': const FhirField(false, 'Element'),
    'manufacturer': const FhirField(true, 'IngredientManufacturer'),
    'substance': const FhirField(false, 'IngredientSubstance'),
  },
  'IngredientManufacturer': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'role': const FhirField(false, 'Code'),
    'roleElement': const FhirField(false, 'Element'),
    'manufacturer': const FhirField(false, 'Reference'),
  },
  'IngredientSubstance': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(false, 'CodeableReference'),
    'strength': const FhirField(true, 'IngredientStrength'),
  },
  'IngredientStrength': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'presentationRatio': const FhirField(false, 'Ratio'),
    'presentationRatioRange': const FhirField(false, 'RatioRange'),
    'presentationCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'presentationQuantity': const FhirField(false, 'Quantity'),
    'textPresentation': const FhirField(false, 'String'),
    'textPresentationElement': const FhirField(false, 'Element'),
    'concentrationRatio': const FhirField(false, 'Ratio'),
    'concentrationRatioRange': const FhirField(false, 'RatioRange'),
    'concentrationCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'concentrationQuantity': const FhirField(false, 'Quantity'),
    'textConcentration': const FhirField(false, 'String'),
    'textConcentrationElement': const FhirField(false, 'Element'),
    'basis': const FhirField(false, 'CodeableConcept'),
    'measurementPoint': const FhirField(false, 'String'),
    'measurementPointElement': const FhirField(false, 'Element'),
    'country': const FhirField(true, 'CodeableConcept'),
    'referenceStrength': const FhirField(true, 'IngredientReferenceStrength'),
  },
  'IngredientReferenceStrength': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'substance': const FhirField(false, 'CodeableReference'),
    'strengthRatio': const FhirField(false, 'Ratio'),
    'strengthRatioRange': const FhirField(false, 'RatioRange'),
    'strengthQuantity': const FhirField(false, 'Quantity'),
    'measurementPoint': const FhirField(false, 'String'),
    'measurementPointElement': const FhirField(false, 'Element'),
    'country': const FhirField(true, 'CodeableConcept'),
  },
  'InsurancePlan': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'type': const FhirField(true, 'CodeableConcept'),
    'name': const FhirField(false, 'String'),
    'nameElement': const FhirField(false, 'Element'),
    'alias': const FhirField(true, 'String'),
    'aliasElement': const FhirField(true, 'Element'),
    'period': const FhirField(false, 'Period'),
    'ownedBy': const FhirField(false, 'Reference'),
    'administeredBy': const FhirField(false, 'Reference'),
    'coverageArea': const FhirField(true, 'Reference'),
    'contact': const FhirField(true, 'ExtendedContactDetail'),
    'endpoint': const FhirField(true, 'Reference'),
    'network': const FhirField(true, 'Reference'),
    'coverage': const FhirField(true, 'InsurancePlanCoverage'),
    'plan': const FhirField(true, 'InsurancePlanPlan'),
  },
  'InsurancePlanCoverage': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'network': const FhirField(true, 'Reference'),
    'benefit': const FhirField(true, 'InsurancePlanBenefit'),
  },
  'InsurancePlanBenefit': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'requirement': const FhirField(false, 'String'),
    'requirementElement': const FhirField(false, 'Element'),
    'limit': const FhirField(true, 'InsurancePlanLimit'),
  },
  'InsurancePlanLimit': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'value': const FhirField(false, 'Quantity'),
    'code': const FhirField(false, 'CodeableConcept'),
  },
  'InsurancePlanPlan': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'type': const FhirField(false, 'CodeableConcept'),
    'coverageArea': const FhirField(true, 'Reference'),
    'network': const FhirField(true, 'Reference'),
    'generalCost': const FhirField(true, 'InsurancePlanGeneralCost'),
    'specificCost': const FhirField(true, 'InsurancePlanSpecificCost'),
  },
  'InsurancePlanGeneralCost': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'groupSize': const FhirField(false, 'PositiveInt'),
    'groupSizeElement': const FhirField(false, 'Element'),
    'cost': const FhirField(false, 'Money'),
    'comment': const FhirField(false, 'String'),
    'commentElement': const FhirField(false, 'Element'),
  },
  'InsurancePlanSpecificCost': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'category': const FhirField(false, 'CodeableConcept'),
    'benefit': const FhirField(true, 'InsurancePlanBenefit1'),
  },
  'InsurancePlanBenefit1': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'cost': const FhirField(true, 'InsurancePlanCost'),
  },
  'InsurancePlanCost': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'applicability': const FhirField(false, 'CodeableConcept'),
    'qualifiers': const FhirField(true, 'CodeableConcept'),
    'value': const FhirField(false, 'Quantity'),
  },
  'InventoryItem': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'category': const FhirField(true, 'CodeableConcept'),
    'code': const FhirField(true, 'CodeableConcept'),
    'name': const FhirField(true, 'InventoryItemName'),
    'responsibleOrganization':
        const FhirField(true, 'InventoryItemResponsibleOrganization'),
    'description': const FhirField(false, 'InventoryItemDescription'),
    'inventoryStatus': const FhirField(true, 'CodeableConcept'),
    'baseUnit': const FhirField(false, 'CodeableConcept'),
    'netContent': const FhirField(false, 'Quantity'),
    'association': const FhirField(true, 'InventoryItemAssociation'),
    'characteristic': const FhirField(true, 'InventoryItemCharacteristic'),
    'instance': const FhirField(false, 'InventoryItemInstance'),
    'productReference': const FhirField(false, 'Reference'),
  },
  'InventoryItemName': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'nameType': const FhirField(false, 'Coding'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'name': const FhirField(false, 'String'),
    'nameElement': const FhirField(false, 'Element'),
  },
  'InventoryItemResponsibleOrganization': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'role': const FhirField(false, 'CodeableConcept'),
    'organization': const FhirField(false, 'Reference'),
  },
  'InventoryItemDescription': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'description': const FhirField(false, 'String'),
    'descriptionElement': const FhirField(false, 'Element'),
  },
  'InventoryItemAssociation': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'associationType': const FhirField(false, 'CodeableConcept'),
    'relatedItem': const FhirField(false, 'Reference'),
    'quantity': const FhirField(false, 'Ratio'),
  },
  'InventoryItemCharacteristic': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'characteristicType': const FhirField(false, 'CodeableConcept'),
    'valueString': const FhirField(false, 'String'),
    'valueStringElement': const FhirField(false, 'Element'),
    'valueInteger': const FhirField(false, 'Integer'),
    'valueIntegerElement': const FhirField(false, 'Element'),
    'valueDecimal': const FhirField(false, 'Decimal'),
    'valueDecimalElement': const FhirField(false, 'Element'),
    'valueBoolean': const FhirField(false, 'Boolean'),
    'valueBooleanElement': const FhirField(false, 'Element'),
    'valueUrl': const FhirField(false, 'FhirUrl'),
    'valueUrlElement': const FhirField(false, 'Element'),
    'valueDateTime': const FhirField(false, 'FhirDateTime'),
    'valueDateTimeElement': const FhirField(false, 'Element'),
    'valueQuantity': const FhirField(false, 'Quantity'),
    'valueRange': const FhirField(false, 'Range'),
    'valueRatio': const FhirField(false, 'Ratio'),
    'valueAnnotation': const FhirField(false, 'Annotation'),
    'valueAddress': const FhirField(false, 'Address'),
    'valueDuration': const FhirField(false, 'FhirDuration'),
    'valueCodeableConcept': const FhirField(false, 'CodeableConcept'),
  },
  'InventoryItemInstance': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'lotNumber': const FhirField(false, 'String'),
    'lotNumberElement': const FhirField(false, 'Element'),
    'expiry': const FhirField(false, 'FhirDateTime'),
    'expiryElement': const FhirField(false, 'Element'),
    'subject': const FhirField(false, 'Reference'),
    'location': const FhirField(false, 'Reference'),
  },
  'InventoryReport': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'countType': const FhirField(false, 'Code'),
    'countTypeElement': const FhirField(false, 'Element'),
    'operationType': const FhirField(false, 'CodeableConcept'),
    'operationTypeReason': const FhirField(false, 'CodeableConcept'),
    'reportedDateTime': const FhirField(false, 'FhirDateTime'),
    'reportedDateTimeElement': const FhirField(false, 'Element'),
    'reporter': const FhirField(false, 'Reference'),
    'reportingPeriod': const FhirField(false, 'Period'),
    'inventoryListing':
        const FhirField(true, 'InventoryReportInventoryListing'),
    'note': const FhirField(true, 'Annotation'),
  },
  'InventoryReportInventoryListing': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'location': const FhirField(false, 'Reference'),
    'itemStatus': const FhirField(false, 'CodeableConcept'),
    'countingDateTime': const FhirField(false, 'FhirDateTime'),
    'countingDateTimeElement': const FhirField(false, 'Element'),
    'item': const FhirField(true, 'InventoryReportItem'),
  },
  'InventoryReportItem': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'category': const FhirField(false, 'CodeableConcept'),
    'quantity': const FhirField(false, 'Quantity'),
    'item': const FhirField(false, 'CodeableReference'),
  },
  'Invoice': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'cancelledReason': const FhirField(false, 'String'),
    'cancelledReasonElement': const FhirField(false, 'Element'),
    'type': const FhirField(false, 'CodeableConcept'),
    'subject': const FhirField(false, 'Reference'),
    'recipient': const FhirField(false, 'Reference'),
    'date': const FhirField(false, 'FhirDateTime'),
    'dateElement': const FhirField(false, 'Element'),
    'creation': const FhirField(false, 'FhirDateTime'),
    'creationElement': const FhirField(false, 'Element'),
    'periodDate': const FhirField(false, 'Date'),
    'periodDateElement': const FhirField(false, 'Element'),
    'periodPeriod': const FhirField(false, 'Period'),
    'participant': const FhirField(true, 'InvoiceParticipant'),
    'issuer': const FhirField(false, 'Reference'),
    'account': const FhirField(false, 'Reference'),
    'lineItem': const FhirField(true, 'InvoiceLineItem'),
    'totalPriceComponent': const FhirField(true, 'MonetaryComponent'),
    'totalNet': const FhirField(false, 'Money'),
    'totalGross': const FhirField(false, 'Money'),
    'paymentTerms': const FhirField(false, 'Markdown'),
    'paymentTermsElement': const FhirField(false, 'Element'),
    'note': const FhirField(true, 'Annotation'),
  },
  'InvoiceParticipant': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'role': const FhirField(false, 'CodeableConcept'),
    'actor': const FhirField(false, 'Reference'),
  },
  'InvoiceLineItem': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'sequence': const FhirField(false, 'PositiveInt'),
    'sequenceElement': const FhirField(false, 'Element'),
    'servicedDate': const FhirField(false, 'Date'),
    'servicedDateElement': const FhirField(false, 'Element'),
    'servicedPeriod': const FhirField(false, 'Period'),
    'chargeItemReference': const FhirField(false, 'Reference'),
    'chargeItemCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'priceComponent': const FhirField(true, 'MonetaryComponent'),
  },
  'Library': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'url': const FhirField(false, 'FhirUri'),
    'urlElement': const FhirField(false, 'Element'),
    'identifier': const FhirField(true, 'Identifier'),
    'version': const FhirField(false, 'String'),
    'versionElement': const FhirField(false, 'Element'),
    'versionAlgorithmString': const FhirField(false, 'String'),
    'versionAlgorithmStringElement': const FhirField(false, 'Element'),
    'versionAlgorithmCoding': const FhirField(false, 'Coding'),
    'name': const FhirField(false, 'String'),
    'nameElement': const FhirField(false, 'Element'),
    'title': const FhirField(false, 'String'),
    'titleElement': const FhirField(false, 'Element'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'experimental': const FhirField(false, 'Boolean'),
    'experimentalElement': const FhirField(false, 'Element'),
    'date': const FhirField(false, 'FhirDateTime'),
    'dateElement': const FhirField(false, 'Element'),
    'publisher': const FhirField(false, 'String'),
    'publisherElement': const FhirField(false, 'Element'),
    'contact': const FhirField(true, 'ContactDetail'),
    'description': const FhirField(false, 'Markdown'),
    'descriptionElement': const FhirField(false, 'Element'),
    'useContext': const FhirField(true, 'UsageContext'),
    'jurisdiction': const FhirField(true, 'CodeableConcept'),
    'purpose': const FhirField(false, 'Markdown'),
    'purposeElement': const FhirField(false, 'Element'),
    'copyright': const FhirField(false, 'Markdown'),
    'copyrightElement': const FhirField(false, 'Element'),
    'copyrightLabel': const FhirField(false, 'String'),
    'copyrightLabelElement': const FhirField(false, 'Element'),
    'approvalDate': const FhirField(false, 'Date'),
    'approvalDateElement': const FhirField(false, 'Element'),
    'lastReviewDate': const FhirField(false, 'Date'),
    'lastReviewDateElement': const FhirField(false, 'Element'),
    'effectivePeriod': const FhirField(false, 'Period'),
    'topic': const FhirField(true, 'CodeableConcept'),
    'author': const FhirField(true, 'ContactDetail'),
    'editor': const FhirField(true, 'ContactDetail'),
    'reviewer': const FhirField(true, 'ContactDetail'),
    'endorser': const FhirField(true, 'ContactDetail'),
    'relatedArtifact': const FhirField(true, 'RelatedArtifact'),
    'subtitle': const FhirField(false, 'String'),
    'subtitleElement': const FhirField(false, 'Element'),
    'type': const FhirField(false, 'CodeableConcept'),
    'subjectCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'subjectReference': const FhirField(false, 'Reference'),
    'usage': const FhirField(false, 'Markdown'),
    'usageElement': const FhirField(false, 'Element'),
    'parameter': const FhirField(true, 'ParameterDefinition'),
    'dataRequirement': const FhirField(true, 'DataRequirement'),
    'content': const FhirField(true, 'Attachment'),
  },
  'Linkage': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'active': const FhirField(false, 'Boolean'),
    'activeElement': const FhirField(false, 'Element'),
    'author': const FhirField(false, 'Reference'),
    'item': const FhirField(true, 'LinkageItem'),
  },
  'LinkageItem': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'Code'),
    'typeElement': const FhirField(false, 'Element'),
    'resource': const FhirField(false, 'Reference'),
  },
  'List': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'mode': const FhirField(false, 'Code'),
    'modeElement': const FhirField(false, 'Element'),
    'title': const FhirField(false, 'String'),
    'titleElement': const FhirField(false, 'Element'),
    'code': const FhirField(false, 'CodeableConcept'),
    'subject': const FhirField(true, 'Reference'),
    'encounter': const FhirField(false, 'Reference'),
    'date': const FhirField(false, 'FhirDateTime'),
    'dateElement': const FhirField(false, 'Element'),
    'source': const FhirField(false, 'Reference'),
    'orderedBy': const FhirField(false, 'CodeableConcept'),
    'note': const FhirField(true, 'Annotation'),
    'entry': const FhirField(true, 'FhirListEntry'),
    'emptyReason': const FhirField(false, 'CodeableConcept'),
  },
  'FhirListEntry': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'flag': const FhirField(false, 'CodeableConcept'),
    'deleted': const FhirField(false, 'Boolean'),
    'deletedElement': const FhirField(false, 'Element'),
    'date': const FhirField(false, 'FhirDateTime'),
    'dateElement': const FhirField(false, 'Element'),
    'item': const FhirField(false, 'Reference'),
  },
  'Location': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'operationalStatus': const FhirField(false, 'Coding'),
    'name': const FhirField(false, 'String'),
    'nameElement': const FhirField(false, 'Element'),
    'alias': const FhirField(true, 'String'),
    'aliasElement': const FhirField(true, 'Element'),
    'description': const FhirField(false, 'Markdown'),
    'descriptionElement': const FhirField(false, 'Element'),
    'mode': const FhirField(false, 'Code'),
    'modeElement': const FhirField(false, 'Element'),
    'type': const FhirField(true, 'CodeableConcept'),
    'contact': const FhirField(true, 'ExtendedContactDetail'),
    'address': const FhirField(false, 'Address'),
    'form': const FhirField(false, 'CodeableConcept'),
    'position': const FhirField(false, 'LocationPosition'),
    'managingOrganization': const FhirField(false, 'Reference'),
    'partOf': const FhirField(false, 'Reference'),
    'characteristic': const FhirField(true, 'CodeableConcept'),
    'hoursOfOperation': const FhirField(true, 'Availability'),
    'virtualService': const FhirField(true, 'VirtualServiceDetail'),
    'endpoint': const FhirField(true, 'Reference'),
  },
  'LocationPosition': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'longitude': const FhirField(false, 'Decimal'),
    'longitudeElement': const FhirField(false, 'Element'),
    'latitude': const FhirField(false, 'Decimal'),
    'latitudeElement': const FhirField(false, 'Element'),
    'altitude': const FhirField(false, 'Decimal'),
    'altitudeElement': const FhirField(false, 'Element'),
  },
  'ManufacturedItemDefinition': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'name': const FhirField(false, 'String'),
    'nameElement': const FhirField(false, 'Element'),
    'manufacturedDoseForm': const FhirField(false, 'CodeableConcept'),
    'unitOfPresentation': const FhirField(false, 'CodeableConcept'),
    'manufacturer': const FhirField(true, 'Reference'),
    'marketingStatus': const FhirField(true, 'MarketingStatus'),
    'ingredient': const FhirField(true, 'CodeableConcept'),
    'property': const FhirField(true, 'ManufacturedItemDefinitionProperty'),
    'component': const FhirField(true, 'ManufacturedItemDefinitionComponent'),
  },
  'ManufacturedItemDefinitionProperty': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'valueCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'valueQuantity': const FhirField(false, 'Quantity'),
    'valueDate': const FhirField(false, 'Date'),
    'valueDateElement': const FhirField(false, 'Element'),
    'valueBoolean': const FhirField(false, 'Boolean'),
    'valueBooleanElement': const FhirField(false, 'Element'),
    'valueMarkdown': const FhirField(false, 'Markdown'),
    'valueMarkdownElement': const FhirField(false, 'Element'),
    'valueAttachment': const FhirField(false, 'Attachment'),
    'valueReference': const FhirField(false, 'Reference'),
  },
  'ManufacturedItemDefinitionComponent': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'function': const FhirField(true, 'CodeableConcept'),
    'amount': const FhirField(true, 'Quantity'),
    'constituent':
        const FhirField(true, 'ManufacturedItemDefinitionConstituent'),
    'property': const FhirField(true, 'ManufacturedItemDefinitionProperty'),
    'component': const FhirField(true, 'ManufacturedItemDefinitionComponent'),
  },
  'ManufacturedItemDefinitionConstituent': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'amount': const FhirField(true, 'Quantity'),
    'location': const FhirField(true, 'CodeableConcept'),
    'function': const FhirField(true, 'CodeableConcept'),
    'hasIngredient': const FhirField(true, 'CodeableReference'),
  },
  'Measure': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'url': const FhirField(false, 'FhirUri'),
    'urlElement': const FhirField(false, 'Element'),
    'identifier': const FhirField(true, 'Identifier'),
    'version': const FhirField(false, 'String'),
    'versionElement': const FhirField(false, 'Element'),
    'versionAlgorithmString': const FhirField(false, 'String'),
    'versionAlgorithmStringElement': const FhirField(false, 'Element'),
    'versionAlgorithmCoding': const FhirField(false, 'Coding'),
    'name': const FhirField(false, 'String'),
    'nameElement': const FhirField(false, 'Element'),
    'title': const FhirField(false, 'String'),
    'titleElement': const FhirField(false, 'Element'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'experimental': const FhirField(false, 'Boolean'),
    'experimentalElement': const FhirField(false, 'Element'),
    'date': const FhirField(false, 'FhirDateTime'),
    'dateElement': const FhirField(false, 'Element'),
    'publisher': const FhirField(false, 'String'),
    'publisherElement': const FhirField(false, 'Element'),
    'contact': const FhirField(true, 'ContactDetail'),
    'description': const FhirField(false, 'Markdown'),
    'descriptionElement': const FhirField(false, 'Element'),
    'useContext': const FhirField(true, 'UsageContext'),
    'jurisdiction': const FhirField(true, 'CodeableConcept'),
    'purpose': const FhirField(false, 'Markdown'),
    'purposeElement': const FhirField(false, 'Element'),
    'copyright': const FhirField(false, 'Markdown'),
    'copyrightElement': const FhirField(false, 'Element'),
    'copyrightLabel': const FhirField(false, 'String'),
    'copyrightLabelElement': const FhirField(false, 'Element'),
    'approvalDate': const FhirField(false, 'Date'),
    'approvalDateElement': const FhirField(false, 'Element'),
    'lastReviewDate': const FhirField(false, 'Date'),
    'lastReviewDateElement': const FhirField(false, 'Element'),
    'effectivePeriod': const FhirField(false, 'Period'),
    'topic': const FhirField(true, 'CodeableConcept'),
    'author': const FhirField(true, 'ContactDetail'),
    'editor': const FhirField(true, 'ContactDetail'),
    'reviewer': const FhirField(true, 'ContactDetail'),
    'endorser': const FhirField(true, 'ContactDetail'),
    'relatedArtifact': const FhirField(true, 'RelatedArtifact'),
    'subtitle': const FhirField(false, 'String'),
    'subtitleElement': const FhirField(false, 'Element'),
    'subjectCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'subjectReference': const FhirField(false, 'Reference'),
    'basis': const FhirField(false, 'Code'),
    'basisElement': const FhirField(false, 'Element'),
    'usage': const FhirField(false, 'Markdown'),
    'usageElement': const FhirField(false, 'Element'),
    'library': const FhirField(true, 'Canonical'),
    'disclaimer': const FhirField(false, 'Markdown'),
    'disclaimerElement': const FhirField(false, 'Element'),
    'scoring': const FhirField(false, 'CodeableConcept'),
    'scoringUnit': const FhirField(false, 'CodeableConcept'),
    'compositeScoring': const FhirField(false, 'CodeableConcept'),
    'type': const FhirField(true, 'CodeableConcept'),
    'riskAdjustment': const FhirField(false, 'Markdown'),
    'riskAdjustmentElement': const FhirField(false, 'Element'),
    'rateAggregation': const FhirField(false, 'Markdown'),
    'rateAggregationElement': const FhirField(false, 'Element'),
    'rationale': const FhirField(false, 'Markdown'),
    'rationaleElement': const FhirField(false, 'Element'),
    'clinicalRecommendationStatement': const FhirField(false, 'Markdown'),
    'clinicalRecommendationStatementElement': const FhirField(false, 'Element'),
    'improvementNotation': const FhirField(false, 'CodeableConcept'),
    'term': const FhirField(true, 'MeasureTerm'),
    'guidance': const FhirField(false, 'Markdown'),
    'guidanceElement': const FhirField(false, 'Element'),
    'group': const FhirField(true, 'MeasureGroup'),
    'supplementalData': const FhirField(true, 'MeasureSupplementalData'),
  },
  'MeasureTerm': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(false, 'CodeableConcept'),
    'definition': const FhirField(false, 'Markdown'),
    'definitionElement': const FhirField(false, 'Element'),
  },
  'MeasureGroup': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'linkId': const FhirField(false, 'String'),
    'linkIdElement': const FhirField(false, 'Element'),
    'code': const FhirField(false, 'CodeableConcept'),
    'description': const FhirField(false, 'Markdown'),
    'descriptionElement': const FhirField(false, 'Element'),
    'type': const FhirField(true, 'CodeableConcept'),
    'subjectCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'subjectReference': const FhirField(false, 'Reference'),
    'basis': const FhirField(false, 'Code'),
    'basisElement': const FhirField(false, 'Element'),
    'scoring': const FhirField(false, 'CodeableConcept'),
    'scoringUnit': const FhirField(false, 'CodeableConcept'),
    'rateAggregation': const FhirField(false, 'Markdown'),
    'rateAggregationElement': const FhirField(false, 'Element'),
    'improvementNotation': const FhirField(false, 'CodeableConcept'),
    'library': const FhirField(true, 'Canonical'),
    'population': const FhirField(true, 'MeasurePopulation'),
    'stratifier': const FhirField(true, 'MeasureStratifier'),
  },
  'MeasurePopulation': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'linkId': const FhirField(false, 'String'),
    'linkIdElement': const FhirField(false, 'Element'),
    'code': const FhirField(false, 'CodeableConcept'),
    'description': const FhirField(false, 'Markdown'),
    'descriptionElement': const FhirField(false, 'Element'),
    'criteria': const FhirField(false, 'Expression'),
    'groupDefinition': const FhirField(false, 'Reference'),
    'inputPopulationId': const FhirField(false, 'String'),
    'inputPopulationIdElement': const FhirField(false, 'Element'),
    'aggregateMethod': const FhirField(false, 'CodeableConcept'),
  },
  'MeasureStratifier': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'linkId': const FhirField(false, 'String'),
    'linkIdElement': const FhirField(false, 'Element'),
    'code': const FhirField(false, 'CodeableConcept'),
    'description': const FhirField(false, 'Markdown'),
    'descriptionElement': const FhirField(false, 'Element'),
    'criteria': const FhirField(false, 'Expression'),
    'groupDefinition': const FhirField(false, 'Reference'),
    'component': const FhirField(true, 'MeasureComponent'),
  },
  'MeasureComponent': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'linkId': const FhirField(false, 'String'),
    'linkIdElement': const FhirField(false, 'Element'),
    'code': const FhirField(false, 'CodeableConcept'),
    'description': const FhirField(false, 'Markdown'),
    'descriptionElement': const FhirField(false, 'Element'),
    'criteria': const FhirField(false, 'Expression'),
    'groupDefinition': const FhirField(false, 'Reference'),
  },
  'MeasureSupplementalData': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'linkId': const FhirField(false, 'String'),
    'linkIdElement': const FhirField(false, 'Element'),
    'code': const FhirField(false, 'CodeableConcept'),
    'usage': const FhirField(true, 'CodeableConcept'),
    'description': const FhirField(false, 'Markdown'),
    'descriptionElement': const FhirField(false, 'Element'),
    'criteria': const FhirField(false, 'Expression'),
  },
  'MeasureReport': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'type': const FhirField(false, 'Code'),
    'typeElement': const FhirField(false, 'Element'),
    'dataUpdateType': const FhirField(false, 'Code'),
    'dataUpdateTypeElement': const FhirField(false, 'Element'),
    'measure': const FhirField(false, 'Canonical'),
    'subject': const FhirField(false, 'Reference'),
    'date': const FhirField(false, 'FhirDateTime'),
    'dateElement': const FhirField(false, 'Element'),
    'reporter': const FhirField(false, 'Reference'),
    'reportingVendor': const FhirField(false, 'Reference'),
    'location': const FhirField(false, 'Reference'),
    'period': const FhirField(false, 'Period'),
    'inputParameters': const FhirField(false, 'Reference'),
    'scoring': const FhirField(false, 'CodeableConcept'),
    'improvementNotation': const FhirField(false, 'CodeableConcept'),
    'group': const FhirField(true, 'MeasureReportGroup'),
    'supplementalData': const FhirField(true, 'Reference'),
    'evaluatedResource': const FhirField(true, 'Reference'),
  },
  'MeasureReportGroup': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'linkId': const FhirField(false, 'String'),
    'linkIdElement': const FhirField(false, 'Element'),
    'code': const FhirField(false, 'CodeableConcept'),
    'subject': const FhirField(false, 'Reference'),
    'population': const FhirField(true, 'MeasureReportPopulation'),
    'measureScoreQuantity': const FhirField(false, 'Quantity'),
    'measureScoreDateTime': const FhirField(false, 'FhirDateTime'),
    'measureScoreDateTimeElement': const FhirField(false, 'Element'),
    'measureScoreCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'measureScorePeriod': const FhirField(false, 'Period'),
    'measureScoreRange': const FhirField(false, 'Range'),
    'measureScoreDuration': const FhirField(false, 'FhirDuration'),
    'stratifier': const FhirField(true, 'MeasureReportStratifier'),
  },
  'MeasureReportPopulation': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'linkId': const FhirField(false, 'String'),
    'linkIdElement': const FhirField(false, 'Element'),
    'code': const FhirField(false, 'CodeableConcept'),
    'count': const FhirField(false, 'Integer'),
    'countElement': const FhirField(false, 'Element'),
    'subjectResults': const FhirField(false, 'Reference'),
    'subjectReport': const FhirField(true, 'Reference'),
    'subjects': const FhirField(false, 'Reference'),
  },
  'MeasureReportStratifier': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'linkId': const FhirField(false, 'String'),
    'linkIdElement': const FhirField(false, 'Element'),
    'code': const FhirField(false, 'CodeableConcept'),
    'stratum': const FhirField(true, 'MeasureReportStratum'),
  },
  'MeasureReportStratum': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'valueCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'valueBoolean': const FhirField(false, 'Boolean'),
    'valueBooleanElement': const FhirField(false, 'Element'),
    'valueQuantity': const FhirField(false, 'Quantity'),
    'valueRange': const FhirField(false, 'Range'),
    'valueReference': const FhirField(false, 'Reference'),
    'component': const FhirField(true, 'MeasureReportComponent'),
    'population': const FhirField(true, 'MeasureReportPopulation1'),
    'measureScoreQuantity': const FhirField(false, 'Quantity'),
    'measureScoreDateTime': const FhirField(false, 'FhirDateTime'),
    'measureScoreDateTimeElement': const FhirField(false, 'Element'),
    'measureScoreCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'measureScorePeriod': const FhirField(false, 'Period'),
    'measureScoreRange': const FhirField(false, 'Range'),
    'measureScoreDuration': const FhirField(false, 'FhirDuration'),
  },
  'MeasureReportComponent': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'linkId': const FhirField(false, 'String'),
    'linkIdElement': const FhirField(false, 'Element'),
    'code': const FhirField(false, 'CodeableConcept'),
    'valueCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'valueBoolean': const FhirField(false, 'Boolean'),
    'valueBooleanElement': const FhirField(false, 'Element'),
    'valueQuantity': const FhirField(false, 'Quantity'),
    'valueRange': const FhirField(false, 'Range'),
    'valueReference': const FhirField(false, 'Reference'),
  },
  'MeasureReportPopulation1': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'linkId': const FhirField(false, 'String'),
    'linkIdElement': const FhirField(false, 'Element'),
    'code': const FhirField(false, 'CodeableConcept'),
    'count': const FhirField(false, 'Integer'),
    'countElement': const FhirField(false, 'Element'),
    'subjectResults': const FhirField(false, 'Reference'),
    'subjectReport': const FhirField(true, 'Reference'),
    'subjects': const FhirField(false, 'Reference'),
  },
  'Medication': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'code': const FhirField(false, 'CodeableConcept'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'marketingAuthorizationHolder': const FhirField(false, 'Reference'),
    'doseForm': const FhirField(false, 'CodeableConcept'),
    'totalVolume': const FhirField(false, 'Quantity'),
    'ingredient': const FhirField(true, 'MedicationIngredient'),
    'batch': const FhirField(false, 'MedicationBatch'),
    'definition': const FhirField(false, 'Reference'),
  },
  'MedicationIngredient': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'item': const FhirField(false, 'CodeableReference'),
    'isActive': const FhirField(false, 'Boolean'),
    'isActiveElement': const FhirField(false, 'Element'),
    'strengthRatio': const FhirField(false, 'Ratio'),
    'strengthCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'strengthQuantity': const FhirField(false, 'Quantity'),
  },
  'MedicationBatch': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'lotNumber': const FhirField(false, 'String'),
    'lotNumberElement': const FhirField(false, 'Element'),
    'expirationDate': const FhirField(false, 'FhirDateTime'),
    'expirationDateElement': const FhirField(false, 'Element'),
  },
  'MedicationAdministration': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'basedOn': const FhirField(true, 'Reference'),
    'partOf': const FhirField(true, 'Reference'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'statusReason': const FhirField(true, 'CodeableConcept'),
    'category': const FhirField(true, 'CodeableConcept'),
    'medication': const FhirField(false, 'CodeableReference'),
    'subject': const FhirField(false, 'Reference'),
    'encounter': const FhirField(false, 'Reference'),
    'supportingInformation': const FhirField(true, 'Reference'),
    'occurenceDateTime': const FhirField(false, 'FhirDateTime'),
    'occurenceDateTimeElement': const FhirField(false, 'Element'),
    'occurencePeriod': const FhirField(false, 'Period'),
    'occurenceTiming': const FhirField(false, 'Timing'),
    'recorded': const FhirField(false, 'FhirDateTime'),
    'recordedElement': const FhirField(false, 'Element'),
    'isSubPotent': const FhirField(false, 'Boolean'),
    'isSubPotentElement': const FhirField(false, 'Element'),
    'subPotentReason': const FhirField(true, 'CodeableConcept'),
    'performer': const FhirField(true, 'MedicationAdministrationPerformer'),
    'reason': const FhirField(true, 'CodeableReference'),
    'request': const FhirField(false, 'Reference'),
    'device': const FhirField(true, 'CodeableReference'),
    'note': const FhirField(true, 'Annotation'),
    'dosage': const FhirField(false, 'MedicationAdministrationDosage'),
    'eventHistory': const FhirField(true, 'Reference'),
  },
  'MedicationAdministrationPerformer': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'function': const FhirField(false, 'CodeableConcept'),
    'actor': const FhirField(false, 'CodeableReference'),
  },
  'MedicationAdministrationDosage': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'text': const FhirField(false, 'String'),
    'textElement': const FhirField(false, 'Element'),
    'site': const FhirField(false, 'CodeableConcept'),
    'route': const FhirField(false, 'CodeableConcept'),
    'method': const FhirField(false, 'CodeableConcept'),
    'dose': const FhirField(false, 'Quantity'),
    'rateRatio': const FhirField(false, 'Ratio'),
    'rateQuantity': const FhirField(false, 'Quantity'),
  },
  'MedicationDispense': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'basedOn': const FhirField(true, 'Reference'),
    'partOf': const FhirField(true, 'Reference'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'notPerformedReason': const FhirField(false, 'CodeableReference'),
    'statusChanged': const FhirField(false, 'FhirDateTime'),
    'statusChangedElement': const FhirField(false, 'Element'),
    'category': const FhirField(true, 'CodeableConcept'),
    'medication': const FhirField(false, 'CodeableReference'),
    'subject': const FhirField(false, 'Reference'),
    'encounter': const FhirField(false, 'Reference'),
    'supportingInformation': const FhirField(true, 'Reference'),
    'performer': const FhirField(true, 'MedicationDispensePerformer'),
    'location': const FhirField(false, 'Reference'),
    'authorizingPrescription': const FhirField(true, 'Reference'),
    'type': const FhirField(false, 'CodeableConcept'),
    'quantity': const FhirField(false, 'Quantity'),
    'daysSupply': const FhirField(false, 'Quantity'),
    'recorded': const FhirField(false, 'FhirDateTime'),
    'recordedElement': const FhirField(false, 'Element'),
    'whenPrepared': const FhirField(false, 'FhirDateTime'),
    'whenPreparedElement': const FhirField(false, 'Element'),
    'whenHandedOver': const FhirField(false, 'FhirDateTime'),
    'whenHandedOverElement': const FhirField(false, 'Element'),
    'destination': const FhirField(false, 'Reference'),
    'receiver': const FhirField(true, 'Reference'),
    'note': const FhirField(true, 'Annotation'),
    'renderedDosageInstruction': const FhirField(false, 'Markdown'),
    'renderedDosageInstructionElement': const FhirField(false, 'Element'),
    'dosageInstruction': const FhirField(true, 'Dosage'),
    'substitution': const FhirField(false, 'MedicationDispenseSubstitution'),
    'eventHistory': const FhirField(true, 'Reference'),
  },
  'MedicationDispensePerformer': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'function': const FhirField(false, 'CodeableConcept'),
    'actor': const FhirField(false, 'Reference'),
  },
  'MedicationDispenseSubstitution': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'wasSubstituted': const FhirField(false, 'Boolean'),
    'wasSubstitutedElement': const FhirField(false, 'Element'),
    'type': const FhirField(false, 'CodeableConcept'),
    'reason': const FhirField(true, 'CodeableConcept'),
    'responsibleParty': const FhirField(false, 'Reference'),
  },
  'MedicationKnowledge': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'url': const FhirField(false, 'FhirUri'),
    'urlElement': const FhirField(false, 'Element'),
    'identifier': const FhirField(true, 'Identifier'),
    'version': const FhirField(false, 'String'),
    'versionElement': const FhirField(false, 'Element'),
    'versionAlgorithmString': const FhirField(false, 'String'),
    'versionAlgorithmStringElement': const FhirField(false, 'Element'),
    'versionAlgorithmCoding': const FhirField(false, 'Coding'),
    'name': const FhirField(true, 'String'),
    'nameElement': const FhirField(true, 'Element'),
    'title': const FhirField(false, 'String'),
    'titleElement': const FhirField(false, 'Element'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'experimental': const FhirField(false, 'Boolean'),
    'experimentalElement': const FhirField(false, 'Element'),
    'date': const FhirField(false, 'FhirDateTime'),
    'dateElement': const FhirField(false, 'Element'),
    'publisher': const FhirField(false, 'String'),
    'publisherElement': const FhirField(false, 'Element'),
    'contact': const FhirField(true, 'ContactDetail'),
    'description': const FhirField(false, 'Markdown'),
    'descriptionElement': const FhirField(false, 'Element'),
    'useContext': const FhirField(true, 'UsageContext'),
    'jurisdiction': const FhirField(true, 'CodeableConcept'),
    'purpose': const FhirField(false, 'Markdown'),
    'purposeElement': const FhirField(false, 'Element'),
    'copyright': const FhirField(false, 'Markdown'),
    'copyrightElement': const FhirField(false, 'Element'),
    'copyrightLabel': const FhirField(false, 'String'),
    'copyrightLabelElement': const FhirField(false, 'Element'),
    'approvalDate': const FhirField(false, 'Date'),
    'approvalDateElement': const FhirField(false, 'Element'),
    'lastReviewDate': const FhirField(false, 'Date'),
    'lastReviewDateElement': const FhirField(false, 'Element'),
    'effectivePeriod': const FhirField(false, 'Period'),
    'topic': const FhirField(true, 'CodeableConcept'),
    'author': const FhirField(false, 'Reference'),
    'editor': const FhirField(true, 'ContactDetail'),
    'reviewer': const FhirField(true, 'ContactDetail'),
    'endorser': const FhirField(true, 'ContactDetail'),
    'relatedArtifact': const FhirField(true, 'RelatedArtifact'),
    'code': const FhirField(false, 'CodeableConcept'),
    'intendedJurisdiction': const FhirField(true, 'CodeableConcept'),
    'relatedMedicationKnowledge':
        const FhirField(true, 'MedicationKnowledgeRelatedMedicationKnowledge'),
    'associatedMedication': const FhirField(true, 'Reference'),
    'productType': const FhirField(true, 'CodeableConcept'),
    'monograph': const FhirField(true, 'MedicationKnowledgeMonograph'),
    'preparationInstruction': const FhirField(false, 'Markdown'),
    'preparationInstructionElement': const FhirField(false, 'Element'),
    'cost': const FhirField(true, 'MedicationKnowledgeCost'),
    'monitoringProgram':
        const FhirField(true, 'MedicationKnowledgeMonitoringProgram'),
    'indicationGuideline':
        const FhirField(true, 'MedicationKnowledgeIndicationGuideline'),
    'medicineClassification':
        const FhirField(true, 'MedicationKnowledgeMedicineClassification'),
    'packaging': const FhirField(true, 'MedicationKnowledgePackaging'),
    'clinicalUseIssue': const FhirField(true, 'Reference'),
    'storageGuideline':
        const FhirField(true, 'MedicationKnowledgeStorageGuideline'),
    'regulatory': const FhirField(true, 'MedicationKnowledgeRegulatory'),
    'definitional': const FhirField(false, 'MedicationKnowledgeDefinitional'),
  },
  'MedicationKnowledgeRelatedMedicationKnowledge': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'reference': const FhirField(true, 'Reference'),
  },
  'MedicationKnowledgeMonograph': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'source': const FhirField(false, 'Reference'),
  },
  'MedicationKnowledgeCost': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'effectiveDate': const FhirField(true, 'Period'),
    'type': const FhirField(false, 'CodeableConcept'),
    'source': const FhirField(false, 'String'),
    'sourceElement': const FhirField(false, 'Element'),
    'costMoney': const FhirField(false, 'Money'),
    'costCodeableConcept': const FhirField(false, 'CodeableConcept'),
  },
  'MedicationKnowledgeMonitoringProgram': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'name': const FhirField(false, 'String'),
    'nameElement': const FhirField(false, 'Element'),
  },
  'MedicationKnowledgeIndicationGuideline': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'indication': const FhirField(true, 'CodeableReference'),
    'dosingGuideline':
        const FhirField(true, 'MedicationKnowledgeDosingGuideline'),
  },
  'MedicationKnowledgeDosingGuideline': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'treatmentIntent': const FhirField(false, 'CodeableConcept'),
    'dosage': const FhirField(true, 'MedicationKnowledgeDosage'),
    'administrationTreatment': const FhirField(false, 'CodeableConcept'),
    'patientCharacteristic':
        const FhirField(true, 'MedicationKnowledgePatientCharacteristic'),
  },
  'MedicationKnowledgeDosage': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'dosage': const FhirField(true, 'Dosage'),
  },
  'MedicationKnowledgePatientCharacteristic': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'valueCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'valueQuantity': const FhirField(false, 'Quantity'),
    'valueRange': const FhirField(false, 'Range'),
  },
  'MedicationKnowledgeMedicineClassification': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'sourceString': const FhirField(false, 'String'),
    'sourceStringElement': const FhirField(false, 'Element'),
    'sourceUri': const FhirField(false, 'FhirUri'),
    'sourceUriElement': const FhirField(false, 'Element'),
    'classification': const FhirField(true, 'CodeableConcept'),
  },
  'MedicationKnowledgePackaging': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'cost': const FhirField(true, 'MedicationKnowledgeCost'),
    'packagedProduct': const FhirField(false, 'Reference'),
  },
  'MedicationKnowledgeStorageGuideline': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'reference': const FhirField(false, 'FhirUri'),
    'referenceElement': const FhirField(false, 'Element'),
    'note': const FhirField(true, 'Annotation'),
    'stabilityDuration': const FhirField(false, 'FhirDuration'),
    'environmentalSetting':
        const FhirField(true, 'MedicationKnowledgeEnvironmentalSetting'),
  },
  'MedicationKnowledgeEnvironmentalSetting': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'valueQuantity': const FhirField(false, 'Quantity'),
    'valueRange': const FhirField(false, 'Range'),
    'valueCodeableConcept': const FhirField(false, 'CodeableConcept'),
  },
  'MedicationKnowledgeRegulatory': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'regulatoryAuthority': const FhirField(false, 'Reference'),
    'substitution': const FhirField(true, 'MedicationKnowledgeSubstitution'),
    'schedule': const FhirField(true, 'CodeableConcept'),
    'maxDispense': const FhirField(false, 'MedicationKnowledgeMaxDispense'),
  },
  'MedicationKnowledgeSubstitution': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'allowed': const FhirField(false, 'Boolean'),
    'allowedElement': const FhirField(false, 'Element'),
  },
  'MedicationKnowledgeMaxDispense': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'quantity': const FhirField(false, 'Quantity'),
    'period': const FhirField(false, 'FhirDuration'),
  },
  'MedicationKnowledgeDefinitional': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'definition': const FhirField(true, 'Reference'),
    'doseForm': const FhirField(false, 'CodeableConcept'),
    'intendedRoute': const FhirField(true, 'CodeableConcept'),
    'ingredient': const FhirField(true, 'MedicationKnowledgeIngredient'),
    'drugCharacteristic':
        const FhirField(true, 'MedicationKnowledgeDrugCharacteristic'),
  },
  'MedicationKnowledgeIngredient': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'item': const FhirField(false, 'CodeableReference'),
    'type': const FhirField(false, 'CodeableConcept'),
    'strengthRatio': const FhirField(false, 'Ratio'),
    'strengthCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'strengthQuantity': const FhirField(false, 'Quantity'),
  },
  'MedicationKnowledgeDrugCharacteristic': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'valueCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'valueString': const FhirField(false, 'String'),
    'valueStringElement': const FhirField(false, 'Element'),
    'valueQuantity': const FhirField(false, 'Quantity'),
    'valueBase64Binary': const FhirField(false, 'Base64Binary'),
    'valueBase64BinaryElement': const FhirField(false, 'Element'),
    'valueAttachment': const FhirField(false, 'Attachment'),
  },
  'MedicationRequest': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'basedOn': const FhirField(true, 'Reference'),
    'priorPrescription': const FhirField(false, 'Reference'),
    'groupIdentifier': const FhirField(false, 'Identifier'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'statusReason': const FhirField(false, 'CodeableConcept'),
    'statusChanged': const FhirField(false, 'FhirDateTime'),
    'statusChangedElement': const FhirField(false, 'Element'),
    'intent': const FhirField(false, 'Code'),
    'intentElement': const FhirField(false, 'Element'),
    'category': const FhirField(true, 'CodeableConcept'),
    'priority': const FhirField(false, 'Code'),
    'priorityElement': const FhirField(false, 'Element'),
    'doNotPerform': const FhirField(false, 'Boolean'),
    'doNotPerformElement': const FhirField(false, 'Element'),
    'medication': const FhirField(false, 'CodeableReference'),
    'subject': const FhirField(false, 'Reference'),
    'informationSource': const FhirField(true, 'Reference'),
    'encounter': const FhirField(false, 'Reference'),
    'supportingInformation': const FhirField(true, 'Reference'),
    'authoredOn': const FhirField(false, 'FhirDateTime'),
    'authoredOnElement': const FhirField(false, 'Element'),
    'requester': const FhirField(false, 'Reference'),
    'reported': const FhirField(false, 'Boolean'),
    'reportedElement': const FhirField(false, 'Element'),
    'performerType': const FhirField(false, 'CodeableConcept'),
    'performer': const FhirField(true, 'Reference'),
    'device': const FhirField(true, 'CodeableReference'),
    'recorder': const FhirField(false, 'Reference'),
    'reason': const FhirField(true, 'CodeableReference'),
    'courseOfTherapyType': const FhirField(false, 'CodeableConcept'),
    'insurance': const FhirField(true, 'Reference'),
    'note': const FhirField(true, 'Annotation'),
    'renderedDosageInstruction': const FhirField(false, 'Markdown'),
    'renderedDosageInstructionElement': const FhirField(false, 'Element'),
    'effectiveDosePeriod': const FhirField(false, 'Period'),
    'dosageInstruction': const FhirField(true, 'Dosage'),
    'dispenseRequest':
        const FhirField(false, 'MedicationRequestDispenseRequest'),
    'substitution': const FhirField(false, 'MedicationRequestSubstitution'),
    'eventHistory': const FhirField(true, 'Reference'),
  },
  'MedicationRequestDose': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'renderedDosageInstruction': const FhirField(false, 'String'),
    'renderedDosageInstructionElement': const FhirField(false, 'Element'),
    'effectiveDosePeriod': const FhirField(false, 'Period'),
    'dosageInstruction': const FhirField(true, 'Dosage'),
  },
  'MedicationRequestDispenseRequest': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'initialFill': const FhirField(false, 'MedicationRequestInitialFill'),
    'dispenseInterval': const FhirField(false, 'FhirDuration'),
    'validityPeriod': const FhirField(false, 'Period'),
    'numberOfRepeatsAllowed': const FhirField(false, 'UnsignedInt'),
    'numberOfRepeatsAllowedElement': const FhirField(false, 'Element'),
    'quantity': const FhirField(false, 'Quantity'),
    'expectedSupplyDuration': const FhirField(false, 'FhirDuration'),
    'dispenser': const FhirField(false, 'Reference'),
    'dispenserInstruction': const FhirField(true, 'Annotation'),
    'doseAdministrationAid': const FhirField(false, 'CodeableConcept'),
  },
  'MedicationRequestInitialFill': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'quantity': const FhirField(false, 'Quantity'),
    'duration': const FhirField(false, 'FhirDuration'),
  },
  'MedicationRequestSubstitution': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'allowedBoolean': const FhirField(false, 'Boolean'),
    'allowedBooleanElement': const FhirField(false, 'Element'),
    'allowedCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'reason': const FhirField(false, 'CodeableConcept'),
  },
  'MedicationStatement': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'partOf': const FhirField(true, 'Reference'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'category': const FhirField(true, 'CodeableConcept'),
    'medication': const FhirField(false, 'CodeableReference'),
    'subject': const FhirField(false, 'Reference'),
    'encounter': const FhirField(false, 'Reference'),
    'effectiveDateTime': const FhirField(false, 'FhirDateTime'),
    'effectiveDateTimeElement': const FhirField(false, 'Element'),
    'effectivePeriod': const FhirField(false, 'Period'),
    'effectiveTiming': const FhirField(false, 'Timing'),
    'dateAsserted': const FhirField(false, 'FhirDateTime'),
    'dateAssertedElement': const FhirField(false, 'Element'),
    'informationSource': const FhirField(true, 'Reference'),
    'derivedFrom': const FhirField(true, 'Reference'),
    'reason': const FhirField(true, 'CodeableReference'),
    'note': const FhirField(true, 'Annotation'),
    'relatedClinicalInformation': const FhirField(true, 'Reference'),
    'renderedDosageInstruction': const FhirField(false, 'Markdown'),
    'renderedDosageInstructionElement': const FhirField(false, 'Element'),
    'dosage': const FhirField(true, 'Dosage'),
    'adherence': const FhirField(false, 'MedicationStatementAdherence'),
  },
  'MedicationStatementAdherence': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(false, 'CodeableConcept'),
    'reason': const FhirField(false, 'CodeableConcept'),
  },
  'MedicinalProductDefinition': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'type': const FhirField(false, 'CodeableConcept'),
    'domain': const FhirField(false, 'CodeableConcept'),
    'version': const FhirField(false, 'String'),
    'versionElement': const FhirField(false, 'Element'),
    'status': const FhirField(false, 'CodeableConcept'),
    'statusDate': const FhirField(false, 'FhirDateTime'),
    'statusDateElement': const FhirField(false, 'Element'),
    'description': const FhirField(false, 'Markdown'),
    'descriptionElement': const FhirField(false, 'Element'),
    'combinedPharmaceuticalDoseForm': const FhirField(false, 'CodeableConcept'),
    'route': const FhirField(true, 'CodeableConcept'),
    'indication': const FhirField(false, 'Markdown'),
    'indicationElement': const FhirField(false, 'Element'),
    'legalStatusOfSupply': const FhirField(false, 'CodeableConcept'),
    'additionalMonitoringIndicator': const FhirField(false, 'CodeableConcept'),
    'specialMeasures': const FhirField(true, 'CodeableConcept'),
    'pediatricUseIndicator': const FhirField(false, 'CodeableConcept'),
    'classification': const FhirField(true, 'CodeableConcept'),
    'marketingStatus': const FhirField(true, 'MarketingStatus'),
    'packagedMedicinalProduct': const FhirField(true, 'CodeableConcept'),
    'comprisedOf': const FhirField(true, 'Reference'),
    'ingredient': const FhirField(true, 'CodeableConcept'),
    'impurity': const FhirField(true, 'CodeableReference'),
    'attachedDocument': const FhirField(true, 'Reference'),
    'masterFile': const FhirField(true, 'Reference'),
    'contact': const FhirField(true, 'MedicinalProductDefinitionContact'),
    'clinicalTrial': const FhirField(true, 'Reference'),
    'code': const FhirField(true, 'Coding'),
    'name': const FhirField(true, 'MedicinalProductDefinitionName'),
    'crossReference':
        const FhirField(true, 'MedicinalProductDefinitionCrossReference'),
    'operation': const FhirField(true, 'MedicinalProductDefinitionOperation'),
    'characteristic':
        const FhirField(true, 'MedicinalProductDefinitionCharacteristic'),
  },
  'MedicinalProductDefinitionContact': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'contact': const FhirField(false, 'Reference'),
  },
  'MedicinalProductDefinitionName': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'productName': const FhirField(false, 'String'),
    'productNameElement': const FhirField(false, 'Element'),
    'type': const FhirField(false, 'CodeableConcept'),
    'part': const FhirField(true, 'MedicinalProductDefinitionPart'),
    'usage': const FhirField(true, 'MedicinalProductDefinitionUsage'),
  },
  'MedicinalProductDefinitionPart': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'part': const FhirField(false, 'String'),
    'partElement': const FhirField(false, 'Element'),
    'type': const FhirField(false, 'CodeableConcept'),
  },
  'MedicinalProductDefinitionUsage': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'country': const FhirField(false, 'CodeableConcept'),
    'jurisdiction': const FhirField(false, 'CodeableConcept'),
    'language': const FhirField(false, 'CodeableConcept'),
  },
  'MedicinalProductDefinitionCrossReference': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'product': const FhirField(false, 'CodeableReference'),
    'type': const FhirField(false, 'CodeableConcept'),
  },
  'MedicinalProductDefinitionOperation': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableReference'),
    'effectiveDate': const FhirField(false, 'Period'),
    'organization': const FhirField(true, 'Reference'),
    'confidentialityIndicator': const FhirField(false, 'CodeableConcept'),
  },
  'MedicinalProductDefinitionCharacteristic': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'valueCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'valueMarkdown': const FhirField(false, 'Markdown'),
    'valueMarkdownElement': const FhirField(false, 'Element'),
    'valueQuantity': const FhirField(false, 'Quantity'),
    'valueInteger': const FhirField(false, 'Integer'),
    'valueIntegerElement': const FhirField(false, 'Element'),
    'valueDate': const FhirField(false, 'Date'),
    'valueDateElement': const FhirField(false, 'Element'),
    'valueBoolean': const FhirField(false, 'Boolean'),
    'valueBooleanElement': const FhirField(false, 'Element'),
    'valueAttachment': const FhirField(false, 'Attachment'),
  },
  'MessageDefinition': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'url': const FhirField(false, 'FhirUri'),
    'urlElement': const FhirField(false, 'Element'),
    'identifier': const FhirField(true, 'Identifier'),
    'version': const FhirField(false, 'String'),
    'versionElement': const FhirField(false, 'Element'),
    'versionAlgorithmString': const FhirField(false, 'String'),
    'versionAlgorithmStringElement': const FhirField(false, 'Element'),
    'versionAlgorithmCoding': const FhirField(false, 'Coding'),
    'name': const FhirField(false, 'String'),
    'nameElement': const FhirField(false, 'Element'),
    'title': const FhirField(false, 'String'),
    'titleElement': const FhirField(false, 'Element'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'experimental': const FhirField(false, 'Boolean'),
    'experimentalElement': const FhirField(false, 'Element'),
    'date': const FhirField(false, 'FhirDateTime'),
    'dateElement': const FhirField(false, 'Element'),
    'publisher': const FhirField(false, 'String'),
    'publisherElement': const FhirField(false, 'Element'),
    'contact': const FhirField(true, 'ContactDetail'),
    'description': const FhirField(false, 'Markdown'),
    'descriptionElement': const FhirField(false, 'Element'),
    'useContext': const FhirField(true, 'UsageContext'),
    'jurisdiction': const FhirField(true, 'CodeableConcept'),
    'purpose': const FhirField(false, 'Markdown'),
    'purposeElement': const FhirField(false, 'Element'),
    'copyright': const FhirField(false, 'Markdown'),
    'copyrightElement': const FhirField(false, 'Element'),
    'copyrightLabel': const FhirField(false, 'String'),
    'copyrightLabelElement': const FhirField(false, 'Element'),
    'replaces': const FhirField(true, 'Canonical'),
    'base': const FhirField(false, 'Canonical'),
    'parent': const FhirField(true, 'Canonical'),
    'eventCoding': const FhirField(false, 'Coding'),
    'eventUri': const FhirField(false, 'FhirUri'),
    'eventUriElement': const FhirField(false, 'Element'),
    'category': const FhirField(false, 'Code'),
    'categoryElement': const FhirField(false, 'Element'),
    'focus': const FhirField(true, 'MessageDefinitionFocus'),
    'responseRequired': const FhirField(false, 'Code'),
    'responseRequiredElement': const FhirField(false, 'Element'),
    'allowedResponse':
        const FhirField(true, 'MessageDefinitionAllowedResponse'),
    'graph': const FhirField(false, 'Canonical'),
  },
  'MessageDefinitionFocus': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(false, 'Code'),
    'codeElement': const FhirField(false, 'Element'),
    'profile': const FhirField(false, 'Canonical'),
    'min': const FhirField(false, 'UnsignedInt'),
    'minElement': const FhirField(false, 'Element'),
    'max': const FhirField(false, 'String'),
    'maxElement': const FhirField(false, 'Element'),
  },
  'MessageDefinitionAllowedResponse': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'message': const FhirField(false, 'Canonical'),
    'situation': const FhirField(false, 'Markdown'),
    'situationElement': const FhirField(false, 'Element'),
  },
  'MessageHeader': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'eventCoding': const FhirField(false, 'Coding'),
    'eventCanonical': const FhirField(false, 'Canonical'),
    'eventCanonicalElement': const FhirField(false, 'Element'),
    'destination': const FhirField(true, 'MessageHeaderDestination'),
    'sender': const FhirField(false, 'Reference'),
    'author': const FhirField(false, 'Reference'),
    'source': const FhirField(false, 'MessageHeaderSource'),
    'responsible': const FhirField(false, 'Reference'),
    'reason': const FhirField(false, 'CodeableConcept'),
    'response': const FhirField(false, 'MessageHeaderResponse'),
    'focus': const FhirField(true, 'Reference'),
    'definition': const FhirField(false, 'Canonical'),
  },
  'MessageHeaderDestination': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'endpointUrl': const FhirField(false, 'FhirUrl'),
    'endpointUrlElement': const FhirField(false, 'Element'),
    'endpointReference': const FhirField(false, 'Reference'),
    'name': const FhirField(false, 'String'),
    'nameElement': const FhirField(false, 'Element'),
    'target': const FhirField(false, 'Reference'),
    'receiver': const FhirField(false, 'Reference'),
  },
  'MessageHeaderSource': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'endpointUrl': const FhirField(false, 'FhirUrl'),
    'endpointUrlElement': const FhirField(false, 'Element'),
    'endpointReference': const FhirField(false, 'Reference'),
    'name': const FhirField(false, 'String'),
    'nameElement': const FhirField(false, 'Element'),
    'software': const FhirField(false, 'String'),
    'softwareElement': const FhirField(false, 'Element'),
    'version': const FhirField(false, 'String'),
    'versionElement': const FhirField(false, 'Element'),
    'contact': const FhirField(false, 'ContactPoint'),
  },
  'MessageHeaderResponse': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(false, 'Identifier'),
    'code': const FhirField(false, 'Code'),
    'codeElement': const FhirField(false, 'Element'),
    'details': const FhirField(false, 'Reference'),
  },
  'MolecularSequence': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'type': const FhirField(false, 'Code'),
    'typeElement': const FhirField(false, 'Element'),
    'subject': const FhirField(false, 'Reference'),
    'focus': const FhirField(true, 'Reference'),
    'specimen': const FhirField(false, 'Reference'),
    'device': const FhirField(false, 'Reference'),
    'performer': const FhirField(false, 'Reference'),
    'literal': const FhirField(false, 'String'),
    'literalElement': const FhirField(false, 'Element'),
    'formatted': const FhirField(true, 'Attachment'),
    'relative': const FhirField(true, 'MolecularSequenceRelative'),
  },
  'MolecularSequenceRelative': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'coordinateSystem': const FhirField(false, 'CodeableConcept'),
    'ordinalPosition': const FhirField(false, 'Integer'),
    'ordinalPositionElement': const FhirField(false, 'Element'),
    'sequenceRange': const FhirField(false, 'Range'),
    'startingSequence':
        const FhirField(false, 'MolecularSequenceStartingSequence'),
    'edit': const FhirField(true, 'MolecularSequenceEdit'),
  },
  'MolecularSequenceStartingSequence': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'genomeAssembly': const FhirField(false, 'CodeableConcept'),
    'chromosome': const FhirField(false, 'CodeableConcept'),
    'sequenceCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'sequenceString': const FhirField(false, 'String'),
    'sequenceStringElement': const FhirField(false, 'Element'),
    'sequenceReference': const FhirField(false, 'Reference'),
    'windowStart': const FhirField(false, 'Integer'),
    'windowStartElement': const FhirField(false, 'Element'),
    'windowEnd': const FhirField(false, 'Integer'),
    'windowEndElement': const FhirField(false, 'Element'),
    'orientation': const FhirField(false, 'Code'),
    'orientationElement': const FhirField(false, 'Element'),
    'strand': const FhirField(false, 'Code'),
    'strandElement': const FhirField(false, 'Element'),
  },
  'MolecularSequenceEdit': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'start': const FhirField(false, 'Integer'),
    'startElement': const FhirField(false, 'Element'),
    'end': const FhirField(false, 'Integer'),
    'endElement': const FhirField(false, 'Element'),
    'replacementSequence': const FhirField(false, 'String'),
    'replacementSequenceElement': const FhirField(false, 'Element'),
    'replacedSequence': const FhirField(false, 'String'),
    'replacedSequenceElement': const FhirField(false, 'Element'),
  },
  'NamingSystem': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'url': const FhirField(false, 'FhirUri'),
    'urlElement': const FhirField(false, 'Element'),
    'identifier': const FhirField(true, 'Identifier'),
    'version': const FhirField(false, 'String'),
    'versionElement': const FhirField(false, 'Element'),
    'versionAlgorithmString': const FhirField(false, 'String'),
    'versionAlgorithmStringElement': const FhirField(false, 'Element'),
    'versionAlgorithmCoding': const FhirField(false, 'Coding'),
    'name': const FhirField(false, 'String'),
    'nameElement': const FhirField(false, 'Element'),
    'title': const FhirField(false, 'String'),
    'titleElement': const FhirField(false, 'Element'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'experimental': const FhirField(false, 'Boolean'),
    'experimentalElement': const FhirField(false, 'Element'),
    'date': const FhirField(false, 'FhirDateTime'),
    'dateElement': const FhirField(false, 'Element'),
    'publisher': const FhirField(false, 'String'),
    'publisherElement': const FhirField(false, 'Element'),
    'contact': const FhirField(true, 'ContactDetail'),
    'description': const FhirField(false, 'Markdown'),
    'descriptionElement': const FhirField(false, 'Element'),
    'useContext': const FhirField(true, 'UsageContext'),
    'jurisdiction': const FhirField(true, 'CodeableConcept'),
    'purpose': const FhirField(false, 'Markdown'),
    'purposeElement': const FhirField(false, 'Element'),
    'copyright': const FhirField(false, 'Markdown'),
    'copyrightElement': const FhirField(false, 'Element'),
    'copyrightLabel': const FhirField(false, 'String'),
    'copyrightLabelElement': const FhirField(false, 'Element'),
    'approvalDate': const FhirField(false, 'Date'),
    'approvalDateElement': const FhirField(false, 'Element'),
    'lastReviewDate': const FhirField(false, 'Date'),
    'lastReviewDateElement': const FhirField(false, 'Element'),
    'effectivePeriod': const FhirField(false, 'Period'),
    'topic': const FhirField(true, 'CodeableConcept'),
    'author': const FhirField(true, 'ContactDetail'),
    'editor': const FhirField(true, 'ContactDetail'),
    'reviewer': const FhirField(true, 'ContactDetail'),
    'endorser': const FhirField(true, 'ContactDetail'),
    'relatedArtifact': const FhirField(true, 'RelatedArtifact'),
    'kind': const FhirField(false, 'Code'),
    'kindElement': const FhirField(false, 'Element'),
    'responsible': const FhirField(false, 'String'),
    'responsibleElement': const FhirField(false, 'Element'),
    'type': const FhirField(false, 'CodeableConcept'),
    'usage': const FhirField(false, 'String'),
    'usageElement': const FhirField(false, 'Element'),
    'uniqueId': const FhirField(true, 'NamingSystemUniqueId'),
  },
  'NamingSystemUniqueId': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'Code'),
    'typeElement': const FhirField(false, 'Element'),
    'value': const FhirField(false, 'String'),
    'valueElement': const FhirField(false, 'Element'),
    'preferred': const FhirField(false, 'Boolean'),
    'preferredElement': const FhirField(false, 'Element'),
    'comment': const FhirField(false, 'String'),
    'commentElement': const FhirField(false, 'Element'),
    'period': const FhirField(false, 'Period'),
    'authoritative': const FhirField(false, 'Boolean'),
    'authoritativeElement': const FhirField(false, 'Element'),
  },
  'NutritionIntake': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'instantiatesCanonical': const FhirField(true, 'Canonical'),
    'instantiatesUri': const FhirField(true, 'FhirUri'),
    'instantiatesUriElement': const FhirField(true, 'Element'),
    'basedOn': const FhirField(true, 'Reference'),
    'partOf': const FhirField(true, 'Reference'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'statusReason': const FhirField(true, 'CodeableConcept'),
    'code': const FhirField(false, 'CodeableConcept'),
    'subject': const FhirField(false, 'Reference'),
    'encounter': const FhirField(false, 'Reference'),
    'occurrenceDateTime': const FhirField(false, 'FhirDateTime'),
    'occurrenceDateTimeElement': const FhirField(false, 'Element'),
    'occurrencePeriod': const FhirField(false, 'Period'),
    'recorded': const FhirField(false, 'FhirDateTime'),
    'recordedElement': const FhirField(false, 'Element'),
    'reportedBoolean': const FhirField(false, 'Boolean'),
    'reportedBooleanElement': const FhirField(false, 'Element'),
    'reportedReference': const FhirField(false, 'Reference'),
    'consumedItem': const FhirField(true, 'NutritionIntakeConsumedItem'),
    'ingredientLabel': const FhirField(true, 'NutritionIntakeIngredientLabel'),
    'performer': const FhirField(true, 'NutritionIntakePerformer'),
    'location': const FhirField(false, 'Reference'),
    'derivedFrom': const FhirField(true, 'Reference'),
    'reason': const FhirField(true, 'CodeableReference'),
    'note': const FhirField(true, 'Annotation'),
  },
  'NutritionIntakeConsumedItem': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'nutritionProduct': const FhirField(false, 'CodeableReference'),
    'schedule': const FhirField(false, 'Timing'),
    'amount': const FhirField(false, 'Quantity'),
    'rate': const FhirField(false, 'Quantity'),
    'notConsumed': const FhirField(false, 'Boolean'),
    'notConsumedElement': const FhirField(false, 'Element'),
    'notConsumedReason': const FhirField(false, 'CodeableConcept'),
  },
  'NutritionIntakeIngredientLabel': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'nutrient': const FhirField(false, 'CodeableReference'),
    'amount': const FhirField(false, 'Quantity'),
  },
  'NutritionIntakePerformer': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'function': const FhirField(false, 'CodeableConcept'),
    'actor': const FhirField(false, 'Reference'),
  },
  'NutritionOrder': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'instantiatesCanonical': const FhirField(true, 'Canonical'),
    'instantiatesUri': const FhirField(true, 'FhirUri'),
    'instantiatesUriElement': const FhirField(true, 'Element'),
    'instantiates': const FhirField(true, 'FhirUri'),
    'instantiatesElement': const FhirField(true, 'Element'),
    'basedOn': const FhirField(true, 'Reference'),
    'groupIdentifier': const FhirField(false, 'Identifier'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'intent': const FhirField(false, 'Code'),
    'intentElement': const FhirField(false, 'Element'),
    'priority': const FhirField(false, 'Code'),
    'priorityElement': const FhirField(false, 'Element'),
    'subject': const FhirField(false, 'Reference'),
    'encounter': const FhirField(false, 'Reference'),
    'supportingInformation': const FhirField(true, 'Reference'),
    'dateTime': const FhirField(false, 'FhirDateTime'),
    'dateTimeElement': const FhirField(false, 'Element'),
    'orderer': const FhirField(false, 'Reference'),
    'performer': const FhirField(true, 'CodeableReference'),
    'allergyIntolerance': const FhirField(true, 'Reference'),
    'foodPreferenceModifier': const FhirField(true, 'CodeableConcept'),
    'excludeFoodModifier': const FhirField(true, 'CodeableConcept'),
    'outsideFoodAllowed': const FhirField(false, 'Boolean'),
    'outsideFoodAllowedElement': const FhirField(false, 'Element'),
    'oralDiet': const FhirField(false, 'NutritionOrderOralDiet'),
    'supplement': const FhirField(true, 'NutritionOrderSupplement'),
    'enteralFormula': const FhirField(false, 'NutritionOrderEnteralFormula'),
    'note': const FhirField(true, 'Annotation'),
  },
  'NutritionOrderOralDiet': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(true, 'CodeableConcept'),
    'schedule': const FhirField(false, 'NutritionOrderSchedule'),
    'nutrient': const FhirField(true, 'NutritionOrderNutrient'),
    'texture': const FhirField(true, 'NutritionOrderTexture'),
    'fluidConsistencyType': const FhirField(true, 'CodeableConcept'),
    'instruction': const FhirField(false, 'String'),
    'instructionElement': const FhirField(false, 'Element'),
  },
  'NutritionOrderSchedule': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'timing': const FhirField(true, 'Timing'),
    'asNeeded': const FhirField(false, 'Boolean'),
    'asNeededElement': const FhirField(false, 'Element'),
    'asNeededFor': const FhirField(false, 'CodeableConcept'),
  },
  'NutritionOrderNutrient': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'modifier': const FhirField(false, 'CodeableConcept'),
    'amount': const FhirField(false, 'Quantity'),
  },
  'NutritionOrderTexture': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'modifier': const FhirField(false, 'CodeableConcept'),
    'foodType': const FhirField(false, 'CodeableConcept'),
  },
  'NutritionOrderSupplement': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableReference'),
    'productName': const FhirField(false, 'String'),
    'productNameElement': const FhirField(false, 'Element'),
    'schedule': const FhirField(false, 'NutritionOrderSchedule1'),
    'quantity': const FhirField(false, 'Quantity'),
    'instruction': const FhirField(false, 'String'),
    'instructionElement': const FhirField(false, 'Element'),
  },
  'NutritionOrderSchedule1': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'timing': const FhirField(true, 'Timing'),
    'asNeeded': const FhirField(false, 'Boolean'),
    'asNeededElement': const FhirField(false, 'Element'),
    'asNeededFor': const FhirField(false, 'CodeableConcept'),
  },
  'NutritionOrderEnteralFormula': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'baseFormulaType': const FhirField(false, 'CodeableReference'),
    'baseFormulaProductName': const FhirField(false, 'String'),
    'baseFormulaProductNameElement': const FhirField(false, 'Element'),
    'deliveryDevice': const FhirField(true, 'CodeableReference'),
    'additive': const FhirField(true, 'NutritionOrderAdditive'),
    'caloricDensity': const FhirField(false, 'Quantity'),
    'routeOfAdministration': const FhirField(false, 'CodeableConcept'),
    'administration': const FhirField(true, 'NutritionOrderAdministration'),
    'maxVolumeToDeliver': const FhirField(false, 'Quantity'),
    'administrationInstruction': const FhirField(false, 'Markdown'),
    'administrationInstructionElement': const FhirField(false, 'Element'),
  },
  'NutritionOrderAdditive': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableReference'),
    'productName': const FhirField(false, 'String'),
    'productNameElement': const FhirField(false, 'Element'),
    'quantity': const FhirField(false, 'Quantity'),
  },
  'NutritionOrderAdministration': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'schedule': const FhirField(false, 'NutritionOrderSchedule2'),
    'quantity': const FhirField(false, 'Quantity'),
    'rateQuantity': const FhirField(false, 'Quantity'),
    'rateRatio': const FhirField(false, 'Ratio'),
  },
  'NutritionOrderSchedule2': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'timing': const FhirField(true, 'Timing'),
    'asNeeded': const FhirField(false, 'Boolean'),
    'asNeededElement': const FhirField(false, 'Element'),
    'asNeededFor': const FhirField(false, 'CodeableConcept'),
  },
  'NutritionProduct': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(false, 'CodeableConcept'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'category': const FhirField(true, 'CodeableConcept'),
    'manufacturer': const FhirField(true, 'Reference'),
    'nutrient': const FhirField(true, 'NutritionProductNutrient'),
    'ingredient': const FhirField(true, 'NutritionProductIngredient'),
    'knownAllergen': const FhirField(true, 'CodeableReference'),
    'characteristic': const FhirField(true, 'NutritionProductCharacteristic'),
    'instance': const FhirField(true, 'NutritionProductInstance'),
    'note': const FhirField(true, 'Annotation'),
  },
  'NutritionProductNutrient': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'item': const FhirField(false, 'CodeableReference'),
    'amount': const FhirField(true, 'Ratio'),
  },
  'NutritionProductIngredient': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'item': const FhirField(false, 'CodeableReference'),
    'amount': const FhirField(true, 'Ratio'),
  },
  'NutritionProductCharacteristic': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'valueCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'valueString': const FhirField(false, 'String'),
    'valueStringElement': const FhirField(false, 'Element'),
    'valueQuantity': const FhirField(false, 'Quantity'),
    'valueBase64Binary': const FhirField(false, 'Base64Binary'),
    'valueBase64BinaryElement': const FhirField(false, 'Element'),
    'valueAttachment': const FhirField(false, 'Attachment'),
    'valueBoolean': const FhirField(false, 'Boolean'),
    'valueBooleanElement': const FhirField(false, 'Element'),
  },
  'NutritionProductInstance': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'quantity': const FhirField(false, 'Quantity'),
    'identifier': const FhirField(true, 'Identifier'),
    'name': const FhirField(false, 'String'),
    'nameElement': const FhirField(false, 'Element'),
    'lotNumber': const FhirField(false, 'String'),
    'lotNumberElement': const FhirField(false, 'Element'),
    'expiry': const FhirField(false, 'FhirDateTime'),
    'expiryElement': const FhirField(false, 'Element'),
    'useBy': const FhirField(false, 'FhirDateTime'),
    'useByElement': const FhirField(false, 'Element'),
    'biologicalSourceEvent': const FhirField(false, 'Identifier'),
  },
  'Observation': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'instantiatesCanonical': const FhirField(false, 'Canonical'),
    'instantiatesCanonicalElement': const FhirField(false, 'Element'),
    'instantiatesReference': const FhirField(false, 'Reference'),
    'basedOn': const FhirField(true, 'Reference'),
    'triggeredBy': const FhirField(true, 'ObservationTriggeredBy'),
    'partOf': const FhirField(true, 'Reference'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'category': const FhirField(true, 'CodeableConcept'),
    'code': const FhirField(false, 'CodeableConcept'),
    'subject': const FhirField(false, 'Reference'),
    'focus': const FhirField(true, 'Reference'),
    'encounter': const FhirField(false, 'Reference'),
    'effectiveDateTime': const FhirField(false, 'FhirDateTime'),
    'effectiveDateTimeElement': const FhirField(false, 'Element'),
    'effectivePeriod': const FhirField(false, 'Period'),
    'effectiveTiming': const FhirField(false, 'Timing'),
    'effectiveInstant': const FhirField(false, 'Instant'),
    'effectiveInstantElement': const FhirField(false, 'Element'),
    'issued': const FhirField(false, 'Instant'),
    'issuedElement': const FhirField(false, 'Element'),
    'performer': const FhirField(true, 'Reference'),
    'valueQuantity': const FhirField(false, 'Quantity'),
    'valueCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'valueString': const FhirField(false, 'String'),
    'valueStringElement': const FhirField(false, 'Element'),
    'valueBoolean': const FhirField(false, 'Boolean'),
    'valueBooleanElement': const FhirField(false, 'Element'),
    'valueInteger': const FhirField(false, 'Integer'),
    'valueIntegerElement': const FhirField(false, 'Element'),
    'valueRange': const FhirField(false, 'Range'),
    'valueRatio': const FhirField(false, 'Ratio'),
    'valueSampledData': const FhirField(false, 'SampledData'),
    'valueTime': const FhirField(false, 'Time'),
    'valueTimeElement': const FhirField(false, 'Element'),
    'valueDateTime': const FhirField(false, 'FhirDateTime'),
    'valueDateTimeElement': const FhirField(false, 'Element'),
    'valuePeriod': const FhirField(false, 'Period'),
    'valueAttachment': const FhirField(false, 'Attachment'),
    'valueReference': const FhirField(false, 'Reference'),
    'dataAbsentReason': const FhirField(false, 'CodeableConcept'),
    'interpretation': const FhirField(true, 'CodeableConcept'),
    'note': const FhirField(true, 'Annotation'),
    'bodySite': const FhirField(false, 'CodeableConcept'),
    'bodyStructure': const FhirField(false, 'Reference'),
    'method': const FhirField(false, 'CodeableConcept'),
    'specimen': const FhirField(false, 'Reference'),
    'device': const FhirField(false, 'Reference'),
    'referenceRange': const FhirField(true, 'ObservationReferenceRange'),
    'hasMember': const FhirField(true, 'Reference'),
    'derivedFrom': const FhirField(true, 'Reference'),
    'component': const FhirField(true, 'ObservationComponent'),
  },
  'ObservationTriggeredBy': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'observation': const FhirField(false, 'Reference'),
    'type': const FhirField(false, 'Code'),
    'typeElement': const FhirField(false, 'Element'),
    'reason': const FhirField(false, 'String'),
    'reasonElement': const FhirField(false, 'Element'),
  },
  'ObservationReferenceRange': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'low': const FhirField(false, 'Quantity'),
    'high': const FhirField(false, 'Quantity'),
    'normalValue': const FhirField(false, 'CodeableConcept'),
    'type': const FhirField(false, 'CodeableConcept'),
    'appliesTo': const FhirField(true, 'CodeableConcept'),
    'age': const FhirField(false, 'Range'),
    'text': const FhirField(false, 'Markdown'),
    'textElement': const FhirField(false, 'Element'),
  },
  'ObservationComponent': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(false, 'CodeableConcept'),
    'valueQuantity': const FhirField(false, 'Quantity'),
    'valueCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'valueString': const FhirField(false, 'String'),
    'valueStringElement': const FhirField(false, 'Element'),
    'valueBoolean': const FhirField(false, 'Boolean'),
    'valueBooleanElement': const FhirField(false, 'Element'),
    'valueInteger': const FhirField(false, 'Integer'),
    'valueIntegerElement': const FhirField(false, 'Element'),
    'valueRange': const FhirField(false, 'Range'),
    'valueRatio': const FhirField(false, 'Ratio'),
    'valueSampledData': const FhirField(false, 'SampledData'),
    'valueTime': const FhirField(false, 'Time'),
    'valueTimeElement': const FhirField(false, 'Element'),
    'valueDateTime': const FhirField(false, 'FhirDateTime'),
    'valueDateTimeElement': const FhirField(false, 'Element'),
    'valuePeriod': const FhirField(false, 'Period'),
    'valueAttachment': const FhirField(false, 'Attachment'),
    'valueReference': const FhirField(false, 'Reference'),
    'dataAbsentReason': const FhirField(false, 'CodeableConcept'),
    'interpretation': const FhirField(true, 'CodeableConcept'),
    'referenceRange': const FhirField(true, 'ObservationReferenceRange'),
  },
  'ObservationDefinition': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'url': const FhirField(false, 'FhirUri'),
    'urlElement': const FhirField(false, 'Element'),
    'identifier': const FhirField(false, 'Identifier'),
    'version': const FhirField(false, 'String'),
    'versionElement': const FhirField(false, 'Element'),
    'versionAlgorithmString': const FhirField(false, 'String'),
    'versionAlgorithmStringElement': const FhirField(false, 'Element'),
    'versionAlgorithmCoding': const FhirField(false, 'Coding'),
    'name': const FhirField(false, 'String'),
    'nameElement': const FhirField(false, 'Element'),
    'title': const FhirField(false, 'String'),
    'titleElement': const FhirField(false, 'Element'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'experimental': const FhirField(false, 'Boolean'),
    'experimentalElement': const FhirField(false, 'Element'),
    'date': const FhirField(false, 'FhirDateTime'),
    'dateElement': const FhirField(false, 'Element'),
    'publisher': const FhirField(false, 'String'),
    'publisherElement': const FhirField(false, 'Element'),
    'contact': const FhirField(true, 'ContactDetail'),
    'description': const FhirField(false, 'Markdown'),
    'descriptionElement': const FhirField(false, 'Element'),
    'useContext': const FhirField(true, 'UsageContext'),
    'jurisdiction': const FhirField(true, 'CodeableConcept'),
    'purpose': const FhirField(false, 'Markdown'),
    'purposeElement': const FhirField(false, 'Element'),
    'copyright': const FhirField(false, 'Markdown'),
    'copyrightElement': const FhirField(false, 'Element'),
    'copyrightLabel': const FhirField(false, 'String'),
    'copyrightLabelElement': const FhirField(false, 'Element'),
    'approvalDate': const FhirField(false, 'Date'),
    'approvalDateElement': const FhirField(false, 'Element'),
    'lastReviewDate': const FhirField(false, 'Date'),
    'lastReviewDateElement': const FhirField(false, 'Element'),
    'effectivePeriod': const FhirField(false, 'Period'),
    'topic': const FhirField(true, 'CodeableConcept'),
    'author': const FhirField(true, 'ContactDetail'),
    'editor': const FhirField(true, 'ContactDetail'),
    'reviewer': const FhirField(true, 'ContactDetail'),
    'endorser': const FhirField(true, 'ContactDetail'),
    'relatedArtifact': const FhirField(true, 'RelatedArtifact'),
    'derivedFromCanonical': const FhirField(true, 'Canonical'),
    'derivedFromUri': const FhirField(true, 'FhirUri'),
    'derivedFromUriElement': const FhirField(true, 'Element'),
    'subject': const FhirField(true, 'CodeableConcept'),
    'performerType': const FhirField(false, 'CodeableConcept'),
    'category': const FhirField(true, 'CodeableConcept'),
    'code': const FhirField(false, 'CodeableConcept'),
    'permittedDataType': const FhirField(true, 'Code'),
    'permittedDataTypeElement': const FhirField(true, 'Element'),
    'multipleResultsAllowed': const FhirField(false, 'Boolean'),
    'multipleResultsAllowedElement': const FhirField(false, 'Element'),
    'bodySite': const FhirField(false, 'CodeableConcept'),
    'method': const FhirField(false, 'CodeableConcept'),
    'specimen': const FhirField(true, 'Reference'),
    'device': const FhirField(true, 'Reference'),
    'preferredReportName': const FhirField(false, 'String'),
    'preferredReportNameElement': const FhirField(false, 'Element'),
    'permittedUnit': const FhirField(true, 'Coding'),
    'qualifiedValue':
        const FhirField(true, 'ObservationDefinitionQualifiedValue'),
    'hasMember': const FhirField(true, 'Reference'),
    'component': const FhirField(true, 'ObservationDefinitionComponent'),
  },
  'ObservationDefinitionQuantitativeDetails': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'unit': const FhirField(false, 'CodeableConcept'),
    'customaryUnit': const FhirField(false, 'CodeableConcept'),
    'conversionFactor': const FhirField(false, 'Decimal'),
    'conversionFactorElement': const FhirField(false, 'Element'),
    'decimalPrecision': const FhirField(false, 'Integer'),
    'decimalPrecisionElement': const FhirField(false, 'Element'),
  },
  'ObservationDefinitionQualifiedValue': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'context': const FhirField(false, 'CodeableConcept'),
    'appliesTo': const FhirField(true, 'CodeableConcept'),
    'gender': const FhirField(false, 'Code'),
    'genderElement': const FhirField(false, 'Element'),
    'age': const FhirField(false, 'Range'),
    'gestationalAge': const FhirField(false, 'Range'),
    'condition': const FhirField(false, 'String'),
    'conditionElement': const FhirField(false, 'Element'),
    'rangeCategory': const FhirField(false, 'Code'),
    'rangeCategoryElement': const FhirField(false, 'Element'),
    'range': const FhirField(false, 'Range'),
    'validCodedValueSet': const FhirField(false, 'Canonical'),
    'normalCodedValueSet': const FhirField(false, 'Canonical'),
    'abnormalCodedValueSet': const FhirField(false, 'Canonical'),
    'criticalCodedValueSet': const FhirField(false, 'Canonical'),
  },
  'ObservationDefinitionComponent': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(false, 'CodeableConcept'),
    'permittedDataType': const FhirField(true, 'Code'),
    'permittedDataTypeElement': const FhirField(true, 'Element'),
    'permittedUnit': const FhirField(true, 'Coding'),
    'qualifiedValue':
        const FhirField(true, 'ObservationDefinitionQualifiedValue'),
  },
  'OperationDefinition': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'url': const FhirField(false, 'FhirUri'),
    'urlElement': const FhirField(false, 'Element'),
    'identifier': const FhirField(true, 'Identifier'),
    'version': const FhirField(false, 'String'),
    'versionElement': const FhirField(false, 'Element'),
    'versionAlgorithmString': const FhirField(false, 'String'),
    'versionAlgorithmStringElement': const FhirField(false, 'Element'),
    'versionAlgorithmCoding': const FhirField(false, 'Coding'),
    'name': const FhirField(false, 'String'),
    'nameElement': const FhirField(false, 'Element'),
    'title': const FhirField(false, 'String'),
    'titleElement': const FhirField(false, 'Element'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'experimental': const FhirField(false, 'Boolean'),
    'experimentalElement': const FhirField(false, 'Element'),
    'date': const FhirField(false, 'FhirDateTime'),
    'dateElement': const FhirField(false, 'Element'),
    'publisher': const FhirField(false, 'String'),
    'publisherElement': const FhirField(false, 'Element'),
    'contact': const FhirField(true, 'ContactDetail'),
    'description': const FhirField(false, 'Markdown'),
    'descriptionElement': const FhirField(false, 'Element'),
    'useContext': const FhirField(true, 'UsageContext'),
    'jurisdiction': const FhirField(true, 'CodeableConcept'),
    'purpose': const FhirField(false, 'Markdown'),
    'purposeElement': const FhirField(false, 'Element'),
    'copyright': const FhirField(false, 'Markdown'),
    'copyrightElement': const FhirField(false, 'Element'),
    'copyrightLabel': const FhirField(false, 'String'),
    'copyrightLabelElement': const FhirField(false, 'Element'),
    'kind': const FhirField(false, 'Code'),
    'kindElement': const FhirField(false, 'Element'),
    'affectsState': const FhirField(false, 'Boolean'),
    'affectsStateElement': const FhirField(false, 'Element'),
    'code': const FhirField(false, 'Code'),
    'codeElement': const FhirField(false, 'Element'),
    'comment': const FhirField(false, 'Markdown'),
    'commentElement': const FhirField(false, 'Element'),
    'base': const FhirField(false, 'Canonical'),
    'resource': const FhirField(true, 'Code'),
    'resourceElement': const FhirField(true, 'Element'),
    'system': const FhirField(false, 'Boolean'),
    'systemElement': const FhirField(false, 'Element'),
    'type': const FhirField(false, 'Boolean'),
    'typeElement': const FhirField(false, 'Element'),
    'instance': const FhirField(false, 'Boolean'),
    'instanceElement': const FhirField(false, 'Element'),
    'inputProfile': const FhirField(false, 'Canonical'),
    'outputProfile': const FhirField(false, 'Canonical'),
    'parameter': const FhirField(true, 'OperationDefinitionParameter'),
    'overload': const FhirField(true, 'OperationDefinitionOverload'),
  },
  'OperationDefinitionParameter': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'name': const FhirField(false, 'Code'),
    'nameElement': const FhirField(false, 'Element'),
    'use': const FhirField(false, 'Code'),
    'useElement': const FhirField(false, 'Element'),
    'scope': const FhirField(true, 'Code'),
    'scopeElement': const FhirField(true, 'Element'),
    'min': const FhirField(false, 'Integer'),
    'minElement': const FhirField(false, 'Element'),
    'max': const FhirField(false, 'String'),
    'maxElement': const FhirField(false, 'Element'),
    'documentation': const FhirField(false, 'Markdown'),
    'documentationElement': const FhirField(false, 'Element'),
    'type': const FhirField(false, 'Code'),
    'typeElement': const FhirField(false, 'Element'),
    'allowedType': const FhirField(true, 'Code'),
    'allowedTypeElement': const FhirField(true, 'Element'),
    'targetProfile': const FhirField(true, 'Canonical'),
    'searchType': const FhirField(false, 'Code'),
    'searchTypeElement': const FhirField(false, 'Element'),
    'binding': const FhirField(false, 'OperationDefinitionBinding'),
    'referencedFrom':
        const FhirField(true, 'OperationDefinitionReferencedFrom'),
    'part': const FhirField(true, 'OperationDefinitionParameter'),
  },
  'OperationDefinitionBinding': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'strength': const FhirField(false, 'Code'),
    'strengthElement': const FhirField(false, 'Element'),
    'valueSet': const FhirField(false, 'Canonical'),
  },
  'OperationDefinitionReferencedFrom': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'source': const FhirField(false, 'String'),
    'sourceElement': const FhirField(false, 'Element'),
    'sourceId': const FhirField(false, 'String'),
    'sourceIdElement': const FhirField(false, 'Element'),
  },
  'OperationDefinitionOverload': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'parameterName': const FhirField(true, 'String'),
    'parameterNameElement': const FhirField(true, 'Element'),
    'comment': const FhirField(false, 'String'),
    'commentElement': const FhirField(false, 'Element'),
  },
  'OperationOutcome': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'issue': const FhirField(true, 'OperationOutcomeIssue'),
  },
  'OperationOutcomeIssue': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'severity': const FhirField(false, 'Code'),
    'severityElement': const FhirField(false, 'Element'),
    'code': const FhirField(false, 'Code'),
    'codeElement': const FhirField(false, 'Element'),
    'details': const FhirField(false, 'CodeableConcept'),
    'diagnostics': const FhirField(false, 'String'),
    'diagnosticsElement': const FhirField(false, 'Element'),
    'location': const FhirField(true, 'String'),
    'locationElement': const FhirField(true, 'Element'),
    'expression': const FhirField(true, 'String'),
    'expressionElement': const FhirField(true, 'Element'),
  },
  'Organization': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'active': const FhirField(false, 'Boolean'),
    'activeElement': const FhirField(false, 'Element'),
    'type': const FhirField(true, 'CodeableConcept'),
    'name': const FhirField(false, 'String'),
    'nameElement': const FhirField(false, 'Element'),
    'alias': const FhirField(true, 'String'),
    'aliasElement': const FhirField(true, 'Element'),
    'description': const FhirField(false, 'Markdown'),
    'descriptionElement': const FhirField(false, 'Element'),
    'contact': const FhirField(true, 'ExtendedContactDetail'),
    'partOf': const FhirField(false, 'Reference'),
    'endpoint': const FhirField(true, 'Reference'),
    'qualification': const FhirField(true, 'OrganizationQualification'),
  },
  'OrganizationQualification': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'code': const FhirField(false, 'CodeableConcept'),
    'period': const FhirField(false, 'Period'),
    'issuer': const FhirField(false, 'Reference'),
  },
  'OrganizationAffiliation': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'active': const FhirField(false, 'Boolean'),
    'activeElement': const FhirField(false, 'Element'),
    'period': const FhirField(false, 'Period'),
    'organization': const FhirField(false, 'Reference'),
    'participatingOrganization': const FhirField(false, 'Reference'),
    'network': const FhirField(true, 'Reference'),
    'code': const FhirField(true, 'CodeableConcept'),
    'specialty': const FhirField(true, 'CodeableConcept'),
    'location': const FhirField(true, 'Reference'),
    'healthcareService': const FhirField(true, 'Reference'),
    'contact': const FhirField(true, 'ExtendedContactDetail'),
    'endpoint': const FhirField(true, 'Reference'),
  },
  'PackagedProductDefinition': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'name': const FhirField(false, 'String'),
    'nameElement': const FhirField(false, 'Element'),
    'type': const FhirField(false, 'CodeableConcept'),
    'packageFor': const FhirField(true, 'Reference'),
    'status': const FhirField(false, 'CodeableConcept'),
    'statusDate': const FhirField(false, 'FhirDateTime'),
    'statusDateElement': const FhirField(false, 'Element'),
    'containedItemQuantity': const FhirField(true, 'Quantity'),
    'description': const FhirField(false, 'Markdown'),
    'descriptionElement': const FhirField(false, 'Element'),
    'legalStatusOfSupply':
        const FhirField(true, 'PackagedProductDefinitionLegalStatusOfSupply'),
    'marketingStatus': const FhirField(true, 'MarketingStatus'),
    'copackagedIndicator': const FhirField(false, 'Boolean'),
    'copackagedIndicatorElement': const FhirField(false, 'Element'),
    'manufacturer': const FhirField(true, 'Reference'),
    'attachedDocument': const FhirField(true, 'Reference'),
    'packaging': const FhirField(false, 'PackagedProductDefinitionPackaging'),
    'characteristic':
        const FhirField(true, 'PackagedProductDefinitionProperty'),
  },
  'PackagedProductDefinitionLegalStatusOfSupply': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(false, 'CodeableConcept'),
    'jurisdiction': const FhirField(false, 'CodeableConcept'),
  },
  'PackagedProductDefinitionPackaging': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'type': const FhirField(false, 'CodeableConcept'),
    'componentPart': const FhirField(false, 'Boolean'),
    'componentPartElement': const FhirField(false, 'Element'),
    'quantity': const FhirField(false, 'Integer'),
    'quantityElement': const FhirField(false, 'Element'),
    'material': const FhirField(true, 'CodeableConcept'),
    'alternateMaterial': const FhirField(true, 'CodeableConcept'),
    'shelfLifeStorage': const FhirField(true, 'ProductShelfLife'),
    'manufacturer': const FhirField(true, 'Reference'),
    'property': const FhirField(true, 'PackagedProductDefinitionProperty'),
    'containedItem':
        const FhirField(true, 'PackagedProductDefinitionContainedItem'),
    'packaging': const FhirField(true, 'PackagedProductDefinitionPackaging'),
  },
  'PackagedProductDefinitionProperty': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'valueCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'valueQuantity': const FhirField(false, 'Quantity'),
    'valueDate': const FhirField(false, 'Date'),
    'valueDateElement': const FhirField(false, 'Element'),
    'valueBoolean': const FhirField(false, 'Boolean'),
    'valueBooleanElement': const FhirField(false, 'Element'),
    'valueAttachment': const FhirField(false, 'Attachment'),
  },
  'PackagedProductDefinitionContainedItem': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'item': const FhirField(false, 'CodeableReference'),
    'amount': const FhirField(false, 'Quantity'),
  },
  'Parameters': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'parameter': const FhirField(true, 'ParametersParameter'),
  },
  'ParametersParameter': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'name': const FhirField(false, 'String'),
    'nameElement': const FhirField(false, 'Element'),
    'valueBase64Binary': const FhirField(false, 'Base64Binary'),
    'valueBase64BinaryElement': const FhirField(false, 'Element'),
    'valueBoolean': const FhirField(false, 'Boolean'),
    'valueBooleanElement': const FhirField(false, 'Element'),
    'valueCanonical': const FhirField(false, 'Canonical'),
    'valueCanonicalElement': const FhirField(false, 'Element'),
    'valueCode': const FhirField(false, 'Code'),
    'valueCodeElement': const FhirField(false, 'Element'),
    'valueDate': const FhirField(false, 'Date'),
    'valueDateElement': const FhirField(false, 'Element'),
    'valueDateTime': const FhirField(false, 'FhirDateTime'),
    'valueDateTimeElement': const FhirField(false, 'Element'),
    'valueDecimal': const FhirField(false, 'Decimal'),
    'valueDecimalElement': const FhirField(false, 'Element'),
    'valueId': const FhirField(false, 'Id'),
    'valueIdElement': const FhirField(false, 'Element'),
    'valueInstant': const FhirField(false, 'Instant'),
    'valueInstantElement': const FhirField(false, 'Element'),
    'valueInteger': const FhirField(false, 'Integer'),
    'valueIntegerElement': const FhirField(false, 'Element'),
    'valueInteger64': const FhirField(false, 'Integer64'),
    'valueInteger64Element': const FhirField(false, 'Element'),
    'valueMarkdown': const FhirField(false, 'Markdown'),
    'valueMarkdownElement': const FhirField(false, 'Element'),
    'valueOid': const FhirField(false, 'Id'),
    'valueOidElement': const FhirField(false, 'Element'),
    'valuePositiveInt': const FhirField(false, 'PositiveInt'),
    'valuePositiveIntElement': const FhirField(false, 'Element'),
    'valueString': const FhirField(false, 'String'),
    'valueStringElement': const FhirField(false, 'Element'),
    'valueTime': const FhirField(false, 'Time'),
    'valueTimeElement': const FhirField(false, 'Element'),
    'valueUnsignedInt': const FhirField(false, 'UnsignedInt'),
    'valueUnsignedIntElement': const FhirField(false, 'Element'),
    'valueUri': const FhirField(false, 'FhirUri'),
    'valueUriElement': const FhirField(false, 'Element'),
    'valueUrl': const FhirField(false, 'FhirUrl'),
    'valueUrlElement': const FhirField(false, 'Element'),
    'valueUuid': const FhirField(false, 'Id'),
    'valueUuidElement': const FhirField(false, 'Element'),
    'valueAddress': const FhirField(false, 'Address'),
    'valueAge': const FhirField(false, 'Age'),
    'valueAnnotation': const FhirField(false, 'Annotation'),
    'valueAttachment': const FhirField(false, 'Attachment'),
    'valueCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'valueCodeableReference': const FhirField(false, 'CodeableReference'),
    'valueCoding': const FhirField(false, 'Coding'),
    'valueContactPoint': const FhirField(false, 'ContactPoint'),
    'valueCount': const FhirField(false, 'Count'),
    'valueDistance': const FhirField(false, 'Distance'),
    'valueDuration': const FhirField(false, 'FhirDuration'),
    'valueHumanName': const FhirField(false, 'HumanName'),
    'valueIdentifier': const FhirField(false, 'Identifier'),
    'valueMoney': const FhirField(false, 'Money'),
    'valuePeriod': const FhirField(false, 'Period'),
    'valueQuantity': const FhirField(false, 'Quantity'),
    'valueRange': const FhirField(false, 'Range'),
    'valueRatio': const FhirField(false, 'Ratio'),
    'valueRatioRange': const FhirField(false, 'RatioRange'),
    'valueReference': const FhirField(false, 'Reference'),
    'valueSampledData': const FhirField(false, 'SampledData'),
    'valueSignature': const FhirField(false, 'Signature'),
    'valueTiming': const FhirField(false, 'Timing'),
    'valueContactDetail': const FhirField(false, 'ContactDetail'),
    'valueDataRequirement': const FhirField(false, 'DataRequirement'),
    'valueExpression': const FhirField(false, 'Expression'),
    'valueParameterDefinition': const FhirField(false, 'ParameterDefinition'),
    'valueRelatedArtifact': const FhirField(false, 'RelatedArtifact'),
    'valueTriggerDefinition': const FhirField(false, 'TriggerDefinition'),
    'valueUsageContext': const FhirField(false, 'UsageContext'),
    'valueAvailability': const FhirField(false, 'Availability'),
    'valueExtendedContactDetail':
        const FhirField(false, 'ExtendedContactDetail'),
    'valueDosage': const FhirField(false, 'Dosage'),
    'valueMeta': const FhirField(false, 'Meta'),
    'resource': const FhirField(false, 'ResourceList'),
    'part': const FhirField(true, 'ParametersParameter'),
  },
  'Patient': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'active': const FhirField(false, 'Boolean'),
    'activeElement': const FhirField(false, 'Element'),
    'name': const FhirField(true, 'HumanName'),
    'telecom': const FhirField(true, 'ContactPoint'),
    'gender': const FhirField(false, 'Code'),
    'genderElement': const FhirField(false, 'Element'),
    'birthDate': const FhirField(false, 'Date'),
    'birthDateElement': const FhirField(false, 'Element'),
    'deceasedBoolean': const FhirField(false, 'Boolean'),
    'deceasedBooleanElement': const FhirField(false, 'Element'),
    'deceasedDateTime': const FhirField(false, 'FhirDateTime'),
    'deceasedDateTimeElement': const FhirField(false, 'Element'),
    'address': const FhirField(true, 'Address'),
    'maritalStatus': const FhirField(false, 'CodeableConcept'),
    'multipleBirthBoolean': const FhirField(false, 'Boolean'),
    'multipleBirthBooleanElement': const FhirField(false, 'Element'),
    'multipleBirthInteger': const FhirField(false, 'Integer'),
    'multipleBirthIntegerElement': const FhirField(false, 'Element'),
    'photo': const FhirField(true, 'Attachment'),
    'contact': const FhirField(true, 'PatientContact'),
    'communication': const FhirField(true, 'PatientCommunication'),
    'generalPractitioner': const FhirField(true, 'Reference'),
    'managingOrganization': const FhirField(false, 'Reference'),
    'link': const FhirField(true, 'PatientLink'),
  },
  'PatientContact': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'relationship': const FhirField(true, 'CodeableConcept'),
    'name': const FhirField(false, 'HumanName'),
    'telecom': const FhirField(true, 'ContactPoint'),
    'address': const FhirField(false, 'Address'),
    'gender': const FhirField(false, 'Code'),
    'genderElement': const FhirField(false, 'Element'),
    'organization': const FhirField(false, 'Reference'),
    'period': const FhirField(false, 'Period'),
  },
  'PatientCommunication': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'language': const FhirField(false, 'CodeableConcept'),
    'preferred': const FhirField(false, 'Boolean'),
    'preferredElement': const FhirField(false, 'Element'),
  },
  'PatientLink': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'other': const FhirField(false, 'Reference'),
    'type': const FhirField(false, 'Code'),
    'typeElement': const FhirField(false, 'Element'),
  },
  'PaymentNotice': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'request': const FhirField(false, 'Reference'),
    'response': const FhirField(false, 'Reference'),
    'created': const FhirField(false, 'FhirDateTime'),
    'createdElement': const FhirField(false, 'Element'),
    'reporter': const FhirField(false, 'Reference'),
    'payment': const FhirField(false, 'Reference'),
    'paymentDate': const FhirField(false, 'Date'),
    'paymentDateElement': const FhirField(false, 'Element'),
    'payee': const FhirField(false, 'Reference'),
    'recipient': const FhirField(false, 'Reference'),
    'amount': const FhirField(false, 'Money'),
    'paymentStatus': const FhirField(false, 'CodeableConcept'),
  },
  'PaymentReconciliation': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'type': const FhirField(false, 'CodeableConcept'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'kind': const FhirField(false, 'CodeableConcept'),
    'period': const FhirField(false, 'Period'),
    'created': const FhirField(false, 'FhirDateTime'),
    'createdElement': const FhirField(false, 'Element'),
    'enterer': const FhirField(false, 'Reference'),
    'issuerType': const FhirField(false, 'CodeableConcept'),
    'paymentIssuer': const FhirField(false, 'Reference'),
    'request': const FhirField(false, 'Reference'),
    'requestor': const FhirField(false, 'Reference'),
    'outcome': const FhirField(false, 'Code'),
    'outcomeElement': const FhirField(false, 'Element'),
    'disposition': const FhirField(false, 'String'),
    'dispositionElement': const FhirField(false, 'Element'),
    'date': const FhirField(false, 'Date'),
    'dateElement': const FhirField(false, 'Element'),
    'location': const FhirField(false, 'Reference'),
    'method': const FhirField(false, 'CodeableConcept'),
    'cardBrand': const FhirField(false, 'String'),
    'cardBrandElement': const FhirField(false, 'Element'),
    'accountNumber': const FhirField(false, 'String'),
    'accountNumberElement': const FhirField(false, 'Element'),
    'expirationDate': const FhirField(false, 'Date'),
    'expirationDateElement': const FhirField(false, 'Element'),
    'processor': const FhirField(false, 'String'),
    'processorElement': const FhirField(false, 'Element'),
    'referenceNumber': const FhirField(false, 'String'),
    'referenceNumberElement': const FhirField(false, 'Element'),
    'authorization': const FhirField(false, 'String'),
    'authorizationElement': const FhirField(false, 'Element'),
    'tenderedAmount': const FhirField(false, 'Money'),
    'returnedAmount': const FhirField(false, 'Money'),
    'amount': const FhirField(false, 'Money'),
    'paymentIdentifier': const FhirField(false, 'Identifier'),
    'allocation': const FhirField(true, 'PaymentReconciliationAllocation'),
    'formCode': const FhirField(false, 'CodeableConcept'),
    'processNote': const FhirField(true, 'PaymentReconciliationProcessNote'),
  },
  'PaymentReconciliationAllocation': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(false, 'Identifier'),
    'predecessor': const FhirField(false, 'Identifier'),
    'target': const FhirField(false, 'Reference'),
    'targetItemString': const FhirField(false, 'String'),
    'targetItemStringElement': const FhirField(false, 'Element'),
    'targetItemIdentifier': const FhirField(false, 'Identifier'),
    'targetItemPositiveInt': const FhirField(false, 'PositiveInt'),
    'targetItemPositiveIntElement': const FhirField(false, 'Element'),
    'encounter': const FhirField(false, 'Reference'),
    'account': const FhirField(false, 'Reference'),
    'type': const FhirField(false, 'CodeableConcept'),
    'submitter': const FhirField(false, 'Reference'),
    'response': const FhirField(false, 'Reference'),
    'date': const FhirField(false, 'Date'),
    'dateElement': const FhirField(false, 'Element'),
    'responsible': const FhirField(false, 'Reference'),
    'payee': const FhirField(false, 'Reference'),
    'amount': const FhirField(false, 'Money'),
  },
  'PaymentReconciliationProcessNote': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'Code'),
    'typeElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'String'),
    'textElement': const FhirField(false, 'Element'),
  },
  'Permission': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'asserter': const FhirField(false, 'Reference'),
    'date': const FhirField(true, 'FhirDateTime'),
    'dateElement': const FhirField(true, 'Element'),
    'validity': const FhirField(false, 'Period'),
    'justification': const FhirField(false, 'PermissionJustification'),
    'combining': const FhirField(false, 'Code'),
    'combiningElement': const FhirField(false, 'Element'),
    'rule': const FhirField(true, 'PermissionRule'),
  },
  'PermissionJustification': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'basis': const FhirField(true, 'CodeableConcept'),
    'evidence': const FhirField(true, 'Reference'),
  },
  'PermissionRule': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'Code'),
    'typeElement': const FhirField(false, 'Element'),
    'data': const FhirField(true, 'PermissionData'),
    'activity': const FhirField(true, 'PermissionActivity'),
    'limit': const FhirField(true, 'CodeableConcept'),
  },
  'PermissionData': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'resource': const FhirField(true, 'PermissionResource'),
    'security': const FhirField(true, 'Coding'),
    'period': const FhirField(true, 'Period'),
    'expression': const FhirField(false, 'Expression'),
  },
  'PermissionResource': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'meaning': const FhirField(false, 'Code'),
    'meaningElement': const FhirField(false, 'Element'),
    'reference': const FhirField(false, 'Reference'),
  },
  'PermissionActivity': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'actor': const FhirField(true, 'Reference'),
    'action': const FhirField(true, 'CodeableConcept'),
    'purpose': const FhirField(true, 'CodeableConcept'),
  },
  'Person': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'active': const FhirField(false, 'Boolean'),
    'activeElement': const FhirField(false, 'Element'),
    'name': const FhirField(true, 'HumanName'),
    'telecom': const FhirField(true, 'ContactPoint'),
    'gender': const FhirField(false, 'Code'),
    'genderElement': const FhirField(false, 'Element'),
    'birthDate': const FhirField(false, 'Date'),
    'birthDateElement': const FhirField(false, 'Element'),
    'deceasedBoolean': const FhirField(false, 'Boolean'),
    'deceasedBooleanElement': const FhirField(false, 'Element'),
    'deceasedDateTime': const FhirField(false, 'FhirDateTime'),
    'deceasedDateTimeElement': const FhirField(false, 'Element'),
    'address': const FhirField(true, 'Address'),
    'maritalStatus': const FhirField(false, 'CodeableConcept'),
    'photo': const FhirField(true, 'Attachment'),
    'communication': const FhirField(true, 'PersonCommunication'),
    'managingOrganization': const FhirField(false, 'Reference'),
    'link': const FhirField(true, 'PersonLink'),
  },
  'PersonCommunication': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'language': const FhirField(false, 'CodeableConcept'),
    'preferred': const FhirField(false, 'Boolean'),
    'preferredElement': const FhirField(false, 'Element'),
  },
  'PersonLink': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'target': const FhirField(false, 'Reference'),
    'assurance': const FhirField(false, 'Code'),
    'assuranceElement': const FhirField(false, 'Element'),
  },
  'PlanDefinition': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'url': const FhirField(false, 'FhirUri'),
    'urlElement': const FhirField(false, 'Element'),
    'identifier': const FhirField(true, 'Identifier'),
    'version': const FhirField(false, 'String'),
    'versionElement': const FhirField(false, 'Element'),
    'versionAlgorithmString': const FhirField(false, 'String'),
    'versionAlgorithmStringElement': const FhirField(false, 'Element'),
    'versionAlgorithmCoding': const FhirField(false, 'Coding'),
    'name': const FhirField(false, 'String'),
    'nameElement': const FhirField(false, 'Element'),
    'title': const FhirField(false, 'String'),
    'titleElement': const FhirField(false, 'Element'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'experimental': const FhirField(false, 'Boolean'),
    'experimentalElement': const FhirField(false, 'Element'),
    'date': const FhirField(false, 'FhirDateTime'),
    'dateElement': const FhirField(false, 'Element'),
    'publisher': const FhirField(false, 'String'),
    'publisherElement': const FhirField(false, 'Element'),
    'contact': const FhirField(true, 'ContactDetail'),
    'description': const FhirField(false, 'Markdown'),
    'descriptionElement': const FhirField(false, 'Element'),
    'useContext': const FhirField(true, 'UsageContext'),
    'jurisdiction': const FhirField(true, 'CodeableConcept'),
    'purpose': const FhirField(false, 'Markdown'),
    'purposeElement': const FhirField(false, 'Element'),
    'copyright': const FhirField(false, 'Markdown'),
    'copyrightElement': const FhirField(false, 'Element'),
    'copyrightLabel': const FhirField(false, 'String'),
    'copyrightLabelElement': const FhirField(false, 'Element'),
    'approvalDate': const FhirField(false, 'Date'),
    'approvalDateElement': const FhirField(false, 'Element'),
    'lastReviewDate': const FhirField(false, 'Date'),
    'lastReviewDateElement': const FhirField(false, 'Element'),
    'effectivePeriod': const FhirField(false, 'Period'),
    'topic': const FhirField(true, 'CodeableConcept'),
    'author': const FhirField(true, 'ContactDetail'),
    'editor': const FhirField(true, 'ContactDetail'),
    'reviewer': const FhirField(true, 'ContactDetail'),
    'endorser': const FhirField(true, 'ContactDetail'),
    'relatedArtifact': const FhirField(true, 'RelatedArtifact'),
    'subtitle': const FhirField(false, 'String'),
    'subtitleElement': const FhirField(false, 'Element'),
    'type': const FhirField(false, 'CodeableConcept'),
    'subjectCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'subjectReference': const FhirField(false, 'Reference'),
    'subjectCanonical': const FhirField(false, 'Canonical'),
    'subjectCanonicalElement': const FhirField(false, 'Element'),
    'usage': const FhirField(false, 'Markdown'),
    'usageElement': const FhirField(false, 'Element'),
    'library': const FhirField(true, 'Canonical'),
    'goal': const FhirField(true, 'PlanDefinitionGoal'),
    'actor': const FhirField(true, 'PlanDefinitionActor'),
    'action': const FhirField(true, 'PlanDefinitionAction'),
    'asNeededBoolean': const FhirField(false, 'Boolean'),
    'asNeededBooleanElement': const FhirField(false, 'Element'),
    'asNeededCodeableConcept': const FhirField(false, 'CodeableConcept'),
  },
  'PlanDefinitionGoal': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'category': const FhirField(false, 'CodeableConcept'),
    'description': const FhirField(false, 'CodeableConcept'),
    'priority': const FhirField(false, 'CodeableConcept'),
    'start': const FhirField(false, 'CodeableConcept'),
    'addresses': const FhirField(true, 'CodeableConcept'),
    'documentation': const FhirField(true, 'RelatedArtifact'),
    'target': const FhirField(true, 'PlanDefinitionTarget'),
  },
  'PlanDefinitionTarget': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'measure': const FhirField(false, 'CodeableConcept'),
    'detailQuantity': const FhirField(false, 'Quantity'),
    'detailRange': const FhirField(false, 'Range'),
    'detailCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'detailString': const FhirField(false, 'String'),
    'detailStringElement': const FhirField(false, 'Element'),
    'detailBoolean': const FhirField(false, 'Boolean'),
    'detailBooleanElement': const FhirField(false, 'Element'),
    'detailInteger': const FhirField(false, 'Integer'),
    'detailIntegerElement': const FhirField(false, 'Element'),
    'detailRatio': const FhirField(false, 'Ratio'),
    'due': const FhirField(false, 'FhirDuration'),
  },
  'PlanDefinitionActor': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'title': const FhirField(false, 'String'),
    'titleElement': const FhirField(false, 'Element'),
    'description': const FhirField(false, 'Markdown'),
    'descriptionElement': const FhirField(false, 'Element'),
    'option': const FhirField(true, 'PlanDefinitionOption'),
  },
  'PlanDefinitionOption': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'Code'),
    'typeElement': const FhirField(false, 'Element'),
    'typeCanonical': const FhirField(false, 'Canonical'),
    'typeReference': const FhirField(false, 'Reference'),
    'role': const FhirField(false, 'CodeableConcept'),
  },
  'PlanDefinitionAction': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'linkId': const FhirField(false, 'String'),
    'linkIdElement': const FhirField(false, 'Element'),
    'prefix': const FhirField(false, 'String'),
    'prefixElement': const FhirField(false, 'Element'),
    'title': const FhirField(false, 'String'),
    'titleElement': const FhirField(false, 'Element'),
    'description': const FhirField(false, 'Markdown'),
    'descriptionElement': const FhirField(false, 'Element'),
    'textEquivalent': const FhirField(false, 'Markdown'),
    'textEquivalentElement': const FhirField(false, 'Element'),
    'priority': const FhirField(false, 'Code'),
    'priorityElement': const FhirField(false, 'Element'),
    'code': const FhirField(false, 'CodeableConcept'),
    'reason': const FhirField(true, 'CodeableConcept'),
    'documentation': const FhirField(true, 'RelatedArtifact'),
    'goalId': const FhirField(true, 'Id'),
    'goalIdElement': const FhirField(true, 'Element'),
    'subjectCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'subjectReference': const FhirField(false, 'Reference'),
    'subjectCanonical': const FhirField(false, 'Canonical'),
    'subjectCanonicalElement': const FhirField(false, 'Element'),
    'trigger': const FhirField(true, 'TriggerDefinition'),
    'condition': const FhirField(true, 'PlanDefinitionCondition'),
    'input': const FhirField(true, 'PlanDefinitionInput'),
    'output': const FhirField(true, 'PlanDefinitionOutput'),
    'relatedAction': const FhirField(true, 'PlanDefinitionRelatedAction'),
    'timingAge': const FhirField(false, 'Age'),
    'timingDuration': const FhirField(false, 'FhirDuration'),
    'timingRange': const FhirField(false, 'Range'),
    'timingTiming': const FhirField(false, 'Timing'),
    'location': const FhirField(false, 'CodeableReference'),
    'participant': const FhirField(true, 'PlanDefinitionParticipant'),
    'type': const FhirField(false, 'CodeableConcept'),
    'groupingBehavior': const FhirField(false, 'Code'),
    'groupingBehaviorElement': const FhirField(false, 'Element'),
    'selectionBehavior': const FhirField(false, 'Code'),
    'selectionBehaviorElement': const FhirField(false, 'Element'),
    'requiredBehavior': const FhirField(false, 'Code'),
    'requiredBehaviorElement': const FhirField(false, 'Element'),
    'precheckBehavior': const FhirField(false, 'Code'),
    'precheckBehaviorElement': const FhirField(false, 'Element'),
    'cardinalityBehavior': const FhirField(false, 'Code'),
    'cardinalityBehaviorElement': const FhirField(false, 'Element'),
    'definitionCanonical': const FhirField(false, 'Canonical'),
    'definitionCanonicalElement': const FhirField(false, 'Element'),
    'definitionUri': const FhirField(false, 'FhirUri'),
    'definitionUriElement': const FhirField(false, 'Element'),
    'transform': const FhirField(false, 'Canonical'),
    'dynamicValue': const FhirField(true, 'PlanDefinitionDynamicValue'),
    'action': const FhirField(true, 'PlanDefinitionAction'),
  },
  'PlanDefinitionCondition': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'kind': const FhirField(false, 'Code'),
    'kindElement': const FhirField(false, 'Element'),
    'expression': const FhirField(false, 'Expression'),
  },
  'PlanDefinitionInput': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'title': const FhirField(false, 'String'),
    'titleElement': const FhirField(false, 'Element'),
    'requirement': const FhirField(false, 'DataRequirement'),
    'relatedData': const FhirField(false, 'Id'),
    'relatedDataElement': const FhirField(false, 'Element'),
  },
  'PlanDefinitionOutput': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'title': const FhirField(false, 'String'),
    'titleElement': const FhirField(false, 'Element'),
    'requirement': const FhirField(false, 'DataRequirement'),
    'relatedData': const FhirField(false, 'String'),
    'relatedDataElement': const FhirField(false, 'Element'),
  },
  'PlanDefinitionRelatedAction': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'targetId': const FhirField(false, 'Id'),
    'targetIdElement': const FhirField(false, 'Element'),
    'relationship': const FhirField(false, 'Code'),
    'relationshipElement': const FhirField(false, 'Element'),
    'endRelationship': const FhirField(false, 'Code'),
    'endRelationshipElement': const FhirField(false, 'Element'),
    'offsetDuration': const FhirField(false, 'FhirDuration'),
    'offsetRange': const FhirField(false, 'Range'),
  },
  'PlanDefinitionParticipant': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'actorId': const FhirField(false, 'String'),
    'actorIdElement': const FhirField(false, 'Element'),
    'type': const FhirField(false, 'Code'),
    'typeElement': const FhirField(false, 'Element'),
    'typeCanonical': const FhirField(false, 'Canonical'),
    'typeReference': const FhirField(false, 'Reference'),
    'role': const FhirField(false, 'CodeableConcept'),
    'function': const FhirField(false, 'CodeableConcept'),
  },
  'PlanDefinitionDynamicValue': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'path': const FhirField(false, 'String'),
    'pathElement': const FhirField(false, 'Element'),
    'expression': const FhirField(false, 'Expression'),
  },
  'Practitioner': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'active': const FhirField(false, 'Boolean'),
    'activeElement': const FhirField(false, 'Element'),
    'name': const FhirField(true, 'HumanName'),
    'telecom': const FhirField(true, 'ContactPoint'),
    'gender': const FhirField(false, 'Code'),
    'genderElement': const FhirField(false, 'Element'),
    'birthDate': const FhirField(false, 'Date'),
    'birthDateElement': const FhirField(false, 'Element'),
    'deceasedBoolean': const FhirField(false, 'Boolean'),
    'deceasedBooleanElement': const FhirField(false, 'Element'),
    'deceasedDateTime': const FhirField(false, 'FhirDateTime'),
    'deceasedDateTimeElement': const FhirField(false, 'Element'),
    'address': const FhirField(true, 'Address'),
    'photo': const FhirField(true, 'Attachment'),
    'qualification': const FhirField(true, 'PractitionerQualification'),
    'communication': const FhirField(true, 'PractitionerCommunication'),
  },
  'PractitionerQualification': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'code': const FhirField(false, 'CodeableConcept'),
    'period': const FhirField(false, 'Period'),
    'issuer': const FhirField(false, 'Reference'),
  },
  'PractitionerCommunication': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'language': const FhirField(false, 'CodeableConcept'),
    'preferred': const FhirField(false, 'Boolean'),
    'preferredElement': const FhirField(false, 'Element'),
  },
  'PractitionerRole': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'active': const FhirField(false, 'Boolean'),
    'activeElement': const FhirField(false, 'Element'),
    'period': const FhirField(false, 'Period'),
    'practitioner': const FhirField(false, 'Reference'),
    'organization': const FhirField(false, 'Reference'),
    'code': const FhirField(true, 'CodeableConcept'),
    'specialty': const FhirField(true, 'CodeableConcept'),
    'location': const FhirField(true, 'Reference'),
    'healthcareService': const FhirField(true, 'Reference'),
    'contact': const FhirField(true, 'ExtendedContactDetail'),
    'characteristic': const FhirField(true, 'CodeableConcept'),
    'communication': const FhirField(true, 'CodeableConcept'),
    'availability': const FhirField(true, 'Availability'),
    'endpoint': const FhirField(true, 'Reference'),
  },
  'Procedure': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'instantiatesCanonical': const FhirField(true, 'Canonical'),
    'instantiatesUri': const FhirField(true, 'FhirUri'),
    'instantiatesUriElement': const FhirField(true, 'Element'),
    'basedOn': const FhirField(true, 'Reference'),
    'partOf': const FhirField(true, 'Reference'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'statusReason': const FhirField(false, 'CodeableConcept'),
    'category': const FhirField(true, 'CodeableConcept'),
    'code': const FhirField(false, 'CodeableConcept'),
    'subject': const FhirField(false, 'Reference'),
    'focus': const FhirField(false, 'Reference'),
    'encounter': const FhirField(false, 'Reference'),
    'occurrenceDateTime': const FhirField(false, 'FhirDateTime'),
    'occurrenceDateTimeElement': const FhirField(false, 'Element'),
    'occurrencePeriod': const FhirField(false, 'Period'),
    'occurrenceString': const FhirField(false, 'String'),
    'occurrenceStringElement': const FhirField(false, 'Element'),
    'occurrenceAge': const FhirField(false, 'Age'),
    'occurrenceRange': const FhirField(false, 'Range'),
    'occurrenceTiming': const FhirField(false, 'Timing'),
    'recorded': const FhirField(false, 'FhirDateTime'),
    'recordedElement': const FhirField(false, 'Element'),
    'recorder': const FhirField(false, 'Reference'),
    'reportedBoolean': const FhirField(false, 'Boolean'),
    'reportedBooleanElement': const FhirField(false, 'Element'),
    'reportedReference': const FhirField(false, 'Reference'),
    'performer': const FhirField(true, 'ProcedurePerformer'),
    'location': const FhirField(false, 'Reference'),
    'reason': const FhirField(true, 'CodeableReference'),
    'bodySite': const FhirField(true, 'CodeableConcept'),
    'outcome': const FhirField(false, 'CodeableConcept'),
    'report': const FhirField(true, 'Reference'),
    'complication': const FhirField(true, 'CodeableReference'),
    'followUp': const FhirField(true, 'CodeableConcept'),
    'note': const FhirField(true, 'Annotation'),
    'focalDevice': const FhirField(true, 'ProcedureFocalDevice'),
    'used': const FhirField(true, 'CodeableReference'),
    'supportingInfo': const FhirField(true, 'Reference'),
  },
  'ProcedurePerformer': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'function': const FhirField(false, 'CodeableConcept'),
    'actor': const FhirField(false, 'Reference'),
    'onBehalfOf': const FhirField(false, 'Reference'),
    'period': const FhirField(false, 'Period'),
  },
  'ProcedureFocalDevice': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'action': const FhirField(false, 'CodeableConcept'),
    'manipulated': const FhirField(false, 'Reference'),
  },
  'Provenance': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'target': const FhirField(true, 'Reference'),
    'occurredPeriod': const FhirField(false, 'Period'),
    'occurredDateTime': const FhirField(false, 'FhirDateTime'),
    'occurredDateTimeElement': const FhirField(false, 'Element'),
    'recorded': const FhirField(false, 'Instant'),
    'recordedElement': const FhirField(false, 'Element'),
    'policy': const FhirField(true, 'FhirUri'),
    'policyElement': const FhirField(true, 'Element'),
    'location': const FhirField(false, 'Reference'),
    'authorization': const FhirField(true, 'CodeableReference'),
    'activity': const FhirField(false, 'CodeableConcept'),
    'basedOn': const FhirField(true, 'Reference'),
    'patient': const FhirField(false, 'Reference'),
    'encounter': const FhirField(false, 'Reference'),
    'agent': const FhirField(true, 'ProvenanceAgent'),
    'entity': const FhirField(true, 'ProvenanceEntity'),
    'signature': const FhirField(true, 'Signature'),
  },
  'ProvenanceAgent': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'role': const FhirField(true, 'CodeableConcept'),
    'who': const FhirField(false, 'Reference'),
    'onBehalfOf': const FhirField(false, 'Reference'),
  },
  'ProvenanceEntity': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'role': const FhirField(false, 'Code'),
    'roleElement': const FhirField(false, 'Element'),
    'what': const FhirField(false, 'Reference'),
    'agent': const FhirField(true, 'ProvenanceAgent'),
  },
  'Questionnaire': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'url': const FhirField(false, 'FhirUri'),
    'urlElement': const FhirField(false, 'Element'),
    'identifier': const FhirField(true, 'Identifier'),
    'version': const FhirField(false, 'String'),
    'versionElement': const FhirField(false, 'Element'),
    'versionAlgorithmString': const FhirField(false, 'String'),
    'versionAlgorithmStringElement': const FhirField(false, 'Element'),
    'versionAlgorithmCoding': const FhirField(false, 'Coding'),
    'name': const FhirField(false, 'String'),
    'nameElement': const FhirField(false, 'Element'),
    'title': const FhirField(false, 'String'),
    'titleElement': const FhirField(false, 'Element'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'experimental': const FhirField(false, 'Boolean'),
    'experimentalElement': const FhirField(false, 'Element'),
    'date': const FhirField(false, 'FhirDateTime'),
    'dateElement': const FhirField(false, 'Element'),
    'publisher': const FhirField(false, 'String'),
    'publisherElement': const FhirField(false, 'Element'),
    'contact': const FhirField(true, 'ContactDetail'),
    'description': const FhirField(false, 'Markdown'),
    'descriptionElement': const FhirField(false, 'Element'),
    'useContext': const FhirField(true, 'UsageContext'),
    'jurisdiction': const FhirField(true, 'CodeableConcept'),
    'purpose': const FhirField(false, 'Markdown'),
    'purposeElement': const FhirField(false, 'Element'),
    'copyright': const FhirField(false, 'Markdown'),
    'copyrightElement': const FhirField(false, 'Element'),
    'copyrightLabel': const FhirField(false, 'String'),
    'copyrightLabelElement': const FhirField(false, 'Element'),
    'approvalDate': const FhirField(false, 'Date'),
    'approvalDateElement': const FhirField(false, 'Element'),
    'lastReviewDate': const FhirField(false, 'Date'),
    'lastReviewDateElement': const FhirField(false, 'Element'),
    'effectivePeriod': const FhirField(false, 'Period'),
    'topic': const FhirField(true, 'CodeableConcept'),
    'author': const FhirField(true, 'ContactDetail'),
    'editor': const FhirField(true, 'ContactDetail'),
    'reviewer': const FhirField(true, 'ContactDetail'),
    'endorser': const FhirField(true, 'ContactDetail'),
    'relatedArtifact': const FhirField(true, 'RelatedArtifact'),
    'derivedFrom': const FhirField(true, 'Canonical'),
    'subjectType': const FhirField(true, 'Code'),
    'subjectTypeElement': const FhirField(true, 'Element'),
    'code': const FhirField(true, 'Coding'),
    'item': const FhirField(true, 'QuestionnaireItem'),
  },
  'QuestionnaireItem': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'linkId': const FhirField(false, 'String'),
    'linkIdElement': const FhirField(false, 'Element'),
    'definition': const FhirField(false, 'FhirUri'),
    'definitionElement': const FhirField(false, 'Element'),
    'code': const FhirField(true, 'Coding'),
    'prefix': const FhirField(false, 'String'),
    'prefixElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'String'),
    'textElement': const FhirField(false, 'Element'),
    'type': const FhirField(false, 'Code'),
    'typeElement': const FhirField(false, 'Element'),
    'enableWhen': const FhirField(true, 'QuestionnaireEnableWhen'),
    'enableBehavior': const FhirField(false, 'Code'),
    'enableBehaviorElement': const FhirField(false, 'Element'),
    'disabledDisplay': const FhirField(false, 'Code'),
    'disabledDisplayElement': const FhirField(false, 'Element'),
    'required': const FhirField(false, 'Boolean'),
    'requiredElement': const FhirField(false, 'Element'),
    'repeats': const FhirField(false, 'Boolean'),
    'repeatsElement': const FhirField(false, 'Element'),
    'readOnly': const FhirField(false, 'Boolean'),
    'readOnlyElement': const FhirField(false, 'Element'),
    'maxLength': const FhirField(false, 'Integer'),
    'maxLengthElement': const FhirField(false, 'Element'),
    'answerConstraint': const FhirField(false, 'Code'),
    'answerConstraintElement': const FhirField(false, 'Element'),
    'answerValueSet': const FhirField(false, 'Canonical'),
    'answerOption': const FhirField(true, 'QuestionnaireAnswerOption'),
    'initial': const FhirField(true, 'QuestionnaireInitial'),
    'item': const FhirField(true, 'QuestionnaireItem'),
  },
  'QuestionnaireEnableWhen': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'question': const FhirField(false, 'String'),
    'questionElement': const FhirField(false, 'Element'),
    'operator': const FhirField(false, 'Code'),
    'operatorElement': const FhirField(false, 'Element'),
    'answerBoolean': const FhirField(false, 'Boolean'),
    'answerBooleanElement': const FhirField(false, 'Element'),
    'answerDecimal': const FhirField(false, 'Decimal'),
    'answerDecimalElement': const FhirField(false, 'Element'),
    'answerInteger': const FhirField(false, 'Integer'),
    'answerIntegerElement': const FhirField(false, 'Element'),
    'answerDate': const FhirField(false, 'Date'),
    'answerDateElement': const FhirField(false, 'Element'),
    'answerDateTime': const FhirField(false, 'FhirDateTime'),
    'answerDateTimeElement': const FhirField(false, 'Element'),
    'answerTime': const FhirField(false, 'Time'),
    'answerTimeElement': const FhirField(false, 'Element'),
    'answerString': const FhirField(false, 'String'),
    'answerStringElement': const FhirField(false, 'Element'),
    'answerCoding': const FhirField(false, 'Coding'),
    'answerQuantity': const FhirField(false, 'Quantity'),
    'answerReference': const FhirField(false, 'Reference'),
  },
  'QuestionnaireAnswerOption': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'valueInteger': const FhirField(false, 'Integer'),
    'valueIntegerElement': const FhirField(false, 'Element'),
    'valueDate': const FhirField(false, 'Date'),
    'valueDateElement': const FhirField(false, 'Element'),
    'valueTime': const FhirField(false, 'Time'),
    'valueTimeElement': const FhirField(false, 'Element'),
    'valueString': const FhirField(false, 'String'),
    'valueStringElement': const FhirField(false, 'Element'),
    'valueCoding': const FhirField(false, 'Coding'),
    'valueReference': const FhirField(false, 'Reference'),
    'initialSelected': const FhirField(false, 'Boolean'),
    'initialSelectedElement': const FhirField(false, 'Element'),
  },
  'QuestionnaireInitial': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'valueBoolean': const FhirField(false, 'Boolean'),
    'valueBooleanElement': const FhirField(false, 'Element'),
    'valueDecimal': const FhirField(false, 'Decimal'),
    'valueDecimalElement': const FhirField(false, 'Element'),
    'valueInteger': const FhirField(false, 'Integer'),
    'valueIntegerElement': const FhirField(false, 'Element'),
    'valueDate': const FhirField(false, 'Date'),
    'valueDateElement': const FhirField(false, 'Element'),
    'valueDateTime': const FhirField(false, 'FhirDateTime'),
    'valueDateTimeElement': const FhirField(false, 'Element'),
    'valueTime': const FhirField(false, 'Time'),
    'valueTimeElement': const FhirField(false, 'Element'),
    'valueString': const FhirField(false, 'String'),
    'valueStringElement': const FhirField(false, 'Element'),
    'valueUri': const FhirField(false, 'FhirUri'),
    'valueUriElement': const FhirField(false, 'Element'),
    'valueAttachment': const FhirField(false, 'Attachment'),
    'valueCoding': const FhirField(false, 'Coding'),
    'valueQuantity': const FhirField(false, 'Quantity'),
    'valueReference': const FhirField(false, 'Reference'),
  },
  'QuestionnaireResponse': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'basedOn': const FhirField(true, 'Reference'),
    'partOf': const FhirField(true, 'Reference'),
    'questionnaire': const FhirField(false, 'Canonical'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'subject': const FhirField(false, 'Reference'),
    'encounter': const FhirField(false, 'Reference'),
    'authored': const FhirField(false, 'FhirDateTime'),
    'authoredElement': const FhirField(false, 'Element'),
    'author': const FhirField(false, 'Reference'),
    'source': const FhirField(false, 'Reference'),
    'item': const FhirField(true, 'QuestionnaireResponseItem'),
  },
  'QuestionnaireResponseItem': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'linkId': const FhirField(false, 'String'),
    'linkIdElement': const FhirField(false, 'Element'),
    'definition': const FhirField(false, 'FhirUri'),
    'definitionElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'String'),
    'textElement': const FhirField(false, 'Element'),
    'answer': const FhirField(true, 'QuestionnaireResponseAnswer'),
    'item': const FhirField(true, 'QuestionnaireResponseItem'),
  },
  'QuestionnaireResponseAnswer': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'valueBoolean': const FhirField(false, 'Boolean'),
    'valueBooleanElement': const FhirField(false, 'Element'),
    'valueDecimal': const FhirField(false, 'Decimal'),
    'valueDecimalElement': const FhirField(false, 'Element'),
    'valueInteger': const FhirField(false, 'Integer'),
    'valueIntegerElement': const FhirField(false, 'Element'),
    'valueDate': const FhirField(false, 'Date'),
    'valueDateElement': const FhirField(false, 'Element'),
    'valueDateTime': const FhirField(false, 'FhirDateTime'),
    'valueDateTimeElement': const FhirField(false, 'Element'),
    'valueTime': const FhirField(false, 'Time'),
    'valueTimeElement': const FhirField(false, 'Element'),
    'valueString': const FhirField(false, 'String'),
    'valueStringElement': const FhirField(false, 'Element'),
    'valueUri': const FhirField(false, 'FhirUri'),
    'valueUriElement': const FhirField(false, 'Element'),
    'valueAttachment': const FhirField(false, 'Attachment'),
    'valueCoding': const FhirField(false, 'Coding'),
    'valueQuantity': const FhirField(false, 'Quantity'),
    'valueReference': const FhirField(false, 'Reference'),
    'item': const FhirField(true, 'QuestionnaireResponseItem'),
  },
  'RegulatedAuthorization': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'subject': const FhirField(true, 'Reference'),
    'type': const FhirField(false, 'CodeableConcept'),
    'description': const FhirField(false, 'Markdown'),
    'descriptionElement': const FhirField(false, 'Element'),
    'region': const FhirField(true, 'CodeableConcept'),
    'status': const FhirField(false, 'CodeableConcept'),
    'statusDate': const FhirField(false, 'FhirDateTime'),
    'statusDateElement': const FhirField(false, 'Element'),
    'validityPeriod': const FhirField(false, 'Period'),
    'indication': const FhirField(true, 'CodeableReference'),
    'intendedUse': const FhirField(false, 'CodeableConcept'),
    'basis': const FhirField(true, 'CodeableConcept'),
    'holder': const FhirField(false, 'Reference'),
    'regulator': const FhirField(false, 'Reference'),
    'attachedDocument': const FhirField(true, 'Reference'),
    'case': const FhirField(false, 'RegulatedAuthorizationCase'),
  },
  'RegulatedAuthorizationCase': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(false, 'Identifier'),
    'type': const FhirField(false, 'CodeableConcept'),
    'status': const FhirField(false, 'CodeableConcept'),
    'datePeriod': const FhirField(false, 'Period'),
    'dateDateTime': const FhirField(false, 'FhirDateTime'),
    'dateDateTimeElement': const FhirField(false, 'Element'),
    'application': const FhirField(true, 'RegulatedAuthorizationCase'),
  },
  'RelatedPerson': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'active': const FhirField(false, 'Boolean'),
    'activeElement': const FhirField(false, 'Element'),
    'patient': const FhirField(false, 'Reference'),
    'relationship': const FhirField(true, 'CodeableConcept'),
    'name': const FhirField(true, 'HumanName'),
    'telecom': const FhirField(true, 'ContactPoint'),
    'gender': const FhirField(false, 'Code'),
    'genderElement': const FhirField(false, 'Element'),
    'birthDate': const FhirField(false, 'Date'),
    'birthDateElement': const FhirField(false, 'Element'),
    'address': const FhirField(true, 'Address'),
    'photo': const FhirField(true, 'Attachment'),
    'period': const FhirField(false, 'Period'),
    'communication': const FhirField(true, 'RelatedPersonCommunication'),
  },
  'RelatedPersonCommunication': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'language': const FhirField(false, 'CodeableConcept'),
    'preferred': const FhirField(false, 'Boolean'),
    'preferredElement': const FhirField(false, 'Element'),
  },
  'RequestOrchestration': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'instantiatesCanonical': const FhirField(true, 'Canonical'),
    'instantiatesCanonicalElement': const FhirField(true, 'Element'),
    'instantiatesUri': const FhirField(true, 'FhirUri'),
    'instantiatesUriElement': const FhirField(true, 'Element'),
    'basedOn': const FhirField(true, 'Reference'),
    'replaces': const FhirField(true, 'Reference'),
    'groupIdentifier': const FhirField(false, 'Identifier'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'intent': const FhirField(false, 'Code'),
    'intentElement': const FhirField(false, 'Element'),
    'priority': const FhirField(false, 'Code'),
    'priorityElement': const FhirField(false, 'Element'),
    'code': const FhirField(false, 'CodeableConcept'),
    'subject': const FhirField(false, 'Reference'),
    'encounter': const FhirField(false, 'Reference'),
    'authoredOn': const FhirField(false, 'FhirDateTime'),
    'authoredOnElement': const FhirField(false, 'Element'),
    'author': const FhirField(false, 'Reference'),
    'reason': const FhirField(true, 'CodeableReference'),
    'goal': const FhirField(true, 'Reference'),
    'note': const FhirField(true, 'Annotation'),
    'action': const FhirField(true, 'RequestOrchestrationAction'),
  },
  'RequestOrchestrationAction': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'linkId': const FhirField(false, 'String'),
    'linkIdElement': const FhirField(false, 'Element'),
    'prefix': const FhirField(false, 'String'),
    'prefixElement': const FhirField(false, 'Element'),
    'title': const FhirField(false, 'String'),
    'titleElement': const FhirField(false, 'Element'),
    'description': const FhirField(false, 'Markdown'),
    'descriptionElement': const FhirField(false, 'Element'),
    'textEquivalent': const FhirField(false, 'Markdown'),
    'textEquivalentElement': const FhirField(false, 'Element'),
    'priority': const FhirField(false, 'Code'),
    'priorityElement': const FhirField(false, 'Element'),
    'code': const FhirField(true, 'CodeableConcept'),
    'documentation': const FhirField(true, 'RelatedArtifact'),
    'goal': const FhirField(true, 'Reference'),
    'condition': const FhirField(true, 'RequestOrchestrationCondition'),
    'input': const FhirField(true, 'RequestOrchestrationInput'),
    'output': const FhirField(true, 'RequestOrchestrationOutput'),
    'relatedAction': const FhirField(true, 'RequestOrchestrationRelatedAction'),
    'timingDateTime': const FhirField(false, 'FhirDateTime'),
    'timingDateTimeElement': const FhirField(false, 'Element'),
    'timingAge': const FhirField(false, 'Age'),
    'timingPeriod': const FhirField(false, 'Period'),
    'timingDuration': const FhirField(false, 'FhirDuration'),
    'timingRange': const FhirField(false, 'Range'),
    'timingTiming': const FhirField(false, 'Timing'),
    'location': const FhirField(false, 'CodeableReference'),
    'participant': const FhirField(true, 'RequestOrchestrationParticipant'),
    'type': const FhirField(false, 'CodeableConcept'),
    'groupingBehavior': const FhirField(false, 'Code'),
    'groupingBehaviorElement': const FhirField(false, 'Element'),
    'selectionBehavior': const FhirField(false, 'Code'),
    'selectionBehaviorElement': const FhirField(false, 'Element'),
    'requiredBehavior': const FhirField(false, 'Code'),
    'requiredBehaviorElement': const FhirField(false, 'Element'),
    'precheckBehavior': const FhirField(false, 'Code'),
    'precheckBehaviorElement': const FhirField(false, 'Element'),
    'cardinalityBehavior': const FhirField(false, 'Code'),
    'cardinalityBehaviorElement': const FhirField(false, 'Element'),
    'resource': const FhirField(false, 'Reference'),
    'definitionCanonical': const FhirField(false, 'Canonical'),
    'definitionCanonicalElement': const FhirField(false, 'Element'),
    'definitionUri': const FhirField(false, 'FhirUri'),
    'definitionUriElement': const FhirField(false, 'Element'),
    'transform': const FhirField(false, 'Canonical'),
    'dynamicValue': const FhirField(true, 'RequestOrchestrationDynamicValue'),
    'action': const FhirField(true, 'RequestOrchestrationAction'),
  },
  'RequestOrchestrationCondition': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'kind': const FhirField(false, 'Code'),
    'kindElement': const FhirField(false, 'Element'),
    'expression': const FhirField(false, 'Expression'),
  },
  'RequestOrchestrationInput': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'title': const FhirField(false, 'String'),
    'titleElement': const FhirField(false, 'Element'),
    'requirement': const FhirField(false, 'DataRequirement'),
    'relatedData': const FhirField(false, 'Id'),
    'relatedDataElement': const FhirField(false, 'Element'),
  },
  'RequestOrchestrationOutput': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'title': const FhirField(false, 'String'),
    'titleElement': const FhirField(false, 'Element'),
    'requirement': const FhirField(false, 'DataRequirement'),
    'relatedData': const FhirField(false, 'String'),
    'relatedDataElement': const FhirField(false, 'Element'),
  },
  'RequestOrchestrationRelatedAction': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'targetId': const FhirField(false, 'Id'),
    'targetIdElement': const FhirField(false, 'Element'),
    'relationship': const FhirField(false, 'Code'),
    'relationshipElement': const FhirField(false, 'Element'),
    'endRelationship': const FhirField(false, 'Code'),
    'endRelationshipElement': const FhirField(false, 'Element'),
    'offsetDuration': const FhirField(false, 'FhirDuration'),
    'offsetRange': const FhirField(false, 'Range'),
  },
  'RequestOrchestrationParticipant': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'Code'),
    'typeElement': const FhirField(false, 'Element'),
    'typeCanonical': const FhirField(false, 'Canonical'),
    'typeReference': const FhirField(false, 'Reference'),
    'role': const FhirField(false, 'CodeableConcept'),
    'function': const FhirField(false, 'CodeableConcept'),
    'actorCanonical': const FhirField(false, 'Canonical'),
    'actorCanonicalElement': const FhirField(false, 'Element'),
    'actorReference': const FhirField(false, 'Reference'),
  },
  'RequestOrchestrationDynamicValue': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'path': const FhirField(false, 'String'),
    'pathElement': const FhirField(false, 'Element'),
    'expression': const FhirField(false, 'Expression'),
  },
  'Requirements': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'url': const FhirField(false, 'FhirUri'),
    'urlElement': const FhirField(false, 'Element'),
    'identifier': const FhirField(true, 'Identifier'),
    'version': const FhirField(false, 'String'),
    'versionElement': const FhirField(false, 'Element'),
    'versionAlgorithmString': const FhirField(false, 'String'),
    'versionAlgorithmStringElement': const FhirField(false, 'Element'),
    'versionAlgorithmCoding': const FhirField(false, 'Coding'),
    'name': const FhirField(false, 'String'),
    'nameElement': const FhirField(false, 'Element'),
    'title': const FhirField(false, 'String'),
    'titleElement': const FhirField(false, 'Element'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'experimental': const FhirField(false, 'Boolean'),
    'experimentalElement': const FhirField(false, 'Element'),
    'date': const FhirField(false, 'FhirDateTime'),
    'dateElement': const FhirField(false, 'Element'),
    'publisher': const FhirField(false, 'String'),
    'publisherElement': const FhirField(false, 'Element'),
    'contact': const FhirField(true, 'ContactDetail'),
    'description': const FhirField(false, 'Markdown'),
    'descriptionElement': const FhirField(false, 'Element'),
    'useContext': const FhirField(true, 'UsageContext'),
    'jurisdiction': const FhirField(true, 'CodeableConcept'),
    'purpose': const FhirField(false, 'Markdown'),
    'purposeElement': const FhirField(false, 'Element'),
    'copyright': const FhirField(false, 'Markdown'),
    'copyrightElement': const FhirField(false, 'Element'),
    'copyrightLabel': const FhirField(false, 'String'),
    'copyrightLabelElement': const FhirField(false, 'Element'),
    'derivedFrom': const FhirField(true, 'Canonical'),
    'reference': const FhirField(true, 'FhirUrl'),
    'referenceElement': const FhirField(true, 'Element'),
    'actor': const FhirField(true, 'Canonical'),
    'statement': const FhirField(true, 'RequirementsStatement'),
  },
  'RequirementsStatement': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'key': const FhirField(false, 'Id'),
    'keyElement': const FhirField(false, 'Element'),
    'label': const FhirField(false, 'String'),
    'labelElement': const FhirField(false, 'Element'),
    'conformance': const FhirField(true, 'Code'),
    'conformanceElement': const FhirField(true, 'Element'),
    'conditionality': const FhirField(false, 'Boolean'),
    'conditionalityElement': const FhirField(false, 'Element'),
    'requirement': const FhirField(false, 'Markdown'),
    'requirementElement': const FhirField(false, 'Element'),
    'derivedFrom': const FhirField(false, 'String'),
    'derivedFromElement': const FhirField(false, 'Element'),
    'parent': const FhirField(false, 'String'),
    'parentElement': const FhirField(false, 'Element'),
    'satisfiedBy': const FhirField(true, 'FhirUrl'),
    'satisfiedByElement': const FhirField(true, 'Element'),
    'reference': const FhirField(true, 'FhirUrl'),
    'referenceElement': const FhirField(true, 'Element'),
    'source': const FhirField(true, 'Reference'),
  },
  'ResearchStudy': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'url': const FhirField(false, 'FhirUri'),
    'urlElement': const FhirField(false, 'Element'),
    'identifier': const FhirField(true, 'Identifier'),
    'version': const FhirField(false, 'String'),
    'versionElement': const FhirField(false, 'Element'),
    'name': const FhirField(false, 'String'),
    'nameElement': const FhirField(false, 'Element'),
    'title': const FhirField(false, 'String'),
    'titleElement': const FhirField(false, 'Element'),
    'label': const FhirField(true, 'ResearchStudyLabel'),
    'protocol': const FhirField(true, 'Reference'),
    'partOf': const FhirField(true, 'Reference'),
    'relatedArtifact': const FhirField(true, 'RelatedArtifact'),
    'date': const FhirField(false, 'FhirDateTime'),
    'dateElement': const FhirField(false, 'Element'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'primaryPurposeType': const FhirField(false, 'CodeableConcept'),
    'phase': const FhirField(false, 'CodeableConcept'),
    'studyDesign': const FhirField(true, 'CodeableConcept'),
    'focus': const FhirField(true, 'CodeableReference'),
    'condition': const FhirField(true, 'CodeableConcept'),
    'keyword': const FhirField(true, 'CodeableConcept'),
    'region': const FhirField(true, 'CodeableConcept'),
    'descriptionSummary': const FhirField(false, 'Markdown'),
    'descriptionSummaryElement': const FhirField(false, 'Element'),
    'description': const FhirField(false, 'Markdown'),
    'descriptionElement': const FhirField(false, 'Element'),
    'period': const FhirField(false, 'Period'),
    'site': const FhirField(true, 'Reference'),
    'note': const FhirField(true, 'Annotation'),
    'classifier': const FhirField(true, 'CodeableConcept'),
    'associatedParty': const FhirField(true, 'ResearchStudyAssociatedParty'),
    'progressStatus': const FhirField(true, 'ResearchStudyProgressStatus'),
    'whyStopped': const FhirField(false, 'CodeableConcept'),
    'recruitment': const FhirField(false, 'ResearchStudyRecruitment'),
    'comparisonGroup': const FhirField(true, 'ResearchStudyComparisonGroup'),
    'objective': const FhirField(true, 'ResearchStudyObjective'),
    'outcomeMeasure': const FhirField(true, 'ResearchStudyOutcomeMeasure'),
    'result': const FhirField(true, 'Reference'),
  },
  'ResearchStudyLabel': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'value': const FhirField(false, 'String'),
    'valueElement': const FhirField(false, 'Element'),
  },
  'ResearchStudyAssociatedParty': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'name': const FhirField(false, 'String'),
    'nameElement': const FhirField(false, 'Element'),
    'role': const FhirField(false, 'CodeableConcept'),
    'period': const FhirField(true, 'Period'),
    'classifier': const FhirField(true, 'CodeableConcept'),
    'party': const FhirField(false, 'Reference'),
  },
  'ResearchStudyProgressStatus': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'state': const FhirField(false, 'CodeableConcept'),
    'actual': const FhirField(false, 'Boolean'),
    'actualElement': const FhirField(false, 'Element'),
    'period': const FhirField(false, 'Period'),
  },
  'ResearchStudyRecruitment': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'targetNumber': const FhirField(false, 'UnsignedInt'),
    'targetNumberElement': const FhirField(false, 'Element'),
    'actualNumber': const FhirField(false, 'UnsignedInt'),
    'actualNumberElement': const FhirField(false, 'Element'),
    'eligibility': const FhirField(false, 'Reference'),
    'actualGroup': const FhirField(false, 'Reference'),
  },
  'ResearchStudyComparisonGroup': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'linkId': const FhirField(false, 'Id'),
    'linkIdElement': const FhirField(false, 'Element'),
    'name': const FhirField(false, 'String'),
    'nameElement': const FhirField(false, 'Element'),
    'type': const FhirField(false, 'CodeableConcept'),
    'description': const FhirField(false, 'Markdown'),
    'descriptionElement': const FhirField(false, 'Element'),
    'intendedExposure': const FhirField(true, 'Reference'),
    'observedGroup': const FhirField(false, 'Reference'),
  },
  'ResearchStudyObjective': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'name': const FhirField(false, 'String'),
    'nameElement': const FhirField(false, 'Element'),
    'type': const FhirField(false, 'CodeableConcept'),
    'description': const FhirField(false, 'Markdown'),
    'descriptionElement': const FhirField(false, 'Element'),
  },
  'ResearchStudyOutcomeMeasure': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'name': const FhirField(false, 'String'),
    'nameElement': const FhirField(false, 'Element'),
    'type': const FhirField(true, 'CodeableConcept'),
    'description': const FhirField(false, 'Markdown'),
    'descriptionElement': const FhirField(false, 'Element'),
    'reference': const FhirField(false, 'Reference'),
  },
  'ResearchSubject': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'progress': const FhirField(true, 'ResearchSubjectProgress'),
    'period': const FhirField(false, 'Period'),
    'study': const FhirField(false, 'Reference'),
    'subject': const FhirField(false, 'Reference'),
    'assignedComparisonGroup': const FhirField(false, 'Id'),
    'assignedComparisonGroupElement': const FhirField(false, 'Element'),
    'actualComparisonGroup': const FhirField(false, 'Id'),
    'actualComparisonGroupElement': const FhirField(false, 'Element'),
    'consent': const FhirField(true, 'Reference'),
  },
  'ResearchSubjectProgress': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'subjectState': const FhirField(false, 'CodeableConcept'),
    'milestone': const FhirField(false, 'CodeableConcept'),
    'reason': const FhirField(false, 'CodeableConcept'),
    'startDate': const FhirField(false, 'FhirDateTime'),
    'startDateElement': const FhirField(false, 'Element'),
    'endDate': const FhirField(false, 'FhirDateTime'),
    'endDateElement': const FhirField(false, 'Element'),
  },
  'RiskAssessment': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'basedOn': const FhirField(false, 'Reference'),
    'parent': const FhirField(false, 'Reference'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'method': const FhirField(false, 'CodeableConcept'),
    'code': const FhirField(false, 'CodeableConcept'),
    'subject': const FhirField(false, 'Reference'),
    'encounter': const FhirField(false, 'Reference'),
    'occurrenceDateTime': const FhirField(false, 'FhirDateTime'),
    'occurrenceDateTimeElement': const FhirField(false, 'Element'),
    'occurrencePeriod': const FhirField(false, 'Period'),
    'condition': const FhirField(false, 'Reference'),
    'performer': const FhirField(false, 'Reference'),
    'reason': const FhirField(true, 'CodeableReference'),
    'basis': const FhirField(true, 'Reference'),
    'prediction': const FhirField(true, 'RiskAssessmentPrediction'),
    'mitigation': const FhirField(false, 'String'),
    'mitigationElement': const FhirField(false, 'Element'),
    'note': const FhirField(true, 'Annotation'),
  },
  'RiskAssessmentPrediction': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'outcome': const FhirField(false, 'CodeableConcept'),
    'probabilityDecimal': const FhirField(false, 'Decimal'),
    'probabilityDecimalElement': const FhirField(false, 'Element'),
    'probabilityRange': const FhirField(false, 'Range'),
    'qualitativeRisk': const FhirField(false, 'CodeableConcept'),
    'relativeRisk': const FhirField(false, 'Decimal'),
    'relativeRiskElement': const FhirField(false, 'Element'),
    'whenPeriod': const FhirField(false, 'Period'),
    'whenRange': const FhirField(false, 'Range'),
    'rationale': const FhirField(false, 'String'),
    'rationaleElement': const FhirField(false, 'Element'),
  },
  'Schedule': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'active': const FhirField(false, 'Boolean'),
    'activeElement': const FhirField(false, 'Element'),
    'serviceCategory': const FhirField(true, 'CodeableConcept'),
    'serviceType': const FhirField(true, 'CodeableReference'),
    'specialty': const FhirField(true, 'CodeableConcept'),
    'name': const FhirField(false, 'String'),
    'nameElement': const FhirField(false, 'Element'),
    'actor': const FhirField(true, 'Reference'),
    'planningHorizon': const FhirField(false, 'Period'),
    'comment': const FhirField(false, 'Markdown'),
    'commentElement': const FhirField(false, 'Element'),
  },
  'SearchParameter': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'url': const FhirField(false, 'FhirUri'),
    'urlElement': const FhirField(false, 'Element'),
    'identifier': const FhirField(true, 'Identifier'),
    'version': const FhirField(false, 'String'),
    'versionElement': const FhirField(false, 'Element'),
    'versionAlgorithmString': const FhirField(false, 'String'),
    'versionAlgorithmStringElement': const FhirField(false, 'Element'),
    'versionAlgorithmCoding': const FhirField(false, 'Coding'),
    'name': const FhirField(false, 'String'),
    'nameElement': const FhirField(false, 'Element'),
    'title': const FhirField(false, 'String'),
    'titleElement': const FhirField(false, 'Element'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'experimental': const FhirField(false, 'Boolean'),
    'experimentalElement': const FhirField(false, 'Element'),
    'date': const FhirField(false, 'FhirDateTime'),
    'dateElement': const FhirField(false, 'Element'),
    'publisher': const FhirField(false, 'String'),
    'publisherElement': const FhirField(false, 'Element'),
    'contact': const FhirField(true, 'ContactDetail'),
    'description': const FhirField(false, 'Markdown'),
    'descriptionElement': const FhirField(false, 'Element'),
    'useContext': const FhirField(true, 'UsageContext'),
    'jurisdiction': const FhirField(true, 'CodeableConcept'),
    'purpose': const FhirField(false, 'Markdown'),
    'purposeElement': const FhirField(false, 'Element'),
    'copyright': const FhirField(false, 'Markdown'),
    'copyrightElement': const FhirField(false, 'Element'),
    'copyrightLabel': const FhirField(false, 'String'),
    'copyrightLabelElement': const FhirField(false, 'Element'),
    'derivedFrom': const FhirField(false, 'Canonical'),
    'code': const FhirField(false, 'Code'),
    'codeElement': const FhirField(false, 'Element'),
    'base': const FhirField(true, 'Code'),
    'baseElement': const FhirField(true, 'Element'),
    'type': const FhirField(false, 'Code'),
    'typeElement': const FhirField(false, 'Element'),
    'expression': const FhirField(false, 'String'),
    'expressionElement': const FhirField(false, 'Element'),
    'processingMode': const FhirField(false, 'Code'),
    'processingModeElement': const FhirField(false, 'Element'),
    'constraint': const FhirField(false, 'String'),
    'constraintElement': const FhirField(false, 'Element'),
    'target': const FhirField(true, 'Code'),
    'targetElement': const FhirField(true, 'Element'),
    'multipleOr': const FhirField(false, 'Boolean'),
    'multipleOrElement': const FhirField(false, 'Element'),
    'multipleAnd': const FhirField(false, 'Boolean'),
    'multipleAndElement': const FhirField(false, 'Element'),
    'comparator': const FhirField(true, 'Code'),
    'comparatorElement': const FhirField(true, 'Element'),
    'modifier': const FhirField(true, 'Code'),
    'modifierElement': const FhirField(true, 'Element'),
    'chain': const FhirField(true, 'String'),
    'chainElement': const FhirField(true, 'Element'),
    'component': const FhirField(true, 'SearchParameterComponent'),
  },
  'SearchParameterComponent': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'definition': const FhirField(false, 'Canonical'),
    'expression': const FhirField(false, 'String'),
    'expressionElement': const FhirField(false, 'Element'),
  },
  'ServiceRequest': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'instantiatesCanonical': const FhirField(true, 'Canonical'),
    'instantiatesUri': const FhirField(true, 'FhirUri'),
    'instantiatesUriElement': const FhirField(true, 'Element'),
    'basedOn': const FhirField(true, 'Reference'),
    'replaces': const FhirField(true, 'Reference'),
    'requisition': const FhirField(false, 'Identifier'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'intent': const FhirField(false, 'Code'),
    'intentElement': const FhirField(false, 'Element'),
    'category': const FhirField(true, 'CodeableConcept'),
    'priority': const FhirField(false, 'Code'),
    'priorityElement': const FhirField(false, 'Element'),
    'doNotPerform': const FhirField(false, 'Boolean'),
    'doNotPerformElement': const FhirField(false, 'Element'),
    'code': const FhirField(false, 'CodeableReference'),
    'orderDetail': const FhirField(true, 'ServiceRequestOrderDetail'),
    'quantityQuantity': const FhirField(false, 'Quantity'),
    'quantityRatio': const FhirField(false, 'Ratio'),
    'quantityRange': const FhirField(false, 'Range'),
    'subject': const FhirField(false, 'Reference'),
    'focus': const FhirField(true, 'Reference'),
    'encounter': const FhirField(false, 'Reference'),
    'occurrenceDateTime': const FhirField(false, 'FhirDateTime'),
    'occurrenceDateTimeElement': const FhirField(false, 'Element'),
    'occurrencePeriod': const FhirField(false, 'Period'),
    'occurrenceTiming': const FhirField(false, 'Timing'),
    'asNeededBoolean': const FhirField(false, 'Boolean'),
    'asNeededBooleanElement': const FhirField(false, 'Element'),
    'asNeededCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'authoredOn': const FhirField(false, 'FhirDateTime'),
    'authoredOnElement': const FhirField(false, 'Element'),
    'requester': const FhirField(false, 'Reference'),
    'performerType': const FhirField(false, 'CodeableConcept'),
    'performer': const FhirField(true, 'Reference'),
    'location': const FhirField(true, 'CodeableReference'),
    'reason': const FhirField(true, 'CodeableReference'),
    'insurance': const FhirField(true, 'Reference'),
    'supportingInfo': const FhirField(true, 'CodeableReference'),
    'specimen': const FhirField(true, 'Reference'),
    'bodySite': const FhirField(true, 'CodeableConcept'),
    'bodyStructure': const FhirField(false, 'Reference'),
    'note': const FhirField(true, 'Annotation'),
    'patientInstruction':
        const FhirField(true, 'ServiceRequestPatientInstruction'),
    'relevantHistory': const FhirField(true, 'Reference'),
  },
  'ServiceRequestOrderDetail': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'parameterFocus': const FhirField(false, 'CodeableReference'),
    'parameter': const FhirField(true, 'ServiceRequestParameter'),
  },
  'ServiceRequestParameter': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(false, 'CodeableConcept'),
    'valueQuantity': const FhirField(false, 'Quantity'),
    'valueRatio': const FhirField(false, 'Ratio'),
    'valueRange': const FhirField(false, 'Range'),
    'valueBoolean': const FhirField(false, 'Boolean'),
    'valueBooleanElement': const FhirField(false, 'Element'),
    'valueCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'valueString': const FhirField(false, 'String'),
    'valueStringElement': const FhirField(false, 'Element'),
    'valuePeriod': const FhirField(false, 'Period'),
  },
  'ServiceRequestPatientInstruction': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'instructionMarkdown': const FhirField(false, 'Markdown'),
    'instructionMarkdownElement': const FhirField(false, 'Element'),
    'instructionReference': const FhirField(false, 'Reference'),
  },
  'Slot': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'serviceCategory': const FhirField(true, 'CodeableConcept'),
    'serviceType': const FhirField(true, 'CodeableReference'),
    'specialty': const FhirField(true, 'CodeableConcept'),
    'appointmentType': const FhirField(true, 'CodeableConcept'),
    'schedule': const FhirField(false, 'Reference'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'start': const FhirField(false, 'Instant'),
    'startElement': const FhirField(false, 'Element'),
    'end': const FhirField(false, 'Instant'),
    'endElement': const FhirField(false, 'Element'),
    'overbooked': const FhirField(false, 'Boolean'),
    'overbookedElement': const FhirField(false, 'Element'),
    'comment': const FhirField(false, 'String'),
    'commentElement': const FhirField(false, 'Element'),
  },
  'Specimen': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'accessionIdentifier': const FhirField(false, 'Identifier'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'type': const FhirField(false, 'CodeableConcept'),
    'subject': const FhirField(false, 'Reference'),
    'receivedTime': const FhirField(false, 'FhirDateTime'),
    'receivedTimeElement': const FhirField(false, 'Element'),
    'parent': const FhirField(true, 'Reference'),
    'request': const FhirField(true, 'Reference'),
    'combined': const FhirField(false, 'Code'),
    'combinedElement': const FhirField(false, 'Element'),
    'role': const FhirField(true, 'CodeableConcept'),
    'feature': const FhirField(true, 'SpecimenFeature'),
    'collection': const FhirField(false, 'SpecimenCollection'),
    'processing': const FhirField(true, 'SpecimenProcessing'),
    'container': const FhirField(true, 'SpecimenContainer'),
    'condition': const FhirField(true, 'CodeableConcept'),
    'note': const FhirField(true, 'Annotation'),
  },
  'SpecimenFeature': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'description': const FhirField(false, 'String'),
    'descriptionElement': const FhirField(false, 'Element'),
  },
  'SpecimenCollection': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'collector': const FhirField(false, 'Reference'),
    'collectedDateTime': const FhirField(false, 'FhirDateTime'),
    'collectedDateTimeElement': const FhirField(false, 'Element'),
    'collectedPeriod': const FhirField(false, 'Period'),
    'duration': const FhirField(false, 'FhirDuration'),
    'quantity': const FhirField(false, 'Quantity'),
    'method': const FhirField(false, 'CodeableConcept'),
    'device': const FhirField(false, 'CodeableReference'),
    'procedure': const FhirField(false, 'Reference'),
    'bodySite': const FhirField(false, 'CodeableReference'),
    'fastingStatusCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'fastingStatusDuration': const FhirField(false, 'FhirDuration'),
  },
  'SpecimenProcessing': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'description': const FhirField(false, 'String'),
    'descriptionElement': const FhirField(false, 'Element'),
    'method': const FhirField(false, 'CodeableConcept'),
    'additive': const FhirField(true, 'Reference'),
    'timeDateTime': const FhirField(false, 'FhirDateTime'),
    'timeDateTimeElement': const FhirField(false, 'Element'),
    'timePeriod': const FhirField(false, 'Period'),
  },
  'SpecimenContainer': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'device': const FhirField(false, 'Reference'),
    'location': const FhirField(false, 'Reference'),
    'specimenQuantity': const FhirField(false, 'Quantity'),
  },
  'SpecimenDefinition': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'url': const FhirField(false, 'FhirUri'),
    'urlElement': const FhirField(false, 'Element'),
    'identifier': const FhirField(false, 'Identifier'),
    'version': const FhirField(false, 'String'),
    'versionElement': const FhirField(false, 'Element'),
    'versionAlgorithmString': const FhirField(false, 'String'),
    'versionAlgorithmStringElement': const FhirField(false, 'Element'),
    'versionAlgorithmCoding': const FhirField(false, 'Coding'),
    'name': const FhirField(false, 'String'),
    'nameElement': const FhirField(false, 'Element'),
    'title': const FhirField(false, 'String'),
    'titleElement': const FhirField(false, 'Element'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'experimental': const FhirField(false, 'Boolean'),
    'experimentalElement': const FhirField(false, 'Element'),
    'date': const FhirField(false, 'FhirDateTime'),
    'dateElement': const FhirField(false, 'Element'),
    'publisher': const FhirField(false, 'String'),
    'publisherElement': const FhirField(false, 'Element'),
    'contact': const FhirField(true, 'ContactDetail'),
    'description': const FhirField(false, 'Markdown'),
    'descriptionElement': const FhirField(false, 'Element'),
    'useContext': const FhirField(true, 'UsageContext'),
    'jurisdiction': const FhirField(true, 'CodeableConcept'),
    'purpose': const FhirField(false, 'Markdown'),
    'purposeElement': const FhirField(false, 'Element'),
    'copyright': const FhirField(false, 'Markdown'),
    'copyrightElement': const FhirField(false, 'Element'),
    'copyrightLabel': const FhirField(false, 'String'),
    'copyrightLabelElement': const FhirField(false, 'Element'),
    'approvalDate': const FhirField(false, 'Date'),
    'approvalDateElement': const FhirField(false, 'Element'),
    'lastReviewDate': const FhirField(false, 'Date'),
    'lastReviewDateElement': const FhirField(false, 'Element'),
    'effectivePeriod': const FhirField(false, 'Period'),
    'topic': const FhirField(true, 'CodeableConcept'),
    'author': const FhirField(true, 'ContactDetail'),
    'editor': const FhirField(true, 'ContactDetail'),
    'reviewer': const FhirField(true, 'ContactDetail'),
    'endorser': const FhirField(true, 'ContactDetail'),
    'relatedArtifact': const FhirField(true, 'RelatedArtifact'),
    'derivedFromCanonical': const FhirField(true, 'Canonical'),
    'derivedFromUri': const FhirField(true, 'FhirUri'),
    'derivedFromUriElement': const FhirField(true, 'Element'),
    'subjectCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'subjectReference': const FhirField(false, 'Reference'),
    'typeCollected': const FhirField(false, 'CodeableConcept'),
    'patientPreparation': const FhirField(true, 'CodeableConcept'),
    'timeAspect': const FhirField(false, 'String'),
    'timeAspectElement': const FhirField(false, 'Element'),
    'collection': const FhirField(true, 'CodeableConcept'),
    'typeTested': const FhirField(true, 'SpecimenDefinitionTypeTested'),
  },
  'SpecimenDefinitionTypeTested': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'isDerived': const FhirField(false, 'Boolean'),
    'isDerivedElement': const FhirField(false, 'Element'),
    'type': const FhirField(false, 'CodeableConcept'),
    'preference': const FhirField(false, 'Code'),
    'preferenceElement': const FhirField(false, 'Element'),
    'container': const FhirField(false, 'SpecimenDefinitionContainer'),
    'requirement': const FhirField(false, 'Markdown'),
    'requirementElement': const FhirField(false, 'Element'),
    'retentionTime': const FhirField(false, 'FhirDuration'),
    'singleUse': const FhirField(false, 'Boolean'),
    'singleUseElement': const FhirField(false, 'Element'),
    'rejectionCriterion': const FhirField(true, 'CodeableConcept'),
    'handling': const FhirField(true, 'SpecimenDefinitionHandling'),
    'testingDestination': const FhirField(true, 'CodeableConcept'),
  },
  'SpecimenDefinitionContainer': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'material': const FhirField(false, 'CodeableConcept'),
    'type': const FhirField(false, 'CodeableConcept'),
    'cap': const FhirField(false, 'CodeableConcept'),
    'description': const FhirField(false, 'Markdown'),
    'descriptionElement': const FhirField(false, 'Element'),
    'capacity': const FhirField(false, 'Quantity'),
    'minimumVolumeQuantity': const FhirField(false, 'Quantity'),
    'minimumVolumeString': const FhirField(false, 'String'),
    'minimumVolumeStringElement': const FhirField(false, 'Element'),
    'additive': const FhirField(true, 'SpecimenDefinitionAdditive'),
    'preparation': const FhirField(false, 'Markdown'),
    'preparationElement': const FhirField(false, 'Element'),
  },
  'SpecimenDefinitionAdditive': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'additiveCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'additiveReference': const FhirField(false, 'Reference'),
  },
  'SpecimenDefinitionHandling': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'temperatureQualifier': const FhirField(false, 'CodeableConcept'),
    'temperatureRange': const FhirField(false, 'Range'),
    'maxDuration': const FhirField(false, 'FhirDuration'),
    'instruction': const FhirField(false, 'Markdown'),
    'instructionElement': const FhirField(false, 'Element'),
  },
  'StructureDefinition': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'url': const FhirField(false, 'FhirUri'),
    'urlElement': const FhirField(false, 'Element'),
    'identifier': const FhirField(true, 'Identifier'),
    'version': const FhirField(false, 'String'),
    'versionElement': const FhirField(false, 'Element'),
    'versionAlgorithmString': const FhirField(false, 'String'),
    'versionAlgorithmStringElement': const FhirField(false, 'Element'),
    'versionAlgorithmCoding': const FhirField(false, 'Coding'),
    'name': const FhirField(false, 'String'),
    'nameElement': const FhirField(false, 'Element'),
    'title': const FhirField(false, 'String'),
    'titleElement': const FhirField(false, 'Element'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'experimental': const FhirField(false, 'Boolean'),
    'experimentalElement': const FhirField(false, 'Element'),
    'date': const FhirField(false, 'FhirDateTime'),
    'dateElement': const FhirField(false, 'Element'),
    'publisher': const FhirField(false, 'String'),
    'publisherElement': const FhirField(false, 'Element'),
    'contact': const FhirField(true, 'ContactDetail'),
    'description': const FhirField(false, 'Markdown'),
    'descriptionElement': const FhirField(false, 'Element'),
    'useContext': const FhirField(true, 'UsageContext'),
    'jurisdiction': const FhirField(true, 'CodeableConcept'),
    'purpose': const FhirField(false, 'Markdown'),
    'purposeElement': const FhirField(false, 'Element'),
    'copyright': const FhirField(false, 'Markdown'),
    'copyrightElement': const FhirField(false, 'Element'),
    'copyrightLabel': const FhirField(false, 'String'),
    'copyrightLabelElement': const FhirField(false, 'Element'),
    'keyword': const FhirField(true, 'Coding'),
    'fhirVersion': const FhirField(false, 'Code'),
    'fhirVersionElement': const FhirField(false, 'Element'),
    'mapping': const FhirField(true, 'StructureDefinitionMapping'),
    'kind': const FhirField(false, 'Code'),
    'kindElement': const FhirField(false, 'Element'),
    'abstract': const FhirField(false, 'Boolean'),
    'abstractElement': const FhirField(false, 'Element'),
    'context': const FhirField(true, 'StructureDefinitionContext'),
    'contextInvariant': const FhirField(true, 'String'),
    'contextInvariantElement': const FhirField(true, 'Element'),
    'type': const FhirField(false, 'FhirUri'),
    'typeElement': const FhirField(false, 'Element'),
    'baseDefinition': const FhirField(false, 'Canonical'),
    'derivation': const FhirField(false, 'Code'),
    'derivationElement': const FhirField(false, 'Element'),
    'snapshot': const FhirField(false, 'StructureDefinitionSnapshot'),
    'differential': const FhirField(false, 'StructureDefinitionDifferential'),
  },
  'StructureDefinitionMapping': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identity': const FhirField(false, 'Id'),
    'identityElement': const FhirField(false, 'Element'),
    'uri': const FhirField(false, 'FhirUri'),
    'uriElement': const FhirField(false, 'Element'),
    'name': const FhirField(false, 'String'),
    'nameElement': const FhirField(false, 'Element'),
    'comment': const FhirField(false, 'String'),
    'commentElement': const FhirField(false, 'Element'),
  },
  'StructureDefinitionContext': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'Code'),
    'typeElement': const FhirField(false, 'Element'),
    'expression': const FhirField(false, 'String'),
    'expressionElement': const FhirField(false, 'Element'),
  },
  'StructureDefinitionSnapshot': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'element': const FhirField(true, 'ElementDefinition'),
  },
  'StructureDefinitionDifferential': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'element': const FhirField(true, 'ElementDefinition'),
  },
  'StructureMap': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'url': const FhirField(false, 'FhirUri'),
    'urlElement': const FhirField(false, 'Element'),
    'identifier': const FhirField(true, 'Identifier'),
    'version': const FhirField(false, 'String'),
    'versionElement': const FhirField(false, 'Element'),
    'versionAlgorithmString': const FhirField(false, 'String'),
    'versionAlgorithmStringElement': const FhirField(false, 'Element'),
    'versionAlgorithmCoding': const FhirField(false, 'Coding'),
    'name': const FhirField(false, 'String'),
    'nameElement': const FhirField(false, 'Element'),
    'title': const FhirField(false, 'String'),
    'titleElement': const FhirField(false, 'Element'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'experimental': const FhirField(false, 'Boolean'),
    'experimentalElement': const FhirField(false, 'Element'),
    'date': const FhirField(false, 'FhirDateTime'),
    'dateElement': const FhirField(false, 'Element'),
    'publisher': const FhirField(false, 'String'),
    'publisherElement': const FhirField(false, 'Element'),
    'contact': const FhirField(true, 'ContactDetail'),
    'description': const FhirField(false, 'Markdown'),
    'descriptionElement': const FhirField(false, 'Element'),
    'useContext': const FhirField(true, 'UsageContext'),
    'jurisdiction': const FhirField(true, 'CodeableConcept'),
    'purpose': const FhirField(false, 'Markdown'),
    'purposeElement': const FhirField(false, 'Element'),
    'copyright': const FhirField(false, 'Markdown'),
    'copyrightElement': const FhirField(false, 'Element'),
    'copyrightLabel': const FhirField(false, 'String'),
    'copyrightLabelElement': const FhirField(false, 'Element'),
    'structure': const FhirField(true, 'StructureMapStructure'),
    'import': const FhirField(true, 'Canonical'),
    'const': const FhirField(true, 'StructureMapConst'),
    'group': const FhirField(true, 'StructureMapGroup'),
  },
  'StructureMapStructure': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'url': const FhirField(false, 'Canonical'),
    'mode': const FhirField(false, 'Code'),
    'modeElement': const FhirField(false, 'Element'),
    'alias': const FhirField(false, 'String'),
    'aliasElement': const FhirField(false, 'Element'),
    'documentation': const FhirField(false, 'String'),
    'documentationElement': const FhirField(false, 'Element'),
  },
  'StructureMapConst': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'name': const FhirField(false, 'Id'),
    'nameElement': const FhirField(false, 'Element'),
    'value': const FhirField(false, 'String'),
    'valueElement': const FhirField(false, 'Element'),
  },
  'StructureMapGroup': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'name': const FhirField(false, 'Id'),
    'nameElement': const FhirField(false, 'Element'),
    'extends': const FhirField(false, 'Id'),
    'extendsElement': const FhirField(false, 'Element'),
    'typeMode': const FhirField(false, 'Code'),
    'typeModeElement': const FhirField(false, 'Element'),
    'documentation': const FhirField(false, 'String'),
    'documentationElement': const FhirField(false, 'Element'),
    'input': const FhirField(true, 'StructureMapInput'),
    'rule': const FhirField(true, 'StructureMapRule'),
  },
  'StructureMapInput': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'name': const FhirField(false, 'Id'),
    'nameElement': const FhirField(false, 'Element'),
    'type': const FhirField(false, 'String'),
    'typeElement': const FhirField(false, 'Element'),
    'mode': const FhirField(false, 'Code'),
    'modeElement': const FhirField(false, 'Element'),
    'documentation': const FhirField(false, 'String'),
    'documentationElement': const FhirField(false, 'Element'),
  },
  'StructureMapRule': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'name': const FhirField(false, 'Id'),
    'nameElement': const FhirField(false, 'Element'),
    'source': const FhirField(true, 'StructureMapSource'),
    'target': const FhirField(true, 'StructureMapTarget'),
    'rule': const FhirField(true, 'StructureMapRule'),
    'dependent': const FhirField(true, 'StructureMapDependent'),
    'documentation': const FhirField(false, 'String'),
    'documentationElement': const FhirField(false, 'Element'),
  },
  'StructureMapSource': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'context': const FhirField(false, 'Id'),
    'contextElement': const FhirField(false, 'Element'),
    'min': const FhirField(false, 'Integer'),
    'minElement': const FhirField(false, 'Element'),
    'max': const FhirField(false, 'String'),
    'maxElement': const FhirField(false, 'Element'),
    'type': const FhirField(false, 'String'),
    'typeElement': const FhirField(false, 'Element'),
    'defaultValue': const FhirField(false, 'String'),
    'defaultValueElement': const FhirField(false, 'Element'),
    'element': const FhirField(false, 'String'),
    'elementElement': const FhirField(false, 'Element'),
    'listMode': const FhirField(false, 'Code'),
    'listModeElement': const FhirField(false, 'Element'),
    'variable': const FhirField(false, 'Id'),
    'variableElement': const FhirField(false, 'Element'),
    'condition': const FhirField(false, 'String'),
    'conditionElement': const FhirField(false, 'Element'),
    'check': const FhirField(false, 'String'),
    'checkElement': const FhirField(false, 'Element'),
    'logMessage': const FhirField(false, 'String'),
    'logMessageElement': const FhirField(false, 'Element'),
  },
  'StructureMapTarget': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'context': const FhirField(false, 'String'),
    'contextElement': const FhirField(false, 'Element'),
    'element': const FhirField(false, 'String'),
    'elementElement': const FhirField(false, 'Element'),
    'variable': const FhirField(false, 'Id'),
    'variableElement': const FhirField(false, 'Element'),
    'listMode': const FhirField(true, 'Code'),
    'listModeElement': const FhirField(true, 'Element'),
    'listRuleId': const FhirField(false, 'Id'),
    'listRuleIdElement': const FhirField(false, 'Element'),
    'transform': const FhirField(false, 'Code'),
    'transformElement': const FhirField(false, 'Element'),
    'parameter': const FhirField(true, 'StructureMapParameter'),
  },
  'StructureMapParameter': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'valueId': const FhirField(false, 'Id'),
    'valueIdElement': const FhirField(false, 'Element'),
    'valueString': const FhirField(false, 'String'),
    'valueStringElement': const FhirField(false, 'Element'),
    'valueBoolean': const FhirField(false, 'Boolean'),
    'valueBooleanElement': const FhirField(false, 'Element'),
    'valueInteger': const FhirField(false, 'Integer'),
    'valueIntegerElement': const FhirField(false, 'Element'),
    'valueDecimal': const FhirField(false, 'Decimal'),
    'valueDecimalElement': const FhirField(false, 'Element'),
    'valueDate': const FhirField(false, 'Date'),
    'valueDateElement': const FhirField(false, 'Element'),
    'valueTime': const FhirField(false, 'Time'),
    'valueTimeElement': const FhirField(false, 'Element'),
    'valueDateTime': const FhirField(false, 'FhirDateTime'),
    'valueDateTimeElement': const FhirField(false, 'Element'),
  },
  'StructureMapDependent': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'name': const FhirField(false, 'Id'),
    'nameElement': const FhirField(false, 'Element'),
    'parameter': const FhirField(true, 'StructureMapParameter'),
  },
  'Subscription': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'name': const FhirField(false, 'String'),
    'nameElement': const FhirField(false, 'Element'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'topic': const FhirField(false, 'Canonical'),
    'contact': const FhirField(true, 'ContactPoint'),
    'end': const FhirField(false, 'Instant'),
    'endElement': const FhirField(false, 'Element'),
    'managingEntity': const FhirField(false, 'Reference'),
    'reason': const FhirField(false, 'String'),
    'reasonElement': const FhirField(false, 'Element'),
    'filterBy': const FhirField(true, 'SubscriptionFilterBy'),
    'channelType': const FhirField(false, 'Coding'),
    'endpoint': const FhirField(false, 'FhirUrl'),
    'endpointElement': const FhirField(false, 'Element'),
    'parameter': const FhirField(true, 'SubscriptionParameter'),
    'heartbeatPeriod': const FhirField(false, 'UnsignedInt'),
    'heartbeatPeriodElement': const FhirField(false, 'Element'),
    'timeout': const FhirField(false, 'UnsignedInt'),
    'timeoutElement': const FhirField(false, 'Element'),
    'contentType': const FhirField(false, 'Code'),
    'contentTypeElement': const FhirField(false, 'Element'),
    'content': const FhirField(false, 'Code'),
    'contentElement': const FhirField(false, 'Element'),
    'maxCount': const FhirField(false, 'PositiveInt'),
    'maxCountElement': const FhirField(false, 'Element'),
  },
  'SubscriptionFilterBy': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'resourceType': const FhirField(false, 'FhirUri'),
    'resourceTypeElement': const FhirField(false, 'Element'),
    'filterParameter': const FhirField(false, 'String'),
    'filterParameterElement': const FhirField(false, 'Element'),
    'comparator': const FhirField(false, 'Code'),
    'comparatorElement': const FhirField(false, 'Element'),
    'modifier': const FhirField(false, 'Code'),
    'modifierElement': const FhirField(false, 'Element'),
    'value': const FhirField(false, 'String'),
    'valueElement': const FhirField(false, 'Element'),
  },
  'SubscriptionParameter': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'name': const FhirField(false, 'String'),
    'nameElement': const FhirField(false, 'Element'),
    'value': const FhirField(false, 'String'),
    'valueElement': const FhirField(false, 'Element'),
  },
  'SubscriptionStatus': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'type': const FhirField(false, 'Code'),
    'typeElement': const FhirField(false, 'Element'),
    'eventsSinceSubscriptionStart': const FhirField(false, 'Integer64'),
    'eventsSinceSubscriptionStartElement': const FhirField(false, 'Element'),
    'notificationEvent':
        const FhirField(true, 'SubscriptionStatusNotificationEvent'),
    'subscription': const FhirField(false, 'Reference'),
    'topic': const FhirField(false, 'Canonical'),
    'error': const FhirField(true, 'CodeableConcept'),
  },
  'SubscriptionStatusNotificationEvent': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'eventNumber': const FhirField(false, 'Integer64'),
    'eventNumberElement': const FhirField(false, 'Element'),
    'timestamp': const FhirField(false, 'Instant'),
    'timestampElement': const FhirField(false, 'Element'),
    'focus': const FhirField(false, 'Reference'),
    'additionalContext': const FhirField(true, 'Reference'),
  },
  'SubscriptionTopic': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'url': const FhirField(false, 'FhirUri'),
    'urlElement': const FhirField(false, 'Element'),
    'identifier': const FhirField(true, 'Identifier'),
    'version': const FhirField(false, 'String'),
    'versionElement': const FhirField(false, 'Element'),
    'versionAlgorithmString': const FhirField(false, 'String'),
    'versionAlgorithmStringElement': const FhirField(false, 'Element'),
    'versionAlgorithmCoding': const FhirField(false, 'Coding'),
    'name': const FhirField(false, 'String'),
    'nameElement': const FhirField(false, 'Element'),
    'title': const FhirField(false, 'String'),
    'titleElement': const FhirField(false, 'Element'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'experimental': const FhirField(false, 'Boolean'),
    'experimentalElement': const FhirField(false, 'Element'),
    'date': const FhirField(false, 'FhirDateTime'),
    'dateElement': const FhirField(false, 'Element'),
    'publisher': const FhirField(false, 'String'),
    'publisherElement': const FhirField(false, 'Element'),
    'contact': const FhirField(true, 'ContactDetail'),
    'description': const FhirField(false, 'Markdown'),
    'descriptionElement': const FhirField(false, 'Element'),
    'useContext': const FhirField(true, 'UsageContext'),
    'jurisdiction': const FhirField(true, 'CodeableConcept'),
    'purpose': const FhirField(false, 'Markdown'),
    'purposeElement': const FhirField(false, 'Element'),
    'copyright': const FhirField(false, 'Markdown'),
    'copyrightElement': const FhirField(false, 'Element'),
    'copyrightLabel': const FhirField(false, 'String'),
    'copyrightLabelElement': const FhirField(false, 'Element'),
    'derivedFrom': const FhirField(true, 'Canonical'),
    'approvalDate': const FhirField(false, 'Date'),
    'approvalDateElement': const FhirField(false, 'Element'),
    'lastReviewDate': const FhirField(false, 'Date'),
    'lastReviewDateElement': const FhirField(false, 'Element'),
    'effectivePeriod': const FhirField(false, 'Period'),
    'resourceTrigger':
        const FhirField(true, 'SubscriptionTopicResourceTrigger'),
    'eventTrigger': const FhirField(true, 'SubscriptionTopicEventTrigger'),
    'canFilterBy': const FhirField(true, 'SubscriptionTopicCanFilterBy'),
    'notificationShape':
        const FhirField(true, 'SubscriptionTopicNotificationShape'),
  },
  'SubscriptionTopicResourceTrigger': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'description': const FhirField(false, 'Markdown'),
    'descriptionElement': const FhirField(false, 'Element'),
    'resource': const FhirField(false, 'FhirUri'),
    'resourceElement': const FhirField(false, 'Element'),
    'supportedInteraction': const FhirField(true, 'Code'),
    'supportedInteractionElement': const FhirField(true, 'Element'),
    'queryCriteria': const FhirField(false, 'SubscriptionTopicQueryCriteria'),
    'fhirPathCriteria': const FhirField(false, 'String'),
    'fhirPathCriteriaElement': const FhirField(false, 'Element'),
  },
  'SubscriptionTopicQueryCriteria': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'previous': const FhirField(false, 'String'),
    'previousElement': const FhirField(false, 'Element'),
    'resultForCreate': const FhirField(false, 'Code'),
    'resultForCreateElement': const FhirField(false, 'Element'),
    'current': const FhirField(false, 'String'),
    'currentElement': const FhirField(false, 'Element'),
    'resultForDelete': const FhirField(false, 'Code'),
    'resultForDeleteElement': const FhirField(false, 'Element'),
    'requireBoth': const FhirField(false, 'Boolean'),
    'requireBothElement': const FhirField(false, 'Element'),
  },
  'SubscriptionTopicEventTrigger': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'description': const FhirField(false, 'Markdown'),
    'descriptionElement': const FhirField(false, 'Element'),
    'event': const FhirField(false, 'CodeableConcept'),
    'resource': const FhirField(false, 'FhirUri'),
    'resourceElement': const FhirField(false, 'Element'),
  },
  'SubscriptionTopicCanFilterBy': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'description': const FhirField(false, 'Markdown'),
    'descriptionElement': const FhirField(false, 'Element'),
    'resource': const FhirField(false, 'FhirUri'),
    'resourceElement': const FhirField(false, 'Element'),
    'filterParameter': const FhirField(false, 'String'),
    'filterParameterElement': const FhirField(false, 'Element'),
    'filterDefinition': const FhirField(false, 'FhirUri'),
    'filterDefinitionElement': const FhirField(false, 'Element'),
    'comparator': const FhirField(true, 'Code'),
    'comparatorElement': const FhirField(true, 'Element'),
    'modifier': const FhirField(true, 'Code'),
    'modifierElement': const FhirField(true, 'Element'),
  },
  'SubscriptionTopicNotificationShape': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'resource': const FhirField(false, 'FhirUri'),
    'resourceElement': const FhirField(false, 'Element'),
    'include': const FhirField(true, 'String'),
    'includeElement': const FhirField(true, 'Element'),
    'revInclude': const FhirField(true, 'String'),
    'revIncludeElement': const FhirField(true, 'Element'),
  },
  'Substance': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'instance': const FhirField(false, 'Boolean'),
    'instanceElement': const FhirField(false, 'Element'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'category': const FhirField(true, 'CodeableConcept'),
    'code': const FhirField(false, 'CodeableReference'),
    'description': const FhirField(false, 'Markdown'),
    'descriptionElement': const FhirField(false, 'Element'),
    'expiry': const FhirField(false, 'FhirDateTime'),
    'expiryElement': const FhirField(false, 'Element'),
    'quantity': const FhirField(false, 'Quantity'),
    'ingredient': const FhirField(true, 'SubstanceIngredient'),
  },
  'SubstanceIngredient': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'quantity': const FhirField(false, 'Ratio'),
    'substanceCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'substanceReference': const FhirField(false, 'Reference'),
  },
  'SubstanceDefinition': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'version': const FhirField(false, 'String'),
    'versionElement': const FhirField(false, 'Element'),
    'status': const FhirField(false, 'CodeableConcept'),
    'classification': const FhirField(true, 'CodeableConcept'),
    'domain': const FhirField(false, 'CodeableConcept'),
    'grade': const FhirField(true, 'CodeableConcept'),
    'description': const FhirField(false, 'Markdown'),
    'descriptionElement': const FhirField(false, 'Element'),
    'informationSource': const FhirField(true, 'Reference'),
    'note': const FhirField(true, 'Annotation'),
    'manufacturer': const FhirField(true, 'Reference'),
    'supplier': const FhirField(true, 'Reference'),
    'moiety': const FhirField(true, 'SubstanceDefinitionMoiety'),
    'characterization':
        const FhirField(true, 'SubstanceDefinitionCharacterization'),
    'property': const FhirField(true, 'SubstanceDefinitionProperty'),
    'referenceInformation': const FhirField(false, 'Reference'),
    'molecularWeight':
        const FhirField(true, 'SubstanceDefinitionMolecularWeight'),
    'structure': const FhirField(false, 'SubstanceDefinitionStructure'),
    'code': const FhirField(true, 'SubstanceDefinitionCode'),
    'name': const FhirField(true, 'SubstanceDefinitionName'),
    'relationship': const FhirField(true, 'SubstanceDefinitionRelationship'),
    'nucleicAcid': const FhirField(false, 'Reference'),
    'polymer': const FhirField(false, 'Reference'),
    'protein': const FhirField(false, 'Reference'),
    'sourceMaterial':
        const FhirField(false, 'SubstanceDefinitionSourceMaterial'),
  },
  'SubstanceDefinitionMoiety': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'role': const FhirField(false, 'CodeableConcept'),
    'identifier': const FhirField(false, 'Identifier'),
    'name': const FhirField(false, 'String'),
    'nameElement': const FhirField(false, 'Element'),
    'stereochemistry': const FhirField(false, 'CodeableConcept'),
    'opticalActivity': const FhirField(false, 'CodeableConcept'),
    'molecularFormula': const FhirField(false, 'String'),
    'molecularFormulaElement': const FhirField(false, 'Element'),
    'amountQuantity': const FhirField(false, 'Quantity'),
    'amountString': const FhirField(false, 'String'),
    'amountStringElement': const FhirField(false, 'Element'),
    'measurementType': const FhirField(false, 'CodeableConcept'),
  },
  'SubstanceDefinitionCharacterization': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'technique': const FhirField(false, 'CodeableConcept'),
    'form': const FhirField(false, 'CodeableConcept'),
    'description': const FhirField(false, 'Markdown'),
    'descriptionElement': const FhirField(false, 'Element'),
    'file': const FhirField(true, 'Attachment'),
  },
  'SubstanceDefinitionProperty': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'valueCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'valueQuantity': const FhirField(false, 'Quantity'),
    'valueDate': const FhirField(false, 'Date'),
    'valueDateElement': const FhirField(false, 'Element'),
    'valueBoolean': const FhirField(false, 'Boolean'),
    'valueBooleanElement': const FhirField(false, 'Element'),
    'valueAttachment': const FhirField(false, 'Attachment'),
  },
  'SubstanceDefinitionMolecularWeight': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'method': const FhirField(false, 'CodeableConcept'),
    'type': const FhirField(false, 'CodeableConcept'),
    'amount': const FhirField(false, 'Quantity'),
  },
  'SubstanceDefinitionStructure': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'stereochemistry': const FhirField(false, 'CodeableConcept'),
    'opticalActivity': const FhirField(false, 'CodeableConcept'),
    'molecularFormula': const FhirField(false, 'String'),
    'molecularFormulaElement': const FhirField(false, 'Element'),
    'molecularFormulaByMoiety': const FhirField(false, 'String'),
    'molecularFormulaByMoietyElement': const FhirField(false, 'Element'),
    'molecularWeight':
        const FhirField(false, 'SubstanceDefinitionMolecularWeight'),
    'technique': const FhirField(true, 'CodeableConcept'),
    'sourceDocument': const FhirField(true, 'Reference'),
    'representation':
        const FhirField(true, 'SubstanceDefinitionRepresentation'),
  },
  'SubstanceDefinitionRepresentation': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'representation': const FhirField(false, 'String'),
    'representationElement': const FhirField(false, 'Element'),
    'format': const FhirField(false, 'CodeableConcept'),
    'document': const FhirField(false, 'Reference'),
  },
  'SubstanceDefinitionCode': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(false, 'CodeableConcept'),
    'status': const FhirField(false, 'CodeableConcept'),
    'statusDate': const FhirField(false, 'FhirDateTime'),
    'statusDateElement': const FhirField(false, 'Element'),
    'note': const FhirField(true, 'Annotation'),
    'source': const FhirField(true, 'Reference'),
  },
  'SubstanceDefinitionName': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'name': const FhirField(false, 'String'),
    'nameElement': const FhirField(false, 'Element'),
    'type': const FhirField(false, 'CodeableConcept'),
    'status': const FhirField(false, 'CodeableConcept'),
    'preferred': const FhirField(false, 'Boolean'),
    'preferredElement': const FhirField(false, 'Element'),
    'language': const FhirField(true, 'CodeableConcept'),
    'domain': const FhirField(true, 'CodeableConcept'),
    'jurisdiction': const FhirField(true, 'CodeableConcept'),
    'synonym': const FhirField(true, 'SubstanceDefinitionName'),
    'translation': const FhirField(true, 'SubstanceDefinitionName'),
    'official': const FhirField(true, 'SubstanceDefinitionOfficial'),
    'source': const FhirField(true, 'Reference'),
  },
  'SubstanceDefinitionOfficial': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'authority': const FhirField(false, 'CodeableConcept'),
    'status': const FhirField(false, 'CodeableConcept'),
    'date': const FhirField(false, 'FhirDateTime'),
    'dateElement': const FhirField(false, 'Element'),
  },
  'SubstanceDefinitionRelationship': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'substanceDefinitionReference': const FhirField(false, 'Reference'),
    'substanceDefinitionCodeableConcept':
        const FhirField(false, 'CodeableConcept'),
    'type': const FhirField(false, 'CodeableConcept'),
    'isDefining': const FhirField(false, 'Boolean'),
    'isDefiningElement': const FhirField(false, 'Element'),
    'amountQuantity': const FhirField(false, 'Quantity'),
    'amountRatio': const FhirField(false, 'Ratio'),
    'amountString': const FhirField(false, 'String'),
    'amountStringElement': const FhirField(false, 'Element'),
    'ratioHighLimitAmount': const FhirField(false, 'Ratio'),
    'comparator': const FhirField(false, 'CodeableConcept'),
    'source': const FhirField(true, 'Reference'),
  },
  'SubstanceDefinitionSourceMaterial': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'genus': const FhirField(false, 'CodeableConcept'),
    'species': const FhirField(false, 'CodeableConcept'),
    'part': const FhirField(false, 'CodeableConcept'),
    'countryOfOrigin': const FhirField(true, 'CodeableConcept'),
  },
  'SubstanceNucleicAcid': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'sequenceType': const FhirField(false, 'CodeableConcept'),
    'numberOfSubunits': const FhirField(false, 'Integer'),
    'numberOfSubunitsElement': const FhirField(false, 'Element'),
    'areaOfHybridisation': const FhirField(false, 'String'),
    'areaOfHybridisationElement': const FhirField(false, 'Element'),
    'oligoNucleotideType': const FhirField(false, 'CodeableConcept'),
    'subunit': const FhirField(true, 'SubstanceNucleicAcidSubunit'),
  },
  'SubstanceNucleicAcidSubunit': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'subunit': const FhirField(false, 'Integer'),
    'subunitElement': const FhirField(false, 'Element'),
    'sequence': const FhirField(false, 'String'),
    'sequenceElement': const FhirField(false, 'Element'),
    'length': const FhirField(false, 'Integer'),
    'lengthElement': const FhirField(false, 'Element'),
    'sequenceAttachment': const FhirField(false, 'Attachment'),
    'fivePrime': const FhirField(false, 'CodeableConcept'),
    'threePrime': const FhirField(false, 'CodeableConcept'),
    'linkage': const FhirField(true, 'SubstanceNucleicAcidLinkage'),
    'sugar': const FhirField(true, 'SubstanceNucleicAcidSugar'),
  },
  'SubstanceNucleicAcidLinkage': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'connectivity': const FhirField(false, 'String'),
    'connectivityElement': const FhirField(false, 'Element'),
    'identifier': const FhirField(false, 'Identifier'),
    'name': const FhirField(false, 'String'),
    'nameElement': const FhirField(false, 'Element'),
    'residueSite': const FhirField(false, 'String'),
    'residueSiteElement': const FhirField(false, 'Element'),
  },
  'SubstanceNucleicAcidSugar': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(false, 'Identifier'),
    'name': const FhirField(false, 'String'),
    'nameElement': const FhirField(false, 'Element'),
    'residueSite': const FhirField(false, 'String'),
    'residueSiteElement': const FhirField(false, 'Element'),
  },
  'SubstancePolymer': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(false, 'Identifier'),
    'class': const FhirField(false, 'CodeableConcept'),
    'geometry': const FhirField(false, 'CodeableConcept'),
    'copolymerConnectivity': const FhirField(true, 'CodeableConcept'),
    'modification': const FhirField(false, 'String'),
    'modificationElement': const FhirField(false, 'Element'),
    'monomerSet': const FhirField(true, 'SubstancePolymerMonomerSet'),
    'repeat': const FhirField(true, 'SubstancePolymerRepeat'),
  },
  'SubstancePolymerMonomerSet': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'ratioType': const FhirField(false, 'CodeableConcept'),
    'startingMaterial':
        const FhirField(true, 'SubstancePolymerStartingMaterial'),
  },
  'SubstancePolymerStartingMaterial': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(false, 'CodeableConcept'),
    'category': const FhirField(false, 'CodeableConcept'),
    'isDefining': const FhirField(false, 'Boolean'),
    'isDefiningElement': const FhirField(false, 'Element'),
    'amount': const FhirField(false, 'Quantity'),
  },
  'SubstancePolymerRepeat': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'averageMolecularFormula': const FhirField(false, 'String'),
    'averageMolecularFormulaElement': const FhirField(false, 'Element'),
    'repeatUnitAmountType': const FhirField(false, 'CodeableConcept'),
    'repeatUnit': const FhirField(true, 'SubstancePolymerRepeatUnit'),
  },
  'SubstancePolymerRepeatUnit': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'unit': const FhirField(false, 'String'),
    'unitElement': const FhirField(false, 'Element'),
    'orientation': const FhirField(false, 'CodeableConcept'),
    'amount': const FhirField(false, 'Integer'),
    'amountElement': const FhirField(false, 'Element'),
    'degreeOfPolymerisation':
        const FhirField(true, 'SubstancePolymerDegreeOfPolymerisation'),
    'structuralRepresentation':
        const FhirField(true, 'SubstancePolymerStructuralRepresentation'),
  },
  'SubstancePolymerDegreeOfPolymerisation': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'average': const FhirField(false, 'Integer'),
    'averageElement': const FhirField(false, 'Element'),
    'low': const FhirField(false, 'Integer'),
    'lowElement': const FhirField(false, 'Element'),
    'high': const FhirField(false, 'Integer'),
    'highElement': const FhirField(false, 'Element'),
  },
  'SubstancePolymerStructuralRepresentation': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'representation': const FhirField(false, 'String'),
    'representationElement': const FhirField(false, 'Element'),
    'format': const FhirField(false, 'CodeableConcept'),
    'attachment': const FhirField(false, 'Attachment'),
  },
  'SubstanceProtein': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'sequenceType': const FhirField(false, 'CodeableConcept'),
    'numberOfSubunits': const FhirField(false, 'Integer'),
    'numberOfSubunitsElement': const FhirField(false, 'Element'),
    'disulfideLinkage': const FhirField(true, 'String'),
    'disulfideLinkageElement': const FhirField(true, 'Element'),
    'subunit': const FhirField(true, 'SubstanceProteinSubunit'),
  },
  'SubstanceProteinSubunit': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'subunit': const FhirField(false, 'Integer'),
    'subunitElement': const FhirField(false, 'Element'),
    'sequence': const FhirField(false, 'String'),
    'sequenceElement': const FhirField(false, 'Element'),
    'length': const FhirField(false, 'Integer'),
    'lengthElement': const FhirField(false, 'Element'),
    'sequenceAttachment': const FhirField(false, 'Attachment'),
    'nTerminalModificationId': const FhirField(false, 'Identifier'),
    'nTerminalModification': const FhirField(false, 'String'),
    'nTerminalModificationElement': const FhirField(false, 'Element'),
    'cTerminalModificationId': const FhirField(false, 'Identifier'),
    'cTerminalModification': const FhirField(false, 'String'),
    'cTerminalModificationElement': const FhirField(false, 'Element'),
  },
  'SubstanceReferenceInformation': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'comment': const FhirField(false, 'String'),
    'commentElement': const FhirField(false, 'Element'),
    'gene': const FhirField(true, 'SubstanceReferenceInformationGene'),
    'geneElement':
        const FhirField(true, 'SubstanceReferenceInformationGeneElement'),
    'target': const FhirField(true, 'SubstanceReferenceInformationTarget'),
  },
  'SubstanceReferenceInformationGene': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'geneSequenceOrigin': const FhirField(false, 'CodeableConcept'),
    'gene': const FhirField(false, 'CodeableConcept'),
    'source': const FhirField(true, 'Reference'),
  },
  'SubstanceReferenceInformationGeneElement': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'element': const FhirField(false, 'Identifier'),
    'source': const FhirField(true, 'Reference'),
  },
  'SubstanceReferenceInformationTarget': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'target': const FhirField(false, 'Identifier'),
    'type': const FhirField(false, 'CodeableConcept'),
    'interaction': const FhirField(false, 'CodeableConcept'),
    'organism': const FhirField(false, 'CodeableConcept'),
    'organismType': const FhirField(false, 'CodeableConcept'),
    'amountQuantity': const FhirField(false, 'Quantity'),
    'amountRange': const FhirField(false, 'Range'),
    'amountString': const FhirField(false, 'String'),
    'amountStringElement': const FhirField(false, 'Element'),
    'amountType': const FhirField(false, 'CodeableConcept'),
    'source': const FhirField(true, 'Reference'),
  },
  'SubstanceSourceMaterial': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'sourceMaterialClass': const FhirField(false, 'CodeableConcept'),
    'sourceMaterialType': const FhirField(false, 'CodeableConcept'),
    'sourceMaterialState': const FhirField(false, 'CodeableConcept'),
    'organismId': const FhirField(false, 'Identifier'),
    'organismName': const FhirField(false, 'String'),
    'organismNameElement': const FhirField(false, 'Element'),
    'parentSubstanceId': const FhirField(true, 'Identifier'),
    'parentSubstanceName': const FhirField(true, 'String'),
    'parentSubstanceNameElement': const FhirField(true, 'Element'),
    'countryOfOrigin': const FhirField(true, 'CodeableConcept'),
    'geographicalLocation': const FhirField(true, 'String'),
    'geographicalLocationElement': const FhirField(true, 'Element'),
    'developmentStage': const FhirField(false, 'CodeableConcept'),
    'fractionDescription':
        const FhirField(true, 'SubstanceSourceMaterialFractionDescription'),
    'organism': const FhirField(false, 'SubstanceSourceMaterialOrganism'),
    'partDescription':
        const FhirField(true, 'SubstanceSourceMaterialPartDescription'),
  },
  'SubstanceSourceMaterialFractionDescription': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'fraction': const FhirField(false, 'String'),
    'fractionElement': const FhirField(false, 'Element'),
    'materialType': const FhirField(false, 'CodeableConcept'),
  },
  'SubstanceSourceMaterialOrganism': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'family': const FhirField(false, 'CodeableConcept'),
    'genus': const FhirField(false, 'CodeableConcept'),
    'species': const FhirField(false, 'CodeableConcept'),
    'intraspecificType': const FhirField(false, 'CodeableConcept'),
    'intraspecificDescription': const FhirField(false, 'String'),
    'intraspecificDescriptionElement': const FhirField(false, 'Element'),
    'author': const FhirField(true, 'SubstanceSourceMaterialAuthor'),
    'hybrid': const FhirField(false, 'Id'),
    'organismGeneral':
        const FhirField(false, 'SubstanceSourceMaterialOrganismGeneral'),
  },
  'SubstanceSourceMaterialAuthor': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'authorType': const FhirField(false, 'CodeableConcept'),
    'authorDescription': const FhirField(false, 'String'),
    'authorDescriptionElement': const FhirField(false, 'Element'),
  },
  'SubstanceSourceMaterialHybrid': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'maternalOrganismId': const FhirField(false, 'String'),
    'maternalOrganismIdElement': const FhirField(false, 'Element'),
    'maternalOrganismName': const FhirField(false, 'String'),
    'maternalOrganismNameElement': const FhirField(false, 'Element'),
    'paternalOrganismId': const FhirField(false, 'String'),
    'paternalOrganismIdElement': const FhirField(false, 'Element'),
    'paternalOrganismName': const FhirField(false, 'String'),
    'paternalOrganismNameElement': const FhirField(false, 'Element'),
    'hybridType': const FhirField(false, 'CodeableConcept'),
  },
  'SubstanceSourceMaterialOrganismGeneral': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'kingdom': const FhirField(false, 'CodeableConcept'),
    'phylum': const FhirField(false, 'CodeableConcept'),
    'class': const FhirField(false, 'CodeableConcept'),
    'order': const FhirField(false, 'CodeableConcept'),
  },
  'SubstanceSourceMaterialPartDescription': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'part': const FhirField(false, 'CodeableConcept'),
    'partLocation': const FhirField(false, 'CodeableConcept'),
  },
  'SupplyDelivery': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'basedOn': const FhirField(true, 'Reference'),
    'partOf': const FhirField(true, 'Reference'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'patient': const FhirField(false, 'Reference'),
    'type': const FhirField(false, 'CodeableConcept'),
    'suppliedItem': const FhirField(true, 'SupplyDeliverySuppliedItem'),
    'occurrenceDateTime': const FhirField(false, 'FhirDateTime'),
    'occurrenceDateTimeElement': const FhirField(false, 'Element'),
    'occurrencePeriod': const FhirField(false, 'Period'),
    'occurrenceTiming': const FhirField(false, 'Timing'),
    'supplier': const FhirField(false, 'Reference'),
    'destination': const FhirField(false, 'Reference'),
    'receiver': const FhirField(true, 'Reference'),
  },
  'SupplyDeliverySuppliedItem': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'quantity': const FhirField(false, 'Quantity'),
    'itemCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'itemReference': const FhirField(false, 'Reference'),
  },
  'SupplyRequest': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'basedOn': const FhirField(true, 'Reference'),
    'category': const FhirField(false, 'CodeableConcept'),
    'priority': const FhirField(false, 'Code'),
    'priorityElement': const FhirField(false, 'Element'),
    'deliverFor': const FhirField(false, 'Reference'),
    'item': const FhirField(false, 'CodeableReference'),
    'quantity': const FhirField(false, 'Quantity'),
    'parameter': const FhirField(true, 'SupplyRequestParameter'),
    'occurrenceDateTime': const FhirField(false, 'FhirDateTime'),
    'occurrenceDateTimeElement': const FhirField(false, 'Element'),
    'occurrencePeriod': const FhirField(false, 'Period'),
    'occurrenceTiming': const FhirField(false, 'Timing'),
    'authoredOn': const FhirField(false, 'FhirDateTime'),
    'authoredOnElement': const FhirField(false, 'Element'),
    'requester': const FhirField(false, 'Reference'),
    'supplier': const FhirField(true, 'Reference'),
    'reason': const FhirField(true, 'CodeableReference'),
    'deliverFrom': const FhirField(false, 'Reference'),
    'deliverTo': const FhirField(false, 'Reference'),
  },
  'SupplyRequestParameter': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(false, 'CodeableConcept'),
    'valueCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'valueQuantity': const FhirField(false, 'Quantity'),
    'valueRange': const FhirField(false, 'Range'),
    'valueBoolean': const FhirField(false, 'Boolean'),
    'valueBooleanElement': const FhirField(false, 'Element'),
  },
  'Task': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'instantiatesCanonical': const FhirField(false, 'Canonical'),
    'instantiatesUri': const FhirField(false, 'FhirUri'),
    'instantiatesUriElement': const FhirField(false, 'Element'),
    'basedOn': const FhirField(true, 'Reference'),
    'groupIdentifier': const FhirField(false, 'Identifier'),
    'partOf': const FhirField(true, 'Reference'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'statusReason': const FhirField(false, 'CodeableReference'),
    'businessStatus': const FhirField(false, 'CodeableConcept'),
    'intent': const FhirField(false, 'Code'),
    'intentElement': const FhirField(false, 'Element'),
    'priority': const FhirField(false, 'Code'),
    'priorityElement': const FhirField(false, 'Element'),
    'doNotPerform': const FhirField(false, 'Boolean'),
    'doNotPerformElement': const FhirField(false, 'Element'),
    'code': const FhirField(false, 'CodeableConcept'),
    'description': const FhirField(false, 'String'),
    'descriptionElement': const FhirField(false, 'Element'),
    'focus': const FhirField(false, 'Reference'),
    'for': const FhirField(false, 'Reference'),
    'encounter': const FhirField(false, 'Reference'),
    'requestedPeriod': const FhirField(false, 'Period'),
    'executionPeriod': const FhirField(false, 'Period'),
    'authoredOn': const FhirField(false, 'FhirDateTime'),
    'authoredOnElement': const FhirField(false, 'Element'),
    'lastModified': const FhirField(false, 'FhirDateTime'),
    'lastModifiedElement': const FhirField(false, 'Element'),
    'requester': const FhirField(false, 'Reference'),
    'requestedPerformer': const FhirField(true, 'CodeableReference'),
    'owner': const FhirField(false, 'Reference'),
    'performer': const FhirField(true, 'TaskPerformer'),
    'location': const FhirField(false, 'Reference'),
    'reason': const FhirField(true, 'CodeableReference'),
    'insurance': const FhirField(true, 'Reference'),
    'note': const FhirField(true, 'Annotation'),
    'relevantHistory': const FhirField(true, 'Reference'),
    'restriction': const FhirField(false, 'TaskRestriction'),
    'input': const FhirField(true, 'TaskInput'),
    'output': const FhirField(true, 'TaskOutput'),
  },
  'TaskPerformer': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'function': const FhirField(false, 'CodeableConcept'),
    'actor': const FhirField(false, 'Reference'),
  },
  'TaskRestriction': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'repetitions': const FhirField(false, 'PositiveInt'),
    'repetitionsElement': const FhirField(false, 'Element'),
    'period': const FhirField(false, 'Period'),
    'recipient': const FhirField(true, 'Reference'),
  },
  'TaskInput': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'valueBase64Binary': const FhirField(false, 'Base64Binary'),
    'valueBase64BinaryElement': const FhirField(false, 'Element'),
    'valueBoolean': const FhirField(false, 'Boolean'),
    'valueBooleanElement': const FhirField(false, 'Element'),
    'valueCanonical': const FhirField(false, 'Canonical'),
    'valueCanonicalElement': const FhirField(false, 'Element'),
    'valueCode': const FhirField(false, 'Code'),
    'valueCodeElement': const FhirField(false, 'Element'),
    'valueDate': const FhirField(false, 'Date'),
    'valueDateElement': const FhirField(false, 'Element'),
    'valueDateTime': const FhirField(false, 'FhirDateTime'),
    'valueDateTimeElement': const FhirField(false, 'Element'),
    'valueDecimal': const FhirField(false, 'Decimal'),
    'valueDecimalElement': const FhirField(false, 'Element'),
    'valueId': const FhirField(false, 'Id'),
    'valueIdElement': const FhirField(false, 'Element'),
    'valueInstant': const FhirField(false, 'Instant'),
    'valueInstantElement': const FhirField(false, 'Element'),
    'valueInteger': const FhirField(false, 'Integer'),
    'valueIntegerElement': const FhirField(false, 'Element'),
    'valueInteger64': const FhirField(false, 'Integer64'),
    'valueInteger64Element': const FhirField(false, 'Element'),
    'valueMarkdown': const FhirField(false, 'Markdown'),
    'valueMarkdownElement': const FhirField(false, 'Element'),
    'valueOid': const FhirField(false, 'Id'),
    'valueOidElement': const FhirField(false, 'Element'),
    'valuePositiveInt': const FhirField(false, 'PositiveInt'),
    'valuePositiveIntElement': const FhirField(false, 'Element'),
    'valueString': const FhirField(false, 'String'),
    'valueStringElement': const FhirField(false, 'Element'),
    'valueTime': const FhirField(false, 'Time'),
    'valueTimeElement': const FhirField(false, 'Element'),
    'valueUnsignedInt': const FhirField(false, 'UnsignedInt'),
    'valueUnsignedIntElement': const FhirField(false, 'Element'),
    'valueUri': const FhirField(false, 'FhirUri'),
    'valueUriElement': const FhirField(false, 'Element'),
    'valueUrl': const FhirField(false, 'FhirUrl'),
    'valueUrlElement': const FhirField(false, 'Element'),
    'valueUuid': const FhirField(false, 'Id'),
    'valueUuidElement': const FhirField(false, 'Element'),
    'valueAddress': const FhirField(false, 'Address'),
    'valueAge': const FhirField(false, 'Age'),
    'valueAnnotation': const FhirField(false, 'Annotation'),
    'valueAttachment': const FhirField(false, 'Attachment'),
    'valueCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'valueCodeableReference': const FhirField(false, 'CodeableReference'),
    'valueCoding': const FhirField(false, 'Coding'),
    'valueContactPoint': const FhirField(false, 'ContactPoint'),
    'valueCount': const FhirField(false, 'Count'),
    'valueDistance': const FhirField(false, 'Distance'),
    'valueDuration': const FhirField(false, 'FhirDuration'),
    'valueHumanName': const FhirField(false, 'HumanName'),
    'valueIdentifier': const FhirField(false, 'Identifier'),
    'valueMoney': const FhirField(false, 'Money'),
    'valuePeriod': const FhirField(false, 'Period'),
    'valueQuantity': const FhirField(false, 'Quantity'),
    'valueRange': const FhirField(false, 'Range'),
    'valueRatio': const FhirField(false, 'Ratio'),
    'valueRatioRange': const FhirField(false, 'RatioRange'),
    'valueReference': const FhirField(false, 'Reference'),
    'valueSampledData': const FhirField(false, 'SampledData'),
    'valueSignature': const FhirField(false, 'Signature'),
    'valueTiming': const FhirField(false, 'Timing'),
    'valueContactDetail': const FhirField(false, 'ContactDetail'),
    'valueDataRequirement': const FhirField(false, 'DataRequirement'),
    'valueExpression': const FhirField(false, 'Expression'),
    'valueParameterDefinition': const FhirField(false, 'ParameterDefinition'),
    'valueRelatedArtifact': const FhirField(false, 'RelatedArtifact'),
    'valueTriggerDefinition': const FhirField(false, 'TriggerDefinition'),
    'valueUsageContext': const FhirField(false, 'UsageContext'),
    'valueAvailability': const FhirField(false, 'Availability'),
    'valueExtendedContactDetail':
        const FhirField(false, 'ExtendedContactDetail'),
    'valueDosage': const FhirField(false, 'Dosage'),
    'valueMeta': const FhirField(false, 'Meta'),
  },
  'TaskOutput': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'valueBase64Binary': const FhirField(false, 'Base64Binary'),
    'valueBase64BinaryElement': const FhirField(false, 'Element'),
    'valueBoolean': const FhirField(false, 'Boolean'),
    'valueBooleanElement': const FhirField(false, 'Element'),
    'valueCanonical': const FhirField(false, 'Canonical'),
    'valueCanonicalElement': const FhirField(false, 'Element'),
    'valueCode': const FhirField(false, 'Code'),
    'valueCodeElement': const FhirField(false, 'Element'),
    'valueDate': const FhirField(false, 'Date'),
    'valueDateElement': const FhirField(false, 'Element'),
    'valueDateTime': const FhirField(false, 'FhirDateTime'),
    'valueDateTimeElement': const FhirField(false, 'Element'),
    'valueDecimal': const FhirField(false, 'Decimal'),
    'valueDecimalElement': const FhirField(false, 'Element'),
    'valueId': const FhirField(false, 'Id'),
    'valueIdElement': const FhirField(false, 'Element'),
    'valueInstant': const FhirField(false, 'Instant'),
    'valueInstantElement': const FhirField(false, 'Element'),
    'valueInteger': const FhirField(false, 'Integer'),
    'valueIntegerElement': const FhirField(false, 'Element'),
    'valueInteger64': const FhirField(false, 'Integer64'),
    'valueInteger64Element': const FhirField(false, 'Element'),
    'valueMarkdown': const FhirField(false, 'Markdown'),
    'valueMarkdownElement': const FhirField(false, 'Element'),
    'valueOid': const FhirField(false, 'Id'),
    'valueOidElement': const FhirField(false, 'Element'),
    'valuePositiveInt': const FhirField(false, 'PositiveInt'),
    'valuePositiveIntElement': const FhirField(false, 'Element'),
    'valueString': const FhirField(false, 'String'),
    'valueStringElement': const FhirField(false, 'Element'),
    'valueTime': const FhirField(false, 'Time'),
    'valueTimeElement': const FhirField(false, 'Element'),
    'valueUnsignedInt': const FhirField(false, 'UnsignedInt'),
    'valueUnsignedIntElement': const FhirField(false, 'Element'),
    'valueUri': const FhirField(false, 'FhirUri'),
    'valueUriElement': const FhirField(false, 'Element'),
    'valueUrl': const FhirField(false, 'FhirUrl'),
    'valueUrlElement': const FhirField(false, 'Element'),
    'valueUuid': const FhirField(false, 'Id'),
    'valueUuidElement': const FhirField(false, 'Element'),
    'valueAddress': const FhirField(false, 'Address'),
    'valueAge': const FhirField(false, 'Age'),
    'valueAnnotation': const FhirField(false, 'Annotation'),
    'valueAttachment': const FhirField(false, 'Attachment'),
    'valueCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'valueCodeableReference': const FhirField(false, 'CodeableReference'),
    'valueCoding': const FhirField(false, 'Coding'),
    'valueContactPoint': const FhirField(false, 'ContactPoint'),
    'valueCount': const FhirField(false, 'Count'),
    'valueDistance': const FhirField(false, 'Distance'),
    'valueDuration': const FhirField(false, 'FhirDuration'),
    'valueHumanName': const FhirField(false, 'HumanName'),
    'valueIdentifier': const FhirField(false, 'Identifier'),
    'valueMoney': const FhirField(false, 'Money'),
    'valuePeriod': const FhirField(false, 'Period'),
    'valueQuantity': const FhirField(false, 'Quantity'),
    'valueRange': const FhirField(false, 'Range'),
    'valueRatio': const FhirField(false, 'Ratio'),
    'valueRatioRange': const FhirField(false, 'RatioRange'),
    'valueReference': const FhirField(false, 'Reference'),
    'valueSampledData': const FhirField(false, 'SampledData'),
    'valueSignature': const FhirField(false, 'Signature'),
    'valueTiming': const FhirField(false, 'Timing'),
    'valueContactDetail': const FhirField(false, 'ContactDetail'),
    'valueDataRequirement': const FhirField(false, 'DataRequirement'),
    'valueExpression': const FhirField(false, 'Expression'),
    'valueParameterDefinition': const FhirField(false, 'ParameterDefinition'),
    'valueRelatedArtifact': const FhirField(false, 'RelatedArtifact'),
    'valueTriggerDefinition': const FhirField(false, 'TriggerDefinition'),
    'valueUsageContext': const FhirField(false, 'UsageContext'),
    'valueAvailability': const FhirField(false, 'Availability'),
    'valueExtendedContactDetail':
        const FhirField(false, 'ExtendedContactDetail'),
    'valueDosage': const FhirField(false, 'Dosage'),
    'valueMeta': const FhirField(false, 'Meta'),
  },
  'TerminologyCapabilities': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'url': const FhirField(false, 'FhirUri'),
    'urlElement': const FhirField(false, 'Element'),
    'identifier': const FhirField(true, 'Identifier'),
    'version': const FhirField(false, 'String'),
    'versionElement': const FhirField(false, 'Element'),
    'versionAlgorithmString': const FhirField(false, 'String'),
    'versionAlgorithmStringElement': const FhirField(false, 'Element'),
    'versionAlgorithmCoding': const FhirField(false, 'Coding'),
    'name': const FhirField(false, 'String'),
    'nameElement': const FhirField(false, 'Element'),
    'title': const FhirField(false, 'String'),
    'titleElement': const FhirField(false, 'Element'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'experimental': const FhirField(false, 'Boolean'),
    'experimentalElement': const FhirField(false, 'Element'),
    'date': const FhirField(false, 'FhirDateTime'),
    'dateElement': const FhirField(false, 'Element'),
    'publisher': const FhirField(false, 'String'),
    'publisherElement': const FhirField(false, 'Element'),
    'contact': const FhirField(true, 'ContactDetail'),
    'description': const FhirField(false, 'Markdown'),
    'descriptionElement': const FhirField(false, 'Element'),
    'useContext': const FhirField(true, 'UsageContext'),
    'jurisdiction': const FhirField(true, 'CodeableConcept'),
    'purpose': const FhirField(false, 'Markdown'),
    'purposeElement': const FhirField(false, 'Element'),
    'copyright': const FhirField(false, 'Markdown'),
    'copyrightElement': const FhirField(false, 'Element'),
    'copyrightLabel': const FhirField(false, 'String'),
    'copyrightLabelElement': const FhirField(false, 'Element'),
    'kind': const FhirField(false, 'Code'),
    'kindElement': const FhirField(false, 'Element'),
    'software': const FhirField(false, 'TerminologyCapabilitiesSoftware'),
    'implementation':
        const FhirField(false, 'TerminologyCapabilitiesImplementation'),
    'lockedDate': const FhirField(false, 'Boolean'),
    'lockedDateElement': const FhirField(false, 'Element'),
    'codeSystem': const FhirField(true, 'TerminologyCapabilitiesCodeSystem'),
    'expansion': const FhirField(false, 'TerminologyCapabilitiesExpansion'),
    'codeSearch': const FhirField(false, 'Code'),
    'codeSearchElement': const FhirField(false, 'Element'),
    'validateCode': const FhirField(false, 'Code'),
    'translation': const FhirField(false, 'TerminologyCapabilitiesTranslation'),
    'closure': const FhirField(false, 'TerminologyCapabilitiesClosure'),
  },
  'TerminologyCapabilitiesSoftware': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'name': const FhirField(false, 'String'),
    'nameElement': const FhirField(false, 'Element'),
    'version': const FhirField(false, 'String'),
    'versionElement': const FhirField(false, 'Element'),
  },
  'TerminologyCapabilitiesImplementation': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'description': const FhirField(false, 'String'),
    'descriptionElement': const FhirField(false, 'Element'),
    'url': const FhirField(false, 'FhirUrl'),
    'urlElement': const FhirField(false, 'Element'),
  },
  'TerminologyCapabilitiesCodeSystem': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'uri': const FhirField(false, 'Canonical'),
    'version': const FhirField(true, 'TerminologyCapabilitiesVersion'),
    'content': const FhirField(false, 'Code'),
    'contentElement': const FhirField(false, 'Element'),
    'subsumption': const FhirField(false, 'Boolean'),
    'subsumptionElement': const FhirField(false, 'Element'),
  },
  'TerminologyCapabilitiesVersion': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(false, 'String'),
    'codeElement': const FhirField(false, 'Element'),
    'isDefault': const FhirField(false, 'Boolean'),
    'isDefaultElement': const FhirField(false, 'Element'),
    'compositional': const FhirField(false, 'Boolean'),
    'compositionalElement': const FhirField(false, 'Element'),
    'language': const FhirField(true, 'Code'),
    'languageElement': const FhirField(true, 'Element'),
    'filter': const FhirField(true, 'TerminologyCapabilitiesFilter'),
    'property': const FhirField(true, 'Code'),
    'propertyElement': const FhirField(true, 'Element'),
  },
  'TerminologyCapabilitiesFilter': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(false, 'Code'),
    'codeElement': const FhirField(false, 'Element'),
    'op': const FhirField(true, 'Code'),
    'opElement': const FhirField(true, 'Element'),
  },
  'TerminologyCapabilitiesExpansion': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'hierarchical': const FhirField(false, 'Boolean'),
    'hierarchicalElement': const FhirField(false, 'Element'),
    'paging': const FhirField(false, 'Boolean'),
    'pagingElement': const FhirField(false, 'Element'),
    'incomplete': const FhirField(false, 'Boolean'),
    'incompleteElement': const FhirField(false, 'Element'),
    'parameter': const FhirField(true, 'TerminologyCapabilitiesParameter'),
    'textFilter': const FhirField(false, 'Markdown'),
    'textFilterElement': const FhirField(false, 'Element'),
  },
  'TerminologyCapabilitiesParameter': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'name': const FhirField(false, 'Code'),
    'nameElement': const FhirField(false, 'Element'),
    'documentation': const FhirField(false, 'String'),
    'documentationElement': const FhirField(false, 'Element'),
  },
  'TerminologyCapabilitiesValidateCode': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'translations': const FhirField(false, 'Boolean'),
    'translationsElement': const FhirField(false, 'Element'),
  },
  'TerminologyCapabilitiesTranslation': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'needsMap': const FhirField(false, 'Boolean'),
    'needsMapElement': const FhirField(false, 'Element'),
  },
  'TerminologyCapabilitiesClosure': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'translation': const FhirField(false, 'Boolean'),
    'translationElement': const FhirField(false, 'Element'),
  },
  'TestPlan': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'url': const FhirField(false, 'FhirUri'),
    'urlElement': const FhirField(false, 'Element'),
    'identifier': const FhirField(true, 'Identifier'),
    'version': const FhirField(false, 'String'),
    'versionElement': const FhirField(false, 'Element'),
    'versionAlgorithmString': const FhirField(false, 'String'),
    'versionAlgorithmStringElement': const FhirField(false, 'Element'),
    'versionAlgorithmCoding': const FhirField(false, 'Coding'),
    'name': const FhirField(false, 'String'),
    'nameElement': const FhirField(false, 'Element'),
    'title': const FhirField(false, 'String'),
    'titleElement': const FhirField(false, 'Element'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'experimental': const FhirField(false, 'Boolean'),
    'experimentalElement': const FhirField(false, 'Element'),
    'date': const FhirField(false, 'FhirDateTime'),
    'dateElement': const FhirField(false, 'Element'),
    'publisher': const FhirField(false, 'String'),
    'publisherElement': const FhirField(false, 'Element'),
    'contact': const FhirField(true, 'ContactDetail'),
    'description': const FhirField(false, 'Markdown'),
    'descriptionElement': const FhirField(false, 'Element'),
    'useContext': const FhirField(true, 'UsageContext'),
    'jurisdiction': const FhirField(true, 'CodeableConcept'),
    'purpose': const FhirField(false, 'Markdown'),
    'purposeElement': const FhirField(false, 'Element'),
    'copyright': const FhirField(false, 'Markdown'),
    'copyrightElement': const FhirField(false, 'Element'),
    'copyrightLabel': const FhirField(false, 'String'),
    'copyrightLabelElement': const FhirField(false, 'Element'),
    'category': const FhirField(true, 'CodeableConcept'),
    'scope': const FhirField(true, 'Reference'),
    'testTools': const FhirField(false, 'Markdown'),
    'testToolsElement': const FhirField(false, 'Element'),
    'dependency': const FhirField(true, 'TestPlanDependency'),
    'exitCriteria': const FhirField(false, 'Markdown'),
    'exitCriteriaElement': const FhirField(false, 'Element'),
    'testCase': const FhirField(true, 'TestPlanTestCase'),
  },
  'TestPlanDependency': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'description': const FhirField(false, 'Markdown'),
    'descriptionElement': const FhirField(false, 'Element'),
    'predecessor': const FhirField(false, 'Reference'),
  },
  'TestPlanTestCase': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'sequence': const FhirField(false, 'Integer'),
    'sequenceElement': const FhirField(false, 'Element'),
    'scope': const FhirField(true, 'Reference'),
    'dependency': const FhirField(true, 'TestPlanDependency1'),
    'testRun': const FhirField(true, 'TestPlanTestRun'),
    'testData': const FhirField(true, 'TestPlanTestData'),
    'assertion': const FhirField(true, 'TestPlanAssertion'),
  },
  'TestPlanDependency1': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'description': const FhirField(false, 'Markdown'),
    'descriptionElement': const FhirField(false, 'Element'),
    'predecessor': const FhirField(false, 'Reference'),
  },
  'TestPlanTestRun': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'narrative': const FhirField(false, 'Markdown'),
    'narrativeElement': const FhirField(false, 'Element'),
    'script': const FhirField(false, 'TestPlanScript'),
  },
  'TestPlanScript': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'language': const FhirField(false, 'CodeableConcept'),
    'sourceString': const FhirField(false, 'String'),
    'sourceStringElement': const FhirField(false, 'Element'),
    'sourceReference': const FhirField(false, 'Reference'),
  },
  'TestPlanTestData': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'Coding'),
    'content': const FhirField(false, 'Reference'),
    'sourceString': const FhirField(false, 'String'),
    'sourceStringElement': const FhirField(false, 'Element'),
    'sourceReference': const FhirField(false, 'Reference'),
  },
  'TestPlanAssertion': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(true, 'CodeableConcept'),
    'object': const FhirField(true, 'CodeableReference'),
    'result': const FhirField(true, 'CodeableReference'),
  },
  'TestReport': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(false, 'Identifier'),
    'name': const FhirField(false, 'String'),
    'nameElement': const FhirField(false, 'Element'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'testScript': const FhirField(false, 'Canonical'),
    'result': const FhirField(false, 'Code'),
    'resultElement': const FhirField(false, 'Element'),
    'score': const FhirField(false, 'Decimal'),
    'scoreElement': const FhirField(false, 'Element'),
    'tester': const FhirField(false, 'String'),
    'testerElement': const FhirField(false, 'Element'),
    'issued': const FhirField(false, 'FhirDateTime'),
    'issuedElement': const FhirField(false, 'Element'),
    'participant': const FhirField(true, 'TestReportParticipant'),
    'setup': const FhirField(false, 'TestReportSetup'),
    'test': const FhirField(true, 'TestReportTest'),
    'teardown': const FhirField(false, 'TestReportTeardown'),
  },
  'TestReportParticipant': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'Code'),
    'typeElement': const FhirField(false, 'Element'),
    'uri': const FhirField(false, 'FhirUri'),
    'uriElement': const FhirField(false, 'Element'),
    'display': const FhirField(false, 'String'),
    'displayElement': const FhirField(false, 'Element'),
  },
  'TestReportSetup': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'action': const FhirField(true, 'TestReportAction'),
  },
  'TestReportAction': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'operation': const FhirField(false, 'TestReportOperation'),
    'assert': const FhirField(false, 'TestReportAssert'),
  },
  'TestReportOperation': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'result': const FhirField(false, 'Code'),
    'resultElement': const FhirField(false, 'Element'),
    'message': const FhirField(false, 'Markdown'),
    'messageElement': const FhirField(false, 'Element'),
    'detail': const FhirField(false, 'FhirUri'),
    'detailElement': const FhirField(false, 'Element'),
  },
  'TestReportAssert': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'result': const FhirField(false, 'Code'),
    'resultElement': const FhirField(false, 'Element'),
    'message': const FhirField(false, 'Markdown'),
    'messageElement': const FhirField(false, 'Element'),
    'detail': const FhirField(false, 'String'),
    'detailElement': const FhirField(false, 'Element'),
    'requirement': const FhirField(true, 'TestReportRequirement'),
  },
  'TestReportRequirement': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'linkUri': const FhirField(false, 'FhirUri'),
    'linkUriElement': const FhirField(false, 'Element'),
    'linkCanonical': const FhirField(false, 'Canonical'),
    'linkCanonicalElement': const FhirField(false, 'Element'),
  },
  'TestReportTest': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'name': const FhirField(false, 'String'),
    'nameElement': const FhirField(false, 'Element'),
    'description': const FhirField(false, 'String'),
    'descriptionElement': const FhirField(false, 'Element'),
    'action': const FhirField(true, 'TestReportAction1'),
  },
  'TestReportAction1': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'operation': const FhirField(false, 'TestReportOperation'),
    'assert': const FhirField(false, 'TestReportAssert'),
  },
  'TestReportTeardown': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'action': const FhirField(true, 'TestReportAction2'),
  },
  'TestReportAction2': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'operation': const FhirField(false, 'TestReportOperation'),
  },
  'TestScript': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'url': const FhirField(false, 'FhirUri'),
    'urlElement': const FhirField(false, 'Element'),
    'identifier': const FhirField(true, 'Identifier'),
    'version': const FhirField(false, 'String'),
    'versionElement': const FhirField(false, 'Element'),
    'versionAlgorithmString': const FhirField(false, 'String'),
    'versionAlgorithmStringElement': const FhirField(false, 'Element'),
    'versionAlgorithmCoding': const FhirField(false, 'Coding'),
    'name': const FhirField(false, 'String'),
    'nameElement': const FhirField(false, 'Element'),
    'title': const FhirField(false, 'String'),
    'titleElement': const FhirField(false, 'Element'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'experimental': const FhirField(false, 'Boolean'),
    'experimentalElement': const FhirField(false, 'Element'),
    'date': const FhirField(false, 'FhirDateTime'),
    'dateElement': const FhirField(false, 'Element'),
    'publisher': const FhirField(false, 'String'),
    'publisherElement': const FhirField(false, 'Element'),
    'contact': const FhirField(true, 'ContactDetail'),
    'description': const FhirField(false, 'Markdown'),
    'descriptionElement': const FhirField(false, 'Element'),
    'useContext': const FhirField(true, 'UsageContext'),
    'jurisdiction': const FhirField(true, 'CodeableConcept'),
    'purpose': const FhirField(false, 'Markdown'),
    'purposeElement': const FhirField(false, 'Element'),
    'copyright': const FhirField(false, 'Markdown'),
    'copyrightElement': const FhirField(false, 'Element'),
    'copyrightLabel': const FhirField(false, 'String'),
    'copyrightLabelElement': const FhirField(false, 'Element'),
    'origin': const FhirField(true, 'TestScriptOrigin'),
    'destination': const FhirField(true, 'TestScriptDestination'),
    'metadata': const FhirField(false, 'TestScriptMetadata'),
    'scope': const FhirField(true, 'TestScriptScope'),
    'fixture': const FhirField(true, 'TestScriptFixture'),
    'profile': const FhirField(true, 'Canonical'),
    'variable': const FhirField(true, 'TestScriptVariable'),
    'setup': const FhirField(false, 'TestScriptSetup'),
    'test': const FhirField(true, 'TestScriptTest'),
    'teardown': const FhirField(false, 'TestScriptTeardown'),
  },
  'TestScriptOrigin': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'index': const FhirField(false, 'Integer'),
    'indexElement': const FhirField(false, 'Element'),
    'profile': const FhirField(false, 'Coding'),
    'url': const FhirField(false, 'FhirUrl'),
    'urlElement': const FhirField(false, 'Element'),
  },
  'TestScriptDestination': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'index': const FhirField(false, 'Integer'),
    'indexElement': const FhirField(false, 'Element'),
    'profile': const FhirField(false, 'Coding'),
    'url': const FhirField(false, 'FhirUrl'),
    'urlElement': const FhirField(false, 'Element'),
  },
  'TestScriptMetadata': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'link': const FhirField(true, 'TestScriptLink'),
    'capability': const FhirField(true, 'TestScriptCapability'),
  },
  'TestScriptLink': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'url': const FhirField(false, 'FhirUri'),
    'urlElement': const FhirField(false, 'Element'),
    'description': const FhirField(false, 'String'),
    'descriptionElement': const FhirField(false, 'Element'),
  },
  'TestScriptCapability': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'required': const FhirField(false, 'Boolean'),
    'requiredElement': const FhirField(false, 'Element'),
    'validated': const FhirField(false, 'Boolean'),
    'validatedElement': const FhirField(false, 'Element'),
    'description': const FhirField(false, 'String'),
    'descriptionElement': const FhirField(false, 'Element'),
    'origin': const FhirField(true, 'Integer'),
    'originElement': const FhirField(true, 'Element'),
    'destination': const FhirField(false, 'Integer'),
    'destinationElement': const FhirField(false, 'Element'),
    'link': const FhirField(true, 'FhirUri'),
    'linkElement': const FhirField(true, 'Element'),
    'capabilities': const FhirField(false, 'Canonical'),
  },
  'TestScriptScope': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'artifact': const FhirField(false, 'Canonical'),
    'conformance': const FhirField(false, 'CodeableConcept'),
    'phase': const FhirField(false, 'CodeableConcept'),
  },
  'TestScriptFixture': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'autocreate': const FhirField(false, 'Boolean'),
    'autocreateElement': const FhirField(false, 'Element'),
    'autodelete': const FhirField(false, 'Boolean'),
    'autodeleteElement': const FhirField(false, 'Element'),
    'resource': const FhirField(false, 'Reference'),
  },
  'TestScriptVariable': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'name': const FhirField(false, 'String'),
    'nameElement': const FhirField(false, 'Element'),
    'defaultValue': const FhirField(false, 'String'),
    'defaultValueElement': const FhirField(false, 'Element'),
    'description': const FhirField(false, 'String'),
    'descriptionElement': const FhirField(false, 'Element'),
    'expression': const FhirField(false, 'String'),
    'expressionElement': const FhirField(false, 'Element'),
    'headerField': const FhirField(false, 'String'),
    'headerFieldElement': const FhirField(false, 'Element'),
    'hint': const FhirField(false, 'String'),
    'hintElement': const FhirField(false, 'Element'),
    'path': const FhirField(false, 'String'),
    'pathElement': const FhirField(false, 'Element'),
    'sourceId': const FhirField(false, 'Id'),
    'sourceIdElement': const FhirField(false, 'Element'),
  },
  'TestScriptSetup': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'action': const FhirField(true, 'TestScriptAction'),
  },
  'TestScriptAction': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'operation': const FhirField(false, 'TestScriptOperation'),
    'assert': const FhirField(false, 'TestScriptAssert'),
  },
  'TestScriptOperation': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'Coding'),
    'resource': const FhirField(false, 'FhirUri'),
    'resourceElement': const FhirField(false, 'Element'),
    'label': const FhirField(false, 'String'),
    'labelElement': const FhirField(false, 'Element'),
    'description': const FhirField(false, 'String'),
    'descriptionElement': const FhirField(false, 'Element'),
    'accept': const FhirField(false, 'Code'),
    'acceptElement': const FhirField(false, 'Element'),
    'contentType': const FhirField(false, 'Code'),
    'contentTypeElement': const FhirField(false, 'Element'),
    'destination': const FhirField(false, 'Integer'),
    'destinationElement': const FhirField(false, 'Element'),
    'encodeRequestUrl': const FhirField(false, 'Boolean'),
    'encodeRequestUrlElement': const FhirField(false, 'Element'),
    'method': const FhirField(false, 'Code'),
    'methodElement': const FhirField(false, 'Element'),
    'origin': const FhirField(false, 'Integer'),
    'originElement': const FhirField(false, 'Element'),
    'params': const FhirField(false, 'String'),
    'paramsElement': const FhirField(false, 'Element'),
    'requestHeader': const FhirField(true, 'TestScriptRequestHeader'),
    'requestId': const FhirField(false, 'Id'),
    'requestIdElement': const FhirField(false, 'Element'),
    'responseId': const FhirField(false, 'Id'),
    'responseIdElement': const FhirField(false, 'Element'),
    'sourceId': const FhirField(false, 'Id'),
    'sourceIdElement': const FhirField(false, 'Element'),
    'targetId': const FhirField(false, 'Id'),
    'targetIdElement': const FhirField(false, 'Element'),
    'url': const FhirField(false, 'String'),
    'urlElement': const FhirField(false, 'Element'),
  },
  'TestScriptRequestHeader': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'field': const FhirField(false, 'String'),
    'fieldElement': const FhirField(false, 'Element'),
    'value': const FhirField(false, 'String'),
    'valueElement': const FhirField(false, 'Element'),
  },
  'TestScriptAssert': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'label': const FhirField(false, 'String'),
    'labelElement': const FhirField(false, 'Element'),
    'description': const FhirField(false, 'String'),
    'descriptionElement': const FhirField(false, 'Element'),
    'direction': const FhirField(false, 'Code'),
    'directionElement': const FhirField(false, 'Element'),
    'compareToSourceId': const FhirField(false, 'String'),
    'compareToSourceIdElement': const FhirField(false, 'Element'),
    'compareToSourceExpression': const FhirField(false, 'String'),
    'compareToSourceExpressionElement': const FhirField(false, 'Element'),
    'compareToSourcePath': const FhirField(false, 'String'),
    'compareToSourcePathElement': const FhirField(false, 'Element'),
    'contentType': const FhirField(false, 'Code'),
    'contentTypeElement': const FhirField(false, 'Element'),
    'defaultManualCompletion': const FhirField(false, 'Code'),
    'defaultManualCompletionElement': const FhirField(false, 'Element'),
    'expression': const FhirField(false, 'String'),
    'expressionElement': const FhirField(false, 'Element'),
    'headerField': const FhirField(false, 'String'),
    'headerFieldElement': const FhirField(false, 'Element'),
    'minimumId': const FhirField(false, 'String'),
    'minimumIdElement': const FhirField(false, 'Element'),
    'navigationLinks': const FhirField(false, 'Boolean'),
    'navigationLinksElement': const FhirField(false, 'Element'),
    'operator': const FhirField(false, 'Code'),
    'operatorElement': const FhirField(false, 'Element'),
    'path': const FhirField(false, 'String'),
    'pathElement': const FhirField(false, 'Element'),
    'requestMethod': const FhirField(false, 'Code'),
    'requestMethodElement': const FhirField(false, 'Element'),
    'requestURL': const FhirField(false, 'String'),
    'requestURLElement': const FhirField(false, 'Element'),
    'resource': const FhirField(false, 'FhirUri'),
    'resourceElement': const FhirField(false, 'Element'),
    'response': const FhirField(false, 'Code'),
    'responseElement': const FhirField(false, 'Element'),
    'responseCode': const FhirField(false, 'String'),
    'responseCodeElement': const FhirField(false, 'Element'),
    'sourceId': const FhirField(false, 'Id'),
    'sourceIdElement': const FhirField(false, 'Element'),
    'stopTestOnFail': const FhirField(false, 'Boolean'),
    'stopTestOnFailElement': const FhirField(false, 'Element'),
    'validateProfileId': const FhirField(false, 'Id'),
    'validateProfileIdElement': const FhirField(false, 'Element'),
    'value': const FhirField(false, 'String'),
    'valueElement': const FhirField(false, 'Element'),
    'warningOnly': const FhirField(false, 'Boolean'),
    'warningOnlyElement': const FhirField(false, 'Element'),
    'requirement': const FhirField(true, 'TestScriptRequirement'),
  },
  'TestScriptRequirement': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'linkUri': const FhirField(false, 'FhirUri'),
    'linkUriElement': const FhirField(false, 'Element'),
    'linkCanonical': const FhirField(false, 'Canonical'),
    'linkCanonicalElement': const FhirField(false, 'Element'),
  },
  'TestScriptTest': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'name': const FhirField(false, 'String'),
    'nameElement': const FhirField(false, 'Element'),
    'description': const FhirField(false, 'String'),
    'descriptionElement': const FhirField(false, 'Element'),
    'action': const FhirField(true, 'TestScriptAction1'),
  },
  'TestScriptAction1': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'operation': const FhirField(false, 'TestScriptOperation'),
    'assert': const FhirField(false, 'TestScriptAssert'),
  },
  'TestScriptTeardown': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'action': const FhirField(true, 'TestScriptAction2'),
  },
  'TestScriptAction2': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'operation': const FhirField(false, 'TestScriptOperation'),
  },
  'Transport': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'instantiatesCanonical': const FhirField(false, 'Canonical'),
    'instantiatesUri': const FhirField(false, 'FhirUri'),
    'instantiatesUriElement': const FhirField(false, 'Element'),
    'basedOn': const FhirField(true, 'Reference'),
    'groupIdentifier': const FhirField(false, 'Identifier'),
    'partOf': const FhirField(true, 'Reference'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'statusReason': const FhirField(false, 'CodeableConcept'),
    'intent': const FhirField(false, 'Code'),
    'intentElement': const FhirField(false, 'Element'),
    'priority': const FhirField(false, 'Code'),
    'priorityElement': const FhirField(false, 'Element'),
    'code': const FhirField(false, 'CodeableConcept'),
    'description': const FhirField(false, 'String'),
    'descriptionElement': const FhirField(false, 'Element'),
    'focus': const FhirField(false, 'Reference'),
    'for': const FhirField(false, 'Reference'),
    'encounter': const FhirField(false, 'Reference'),
    'completionTime': const FhirField(false, 'FhirDateTime'),
    'completionTimeElement': const FhirField(false, 'Element'),
    'authoredOn': const FhirField(false, 'FhirDateTime'),
    'authoredOnElement': const FhirField(false, 'Element'),
    'lastModified': const FhirField(false, 'FhirDateTime'),
    'lastModifiedElement': const FhirField(false, 'Element'),
    'requester': const FhirField(false, 'Reference'),
    'performerType': const FhirField(true, 'CodeableConcept'),
    'owner': const FhirField(false, 'Reference'),
    'location': const FhirField(false, 'Reference'),
    'insurance': const FhirField(true, 'Reference'),
    'note': const FhirField(true, 'Annotation'),
    'relevantHistory': const FhirField(true, 'Reference'),
    'restriction': const FhirField(false, 'TransportRestriction'),
    'input': const FhirField(true, 'TransportInput'),
    'output': const FhirField(true, 'TransportOutput'),
    'requestedLocation': const FhirField(false, 'Reference'),
    'currentLocation': const FhirField(false, 'Reference'),
    'reason': const FhirField(false, 'CodeableReference'),
    'history': const FhirField(false, 'Reference'),
  },
  'TransportRestriction': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'repetitions': const FhirField(false, 'PositiveInt'),
    'repetitionsElement': const FhirField(false, 'Element'),
    'period': const FhirField(false, 'Period'),
    'recipient': const FhirField(true, 'Reference'),
  },
  'TransportInput': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'valueBase64Binary': const FhirField(false, 'Base64Binary'),
    'valueBase64BinaryElement': const FhirField(false, 'Element'),
    'valueBoolean': const FhirField(false, 'Boolean'),
    'valueBooleanElement': const FhirField(false, 'Element'),
    'valueCanonical': const FhirField(false, 'Canonical'),
    'valueCanonicalElement': const FhirField(false, 'Element'),
    'valueCode': const FhirField(false, 'Code'),
    'valueCodeElement': const FhirField(false, 'Element'),
    'valueDate': const FhirField(false, 'Date'),
    'valueDateElement': const FhirField(false, 'Element'),
    'valueDateTime': const FhirField(false, 'FhirDateTime'),
    'valueDateTimeElement': const FhirField(false, 'Element'),
    'valueDecimal': const FhirField(false, 'Decimal'),
    'valueDecimalElement': const FhirField(false, 'Element'),
    'valueId': const FhirField(false, 'Id'),
    'valueIdElement': const FhirField(false, 'Element'),
    'valueInstant': const FhirField(false, 'Instant'),
    'valueInstantElement': const FhirField(false, 'Element'),
    'valueInteger': const FhirField(false, 'Integer'),
    'valueIntegerElement': const FhirField(false, 'Element'),
    'valueInteger64': const FhirField(false, 'Integer64'),
    'valueInteger64Element': const FhirField(false, 'Element'),
    'valueMarkdown': const FhirField(false, 'Markdown'),
    'valueMarkdownElement': const FhirField(false, 'Element'),
    'valueOid': const FhirField(false, 'Id'),
    'valueOidElement': const FhirField(false, 'Element'),
    'valuePositiveInt': const FhirField(false, 'PositiveInt'),
    'valuePositiveIntElement': const FhirField(false, 'Element'),
    'valueString': const FhirField(false, 'String'),
    'valueStringElement': const FhirField(false, 'Element'),
    'valueTime': const FhirField(false, 'Time'),
    'valueTimeElement': const FhirField(false, 'Element'),
    'valueUnsignedInt': const FhirField(false, 'UnsignedInt'),
    'valueUnsignedIntElement': const FhirField(false, 'Element'),
    'valueUri': const FhirField(false, 'FhirUri'),
    'valueUriElement': const FhirField(false, 'Element'),
    'valueUrl': const FhirField(false, 'FhirUrl'),
    'valueUrlElement': const FhirField(false, 'Element'),
    'valueUuid': const FhirField(false, 'Id'),
    'valueUuidElement': const FhirField(false, 'Element'),
    'valueAddress': const FhirField(false, 'Address'),
    'valueAge': const FhirField(false, 'Age'),
    'valueAnnotation': const FhirField(false, 'Annotation'),
    'valueAttachment': const FhirField(false, 'Attachment'),
    'valueCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'valueCodeableReference': const FhirField(false, 'CodeableReference'),
    'valueCoding': const FhirField(false, 'Coding'),
    'valueContactPoint': const FhirField(false, 'ContactPoint'),
    'valueCount': const FhirField(false, 'Count'),
    'valueDistance': const FhirField(false, 'Distance'),
    'valueDuration': const FhirField(false, 'FhirDuration'),
    'valueHumanName': const FhirField(false, 'HumanName'),
    'valueIdentifier': const FhirField(false, 'Identifier'),
    'valueMoney': const FhirField(false, 'Money'),
    'valuePeriod': const FhirField(false, 'Period'),
    'valueQuantity': const FhirField(false, 'Quantity'),
    'valueRange': const FhirField(false, 'Range'),
    'valueRatio': const FhirField(false, 'Ratio'),
    'valueRatioRange': const FhirField(false, 'RatioRange'),
    'valueReference': const FhirField(false, 'Reference'),
    'valueSampledData': const FhirField(false, 'SampledData'),
    'valueSignature': const FhirField(false, 'Signature'),
    'valueTiming': const FhirField(false, 'Timing'),
    'valueContactDetail': const FhirField(false, 'ContactDetail'),
    'valueDataRequirement': const FhirField(false, 'DataRequirement'),
    'valueExpression': const FhirField(false, 'Expression'),
    'valueParameterDefinition': const FhirField(false, 'ParameterDefinition'),
    'valueRelatedArtifact': const FhirField(false, 'RelatedArtifact'),
    'valueTriggerDefinition': const FhirField(false, 'TriggerDefinition'),
    'valueUsageContext': const FhirField(false, 'UsageContext'),
    'valueAvailability': const FhirField(false, 'Availability'),
    'valueExtendedContactDetail':
        const FhirField(false, 'ExtendedContactDetail'),
    'valueDosage': const FhirField(false, 'Dosage'),
    'valueMeta': const FhirField(false, 'Meta'),
  },
  'TransportOutput': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'type': const FhirField(false, 'CodeableConcept'),
    'valueBase64Binary': const FhirField(false, 'Base64Binary'),
    'valueBase64BinaryElement': const FhirField(false, 'Element'),
    'valueBoolean': const FhirField(false, 'Boolean'),
    'valueBooleanElement': const FhirField(false, 'Element'),
    'valueCanonical': const FhirField(false, 'Canonical'),
    'valueCanonicalElement': const FhirField(false, 'Element'),
    'valueCode': const FhirField(false, 'Code'),
    'valueCodeElement': const FhirField(false, 'Element'),
    'valueDate': const FhirField(false, 'Date'),
    'valueDateElement': const FhirField(false, 'Element'),
    'valueDateTime': const FhirField(false, 'FhirDateTime'),
    'valueDateTimeElement': const FhirField(false, 'Element'),
    'valueDecimal': const FhirField(false, 'Decimal'),
    'valueDecimalElement': const FhirField(false, 'Element'),
    'valueId': const FhirField(false, 'Id'),
    'valueIdElement': const FhirField(false, 'Element'),
    'valueInstant': const FhirField(false, 'Instant'),
    'valueInstantElement': const FhirField(false, 'Element'),
    'valueInteger': const FhirField(false, 'Integer'),
    'valueIntegerElement': const FhirField(false, 'Element'),
    'valueInteger64': const FhirField(false, 'Integer64'),
    'valueInteger64Element': const FhirField(false, 'Element'),
    'valueMarkdown': const FhirField(false, 'Markdown'),
    'valueMarkdownElement': const FhirField(false, 'Element'),
    'valueOid': const FhirField(false, 'Id'),
    'valueOidElement': const FhirField(false, 'Element'),
    'valuePositiveInt': const FhirField(false, 'PositiveInt'),
    'valuePositiveIntElement': const FhirField(false, 'Element'),
    'valueString': const FhirField(false, 'String'),
    'valueStringElement': const FhirField(false, 'Element'),
    'valueTime': const FhirField(false, 'Time'),
    'valueTimeElement': const FhirField(false, 'Element'),
    'valueUnsignedInt': const FhirField(false, 'UnsignedInt'),
    'valueUnsignedIntElement': const FhirField(false, 'Element'),
    'valueUri': const FhirField(false, 'FhirUri'),
    'valueUriElement': const FhirField(false, 'Element'),
    'valueUrl': const FhirField(false, 'FhirUrl'),
    'valueUrlElement': const FhirField(false, 'Element'),
    'valueUuid': const FhirField(false, 'Id'),
    'valueUuidElement': const FhirField(false, 'Element'),
    'valueAddress': const FhirField(false, 'Address'),
    'valueAge': const FhirField(false, 'Age'),
    'valueAnnotation': const FhirField(false, 'Annotation'),
    'valueAttachment': const FhirField(false, 'Attachment'),
    'valueCodeableConcept': const FhirField(false, 'CodeableConcept'),
    'valueCodeableReference': const FhirField(false, 'CodeableReference'),
    'valueCoding': const FhirField(false, 'Coding'),
    'valueContactPoint': const FhirField(false, 'ContactPoint'),
    'valueCount': const FhirField(false, 'Count'),
    'valueDistance': const FhirField(false, 'Distance'),
    'valueDuration': const FhirField(false, 'FhirDuration'),
    'valueHumanName': const FhirField(false, 'HumanName'),
    'valueIdentifier': const FhirField(false, 'Identifier'),
    'valueMoney': const FhirField(false, 'Money'),
    'valuePeriod': const FhirField(false, 'Period'),
    'valueQuantity': const FhirField(false, 'Quantity'),
    'valueRange': const FhirField(false, 'Range'),
    'valueRatio': const FhirField(false, 'Ratio'),
    'valueRatioRange': const FhirField(false, 'RatioRange'),
    'valueReference': const FhirField(false, 'Reference'),
    'valueSampledData': const FhirField(false, 'SampledData'),
    'valueSignature': const FhirField(false, 'Signature'),
    'valueTiming': const FhirField(false, 'Timing'),
    'valueContactDetail': const FhirField(false, 'ContactDetail'),
    'valueDataRequirement': const FhirField(false, 'DataRequirement'),
    'valueExpression': const FhirField(false, 'Expression'),
    'valueParameterDefinition': const FhirField(false, 'ParameterDefinition'),
    'valueRelatedArtifact': const FhirField(false, 'RelatedArtifact'),
    'valueTriggerDefinition': const FhirField(false, 'TriggerDefinition'),
    'valueUsageContext': const FhirField(false, 'UsageContext'),
    'valueAvailability': const FhirField(false, 'Availability'),
    'valueExtendedContactDetail':
        const FhirField(false, 'ExtendedContactDetail'),
    'valueDosage': const FhirField(false, 'Dosage'),
    'valueMeta': const FhirField(false, 'Meta'),
  },
  'ValueSet': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'url': const FhirField(false, 'FhirUri'),
    'urlElement': const FhirField(false, 'Element'),
    'identifier': const FhirField(true, 'Identifier'),
    'version': const FhirField(false, 'String'),
    'versionElement': const FhirField(false, 'Element'),
    'versionAlgorithmString': const FhirField(false, 'String'),
    'versionAlgorithmStringElement': const FhirField(false, 'Element'),
    'versionAlgorithmCoding': const FhirField(false, 'Coding'),
    'name': const FhirField(false, 'String'),
    'nameElement': const FhirField(false, 'Element'),
    'title': const FhirField(false, 'String'),
    'titleElement': const FhirField(false, 'Element'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'experimental': const FhirField(false, 'Boolean'),
    'experimentalElement': const FhirField(false, 'Element'),
    'date': const FhirField(false, 'FhirDateTime'),
    'dateElement': const FhirField(false, 'Element'),
    'publisher': const FhirField(false, 'String'),
    'publisherElement': const FhirField(false, 'Element'),
    'contact': const FhirField(true, 'ContactDetail'),
    'description': const FhirField(false, 'Markdown'),
    'descriptionElement': const FhirField(false, 'Element'),
    'useContext': const FhirField(true, 'UsageContext'),
    'jurisdiction': const FhirField(true, 'CodeableConcept'),
    'purpose': const FhirField(false, 'Markdown'),
    'purposeElement': const FhirField(false, 'Element'),
    'copyright': const FhirField(false, 'Markdown'),
    'copyrightElement': const FhirField(false, 'Element'),
    'copyrightLabel': const FhirField(false, 'String'),
    'copyrightLabelElement': const FhirField(false, 'Element'),
    'approvalDate': const FhirField(false, 'Date'),
    'approvalDateElement': const FhirField(false, 'Element'),
    'lastReviewDate': const FhirField(false, 'Date'),
    'lastReviewDateElement': const FhirField(false, 'Element'),
    'effectivePeriod': const FhirField(false, 'Period'),
    'topic': const FhirField(true, 'CodeableConcept'),
    'author': const FhirField(true, 'ContactDetail'),
    'editor': const FhirField(true, 'ContactDetail'),
    'reviewer': const FhirField(true, 'ContactDetail'),
    'endorser': const FhirField(true, 'ContactDetail'),
    'relatedArtifact': const FhirField(true, 'RelatedArtifact'),
    'immutable': const FhirField(false, 'Boolean'),
    'immutableElement': const FhirField(false, 'Element'),
    'compose': const FhirField(false, 'ValueSetCompose'),
    'expansion': const FhirField(false, 'ValueSetExpansion'),
    'scope': const FhirField(false, 'ValueSetScope'),
  },
  'ValueSetCompose': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'lockedDate': const FhirField(false, 'Date'),
    'lockedDateElement': const FhirField(false, 'Element'),
    'inactive': const FhirField(false, 'Boolean'),
    'inactiveElement': const FhirField(false, 'Element'),
    'include': const FhirField(true, 'ValueSetInclude'),
    'exclude': const FhirField(true, 'ValueSetInclude'),
    'property': const FhirField(true, 'String'),
    'propertyElement': const FhirField(true, 'Element'),
  },
  'ValueSetInclude': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'system': const FhirField(false, 'FhirUri'),
    'systemElement': const FhirField(false, 'Element'),
    'version': const FhirField(false, 'String'),
    'versionElement': const FhirField(false, 'Element'),
    'concept': const FhirField(true, 'ValueSetConcept'),
    'filter': const FhirField(true, 'ValueSetFilter'),
    'valueSet': const FhirField(true, 'Canonical'),
    'copyright': const FhirField(false, 'String'),
    'copyrightElement': const FhirField(false, 'Element'),
  },
  'ValueSetConcept': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(false, 'Code'),
    'codeElement': const FhirField(false, 'Element'),
    'display': const FhirField(false, 'String'),
    'displayElement': const FhirField(false, 'Element'),
    'designation': const FhirField(true, 'ValueSetDesignation'),
  },
  'ValueSetDesignation': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'use': const FhirField(false, 'Coding'),
    'additionalUse': const FhirField(true, 'Coding'),
    'value': const FhirField(false, 'String'),
    'valueElement': const FhirField(false, 'Element'),
  },
  'ValueSetFilter': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'property': const FhirField(false, 'Code'),
    'propertyElement': const FhirField(false, 'Element'),
    'op': const FhirField(false, 'Code'),
    'opElement': const FhirField(false, 'Element'),
    'value': const FhirField(false, 'String'),
    'valueElement': const FhirField(false, 'Element'),
  },
  'ValueSetExpansion': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(false, 'FhirUri'),
    'identifierElement': const FhirField(false, 'Element'),
    'next': const FhirField(false, 'FhirUri'),
    'nextElement': const FhirField(false, 'Element'),
    'timestamp': const FhirField(false, 'FhirDateTime'),
    'timestampElement': const FhirField(false, 'Element'),
    'total': const FhirField(false, 'Integer'),
    'totalElement': const FhirField(false, 'Element'),
    'offset': const FhirField(false, 'Integer'),
    'offsetElement': const FhirField(false, 'Element'),
    'parameter': const FhirField(true, 'ValueSetParameter'),
    'property': const FhirField(true, 'ValueSetProperty'),
    'contains': const FhirField(true, 'ValueSetContains'),
  },
  'ValueSetParameter': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'name': const FhirField(false, 'String'),
    'nameElement': const FhirField(false, 'Element'),
    'valueString': const FhirField(false, 'String'),
    'valueStringElement': const FhirField(false, 'Element'),
    'valueBoolean': const FhirField(false, 'Boolean'),
    'valueBooleanElement': const FhirField(false, 'Element'),
    'valueInteger': const FhirField(false, 'Integer'),
    'valueIntegerElement': const FhirField(false, 'Element'),
    'valueDecimal': const FhirField(false, 'Decimal'),
    'valueDecimalElement': const FhirField(false, 'Element'),
    'valueUri': const FhirField(false, 'FhirUri'),
    'valueUriElement': const FhirField(false, 'Element'),
    'valueCode': const FhirField(false, 'Code'),
    'valueCodeElement': const FhirField(false, 'Element'),
    'valueDateTime': const FhirField(false, 'FhirDateTime'),
    'valueDateTimeElement': const FhirField(false, 'Element'),
  },
  'ValueSetProperty': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(false, 'Code'),
    'codeElement': const FhirField(false, 'Element'),
    'uri': const FhirField(false, 'FhirUri'),
    'uriElement': const FhirField(false, 'Element'),
  },
  'ValueSetContains': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'system': const FhirField(false, 'FhirUri'),
    'systemElement': const FhirField(false, 'Element'),
    'abstract': const FhirField(false, 'Boolean'),
    'abstractElement': const FhirField(false, 'Element'),
    'inactive': const FhirField(false, 'Boolean'),
    'inactiveElement': const FhirField(false, 'Element'),
    'version': const FhirField(false, 'String'),
    'versionElement': const FhirField(false, 'Element'),
    'code': const FhirField(false, 'Code'),
    'codeElement': const FhirField(false, 'Element'),
    'display': const FhirField(false, 'String'),
    'displayElement': const FhirField(false, 'Element'),
    'designation': const FhirField(true, 'ValueSetDesignation'),
    'property': const FhirField(true, 'ValueSetProperty1'),
    'contains': const FhirField(true, 'ValueSetContains'),
  },
  'ValueSetProperty1': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(false, 'Code'),
    'codeElement': const FhirField(false, 'Element'),
    'valueCode': const FhirField(false, 'Code'),
    'valueCodeElement': const FhirField(false, 'Element'),
    'valueCoding': const FhirField(false, 'Coding'),
    'valueString': const FhirField(false, 'String'),
    'valueStringElement': const FhirField(false, 'Element'),
    'valueInteger': const FhirField(false, 'Integer'),
    'valueIntegerElement': const FhirField(false, 'Element'),
    'valueBoolean': const FhirField(false, 'Boolean'),
    'valueBooleanElement': const FhirField(false, 'Element'),
    'valueDateTime': const FhirField(false, 'FhirDateTime'),
    'valueDateTimeElement': const FhirField(false, 'Element'),
    'valueDecimal': const FhirField(false, 'Decimal'),
    'valueDecimalElement': const FhirField(false, 'Element'),
    'subProperty': const FhirField(true, 'ValueSetSubProperty'),
  },
  'ValueSetSubProperty': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'code': const FhirField(false, 'Code'),
    'codeElement': const FhirField(false, 'Element'),
    'valueCode': const FhirField(false, 'Code'),
    'valueCodeElement': const FhirField(false, 'Element'),
    'valueCoding': const FhirField(false, 'Coding'),
    'valueString': const FhirField(false, 'String'),
    'valueStringElement': const FhirField(false, 'Element'),
    'valueInteger': const FhirField(false, 'Integer'),
    'valueIntegerElement': const FhirField(false, 'Element'),
    'valueBoolean': const FhirField(false, 'Boolean'),
    'valueBooleanElement': const FhirField(false, 'Element'),
    'valueDateTime': const FhirField(false, 'FhirDateTime'),
    'valueDateTimeElement': const FhirField(false, 'Element'),
    'valueDecimal': const FhirField(false, 'Decimal'),
    'valueDecimalElement': const FhirField(false, 'Element'),
  },
  'ValueSetScope': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'inclusionCriteria': const FhirField(false, 'String'),
    'inclusionCriteriaElement': const FhirField(false, 'Element'),
    'exclusionCriteria': const FhirField(false, 'String'),
    'exclusionCriteriaElement': const FhirField(false, 'Element'),
  },
  'VerificationResult': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'target': const FhirField(true, 'Reference'),
    'targetLocation': const FhirField(true, 'String'),
    'targetLocationElement': const FhirField(true, 'Element'),
    'need': const FhirField(false, 'CodeableConcept'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'statusDate': const FhirField(false, 'FhirDateTime'),
    'statusDateElement': const FhirField(false, 'Element'),
    'validationType': const FhirField(false, 'CodeableConcept'),
    'validationProcess': const FhirField(true, 'CodeableConcept'),
    'frequency': const FhirField(false, 'Timing'),
    'lastPerformed': const FhirField(false, 'FhirDateTime'),
    'lastPerformedElement': const FhirField(false, 'Element'),
    'nextScheduled': const FhirField(false, 'Date'),
    'nextScheduledElement': const FhirField(false, 'Element'),
    'failureAction': const FhirField(false, 'CodeableConcept'),
    'primarySource': const FhirField(true, 'VerificationResultPrimarySource'),
    'attestation': const FhirField(false, 'VerificationResultAttestation'),
    'validator': const FhirField(true, 'VerificationResultValidator'),
  },
  'VerificationResultPrimarySource': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'who': const FhirField(false, 'Reference'),
    'type': const FhirField(true, 'CodeableConcept'),
    'communicationMethod': const FhirField(true, 'CodeableConcept'),
    'validationStatus': const FhirField(false, 'CodeableConcept'),
    'validationDate': const FhirField(false, 'FhirDateTime'),
    'validationDateElement': const FhirField(false, 'Element'),
    'canPushUpdates': const FhirField(false, 'CodeableConcept'),
    'pushTypeAvailable': const FhirField(true, 'CodeableConcept'),
  },
  'VerificationResultAttestation': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'who': const FhirField(false, 'Reference'),
    'onBehalfOf': const FhirField(false, 'Reference'),
    'communicationMethod': const FhirField(false, 'CodeableConcept'),
    'date': const FhirField(false, 'Date'),
    'dateElement': const FhirField(false, 'Element'),
    'sourceIdentityCertificate': const FhirField(false, 'String'),
    'sourceIdentityCertificateElement': const FhirField(false, 'Element'),
    'proxyIdentityCertificate': const FhirField(false, 'String'),
    'proxyIdentityCertificateElement': const FhirField(false, 'Element'),
    'proxySignature': const FhirField(false, 'Signature'),
    'sourceSignature': const FhirField(false, 'Signature'),
  },
  'VerificationResultValidator': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'organization': const FhirField(false, 'Reference'),
    'identityCertificate': const FhirField(false, 'String'),
    'identityCertificateElement': const FhirField(false, 'Element'),
    'attestationSignature': const FhirField(false, 'Signature'),
  },
  'VisionPrescription': <String, FhirField>{
    'id': const FhirField(false, 'Id'),
    'meta': const FhirField(false, 'Meta'),
    'implicitRules': const FhirField(false, 'FhirUri'),
    'implicitRulesElement': const FhirField(false, 'Element'),
    'language': const FhirField(false, 'Code'),
    'languageElement': const FhirField(false, 'Element'),
    'text': const FhirField(false, 'Narrative'),
    'contained': const FhirField(true, 'ResourceList'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'identifier': const FhirField(true, 'Identifier'),
    'status': const FhirField(false, 'Code'),
    'statusElement': const FhirField(false, 'Element'),
    'created': const FhirField(false, 'FhirDateTime'),
    'createdElement': const FhirField(false, 'Element'),
    'patient': const FhirField(false, 'Reference'),
    'encounter': const FhirField(false, 'Reference'),
    'dateWritten': const FhirField(false, 'FhirDateTime'),
    'dateWrittenElement': const FhirField(false, 'Element'),
    'prescriber': const FhirField(false, 'Reference'),
    'lensSpecification':
        const FhirField(true, 'VisionPrescriptionLensSpecification'),
  },
  'VisionPrescriptionLensSpecification': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'product': const FhirField(false, 'CodeableConcept'),
    'eye': const FhirField(false, 'Code'),
    'eyeElement': const FhirField(false, 'Element'),
    'sphere': const FhirField(false, 'Decimal'),
    'sphereElement': const FhirField(false, 'Element'),
    'cylinder': const FhirField(false, 'Decimal'),
    'cylinderElement': const FhirField(false, 'Element'),
    'axis': const FhirField(false, 'Integer'),
    'axisElement': const FhirField(false, 'Element'),
    'prism': const FhirField(true, 'VisionPrescriptionPrism'),
    'add': const FhirField(false, 'Decimal'),
    'addElement': const FhirField(false, 'Element'),
    'power': const FhirField(false, 'Decimal'),
    'powerElement': const FhirField(false, 'Element'),
    'backCurve': const FhirField(false, 'Decimal'),
    'backCurveElement': const FhirField(false, 'Element'),
    'diameter': const FhirField(false, 'Decimal'),
    'diameterElement': const FhirField(false, 'Element'),
    'duration': const FhirField(false, 'Quantity'),
    'color': const FhirField(false, 'String'),
    'colorElement': const FhirField(false, 'Element'),
    'brand': const FhirField(false, 'String'),
    'brandElement': const FhirField(false, 'Element'),
    'note': const FhirField(true, 'Annotation'),
  },
  'VisionPrescriptionPrism': <String, FhirField>{
    'id': const FhirField(false, 'String'),
    'extension': const FhirField(true, 'FhirExtension'),
    'modifierExtension': const FhirField(true, 'FhirExtension'),
    'amount': const FhirField(false, 'Decimal'),
    'amountElement': const FhirField(false, 'Element'),
    'base': const FhirField(false, 'Code'),
    'baseElement': const FhirField(false, 'Element'),
  },
};
